@@ -57,7 +57,7 @@
 #include <math.h>
 #include "allheaders.h"
 
-static void test_gif(const char *fname, L_REGPARAMS *rp);
+static void test_gif(const char *fname, PIXA *pixa, L_REGPARAMS *rp);
 static l_int32 test_mem_gif(const char *fname, l_int32 index);
 
 
@@ -75,12 +75,12 @@ static l_int32 test_mem_gif(const char *fname, l_int32 index);
 #define   FILE_16BPP    "test16.tif"
 #define   FILE_32BPP    "marge.jpg"
 
-#define   REDUCTION     1
-
 int main(int    argc,
          char **argv)
 {
 l_int32       success;
+PIX          *pix;
+PIXA         *pixa;
 L_REGPARAMS  *rp;
 
 #if !HAVE_LIBGIF && !HAVE_LIBUNGIF
@@ -93,34 +93,40 @@ L_REGPARAMS  *rp;
 
     if (regTestSetup(argc, argv, &rp))
         return 1;
-    pixDisplayWrite(NULL, -1);
+
+        /* Set up for display output */
+    pixa = (rp->display) ? pixaCreate(0) : NULL;
 
     lept_rmdir("lept/gif");
     lept_mkdir("lept/gif");
 
     /* ------------ Part 1: Test lossless r/w to file ------------*/
-    test_gif(FILE_1BPP, rp);
-    test_gif(FILE_2BPP, rp);
-    test_gif(FILE_4BPP, rp);
-    test_gif(FILE_8BPP_1, rp);
-    test_gif(FILE_8BPP_2, rp);
-    test_gif(FILE_8BPP_3, rp);
-    test_gif(FILE_16BPP, rp);
-    test_gif(FILE_32BPP, rp);
-    if (rp->success)
+    test_gif(FILE_1BPP, pixa, rp);
+    test_gif(FILE_2BPP, pixa, rp);
+    test_gif(FILE_4BPP, pixa, rp);
+    test_gif(FILE_8BPP_1, pixa, rp);
+    test_gif(FILE_8BPP_2, pixa, rp);
+    test_gif(FILE_8BPP_3, pixa, rp);
+    test_gif(FILE_16BPP, pixa, rp);
+    test_gif(FILE_32BPP, pixa, rp);
+    if (rp->success) {
         fprintf(stderr,
             "\n  ****** Success on lossless r/w to file *****\n\n");
-    else
+    } else {
         fprintf(stderr,
             "\n  ******* Failure on at least one r/w to file ******\n\n");
+    }
 
-    if (rp->display)
-        pixDisplayMultiple("/tmp/display/file*");
+    if (rp->display) {
+        pix = pixaDisplayTiledAndScaled(pixa, 32, 450, 3, 0, 20, 2);
+        pixWrite("/tmp/lept/gif/giftest.jpg", pix, IFF_JFIF_JPEG);
+        pixDisplay(pix, 100, 100);
+        pixDestroy(&pix);
+        pixaDestroy(&pixa);
+    }
 
     /* ------------ Part 2: Test lossless r/w to memory ------------ */
     success = TRUE;
-#if HAVE_FMEMOPEN
-    pixDisplayWrite(NULL, -1);
     if (test_mem_gif(FILE_1BPP, 0)) success = FALSE;
     if (test_mem_gif(FILE_2BPP, 1)) success = FALSE;
     if (test_mem_gif(FILE_4BPP, 2)) success = FALSE;
@@ -129,17 +135,13 @@ L_REGPARAMS  *rp;
     if (test_mem_gif(FILE_8BPP_3, 5)) success = FALSE;
     if (test_mem_gif(FILE_16BPP, 6)) success = FALSE;
     if (test_mem_gif(FILE_32BPP, 7)) success = FALSE;
-    if (success)
+    if (success) {
         fprintf(stderr,
             "\n  ****** Success on lossless r/w to memory *****\n\n");
-    else
+    } else {
         fprintf(stderr,
             "\n  ******* Failure on at least one r/w to memory ******\n\n");
-
-#else
-        fprintf(stderr,
-            "\n  *****  r/w to memory not enabled *****\n\n");
-#endif  /*  HAVE_FMEMOPEN  */
+    }
 
         /* Success only if all tests are passed */
     if (rp->success == TRUE) rp->success = success;
@@ -150,6 +152,7 @@ L_REGPARAMS  *rp;
 
 static void
 test_gif(const char   *fname,
+         PIXA         *pixa,
          L_REGPARAMS  *rp)
 {
 char     buf[256];
@@ -165,6 +168,7 @@ PIX     *pixs, *pix1, *pix2;
     pix2 = pixRead(buf);
     regTestWritePixAndCheck(rp, pix2, IFF_GIF);
     pixEqual(pixs, pix2, &same);
+
     if (!same && rp->index < 6) {
         fprintf(stderr, "Error for %s\n", fname);
         rp->success = FALSE;
@@ -173,7 +177,7 @@ PIX     *pixs, *pix1, *pix2;
         fprintf(stderr,
                 " depth: pixs = %d, pix1 = %d\n", pixGetDepth(pixs),
                 pixGetDepth(pix1));
-        pixDisplayWrite(pix2, REDUCTION);
+        pixaAddPix(pixa, pix2, L_CLONE);
     }
     pixDestroy(&pixs);
     pixDestroy(&pix1);
@@ -194,15 +198,15 @@ PIX      *pixs;
 PIX      *pixd = NULL;
 
     if ((pixs = pixRead(fname)) == NULL) {
-        fprintf(stderr, "Failure to read %s\n", fname);
+        fprintf(stderr, "Failure to read gif file: %s\n", fname);
         return 1;
     }
     if (pixWriteMem(&data, &size, pixs, IFF_GIF)) {
-        fprintf(stderr, "Mem write fail for gif\n");
+        fprintf(stderr, "Mem gif write fail on image %d\n", index);
         return 1;
     }
     if ((pixd = pixReadMem(data, size)) == NULL) {
-        fprintf(stderr, "Mem read fail for gif\n");
+        fprintf(stderr, "Mem gif read fail on image %d\n", index);
         lept_free(data);
         return 1;
     }
@@ -212,7 +216,7 @@ PIX      *pixd = NULL;
     pixDestroy(&pixd);
     lept_free(data);
     if (!same && index < 6) {
-        fprintf(stderr, "Mem write/read fail for file %s\n", fname);
+        fprintf(stderr, "Mem gif write/read fail for file %s\n", fname);
         return 1;
     }
     else
