@@ -61,9 +61,9 @@ static const char *textsel4 = "xxxxxx"
 int main(int    argc,
          char **argv)
 {
-PIX         *pixd;
-SEL         *sel1, *sel2, *sel3, *sel4;
-SELA        *sela;
+PIX   *pix1;
+SEL   *sel1, *sel2, *sel3, *sel4;
+SELA  *sela;
 
     sel1 = selCreateFromString(textsel1, 5, 6, NULL);
     sel2 = selCreateFromString(textsel2, 5, 6, NULL);
@@ -76,11 +76,12 @@ SELA        *sela;
     selaAddSel(sela, sel3, "textsel3", L_INSERT);
     selaAddSel(sela, sel4, "textsel4", L_INSERT);
 
-    pixd = selaDisplayInPix(sela, 28, 3, 30, 4);
-    pixWrite("/tmp/orient.png", pixd, IFF_PNG);
-    pixDisplay(pixd, 100, 100);
+    lept_mkdir("lept/livre");
+    pix1 = selaDisplayInPix(sela, 28, 3, 30, 4);
+    pixWrite("/tmp/lept/livre/orient.png", pix1, IFF_PNG);
+    pixDisplay(pix1, 1200, 1200);
 
-    pixDestroy(&pixd);
+    pixDestroy(&pix1);
     selaDestroy(&sela);
     return 0;
 }
