@@ -49,11 +49,10 @@
      *    GPLOT_PNG
      *    GPLOT_PS
      *    GPLOT_EPS
-     *    GPLOT_X11
      *    GPLOT_LATEX
      */
-#define  GPLOT_OUTPUT   GPLOT_X11
 
+#define  GPLOT_OUTPUT   GPLOT_PNG
 
 int main(int    argc,
          char **argv)
@@ -69,7 +68,7 @@ static char  mainName[] = "plottest";
     if (argc != 1)
         return ERROR_INT(" Syntax:  plottest", mainName, 1);
 
-    lept_mkdir("plot");
+    lept_mkdir("lept/plot");
 
         /* Generate plot data */
     nax = numaCreate(0);
@@ -86,7 +85,7 @@ static char  mainName[] = "plottest";
     }
 
         /* Show the plot */
-    gplot1 = gplotCreate("/tmp/plot/root1", GPLOT_OUTPUT, "Example plots",
+    gplot1 = gplotCreate("/tmp/lept/plot/set1", GPLOT_OUTPUT, "Example plots",
                          "theta", "f(theta)");
     gplotAddPlot(gplot1, nax, nay1, GPLOT_STYLE, "sin (2.4 * theta)");
     gplotAddPlot(gplot1, nax, nay2, GPLOT_STYLE, "cos (2.4 * theta)");
@@ -94,20 +93,21 @@ static char  mainName[] = "plottest";
 
         /* Also save the plot to png */
     gplot1->outformat = GPLOT_PNG;
-    pngname = genPathname("/tmp/plot", "root1.png");
+    pngname = genPathname("/tmp/lept/plot", "set1.png");
     stringReplace(&gplot1->outname, pngname);
     gplotMakeOutput(gplot1);
+    l_fileDisplay("/tmp/lept/plot/set1.png", 100, 100, 1.0);
     lept_free(pngname);
 
         /* Test gplot serialization */
-    gplotWrite("/tmp/plot/gplot1", gplot1);
-    if ((gplot2 = gplotRead("/tmp/plot/gplot1")) == NULL)
+    gplotWrite("/tmp/lept/plot/plot1.gp", gplot1);
+    if ((gplot2 = gplotRead("/tmp/lept/plot/plot1.gp")) == NULL)
         return ERROR_INT("gplotRead failure!", mainName, 1);
-    gplotWrite("/tmp/plot/gplot2", gplot2);
+    gplotWrite("/tmp/lept/plot/plot2.gp", gplot2);
 
         /* Are the two written gplot files the same? */
-    str1 = (char *)l_binaryRead("/tmp/plot/gplot1", &size1);
-    str2 = (char *)l_binaryRead("/tmp/plot/gplot2", &size2);
+    str1 = (char *)l_binaryRead("/tmp/lept/plot/plot1.gp", &size1);
+    str2 = (char *)l_binaryRead("/tmp/lept/plot/plot2.gp", &size2);
     if (size1 != size2)
         fprintf(stderr, "Error: size1 = %lu, size2 = %lu\n",
                 (unsigned long)size1, (unsigned long)size2);
@@ -121,22 +121,23 @@ static char  mainName[] = "plottest";
     lept_free(str2);
 
         /* Read from file and regenerate the plot */
-    gplot3 = gplotRead("/tmp/plot/gplot2");
+    gplot3 = gplotRead("/tmp/lept/plot/plot2.gp");
     stringReplace(&gplot3->title , "Example plots regen");
-    gplot3->outformat = GPLOT_X11;
+    gplot3->outformat = GPLOT_PNG;
     gplotMakeOutput(gplot3);
 
         /* Build gplot but do not make the output formatted stuff */
-    gplot4 = gplotCreate("/tmp/plot/root2", GPLOT_OUTPUT, "Example plots 2",
-                         "theta", "f(theta)");
+    gplot4 = gplotCreate("/tmp/lept/plot/set2", GPLOT_OUTPUT,
+                         "Example plots 2", "theta", "f(theta)");
     gplotAddPlot(gplot4, nax, nay1, GPLOT_STYLE, "sin (2.4 * theta)");
     gplotAddPlot(gplot4, nax, nay2, GPLOT_STYLE, "cos (2.4 * theta)");
 
         /* Write, read back, and generate the plot */
-    gplotWrite("/tmp/plot/gplot4", gplot4);
-    if ((gplot5 = gplotRead("/tmp/plot/gplot4")) == NULL)
+    gplotWrite("/tmp/lept/plot/plot4.gp", gplot4);
+    if ((gplot5 = gplotRead("/tmp/lept/plot/plot4.gp")) == NULL)
         return ERROR_INT("gplotRead failure!", mainName, 1);
     gplotMakeOutput(gplot5);
+    l_fileDisplay("/tmp/lept/plot/set2.png", 750, 100, 1.0);
 
     gplotDestroy(&gplot1);
     gplotDestroy(&gplot2);
