@@ -25,8 +25,9 @@
  *====================================================================*/
 
 
-/*
- *  affine.c
+/*!
+ * \file affine.c
+ * <pre>
  *
  *      Affine (3 pt) image transformation using a sampled
  *      (to nearest integer) transform on each dest point
@@ -159,10 +160,10 @@
  *      that is undergoing a general affine transformation.)
  *
  *      Choose the 3 corresponding points as follows:
- *         - The 1st point is an origin
- *         - The 2nd point gives the orientation and scaling of the
+ *         ~ The 1st point is an origin
+ *         ~ The 2nd point gives the orientation and scaling of the
  *           "x" axis with respect to the origin
- *         - The 3rd point does likewise for the "y" axis.
+ *         ~ The 3rd point does likewise for the "y" axis.
  *      These "axes" must not be collinear; otherwise they are
  *      arbitrary (although some strange things will happen if
  *      the handedness sweeping through the minimum angle between
@@ -227,6 +228,7 @@
  *               interpolated   1.8
  *      Additionally, the computation time/pixel is nearly the same
  *      for 8 bpp and 32 bpp, for both sampled and interpolated.
+ * </pre>
  */
 
 
@@ -245,15 +247,16 @@ extern l_float32  AlphaMaskBorderVals[2];
  *               Sampled affine image transformation           *
  *-------------------------------------------------------------*/
 /*!
- *  pixAffineSampledPta()
+ * \brief   pixAffineSampledPta()
  *
- *      Input:  pixs (all depths)
- *              ptad  (3 pts of final coordinate space)
- *              ptas  (3 pts of initial coordinate space)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths
+ * \param[in]    ptad  3 pts of final coordinate space
+ * \param[in]    ptas  3 pts of initial coordinate space
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary.
  *      (2) Retains colormap, which you can do for a sampled transform..
  *      (3) The 3 points must not be collinear.
@@ -271,6 +274,7 @@ extern l_float32  AlphaMaskBorderVals[2];
  *          for relative timings between sampled and interpolated.
  *      (7) To repeat, use of the sequential transform,
  *          pixAffineSequential(), for any images, is discouraged.
+ * </pre>
  */
 PIX *
 pixAffineSampledPta(PIX     *pixs,
@@ -299,26 +303,28 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getAffineXformCoeffs(ptad, ptas, &vc);
     pixd = pixAffineSampled(pixs, vc, incolor);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
 
 
 /*!
- *  pixAffineSampled()
+ * \brief   pixAffineSampled()
  *
- *      Input:  pixs (all depths)
- *              vc  (vector of 6 coefficients for affine transformation)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths
+ * \param[in]    vc  vector of 6 coefficients for affine transformation
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary.
  *      (2) Retains colormap, which you can do for a sampled transform..
  *      (3) For 8 or 32 bpp, much better quality is obtained by the
  *          somewhat slower pixAffine().  See that function
  *          for relative timings between sampled and interpolated.
+ * </pre>
  */
 PIX *
 pixAffineSampled(PIX        *pixs,
@@ -399,17 +405,19 @@ PIXCMAP    *cmap;
  *               Interpolated affine image transformation              *
  *---------------------------------------------------------------------*/
 /*!
- *  pixAffinePta()
+ * \brief   pixAffinePta()
  *
- *      Input:  pixs (all depths; colormap ok)
- *              ptad  (3 pts of final coordinate space)
- *              ptas  (3 pts of initial coordinate space)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths; colormap ok
+ * \param[in]    ptad  3 pts of final coordinate space
+ * \param[in]    ptas  3 pts of initial coordinate space
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary
  *      (2) Removes any existing colormap, if necessary, before transforming
+ * </pre>
  */
 PIX *
 pixAffinePta(PIX     *pixs,
@@ -468,16 +476,18 @@ PIX      *pixt1, *pixt2, *pixd;
 
 
 /*!
- *  pixAffine()
+ * \brief   pixAffine()
  *
- *      Input:  pixs (all depths; colormap ok)
- *              vc  (vector of 6 coefficients for affine transformation)
- *              incolor (L_BRING_IN_WHITE, L_BRING_IN_BLACK)
- *      Return: pixd, or null on error
+ * \param[in]    pixs all depths; colormap ok
+ * \param[in]    vc  vector of 6 coefficients for affine transformation
+ * \param[in]    incolor L_BRING_IN_WHITE, L_BRING_IN_BLACK
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Brings in either black or white pixels from the boundary
  *      (2) Removes any existing colormap, if necessary, before transforming
+ * </pre>
  */
 PIX *
 pixAffine(PIX        *pixs,
@@ -527,13 +537,13 @@ PIX      *pixt1, *pixt2, *pixd;
 
 
 /*!
- *  pixAffinePtaColor()
+ * \brief   pixAffinePtaColor()
  *
- *      Input:  pixs (32 bpp)
- *              ptad  (3 pts of final coordinate space)
- *              ptas  (3 pts of initial coordinate space)
- *              colorval (e.g., 0 to bring in BLACK, 0xffffff00 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 32 bpp
+ * \param[in]    ptad  3 pts of final coordinate space
+ * \param[in]    ptas  3 pts of initial coordinate space
+ * \param[in]    colorval e.g., 0 to bring in BLACK, 0xffffff00 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixAffinePtaColor(PIX      *pixs,
@@ -562,19 +572,19 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getAffineXformCoeffs(ptad, ptas, &vc);
     pixd = pixAffineColor(pixs, vc, colorval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
 
 
 /*!
- *  pixAffineColor()
+ * \brief   pixAffineColor()
  *
- *      Input:  pixs (32 bpp)
- *              vc  (vector of 6 coefficients for affine transformation)
- *              colorval (e.g., 0 to bring in BLACK, 0xffffff00 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 32 bpp
+ * \param[in]    vc  vector of 6 coefficients for affine transformation
+ * \param[in]    colorval e.g., 0 to bring in BLACK, 0xffffff00 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixAffineColor(PIX        *pixs,
@@ -630,13 +640,13 @@ PIX       *pix1, *pix2, *pixd;
 
 
 /*!
- *  pixAffinePtaGray()
+ * \brief   pixAffinePtaGray()
  *
- *      Input:  pixs (8 bpp)
- *              ptad  (3 pts of final coordinate space)
- *              ptas  (3 pts of initial coordinate space)
- *              grayval (0 to bring in BLACK, 255 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 8 bpp
+ * \param[in]    ptad  3 pts of final coordinate space
+ * \param[in]    ptas  3 pts of initial coordinate space
+ * \param[in]    grayval 0 to bring in BLACK, 255 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixAffinePtaGray(PIX     *pixs,
@@ -665,7 +675,7 @@ PIX        *pixd;
         /* Get backwards transform from dest to src, and apply it */
     getAffineXformCoeffs(ptad, ptas, &vc);
     pixd = pixAffineGray(pixs, vc, grayval);
-    FREE(vc);
+    LEPT_FREE(vc);
 
     return pixd;
 }
@@ -673,12 +683,12 @@ PIX        *pixd;
 
 
 /*!
- *  pixAffineGray()
+ * \brief   pixAffineGray()
  *
- *      Input:  pixs (8 bpp)
- *              vc  (vector of 6 coefficients for affine transformation)
- *              grayval (0 to bring in BLACK, 255 for WHITE)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 8 bpp
+ * \param[in]    vc  vector of 6 coefficients for affine transformation
+ * \param[in]    grayval 0 to bring in BLACK, 255 for WHITE
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixAffineGray(PIX        *pixs,
@@ -723,29 +733,30 @@ PIX       *pixd;
 
 
 /*---------------------------------------------------------------------------*
- *   Affine transform including alpha (blend) component and gamma transform  *
+ *            Affine transform including alpha (blend) component             *
  *---------------------------------------------------------------------------*/
 /*!
- *  pixAffinePtaWithAlpha()
+ * \brief   pixAffinePtaWithAlpha()
  *
- *      Input:  pixs (32 bpp rgb)
- *              ptad  (3 pts of final coordinate space)
- *              ptas  (3 pts of initial coordinate space)
- *              pixg (<optional> 8 bpp, can be null)
- *              fract (between 0.0 and 1.0, with 0.0 fully transparent
- *                     and 1.0 fully opaque)
- *              border (of pixels added to capture transformed source pixels)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 32 bpp rgb
+ * \param[in]    ptad  3 pts of final coordinate space
+ * \param[in]    ptas  3 pts of initial coordinate space
+ * \param[in]    pixg [optional] 8 bpp, can be null
+ * \param[in]    fract between 0.0 and 1.0, with 0.0 fully transparent
+ *                     and 1.0 fully opaque
+ * \param[in]    border of pixels added to capture transformed source pixels
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The alpha channel is transformed separately from pixs,
  *          and aligns with it, being fully transparent outside the
  *          boundary of the transformed pixs.  For pixels that are fully
  *          transparent, a blending function like pixBlendWithGrayMask()
  *          will give zero weight to corresponding pixels in pixs.
  *      (2) If pixg is NULL, it is generated as an alpha layer that is
- *          partially opaque, using @fract.  Otherwise, it is cropped
- *          to pixs if required and @fract is ignored.  The alpha channel
+ *          partially opaque, using %fract.  Otherwise, it is cropped
+ *          to pixs if required and %fract is ignored.  The alpha channel
  *          in pixs is never used.
  *      (3) Colormaps are removed.
  *      (4) When pixs is transformed, it doesn't matter what color is brought
@@ -753,7 +764,7 @@ PIX       *pixd;
  *      (5) To avoid losing source pixels in the destination, it may be
  *          necessary to add a border to the source pix before doing
  *          the affine transformation.  This can be any non-negative number.
- *      (6) The input @ptad and @ptas are in a coordinate space before
+ *      (6) The input %ptad and %ptas are in a coordinate space before
  *          the border is added.  Internally, we compensate for this
  *          before doing the affine transform on the image after the border
  *          is added.
@@ -765,17 +776,7 @@ PIX       *pixd;
  *              with an image below, and
  *          (b) softens the edges by weakening the aliasing there.
  *          Use l_setAlphaMaskBorder() to change these values.
- *      (8) A subtle use of gamma correction is to remove gamma correction
- *          before scaling and restore it afterwards.  This is done
- *          by sandwiching this function between a gamma/inverse-gamma
- *          photometric transform:
- *              pixt = pixGammaTRCWithAlpha(NULL, pixs, 1.0 / gamma, 0, 255);
- *              pixd = pixAffinePtaWithAlpha(pixg, ptad, ptas, NULL,
- *                                           fract, border);
- *              pixGammaTRCWithAlpha(pixd, pixd, gamma, 0, 255);
- *              pixDestroy(&pixt);
- *          This has the side-effect of producing artifacts in the very
- *          dark regions.
+ * </pre>
  */
 PIX *
 pixAffinePtaWithAlpha(PIX       *pixs,
@@ -797,7 +798,8 @@ PTA     *ptad2, *ptas2;
     if (d != 32 && pixGetColormap(pixs) == NULL)
         return (PIX *)ERROR_PTR("pixs not cmapped or 32 bpp", procName, NULL);
     if (pixg && pixGetDepth(pixg) != 8) {
-        L_WARNING("pixg not 8 bpp; using @fract transparent alpha\n", procName);
+        L_WARNING("pixg not 8 bpp; using 'fract' transparent alpha\n",
+                  procName);
         pixg = NULL;
     }
     if (!pixg && (fract < 0.0 || fract > 1.0)) {
@@ -855,16 +857,17 @@ PTA     *ptad2, *ptas2;
  *                 Affine coordinate transformation            *
  *-------------------------------------------------------------*/
 /*!
- *  getAffineXformCoeffs()
+ * \brief   getAffineXformCoeffs()
  *
- *      Input:  ptas  (source 3 points; unprimed)
- *              ptad  (transformed 3 points; primed)
- *              &vc   (<return> vector of coefficients of transform)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    ptas  source 3 points; unprimed
+ * \param[in]    ptad  transformed 3 points; primed
+ * \param[out]   pvc   vector of coefficients of transform
+ * \return  0 if OK; 1 on error
  *
+ * <pre>
  *  We have a set of six equations, describing the affine
- *  transformation that takes 3 points (ptas) into 3 other
- *  points (ptad).  These equations are:
+ *  transformation that takes 3 points ptas into 3 other
+ *  points ptad.  These equations are:
  *
  *          x1' = c[0]*x1 + c[1]*y1 + c[2]
  *          y1' = c[3]*x1 + c[4]*y1 + c[5]
@@ -894,13 +897,13 @@ PTA     *ptad2, *ptas2;
  *  These six equations are solved here for the coefficients C.
  *
  *  These six coefficients can then be used to find the dest
- *  point (x',y') corresponding to any src point (x,y), according
+ *  point x',y') corresponding to any src point (x,y, according
  *  to the equations
  *
  *           x' = c[0]x + c[1]y + c[2]
  *           y' = c[3]x + c[4]y + c[5]
  *
- *  that are implemented in affineXformPt().
+ *  that are implemented in affineXformPt.
  *
  *  !!!!!!!!!!!!!!!!!!   Very important   !!!!!!!!!!!!!!!!!!!!!!
  *
@@ -916,12 +919,13 @@ PTA     *ptad2, *ptas2;
  *
  *  For example, if you transform a boxa with a 3x3 affine matrix
  *  'mat', the analogous image transformation must use 'matinv':
- *
+ * \code
  *     boxad = boxaAffineTransform(boxas, mat);
  *     affineInvertXform(mat, &matinv);
  *     pixd = pixAffine(pixs, matinv, L_BRING_IN_WHITE);
- *
+ * \endcode
  *  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+ * </pre>
  */
 l_int32
 getAffineXformCoeffs(PTA         *ptas,
@@ -942,7 +946,7 @@ l_float32  *a[6];  /* 6x6 matrix A  */
     if (!pvc)
         return ERROR_INT("&vc not defined", procName, 1);
 
-    if ((b = (l_float32 *)CALLOC(6, sizeof(l_float32))) == NULL)
+    if ((b = (l_float32 *)LEPT_CALLOC(6, sizeof(l_float32))) == NULL)
         return ERROR_INT("b not made", procName, 1);
     *pvc = b;
 
@@ -954,7 +958,7 @@ l_float32  *a[6];  /* 6x6 matrix A  */
     ptaGetPt(ptad, 2, &b[4], &b[5]);
 
     for (i = 0; i < 6; i++)
-        if ((a[i] = (l_float32 *)CALLOC(6, sizeof(l_float32))) == NULL)
+        if ((a[i] = (l_float32 *)LEPT_CALLOC(6, sizeof(l_float32))) == NULL)
             return ERROR_INT("a[i] not made", procName, 1);
 
     a[0][0] = x1;
@@ -979,20 +983,21 @@ l_float32  *a[6];  /* 6x6 matrix A  */
     gaussjordan(a, b, 6);
 
     for (i = 0; i < 6; i++)
-        FREE(a[i]);
+        LEPT_FREE(a[i]);
 
     return 0;
 }
 
 
 /*!
- *  affineInvertXform()
+ * \brief   affineInvertXform()
  *
- *      Input:  vc (vector of 6 coefficients)
- *              *vci (<return> inverted transform)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    vc vector of 6 coefficients
+ * \param[out]   pvci inverted transform
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The 6 affine transform coefficients are the first
  *          two rows of a 3x3 matrix where the last row has
  *          only a 1 in the third column.  We invert this
@@ -1013,6 +1018,7 @@ l_float32  *a[6];  /* 6x6 matrix A  */
  *          and puts the solution for F, which is A'R,
  *          into the input R vector.
  *
+ * </pre>
  */
 l_int32
 affineInvertXform(l_float32   *vc,
@@ -1031,8 +1037,9 @@ l_float32   b[3] = {1.0, 1.0, 1.0};   /* anything; results ignored */
     if (!vc)
         return ERROR_INT("vc not defined", procName, 1);
 
+#if 1
     for (i = 0; i < 3; i++)
-        a[i] = (l_float32 *)CALLOC(3, sizeof(l_float32));
+        a[i] = (l_float32 *)LEPT_CALLOC(3, sizeof(l_float32));
     a[0][0] = vc[0];
     a[0][1] = vc[1];
     a[0][2] = vc[2];
@@ -1040,8 +1047,8 @@ l_float32   b[3] = {1.0, 1.0, 1.0};   /* anything; results ignored */
     a[1][1] = vc[4];
     a[1][2] = vc[5];
     a[2][2] = 1.0;
-    gaussjordan(a, b, 3);  /* now matrix a contains the inverse */
-    vci = (l_float32 *)CALLOC(6, sizeof(l_float32));
+    gaussjordan(a, b, 3);  /* this inverts matrix a */
+    vci = (l_float32 *)LEPT_CALLOC(6, sizeof(l_float32));
     *pvci = vci;
     vci[0] = a[0][0];
     vci[1] = a[0][1];
@@ -1049,26 +1056,33 @@ l_float32   b[3] = {1.0, 1.0, 1.0};   /* anything; results ignored */
     vci[3] = a[1][0];
     vci[4] = a[1][1];
     vci[5] = a[1][2];
+    for (i = 0; i < 3; i++)
+        LEPT_FREE(a[i]);
+
+#else
 
-#if 0
         /* Alternative version, inverting a 2x2 matrix */
+    { l_float32 *a2[2];
     for (i = 0; i < 2; i++)
-        a[i] = (l_float32 *)CALLOC(2, sizeof(l_float32));
-    a[0][0] = vc[0];
-    a[0][1] = vc[1];
-    a[1][0] = vc[3];
-    a[1][1] = vc[4];
+        a2[i] = (l_float32 *)LEPT_CALLOC(2, sizeof(l_float32));
+    a2[0][0] = vc[0];
+    a2[0][1] = vc[1];
+    a2[1][0] = vc[3];
+    a2[1][1] = vc[4];
     b[0] = vc[2];
     b[1] = vc[5];
-    gaussjordan(a, b, 2);  /* now matrix a contains the inverse */
-    vci = (l_float32 *)CALLOC(6, sizeof(l_float32));
+    gaussjordan(a2, b, 2);  /* this inverts matrix a2 */
+    vci = (l_float32 *)LEPT_CALLOC(6, sizeof(l_float32));
     *pvci = vci;
-    vci[0] = a[0][0];
-    vci[1] = a[0][1];
+    vci[0] = a2[0][0];
+    vci[1] = a2[0][1];
     vci[2] = -b[0];   /* note sign */
-    vci[3] = a[1][0];
-    vci[4] = a[1][1];
+    vci[3] = a2[1][0];
+    vci[4] = a2[1][1];
     vci[5] = -b[1];   /* note sign */
+    for (i = 0; i < 2; i++)
+        LEPT_FREE(a2[i]);
+    }
 #endif
 
     return 0;
@@ -1076,16 +1090,18 @@ l_float32   b[3] = {1.0, 1.0, 1.0};   /* anything; results ignored */
 
 
 /*!
- *  affineXformSampledPt()
+ * \brief   affineXformSampledPt()
  *
- *      Input:  vc (vector of 6 coefficients)
- *              (x, y)  (initial point)
- *              (&xp, &yp)   (<return> transformed point)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    vc vector of 6 coefficients
+ * \param[in]    x, y  initial point
+ * \param[out]   pxp, pyp   transformed point
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This finds the nearest pixel coordinates of the transformed point.
  *      (2) It does not check ptrs for returned data!
+ * </pre>
  */
 l_int32
 affineXformSampledPt(l_float32  *vc,
@@ -1106,16 +1122,18 @@ affineXformSampledPt(l_float32  *vc,
 
 
 /*!
- *  affineXformPt()
+ * \brief   affineXformPt()
  *
- *      Input:  vc (vector of 6 coefficients)
- *              (x, y)  (initial point)
- *              (&xp, &yp)   (<return> transformed point)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    vc vector of 6 coefficients
+ * \param[in]    x, y  initial point
+ * \param[out]   pxp, pyp   transformed point
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This computes the floating point location of the transformed point.
  *      (2) It does not check ptrs for returned data!
+ * </pre>
  */
 l_int32
 affineXformPt(l_float32  *vc,
@@ -1139,22 +1157,24 @@ affineXformPt(l_float32  *vc,
  *                 Interpolation helper functions              *
  *-------------------------------------------------------------*/
 /*!
- *  linearInterpolatePixelColor()
+ * \brief   linearInterpolatePixelColor()
  *
- *      Input:  datas (ptr to beginning of image data)
- *              wpls (32-bit word/line for this data array)
- *              w, h (of image)
- *              x, y (floating pt location for evaluation)
- *              colorval (color brought in from the outside when the
+ * \param[in]    datas ptr to beginning of image data
+ * \param[in]    wpls 32-bit word/line for this data array
+ * \param[in]    w, h of image
+ * \param[in]    x, y floating pt location for evaluation
+ * \param[in]    colorval color brought in from the outside when the
  *                        input x,y location is outside the image;
- *                        in 0xrrggbb00 format))
- *              &val (<return> interpolated color value)
- *      Return: 0 if OK, 1 on error
+ *                        in 0xrrggbb00 format)
+ * \param[out]   pval interpolated color value
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is a standard linear interpolation function.  It is
  *          equivalent to area weighting on each component, and
  *          avoids "jaggies" when rendering sharp edges.
+ * </pre>
  */
 l_int32
 linearInterpolatePixelColor(l_uint32  *datas,
@@ -1166,7 +1186,7 @@ linearInterpolatePixelColor(l_uint32  *datas,
                             l_uint32   colorval,
                             l_uint32  *pval)
 {
-l_int32    xpm, ypm, xp, xp2, yp, xf, yf;
+l_int32    valid, xpm, ypm, xp, xp2, yp, xf, yf;
 l_int32    rval, gval, bval;
 l_uint32   word00, word01, word10, word11;
 l_uint32  *lines;
@@ -1179,9 +1199,12 @@ l_uint32  *lines;
     if (!datas)
         return ERROR_INT("datas not defined", procName, 1);
 
-        /* Skip if off the edge */
-    if (x < 0.0 || y < 0.0 || x >= w || y >= h)
-        return 0;
+        /* Skip if x or y are invalid. (x,y) must be in the source image.
+         * Failure to detect an invalid point will cause a mem address fault.
+         * Occasionally, x or y will be a nan, and relational checks always
+         * fail for nans.  Therefore we check if the point is inside the pix */
+    valid = (x >= 0.0 && y >= 0.0 && x < w && y < h);
+    if (!valid) return 0;
 
     xpm = (l_int32)(16.0 * x);
     ypm = (l_int32)(16.0 * y);
@@ -1222,21 +1245,23 @@ l_uint32  *lines;
 
 
 /*!
- *  linearInterpolatePixelGray()
+ * \brief   linearInterpolatePixelGray()
  *
- *      Input:  datas (ptr to beginning of image data)
- *              wpls (32-bit word/line for this data array)
- *              w, h (of image)
- *              x, y (floating pt location for evaluation)
- *              grayval (color brought in from the outside when the
- *                       input x,y location is outside the image)
- *              &val (<return> interpolated gray value)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    datas ptr to beginning of image data
+ * \param[in]    wpls 32-bit word/line for this data array
+ * \param[in]    w, h of image
+ * \param[in]    x, y floating pt location for evaluation
+ * \param[in]    grayval color brought in from the outside when the
+ *                       input x,y location is outside the image
+ * \param[out]   pval interpolated gray value
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is a standard linear interpolation function.  It is
  *          equivalent to area weighting on each component, and
  *          avoids "jaggies" when rendering sharp edges.
+ * </pre>
  */
 l_int32
 linearInterpolatePixelGray(l_uint32  *datas,
@@ -1248,7 +1273,7 @@ linearInterpolatePixelGray(l_uint32  *datas,
                            l_int32    grayval,
                            l_int32   *pval)
 {
-l_int32    xpm, ypm, xp, xp2, yp, xf, yf, v00, v10, v01, v11;
+l_int32    valid, xpm, ypm, xp, xp2, yp, xf, yf, v00, v10, v01, v11;
 l_uint32  *lines;
 
     PROCNAME("linearInterpolatePixelGray");
@@ -1259,9 +1284,12 @@ l_uint32  *lines;
     if (!datas)
         return ERROR_INT("datas not defined", procName, 1);
 
-        /* Skip if really off the edge */
-    if (x < 0.0 || y < 0.0 || x >= w || y >= h)
-        return 0;
+        /* Skip if x or y is invalid. (x,y) must be in the source image.
+         * Failure to detect an invalid point will cause a mem address fault.
+         * Occasionally, x or y will be a nan, and relational checks always
+         * fail for nans.  Therefore we check if the point is inside the pix */
+    valid = (x >= 0.0 && y >= 0.0 && x < w && y < h);
+    if (!valid) return 0;
 
     xpm = (l_int32)(16.0 * x);
     ypm = (l_int32)(16.0 * y);
@@ -1295,29 +1323,33 @@ l_uint32  *lines;
 #define  SWAP(a,b)   {temp = (a); (a) = (b); (b) = temp;}
 
 /*!
- *  gaussjordan()
+ * \brief   gaussjordan()
  *
- *      Input:   a  (n x n matrix)
- *               b  (rhs column vector)
- *               n  (dimension)
- *      Return:  0 if ok, 1 on error
+ * \param[in]    a  n x n matrix
+ * \param[in]    b  n x 1 right-hand side column vector
+ * \param[in]    n  dimension
+ * \return  0 if ok, 1 on error
  *
- *      Note side effects:
- *            (1) the matrix a is transformed to its inverse
- *            (2) the vector b is transformed to the solution X to the
- *                linear equation AX = B
- *
- *      Adapted from "Numerical Recipes in C, Second Edition", 1992
- *      pp. 36-41 (gauss-jordan elimination)
+ * <pre>
+ * Notes:
+ *      (1) There are two side-effects:
+ *          * The matrix a is transformed to its inverse A
+ *          * The rhs vector b is transformed to the solution x
+ *            of the linear equation ax = b
+ *      (2) The inverse A can then be used to solve the same equation with
+ *          different rhs vectors c by multiplication: x = Ac
+ *      (3) Adapted from "Numerical Recipes in C, Second Edition", 1992,
+ *          pp. 36-41 (gauss-jordan elimination)
+ * </pre>
  */
 l_int32
 gaussjordan(l_float32  **a,
             l_float32   *b,
             l_int32      n)
 {
-l_int32    i, icol, irow, j, k, l, ll;
+l_int32    i, icol, irow, j, k, col, row, success;
 l_int32   *indexc, *indexr, *ipiv;
-l_float32  big, dum, pivinv, temp;
+l_float32  maxval, val, pivinv, temp;
 
     PROCNAME("gaussjordan");
 
@@ -1326,67 +1358,80 @@ l_float32  big, dum, pivinv, temp;
     if (!b)
         return ERROR_INT("b not defined", procName, 1);
 
-    if ((indexc = (l_int32 *)CALLOC(n, sizeof(l_int32))) == NULL)
-        return ERROR_INT("indexc not made", procName, 1);
-    if ((indexr = (l_int32 *)CALLOC(n, sizeof(l_int32))) == NULL)
-        return ERROR_INT("indexr not made", procName, 1);
-    if ((ipiv = (l_int32 *)CALLOC(n, sizeof(l_int32))) == NULL)
-        return ERROR_INT("ipiv not made", procName, 1);
+    success = TRUE;
+    indexc = (l_int32 *)LEPT_CALLOC(n, sizeof(l_int32));
+    indexr = (l_int32 *)LEPT_CALLOC(n, sizeof(l_int32));
+    ipiv = (l_int32 *)LEPT_CALLOC(n, sizeof(l_int32));
+    if (!indexc || !indexr || !ipiv) {
+        L_ERROR("array not made\n", procName);
+        success = FALSE;
+        goto cleanup_arrays;
+    }
 
+    icol = irow = 0;  /* silence static checker */
     for (i = 0; i < n; i++) {
-        big = 0.0;
+        maxval = 0.0;
         for (j = 0; j < n; j++) {
-            if (ipiv[j] != 1)
+            if (ipiv[j] != 1) {
                 for (k = 0; k < n; k++) {
                     if (ipiv[k] == 0) {
-                        if (fabs(a[j][k]) >= big) {
-                            big = fabs(a[j][k]);
+                        if (fabs(a[j][k]) >= maxval) {
+                            maxval = fabs(a[j][k]);
                             irow = j;
                             icol = k;
                         }
                     } else if (ipiv[k] > 1) {
-                        return ERROR_INT("singular matrix", procName, 1);
+                        L_ERROR("singular matrix\n", procName);
+                        success = FALSE;
+                        goto cleanup_arrays;
                     }
                 }
+            }
         }
         ++(ipiv[icol]);
 
         if (irow != icol) {
-            for (l = 0; l < n; l++)
-                SWAP(a[irow][l], a[icol][l]);
+            for (col = 0; col < n; col++)
+                SWAP(a[irow][col], a[icol][col]);
             SWAP(b[irow], b[icol]);
         }
 
         indexr[i] = irow;
         indexc[i] = icol;
-        if (a[icol][icol] == 0.0)
-            return ERROR_INT("singular matrix", procName, 1);
+        if (a[icol][icol] == 0.0) {
+            L_ERROR("singular matrix\n", procName);
+            success = FALSE;
+            goto cleanup_arrays;
+        }
         pivinv = 1.0 / a[icol][icol];
         a[icol][icol] = 1.0;
-        for (l = 0; l < n; l++)
-            a[icol][l] *= pivinv;
+        for (col = 0; col < n; col++)
+            a[icol][col] *= pivinv;
         b[icol] *= pivinv;
 
-        for (ll = 0; ll < n; ll++)
-            if (ll != icol) {
-                dum = a[ll][icol];
-                a[ll][icol] = 0.0;
-                for (l = 0; l < n; l++)
-                    a[ll][l] -= a[icol][l] * dum;
-                b[ll] -= b[icol] * dum;
+        for (row = 0; row < n; row++) {
+            if (row != icol) {
+                val = a[row][icol];
+                a[row][icol] = 0.0;
+                for (col = 0; col < n; col++)
+                    a[row][col] -= a[icol][col] * val;
+                b[row] -= b[icol] * val;
             }
+        }
     }
 
-    for (l = n - 1; l >= 0; l--) {
-        if (indexr[l] != indexc[l])
+    for (col = n - 1; col >= 0; col--) {
+        if (indexr[col] != indexc[col]) {
             for (k = 0; k < n; k++)
-                SWAP(a[k][indexr[l]], a[k][indexc[l]]);
+                SWAP(a[k][indexr[col]], a[k][indexc[col]]);
+        }
     }
 
-    FREE(indexr);
-    FREE(indexc);
-    FREE(ipiv);
-    return 0;
+cleanup_arrays:
+    LEPT_FREE(indexr);
+    LEPT_FREE(indexc);
+    LEPT_FREE(ipiv);
+    return (success) ? 0 : 1;
 }
 
 
@@ -1394,21 +1439,22 @@ l_float32  big, dum, pivinv, temp;
  *              Sequential affine image transformation         *
  *-------------------------------------------------------------*/
 /*!
- *  pixAffineSequential()
+ * \brief   pixAffineSequential()
  *
- *      Input:  pixs
- *              ptad  (3 pts of final coordinate space)
- *              ptas  (3 pts of initial coordinate space)
- *              bw    (pixels of additional border width during computation)
- *              bh    (pixels of additional border height during computation)
- *      Return: pixd, or null on error
+ * \param[in]    pixs
+ * \param[in]    ptad  3 pts of final coordinate space
+ * \param[in]    ptas  3 pts of initial coordinate space
+ * \param[in]    bw    pixels of additional border width during computation
+ * \param[in]    bh    pixels of additional border height during computation
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The 3 pts must not be collinear.
  *      (2) The 3 pts must be given in this order:
- *           - origin
- *           - a location along the x-axis
- *           - a location along the y-axis.
+ *           ~ origin
+ *           ~ a location along the x-axis
+ *           ~ a location along the y-axis.
  *      (3) You must guess how much border must be added so that no
  *          pixels are lost in the transformations from src to
  *          dest coordinate space.  (This can be calculated but it
@@ -1418,6 +1464,7 @@ l_float32  big, dum, pivinv, temp;
  *      (4) This is here for pedagogical reasons.  It is about 3x faster
  *          on 1 bpp images than pixAffineSampled(), but the results
  *          on text are much inferior.
+ * </pre>
  */
 PIX *
 pixAffineSequential(PIX     *pixs,
@@ -1432,7 +1479,7 @@ l_int32    x1sc, y1sc;  /* scaled origin */
 l_float32  x2s, x2sp, scalex, scaley;
 l_float32  th3, th3p, ph2, ph2p;
 l_float32  rad2deg;
-PIX       *pixt1, *pixt2, *pixd;
+PIX       *pix1, *pix2, *pixd;
 
     PROCNAME("pixAffineSequential");
 
@@ -1455,6 +1502,7 @@ PIX       *pixt1, *pixt2, *pixd;
     ptaGetIPt(ptad, 2, &x3p, &y3p);
 
     rad2deg = 180. / 3.1415926535;
+    pix1 = pix2 = pixd = NULL;
 
     if (y1 == y3)
         return (PIX *)ERROR_PTR("y1 == y3!", procName, NULL);
@@ -1476,10 +1524,10 @@ PIX       *pixt1, *pixt2, *pixd;
         x3p = x3p + bw;
         y3p = y3p + bh;
 
-        if ((pixt1 = pixAddBorderGeneral(pixs, bw, bw, bh, bh, 0)) == NULL)
-            return (PIX *)ERROR_PTR("pixt1 not made", procName, NULL);
+        if ((pix1 = pixAddBorderGeneral(pixs, bw, bw, bh, bh, 0)) == NULL)
+            return (PIX *)ERROR_PTR("pix1 not made", procName, NULL);
     } else {
-        pixt1 = pixCopy(NULL, pixs);
+        pix1 = pixCopy(NULL, pixs);
     }
 
     /*-------------------------------------------------------------*
@@ -1496,8 +1544,10 @@ PIX       *pixt1, *pixt2, *pixd;
         /* Shear angles required to put src points on x and y axes */
     th3 = atan2((l_float64)(x1 - x3), (l_float64)(y1 - y3));
     x2s = (l_float32)(x2 - ((l_float32)(y1 - y2) * (x3 - x1)) / (y1 - y3));
-    if (x2s == (l_float32)x1)
-        return (PIX *)ERROR_PTR("x2s == x1!", procName, NULL);
+    if (x2s == (l_float32)x1) {
+        L_ERROR("x2s == x1!\n", procName);
+        goto cleanup_pix;
+    }
     ph2 = atan2((l_float64)(y1 - y2), (l_float64)(x2s - x1));
 
         /* Shear angles required to put dest points on x and y axes.
@@ -1505,15 +1555,18 @@ PIX       *pixt1, *pixt2, *pixd;
          * src points from the axes to the actual dest position.
          * These values are also needed to scale the image. */
     th3p = atan2((l_float64)(x1p - x3p), (l_float64)(y1p - y3p));
-    x2sp = (l_float32)(x2p - ((l_float32)(y1p - y2p) * (x3p - x1p)) / (y1p - y3p));
-    if (x2sp == (l_float32)x1p)
-        return (PIX *)ERROR_PTR("x2sp == x1p!", procName, NULL);
+    x2sp = (l_float32)(x2p -
+                       ((l_float32)(y1p - y2p) * (x3p - x1p)) / (y1p - y3p));
+    if (x2sp == (l_float32)x1p) {
+        L_ERROR("x2sp == x1p!\n", procName);
+        goto cleanup_pix;
+    }
     ph2p = atan2((l_float64)(y1p - y2p), (l_float64)(x2sp - x1p));
 
         /* Shear image to first put src point 3 on the y axis,
          * and then to put src point 2 on the x axis */
-    pixHShearIP(pixt1, y1, th3, L_BRING_IN_WHITE);
-    pixVShearIP(pixt1, x1, ph2, L_BRING_IN_WHITE);
+    pixHShearIP(pix1, y1, th3, L_BRING_IN_WHITE);
+    pixVShearIP(pix1, x1, ph2, L_BRING_IN_WHITE);
 
         /* Scale image to match dest scale.  The dest scale
          * is calculated above from the angles th3p and ph2p
@@ -1521,8 +1574,10 @@ PIX       *pixt1, *pixt2, *pixd;
          * the x and y axes. */
     scalex = (l_float32)(x2sp - x1p) / (x2s - x1);
     scaley = (l_float32)(y3p - y1p) / (y3 - y1);
-    if ((pixt2 = pixScale(pixt1, scalex, scaley)) == NULL)
-        return (PIX *)ERROR_PTR("pixt2 not made", procName, NULL);
+    if ((pix2 = pixScale(pix1, scalex, scaley)) == NULL) {
+        L_ERROR("pix2 not made\n", procName);
+        goto cleanup_pix;
+    }
 
 #if  DEBUG
     fprintf(stderr, "th3 = %5.1f deg, ph2 = %5.1f deg\n",
@@ -1545,21 +1600,22 @@ PIX       *pixt1, *pixt2, *pixd;
         /* Shift image to match dest origin. */
     x1sc = (l_int32)(scalex * x1 + 0.5);   /* x comp of origin after scaling */
     y1sc = (l_int32)(scaley * y1 + 0.5);   /* y comp of origin after scaling */
-    pixRasteropIP(pixt2, x1p - x1sc, y1p - y1sc, L_BRING_IN_WHITE);
+    pixRasteropIP(pix2, x1p - x1sc, y1p - y1sc, L_BRING_IN_WHITE);
 
         /* Shear image to take points 2 and 3 off the axis and
          * put them in the original dest position */
-    pixVShearIP(pixt2, x1p, -ph2p, L_BRING_IN_WHITE);
-    pixHShearIP(pixt2, y1p, -th3p, L_BRING_IN_WHITE);
+    pixVShearIP(pix2, x1p, -ph2p, L_BRING_IN_WHITE);
+    pixHShearIP(pix2, y1p, -th3p, L_BRING_IN_WHITE);
 
     if (bw != 0 || bh != 0) {
-        if ((pixd = pixRemoveBorderGeneral(pixt2, bw, bw, bh, bh)) == NULL)
-            return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+        if ((pixd = pixRemoveBorderGeneral(pix2, bw, bw, bh, bh)) == NULL)
+            L_ERROR("pixd not made\n", procName);
     } else {
-        pixd = pixClone(pixt2);
+        pixd = pixClone(pix2);
     }
 
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
+cleanup_pix:
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
     return pixd;
 }
