@@ -24,15 +24,17 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  binarize.c
+/*!
+ * \file binarize.c
+ * <pre>
  *
  *  ===================================================================
  *  Image binarization algorithms are found in:
  *    grayquant.c:   standard, simple, general grayscale quantization
  *    adaptmap.c:    local adaptive; mostly gray-to-gray in preparation
  *                   for binarization
- *    binarize.c:    special binarization methods, locally adaptive.
+ *    binarize.c:    special binarization methods, locally adaptive and
+ *                   global.
  *  ===================================================================
  *
  *      Adaptive Otsu-based thresholding
@@ -50,6 +52,9 @@
  *          PIX       *pixSauvolaGetThreshold()
  *          PIX       *pixApplyLocalThreshold();
  *
+ *      Thresholding using connected components
+ *          PIX       *pixThresholdByConnComp()
+ *
  *  Notes:
  *      (1) pixOtsuAdaptiveThreshold() computes a global threshold over each
  *          tile and performs the threshold operation, resulting in a
@@ -63,6 +68,11 @@
  *          the window size for the measurment at each pixel and a
  *          parameter that determines the amount of normalized local
  *          standard deviation to subtract from the local average value.
+ *      (4) pixThresholdByCC() uses the numbers of 4 and 8 connected
+ *          components at different thresholding to determine if a
+ *          global threshold can be used (for text or line-art) and the
+ *          value it should have.
+ * </pre>
  */
 
 #include <math.h>
@@ -72,28 +82,29 @@
  *                 Adaptive Otsu-based thresholding                 *
  *------------------------------------------------------------------*/
 /*!
- *  pixOtsuAdaptiveThreshold()
+ * \brief   pixOtsuAdaptiveThreshold()
  *
- *      Input:  pixs (8 bpp)
- *              sx, sy (desired tile dimensions; actual size may vary)
- *              smoothx, smoothy (half-width of convolution kernel applied to
- *                                threshold array: use 0 for no smoothing)
- *              scorefract (fraction of the max Otsu score; typ. 0.1;
- *                          use 0.0 for standard Otsu)
- *              &pixth (<optional return> array of threshold values
- *                      found for each tile)
- *              &pixd (<optional return> thresholded input pixs, based on
- *                     the threshold array)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs 8 bpp
+ * \param[in]    sx, sy desired tile dimensions; actual size may vary
+ * \param[in]    smoothx, smoothy half-width of convolution kernel applied to
+ *                                threshold array: use 0 for no smoothing
+ * \param[in]    scorefract fraction of the max Otsu score; typ. 0.1;
+ *                          use 0.0 for standard Otsu
+ * \param[out]   ppixth [optional] array of threshold values
+ *                      found for each tile
+ * \param[out]   ppixd [optional] thresholded input pixs, based on
+ *                     the threshold array
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The Otsu method finds a single global threshold for an image.
  *          This function allows a locally adapted threshold to be
  *          found for each tile into which the image is broken up.
  *      (2) The array of threshold values, one for each tile, constitutes
  *          a highly downscaled image.  This array is optionally
  *          smoothed using a convolution.  The full width and height of the
- *          convolution kernel are (2 * @smoothx + 1) and (2 * @smoothy + 1).
+ *          convolution kernel are (2 * %smoothx + 1) and (2 * %smoothy + 1).
  *      (3) The minimum tile dimension allowed is 16.  If such small
  *          tiles are used, it is recommended to use smoothing, because
  *          without smoothing, each small tile determines the splitting
@@ -103,7 +114,7 @@
  *          tile is only influenced by one type (fg or bg) of pixels,
  *          because it will force a split of its pixels.
  *      (4) To get a single global threshold for the entire image, use
- *          input values of @sx and @sy that are larger than the image.
+ *          input values of %sx and %sy that are larger than the image.
  *          For this situation, the smoothing parameters are ignored.
  *      (5) The threshold values partition the image pixels into two classes:
  *          one whose values are less than the threshold and another
@@ -117,9 +128,13 @@
  *          splitting the distribution of pixels in each tile into a
  *          fg and bg part.  The modification consists of searching for
  *          a minimum in the histogram over a range of pixel values where
- *          the Otsu score is within a defined fraction, @scorefract,
+ *          the Otsu score is within a defined fraction, %scorefract,
  *          of the max score.  To get the original Otsu algorithm, set
- *          @scorefract == 0.
+ *          %scorefract == 0.
+ *      (8) N.B. This method is NOT recommended for images with weak text
+ *          and significant background noise, such as bleedthrough, because
+ *          of the problem noted in (3) above for tiling.  Use Sauvola.
+ * </pre>
  */
 l_int32
 pixOtsuAdaptiveThreshold(PIX       *pixs,
@@ -159,7 +174,7 @@ PIXTILING  *pt;
         for (j = 0; j < nx; j++) {
             pixt = pixTilingGetTile(pt, i, j);
             pixSplitDistributionFgBg(pixt, scorefract, 1, &thresh,
-                                     NULL, NULL, 0);
+                                     NULL, NULL, NULL);
             pixSetPixel(pixthresh, j, i, thresh);  /* see note (4) */
             pixDestroy(&pixt);
         }
@@ -175,6 +190,7 @@ PIXTILING  *pt;
         /* Optionally apply the threshold array to binarize pixs */
     if (ppixd) {
         pixd = pixCreate(w, h, 1);
+        pixCopyResolution(pixd, pixs);
         for (i = 0; i < ny; i++) {
             for (j = 0; j < nx; j++) {
                 pixt = pixTilingGetTile(pt, i, j);
@@ -202,22 +218,23 @@ PIXTILING  *pt;
  *      Otsu thresholding on adaptive background normalization      *
  *------------------------------------------------------------------*/
 /*!
- *  pixOtsuThreshOnBackgroundNorm()
+ * \brief   pixOtsuThreshOnBackgroundNorm()
  *
- *      Input:  pixs (8 bpp grayscale; not colormapped)
- *              pixim (<optional> 1 bpp 'image' mask; can be null)
- *              sx, sy (tile size in pixels)
- *              thresh (threshold for determining foreground)
- *              mincount (min threshold on counts in a tile)
- *              bgval (target bg val; typ. > 128)
- *              smoothx (half-width of block convolution kernel width)
- *              smoothy (half-width of block convolution kernel height)
- *              scorefract (fraction of the max Otsu score; typ. 0.1)
- *              &thresh (<optional return> threshold value that was
- *                       used on the normalized image)
- *      Return: pixd (1 bpp thresholded image), or null on error
+ * \param[in]    pixs 8 bpp grayscale; not colormapped
+ * \param[in]    pixim [optional] 1 bpp 'image' mask; can be null
+ * \param[in]    sx, sy tile size in pixels
+ * \param[in]    thresh threshold for determining foreground
+ * \param[in]    mincount min threshold on counts in a tile
+ * \param[in]    bgval target bg val; typ. > 128
+ * \param[in]    smoothx half-width of block convolution kernel width
+ * \param[in]    smoothy half-width of block convolution kernel height
+ * \param[in]    scorefract fraction of the max Otsu score; typ. 0.1
+ * \param[out]   pthresh [optional] threshold value that was
+ *                       used on the normalized image
+ * \return  pixd 1 bpp thresholded image, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This does background normalization followed by Otsu
  *          thresholding.  Otsu binarization attempts to split the
  *          image into two roughly equal sets of pixels, and it does
@@ -233,6 +250,7 @@ PIXTILING  *pt;
  *            mincount = 50
  *            bgval = 255
  *            smoothx, smoothy = 2
+ * </pre>
  */
 PIX *
 pixOtsuThreshOnBackgroundNorm(PIX       *pixs,
@@ -294,21 +312,22 @@ PIX      *pixn, *pixt, *pixd;
  *    Masking and Otsu estimate on adaptive background normalization    *
  *----------------------------------------------------------------------*/
 /*!
- *  pixMaskedThreshOnBackgroundNorm()
+ * \brief   pixMaskedThreshOnBackgroundNorm()
  *
- *      Input:  pixs (8 bpp grayscale; not colormapped)
- *              pixim (<optional> 1 bpp 'image' mask; can be null)
- *              sx, sy (tile size in pixels)
- *              thresh (threshold for determining foreground)
- *              mincount (min threshold on counts in a tile)
- *              smoothx (half-width of block convolution kernel width)
- *              smoothy (half-width of block convolution kernel height)
- *              scorefract (fraction of the max Otsu score; typ. ~ 0.1)
- *              &thresh (<optional return> threshold value that was
- *                       used on the normalized image)
- *      Return: pixd (1 bpp thresholded image), or null on error
+ * \param[in]    pixs 8 bpp grayscale; not colormapped
+ * \param[in]    pixim [optional] 1 bpp 'image' mask; can be null
+ * \param[in]    sx, sy tile size in pixels
+ * \param[in]    thresh threshold for determining foreground
+ * \param[in]    mincount min threshold on counts in a tile
+ * \param[in]    smoothx half-width of block convolution kernel width
+ * \param[in]    smoothy half-width of block convolution kernel height
+ * \param[in]    scorefract fraction of the max Otsu score; typ. ~ 0.1
+ * \param[out]   pthresh [optional] threshold value that was
+ *                       used on the normalized image
+ * \return  pixd 1 bpp thresholded image, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This begins with a standard background normalization.
  *          Additionally, there is a flexible background norm, that
  *          will adapt to a rapidly varying background, and this
@@ -329,6 +348,7 @@ PIX      *pixn, *pixt, *pixd;
  *            thresh = 100
  *            mincount = 50
  *            smoothx, smoothy = 2
+ * </pre>
  */
 PIX *
 pixMaskedThreshOnBackgroundNorm(PIX       *pixs,
@@ -342,9 +362,9 @@ pixMaskedThreshOnBackgroundNorm(PIX       *pixs,
                                 l_float32  scorefract,
                                 l_int32   *pthresh)
 {
-l_int32   w, h;
+l_int32   w, h, highthresh;
 l_uint32  val;
-PIX      *pixn, *pixm, *pixd, *pixt1, *pixt2, *pixt3, *pixt4;
+PIX      *pixn, *pixm, *pixd, *pix1, *pix2, *pix3, *pix4;
 
     PROCNAME("pixMaskedThreshOnBackgroundNorm");
 
@@ -372,22 +392,22 @@ PIX      *pixn, *pixm, *pixd, *pixt1, *pixt2, *pixt3, *pixt4;
          * form a mask over regions that are typically text.  The
          * dilation size is chosen to cover the text completely,
          * except for very thick fonts. */
-    pixt1 = pixBackgroundNormFlex(pixs, 7, 7, 1, 1, 20);
-    pixt2 = pixThresholdToBinary(pixt1, 240);
-    pixInvert(pixt2, pixt2);
-    pixm = pixMorphSequence(pixt2, "d21.21", 0);
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
+    pix1 = pixBackgroundNormFlex(pixs, 7, 7, 1, 1, 20);
+    pix2 = pixThresholdToBinary(pix1, 240);
+    pixInvert(pix2, pix2);
+    pixm = pixMorphSequence(pix2, "d21.21", 0);
+    pixDestroy(&pix1);
+    pixDestroy(&pix2);
 
         /* Use Otsu to get a global threshold estimate for the image,
-         * which is stored as a single pixel in pixt3. */
+         * which is stored as a single pixel in pix3. */
     pixGetDimensions(pixs, &w, &h, NULL);
-    pixOtsuAdaptiveThreshold(pixs, w, h, 0, 0, scorefract, &pixt3, NULL);
-    if (pixt3 && pthresh) {
-        pixGetPixel(pixt3, 0, 0, &val);
+    pixOtsuAdaptiveThreshold(pixs, w, h, 0, 0, scorefract, &pix3, NULL);
+    if (pix3 && pthresh) {
+        pixGetPixel(pix3, 0, 0, &val);
         *pthresh = val;
     }
-    pixDestroy(&pixt3);
+    pixDestroy(&pix3);
 
         /* Threshold the background normalized images differentially,
          * using a high value correlated with the background normalization
@@ -398,10 +418,11 @@ PIX      *pixn, *pixm, *pixd, *pixt1, *pixt2, *pixt3, *pixt4;
          * while allowing the background and light foreground to be
          * reasonably well cleaned using a threshold adapted to the
          * input image. */
-    pixd = pixThresholdToBinary(pixn, val + 30);  /* for bg and light fg */
-    pixt4 = pixThresholdToBinary(pixn, 190);  /* for heavier fg */
-    pixCombineMasked(pixd, pixt4, pixm);
-    pixDestroy(&pixt4);
+    highthresh = L_MIN(256, val + 30);
+    pixd = pixThresholdToBinary(pixn, highthresh);  /* for bg and light fg */
+    pix4 = pixThresholdToBinary(pixn, 190);  /* for heavier fg */
+    pixCombineMasked(pixd, pix4, pixm);
+    pixDestroy(&pix4);
     pixDestroy(&pixm);
     pixDestroy(&pixn);
 
@@ -416,19 +437,20 @@ PIX      *pixn, *pixm, *pixd, *pixt1, *pixt2, *pixt3, *pixt4;
  *                           Sauvola binarization                       *
  *----------------------------------------------------------------------*/
 /*!
- *  pixSauvolaBinarizeTiled()
+ * \brief   pixSauvolaBinarizeTiled()
  *
- *      Input:  pixs (8 bpp grayscale, not colormapped)
- *              whsize (window half-width for measuring local statistics)
- *              factor (factor for reducing threshold due to variance; >= 0)
- *              nx, ny (subdivision into tiles; >= 1)
- *              &pixth (<optional return> Sauvola threshold values)
- *              &pixd (<optional return> thresholded image)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs 8 bpp grayscale, not colormapped
+ * \param[in]    whsize window half-width for measuring local statistics
+ * \param[in]    factor factor for reducing threshold due to variance; >= 0
+ * \param[in]    nx, ny subdivision into tiles; >= 1
+ * \param[out]   ppixth [optional] Sauvola threshold values
+ * \param[out]   ppixd [optional] thresholded image
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
- *      (1) The window width and height are 2 * @whsize + 1.  The minimum
- *          value for @whsize is 2; typically it is >= 7..
+ * <pre>
+ * Notes:
+ *      (1) The window width and height are 2 * %whsize + 1.  The minimum
+ *          value for %whsize is 2; typically it is \>= 7..
  *      (2) For nx == ny == 1, this defaults to pixSauvolaBinarize().
  *      (3) Why a tiled version?
  *          (a) Because the mean value accumulator is a uint32, overflow
@@ -441,6 +463,7 @@ PIX      *pixn, *pixm, *pixd, *pixt1, *pixt2, *pixt3, *pixt4;
  *      (4) The Sauvola threshold is determined from the formula:
  *              t = m * (1 - k * (1 - s / 128))
  *          See pixSauvolaBinarize() for details.
+ * </pre>
  */
 l_int32
 pixSauvolaBinarizeTiled(PIX       *pixs,
@@ -532,33 +555,34 @@ PIXTILING  *pt;
 
 
 /*!
- *  pixSauvolaBinarize()
+ * \brief   pixSauvolaBinarize()
  *
- *      Input:  pixs (8 bpp grayscale; not colormapped)
- *              whsize (window half-width for measuring local statistics)
- *              factor (factor for reducing threshold due to variance; >= 0)
- *              addborder (1 to add border of width (@whsize + 1) on all sides)
- *              &pixm (<optional return> local mean values)
- *              &pixsd (<optional return> local standard deviation values)
- *              &pixth (<optional return> threshold values)
- *              &pixd (<optional return> thresholded image)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs 8 bpp grayscale; not colormapped
+ * \param[in]    whsize window half-width for measuring local statistics
+ * \param[in]    factor factor for reducing threshold due to variance; >= 0
+ * \param[in]    addborder 1 to add border of width (%whsize + 1) on all sides
+ * \param[out]   ppixm [optional] local mean values
+ * \param[out]   ppixsd [optional] local standard deviation values
+ * \param[out]   ppixth [optional] threshold values
+ * \param[out]   ppixd [optional] thresholded image
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
- *      (1) The window width and height are 2 * @whsize + 1.  The minimum
- *          value for @whsize is 2; typically it is >= 7..
+ * <pre>
+ * Notes:
+ *      (1) The window width and height are 2 * %whsize + 1.  The minimum
+ *          value for %whsize is 2; typically it is \>= 7..
  *      (2) The local statistics, measured over the window, are the
  *          average and standard deviation.
  *      (3) The measurements of the mean and standard deviation are
- *          performed inside a border of (@whsize + 1) pixels.  If pixs does
- *          not have these added border pixels, use @addborder = 1 to add
- *          it here; otherwise use @addborder = 0.
+ *          performed inside a border of (%whsize + 1) pixels.  If pixs does
+ *          not have these added border pixels, use %addborder = 1 to add
+ *          it here; otherwise use %addborder = 0.
  *      (4) The Sauvola threshold is determined from the formula:
  *            t = m * (1 - k * (1 - s / 128))
  *          where:
  *            t = local threshold
  *            m = local mean
- *            k = @factor (>= 0)   [ typ. 0.35 ]
+ *            k = %factor (\>= 0)   [ typ. 0.35 ]
  *            s = local standard deviation, which is maximized at
  *                127.5 when half the samples are 0 and half are 255.
  *      (5) The basic idea of Niblack and Sauvola binarization is that
@@ -566,6 +590,7 @@ PIXTILING  *pt;
  *          and the larger the variance, the closer to the median
  *          it should be chosen.  Typical values for k are between
  *          0.2 and 0.5.
+ * </pre>
  */
 l_int32
 pixSauvolaBinarize(PIX       *pixs,
@@ -582,13 +607,12 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
 
     PROCNAME("pixSauvolaBinarize");
 
-
-    if (!ppixm && !ppixsd && !ppixth && !ppixd)
-        return ERROR_INT("no outputs", procName, 1);
     if (ppixm) *ppixm = NULL;
     if (ppixsd) *ppixsd = NULL;
     if (ppixth) *ppixth = NULL;
     if (ppixd) *ppixd = NULL;
+    if (!ppixm && !ppixsd && !ppixth && !ppixd)
+        return ERROR_INT("no outputs", procName, 1);
     if (!pixs || pixGetDepth(pixs) != 8)
         return ERROR_INT("pixs undefined or not 8 bpp", procName, 1);
     if (pixGetColormap(pixs))
@@ -619,8 +643,10 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
         pixms = pixWindowedMeanSquare(pixg, whsize, whsize, 1);
     if (ppixth || ppixd)
         pixth = pixSauvolaGetThreshold(pixm, pixms, factor, ppixsd);
-    if (ppixd)
+    if (ppixd) {
         pixd = pixApplyLocalThreshold(pixsc, pixth, 1);
+        pixCopyResolution(pixd, pixs);
+    }
 
     if (ppixm)
         *ppixm = pixm;
@@ -642,21 +668,22 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
 
 
 /*!
- *  pixSauvolaGetThreshold()
+ * \brief   pixSauvolaGetThreshold()
  *
- *      Input:  pixm (8 bpp grayscale; not colormapped)
- *              pixms (32 bpp)
- *              factor (factor for reducing threshold due to variance; >= 0)
- *              &pixsd (<optional return> local standard deviation)
- *      Return: pixd (8 bpp, sauvola threshold values), or null on error
+ * \param[in]    pixm 8 bpp grayscale; not colormapped
+ * \param[in]    pixms 32 bpp
+ * \param[in]    factor factor for reducing threshold due to variance; >= 0
+ * \param[out]   ppixsd [optional] local standard deviation
+ * \return  pixd 8 bpp, sauvola threshold values, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The Sauvola threshold is determined from the formula:
  *            t = m * (1 - k * (1 - s / 128))
  *          where:
  *            t = local threshold
  *            m = local mean
- *            k = @factor (>= 0)   [ typ. 0.35 ]
+ *            k = %factor (\>= 0)   [ typ. 0.35 ]
  *            s = local standard deviation, which is maximized at
  *                127.5 when half the samples are 0 and half are 255.
  *      (2) See pixSauvolaBinarize() for other details.
@@ -675,6 +702,7 @@ PIX     *pixg, *pixsc, *pixm, *pixms, *pixth, *pixd;
  *          So for evaluating the standard deviation in the Sauvola
  *          threshold, we take
  *            s = sqrt(ms - mv * mv)
+ * </pre>
  */
 PIX *
 pixSauvolaGetThreshold(PIX       *pixm,
@@ -709,9 +737,9 @@ PIX        *pixsd, *pixd;
     usetab = (w * h > 100000) ? 1 : 0;
     if (usetab) {
         tabsize = 1 << 16;
-        tab = (l_float32 *)CALLOC(tabsize, sizeof(l_float32));
+        tab = (l_float32 *)LEPT_CALLOC(tabsize, sizeof(l_float32));
         for (i = 0; i < tabsize; i++)
-            tab[i] = (l_float32)sqrt((l_float64)i);
+            tab[i] = sqrtf((l_float32)i);
     }
 
     pixd = pixCreate(w, h, 8);
@@ -739,25 +767,25 @@ PIX        *pixsd, *pixd;
             if (usetab)
                 sd = tab[var];
             else
-                sd = (l_float32)sqrt((l_float32)var);
+                sd = sqrtf((l_float32)var);
             if (ppixsd) SET_DATA_BYTE(linesd, j, (l_int32)sd);
             thresh = (l_int32)(mv * (1.0 - factor * (1.0 - sd / 128.)));
             SET_DATA_BYTE(lined, j, thresh);
         }
     }
 
-    if (usetab) FREE(tab);
+    if (usetab) LEPT_FREE(tab);
     return pixd;
 }
 
 
 /*!
- *  pixApplyLocalThreshold()
+ * \brief   pixApplyLocalThreshold()
  *
- *      Input:  pixs (8 bpp grayscale; not colormapped)
- *              pixth (8 bpp array of local thresholds)
- *              redfactor ( ... )
- *      Return: pixd (1 bpp, thresholded image), or null on error
+ * \param[in]    pixs 8 bpp grayscale; not colormapped
+ * \param[in]    pixth 8 bpp array of local thresholds
+ * \param[in]    redfactor  ...
+ * \return  pixd 1 bpp, thresholded image, or NULL on error
  */
 PIX *
 pixApplyLocalThreshold(PIX     *pixs,
@@ -799,3 +827,189 @@ PIX       *pixd;
 
     return pixd;
 }
+
+
+/*----------------------------------------------------------------------*
+ *                  Thresholding using connected components             *
+ *----------------------------------------------------------------------*/
+/*!
+ * \brief   pixThresholdByConnComp()
+ *
+ * \param[in]    pixs depth > 1, colormap OK
+ * \param[in]    pixm [optional] 1 bpp mask giving region to ignore by setting
+ *                    pixels to white; use NULL if no mask
+ * \param[in]    start, end, incr binarization threshold levels to test
+ * \param[in]    thresh48 threshold on normalized difference between the
+ *                        numbers of 4 and 8 connected components
+ * \param[in]    threshdiff threshold on normalized difference between the
+ *                          number of 4 cc at successive iterations
+ * \param[out]   pglobthresh [optional] best global threshold; 0
+ *                           if no threshold is found
+ * \param[out]   ppixd [optional] image thresholded to binary, or
+ *                     null if no threshold is found
+ * \param[in]    debugflag 1 for plotted results
+ * \return  0 if OK, 1 on error or if no threshold is found
+ *
+ * <pre>
+ * Notes:
+ *      (1) This finds a global threshold based on connected components.
+ *          Although slow, it is reasonable to use it in a situation where
+ *          (a) the background in the image is relatively uniform, and
+ *          (b) the result will be fed to an OCR program that accepts 1 bpp
+ *              images and works best with easily segmented characters.
+ *          The reason for (b) is that this selects a threshold with a
+ *          minimum number of both broken characters and merged characters.
+ *      (2) If the pix has color, it is converted to gray using the
+ *          max component.
+ *      (3) Input 0 to use default values for any of these inputs:
+ *          %start, %end, %incr, %thresh48, %threshdiff.
+ *      (4) This approach can be understood as follows.  When the
+ *          binarization threshold is varied, the numbers of c.c. identify
+ *          four regimes:
+ *          (a) For low thresholds, text is broken into small pieces, and
+ *              the number of c.c. is large, with the 4 c.c. significantly
+ *              exceeding the 8 c.c.
+ *          (b) As the threshold rises toward the optimum value, the text
+ *              characters coalesce and there is very little difference
+ *              between the numbers of 4 and 8 c.c, which both go
+ *              through a minimum.
+ *          (c) Above this, the image background gets noisy because some
+ *              pixels are(thresholded to foreground, and the numbers
+ *              of c.c. quickly increase, with the 4 c.c. significantly
+ *              larger than the 8 c.c.
+ *          (d) At even higher thresholds, the image background noise
+ *              coalesces as it becomes mostly foreground, and the
+ *              number of c.c. drops quickly.
+ *      (5) If there is no global threshold that distinguishes foreground
+ *          text from background (e.g., weak text over a background that
+ *          has significant variation and/or bleedthrough), this returns 1,
+ *          which the caller should check.
+ * </pre>
+ */
+l_int32
+pixThresholdByConnComp(PIX       *pixs,
+                       PIX       *pixm,
+                       l_int32    start,
+                       l_int32    end,
+                       l_int32    incr,
+                       l_float32  thresh48,
+                       l_float32  threshdiff,
+                       l_int32   *pglobthresh,
+                       PIX      **ppixd,
+                       l_int32    debugflag)
+{
+l_int32    i, thresh, n, n4, n8, mincounts, found, globthresh;
+l_float32  count4, count8, firstcount4, prevcount4, diff48, diff4;
+GPLOT     *gplot;
+NUMA      *na4, *na8;
+PIX       *pix1, *pix2, *pix3;
+
+    PROCNAME("pixThresholdByConnComp");
+
+    if (pglobthresh) *pglobthresh = 0;
+    if (ppixd) *ppixd = NULL;
+    if (!pixs || pixGetDepth(pixs) == 1)
+        return ERROR_INT("pixs undefined or 1 bpp", procName, 1);
+    if (pixm && pixGetDepth(pixm) != 1)
+        return ERROR_INT("pixm must be 1 bpp", procName, 1);
+
+        /* Assign default values if requested */
+    if (start <= 0) start = 80;
+    if (end <= 0) end = 200;
+    if (incr <= 0) incr = 10;
+    if (thresh48 <= 0.0) thresh48 = 0.01;
+    if (threshdiff <= 0.0) threshdiff = 0.01;
+    if (start > end)
+        return ERROR_INT("invalid start,end", procName, 1);
+
+        /* Make 8 bpp, using the max component if color. */
+    if (pixGetColormap(pixs))
+        pix1 = pixRemoveColormap(pixs, REMOVE_CMAP_BASED_ON_SRC);
+    else
+        pix1 = pixClone(pixs);
+    if (pixGetDepth(pix1) == 32)
+        pix2 = pixConvertRGBToGrayMinMax(pix1, L_CHOOSE_MAX);
+    else
+        pix2 = pixConvertTo8(pix1, 0);
+    pixDestroy(&pix1);
+
+        /* Mask out any non-text regions.  Do this in-place, because pix2
+         * can never be the same pix as pixs. */
+    if (pixm)
+        pixSetMasked(pix2, pixm, 255);
+
+        /* Make sure there are enough components to get a valid signal */
+    pix3 = pixConvertTo1(pix2, start);
+    pixCountConnComp(pix3, 4, &n4);
+    pixDestroy(&pix3);
+    mincounts = 500;
+    if (n4 < mincounts) {
+        L_INFO("Insufficient component count: %d\n", procName, n4);
+        pixDestroy(&pix2);
+        return 1;
+    }
+
+        /* Compute the c.c. data */
+    na4 = numaCreate(0);
+    na8 = numaCreate(0);
+    numaSetParameters(na4, start, incr);
+    numaSetParameters(na8, start, incr);
+    for (thresh = start, i = 0; thresh <= end; thresh += incr, i++) {
+        pix3 = pixConvertTo1(pix2, thresh);
+        pixCountConnComp(pix3, 4, &n4);
+        pixCountConnComp(pix3, 8, &n8);
+        numaAddNumber(na4, n4);
+        numaAddNumber(na8, n8);
+        pixDestroy(&pix3);
+    }
+    if (debugflag) {
+        gplot = gplotCreate("/tmp/threshroot", GPLOT_PNG,
+                            "number of cc vs. threshold",
+                            "threshold", "number of cc");
+        gplotAddPlot(gplot, NULL, na4, GPLOT_LINES, "plot 4cc");
+        gplotAddPlot(gplot, NULL, na8, GPLOT_LINES, "plot 8cc");
+        gplotMakeOutput(gplot);
+        gplotDestroy(&gplot);
+    }
+
+    n = numaGetCount(na4);
+    found = FALSE;
+    for (i = 0; i < n; i++) {
+        if (i == 0) {
+            numaGetFValue(na4, i, &firstcount4);
+            prevcount4 = firstcount4;
+        } else {
+            numaGetFValue(na4, i, &count4);
+            numaGetFValue(na8, i, &count8);
+            diff48 = (count4 - count8) / firstcount4;
+            diff4 = L_ABS(prevcount4 - count4) / firstcount4;
+            if (debugflag) {
+                fprintf(stderr, "diff48 = %7.3f, diff4 = %7.3f\n",
+                        diff48, diff4);
+            }
+            if (diff48 < thresh48 && diff4 < threshdiff) {
+                found = TRUE;
+                break;
+            }
+            prevcount4 = count4;
+        }
+    }
+    numaDestroy(&na4);
+    numaDestroy(&na8);
+
+    if (found) {
+        globthresh = start + i * incr;
+        if (pglobthresh) *pglobthresh = globthresh;
+        if (ppixd) {
+            *ppixd = pixConvertTo1(pix2, globthresh);
+            pixCopyResolution(*ppixd, pixs);
+        }
+        if (debugflag) fprintf(stderr, "global threshold = %d\n", globthresh);
+        pixDestroy(&pix2);
+        return 0;
+    }
+
+    if (debugflag) fprintf(stderr, "no global threshold found\n");
+    pixDestroy(&pix2);
+    return 1;
+}
