@@ -24,8 +24,9 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  bmpio.c
+/*!
+ * \file bmpio.c
+ * <pre>
  *
  *      Read bmp from file
  *           PIX          *pixReadStreamBmp()
@@ -41,8 +42,13 @@
  *    we write data to a temp file and read it back for operations
  *    between pix and compressed-data, such as pixReadMemPng() and
  *    pixWriteMemPng().
+ * </pre>
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config_auto.h"
+#endif  /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include "allheaders.h"
 #include "bmp.h"
@@ -51,26 +57,37 @@
 #if  USE_BMPIO   /* defined in environ.h */
 /* --------------------------------------------*/
 
-RGBA_QUAD   bwmap[2] = { {255,255,255,0}, {0,0,0,0} };
+    /* Here we're setting the pixel value 0 to white (255) and the
+     * value 1 to black (0).  This is the convention for grayscale, but
+     * the opposite of the convention for 1 bpp, where 0 is white
+     * and 1 is black.  Both colormap entries are opaque (alpha = 255) */
+RGBA_QUAD   bwmap[2] = { {255,255,255,255}, {0,0,0,255} };
 
     /* Colormap size limit */
 static const l_int32  L_MAX_ALLOWED_NUM_COLORS = 256;
 
+    /* Image dimension limits */
+static const l_int32  L_MAX_ALLOWED_WIDTH = 1000000;
+static const l_int32  L_MAX_ALLOWED_HEIGHT = 1000000;
+static const l_int64  L_MAX_ALLOWED_AREA = 400000000LL;
+
 #ifndef  NO_CONSOLE_IO
 #define  DEBUG     0
 #endif  /* ~NO_CONSOLE_IO */
 
 
 /*!
- *  pixReadStreamBmp()
+ * \brief   pixReadStreamBmp()
  *
- *      Input:  stream opened for read
- *      Return: pix, or null on error
+ * \param[in]    fp file stream opened for read
+ * \return  pix, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Here are references on the bmp file format:
  *          http://en.wikipedia.org/wiki/BMP_file_format
  *          http://www.fortunecity.com/skyscraper/windows/364/bmpffrmt.html
+ * </pre>
  */
 PIX *
 pixReadStreamBmp(FILE  *fp)
@@ -81,11 +98,13 @@ l_int16    bfType, bfSize, bfFill1, bfReserved1, bfReserved2;
 l_int16    offset, bfFill2, biPlanes, depth, d;
 l_int32    biSize, width, height, xres, yres, compression;
 l_int32    imagebytes, biClrUsed, biClrImportant;
+size_t     filesize;
 l_uint8   *colormapBuf;
 l_int32    colormapEntries;
 l_int32    fileBpl, extrabytes;
 l_int32    pixWpl, pixBpl;
 l_int32    i, j, k;
+l_int64    area;
 l_uint8    pel[4];
 l_uint8   *data;
 l_uint32  *line, *pword;
@@ -162,43 +181,53 @@ PIXCMAP   *cmap;
         return (PIX *)ERROR_PTR("cannot read compressed BMP files",
                                 procName, NULL);
 
-        /* A little sanity checking.  It would be nice to check
-         * if the number of bytes in the file equals the offset to
-         * the data plus the imagedata, but this won't work when
-         * reading from memory, because fmemopen() doesn't implement
-         * ftell().  So we can't do that check.  The imagebytes for
-         * uncompressed images is either 0 or the size of the file data.
-         * (The fact that it can be 0 is perhaps some legacy glitch).  */
+        /* Some sanity checking.  We impose limits on the image
+         * dimensions and number of pixels.  We make sure the file
+         * is large enough to hold the amount of uncompressed data
+         * that is specified in the header.  The number of colormap
+         * entries is checked: it can be either 0 (no cmap) or some
+         * number between 2 and 256.
+         * Note that the imagebytes for uncompressed images is either
+         * 0 or the size of the file data.  (The fact that it can
+         * be 0 is perhaps some legacy glitch). */
     if (width < 1)
         return (PIX *)ERROR_PTR("width < 1", procName, NULL);
+    if (width > L_MAX_ALLOWED_WIDTH)
+        return (PIX *)ERROR_PTR("width too large", procName, NULL);
     if (height < 1)
         return (PIX *)ERROR_PTR("height < 1", procName, NULL);
+    if (height > L_MAX_ALLOWED_HEIGHT)
+        return (PIX *)ERROR_PTR("height too large", procName, NULL);
+    area = 1LL * width * height;
+    if (area > L_MAX_ALLOWED_AREA)
+        return (PIX *)ERROR_PTR("area too large", procName, NULL);
     if (depth != 1 && depth != 2 && depth != 4 && depth != 8 &&
         depth != 16 && depth != 24 && depth != 32)
         return (PIX *)ERROR_PTR("depth not in {1, 2, 4, 8, 16, 24, 32}",
                                 procName,NULL);
-    fileBpl = 4 * ((width * depth + 31)/32);
+    fileBpl = 4 * ((1LL * width * depth + 31)/32);
     if (imagebytes != 0 && imagebytes != fileBpl * height)
         return (PIX *)ERROR_PTR("invalid imagebytes", procName, NULL);
-    if (offset < BMP_FHBYTES + BMP_IHBYTES)
-        return (PIX *)ERROR_PTR("invalid offset: too small", procName, NULL);
-    if (offset > BMP_FHBYTES + BMP_IHBYTES + 4 * 256)
-        return (PIX *)ERROR_PTR("invalid offset: too large", procName, NULL);
+    colormapEntries = (offset - BMP_FHBYTES - BMP_IHBYTES) / sizeof(RGBA_QUAD);
+    if (colormapEntries < 0 || colormapEntries == 1)
+        return (PIX *)ERROR_PTR("invalid: cmap size < 0 or 1", procName, NULL);
+    if (colormapEntries > L_MAX_ALLOWED_NUM_COLORS)
+        return (PIX *)ERROR_PTR("invalid cmap: too large", procName,NULL);
+    filesize = fnbytesInFile(fp);
+    if (filesize < 1LL * fileBpl * height)
+        return (PIX *)ERROR_PTR("file too small to hold data", procName,NULL);
 
         /* Handle the colormap */
-    colormapEntries = (offset - BMP_FHBYTES - BMP_IHBYTES) / sizeof(RGBA_QUAD);
     colormapBuf = NULL;
-    if (colormapEntries > L_MAX_ALLOWED_NUM_COLORS)
-        return (PIX *)ERROR_PTR("colormap too large", procName,NULL);
     if (colormapEntries > 0) {
-        if ((colormapBuf = (l_uint8 *)CALLOC(colormapEntries,
+        if ((colormapBuf = (l_uint8 *)LEPT_CALLOC(colormapEntries,
                                              sizeof(RGBA_QUAD))) == NULL)
             return (PIX *)ERROR_PTR("colormapBuf alloc fail", procName, NULL );
 
             /* Read colormap */
         if (fread(colormapBuf, sizeof(RGBA_QUAD), colormapEntries, fp)
                  != colormapEntries) {
-            FREE(colormapBuf);
+            LEPT_FREE(colormapBuf);
             return (PIX *)ERROR_PTR( "colormap read fail", procName, NULL);
         }
     }
@@ -208,26 +237,25 @@ PIXCMAP   *cmap;
     if (depth == 24)
         d = 32;
     if ((pix = pixCreate(width, height, d)) == NULL) {
-        FREE(colormapBuf);
+        LEPT_FREE(colormapBuf);
         return (PIX *)ERROR_PTR( "pix not made", procName, NULL);
     }
     pixSetXRes(pix, (l_int32)((l_float32)xres / 39.37 + 0.5));  /* to ppi */
     pixSetYRes(pix, (l_int32)((l_float32)yres / 39.37 + 0.5));  /* to ppi */
+    pixSetInputFormat(pix, IFF_BMP);
     pixWpl = pixGetWpl(pix);
     pixBpl = 4 * pixWpl;
 
     cmap = NULL;
-    if (colormapEntries > 256)
-        L_WARNING("more than 256 colormap entries!\n", procName);
     if (colormapEntries > 0) {  /* import the colormap to the pix cmap */
         cmap = pixcmapCreate(L_MIN(d, 8));
-        FREE(cmap->array);  /* remove generated cmap array */
+        LEPT_FREE(cmap->array);  /* remove generated cmap array */
         cmap->array  = (void *)colormapBuf;  /* and replace */
         cmap->n = L_MIN(colormapEntries, 256);
     }
     pixSetColormap(pix, cmap);
 
-        /* Seek to the start of the bitmap in the file */
+        /* Seek to the start of the image data in the file */
     fseek(fp, offset, 0);
 
     if (depth != 24) {  /* typ. 1 or 8 bpp */
@@ -307,14 +335,15 @@ PIXCMAP   *cmap;
         /* ----------------------------------------------
          * The bmp colormap determines the values of black
          * and white pixels for binary in the following way:
-         * if black = 1 (255), white = 0
-         *      255, 255, 255, 0, 0, 0, 0, 0
-         * if black = 0, white = 1 (255)
-         *      0, 0, 0, 0, 255, 255, 255, 0
+         * (a) white = 0 [255], black = 1 [0]
+         *      255, 255, 255, 255, 0, 0, 0, 255
+         * (b) black = 0 [0], white = 1 [255]
+         *      0, 0, 0, 255, 255, 255, 255, 255
          * We have no need for a 1 bpp pix with a colormap!
+         * Note: the alpha component here is 255 (opaque)
          * ---------------------------------------------- */
     if (depth == 1 && cmap) {
-        pixt = pixRemoveColormap(pix, REMOVE_CMAP_BASED_ON_SRC);
+        pixt = pixRemoveColormap(pix, REMOVE_CMAP_TO_BINARY);
         pixDestroy(&pix);
         pix = pixt;  /* rename */
     }
@@ -325,17 +354,19 @@ PIXCMAP   *cmap;
 
 
 /*!
- *  pixWriteStreamBmp()
+ * \brief   pixWriteStreamBmp()
  *
- *      Input:  stream opened for write
- *              pix (1, 4, 8, 32 bpp)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    fp file stream opened for write
+ * \param[in]    pix 1, 4, 8, 32 bpp
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) We position fp at the beginning of the stream, so it
  *          truncates any existing data
  *      (2) 2 bpp Bmp files are apparently not valid!.  We can
  *          write and read them, but nobody else can read ours.
+ * </pre>
  */
 l_int32
 pixWriteStreamBmp(FILE  *fp,
@@ -403,7 +434,7 @@ RGBA_QUAD  *pquad;
             cmaplen = ncolors * sizeof(RGBA_QUAD);
 
             heapcm = 1;
-            if ((cta = (l_uint8 *)CALLOC(cmaplen, 1)) == NULL)
+            if ((cta = (l_uint8 *)LEPT_CALLOC(cmaplen, 1)) == NULL)
                 return ERROR_INT("colormap alloc fail", procName, 1);
 
             stepsize = 255 / (ncolors - 1);
@@ -411,6 +442,7 @@ RGBA_QUAD  *pquad;
                  i < ncolors;
                  i++, val += stepsize, pquad++) {
                 pquad->blue = pquad->green = pquad->red = val;
+                pquad->alpha = 255;  /* opaque */
             }
         }
     }
@@ -445,7 +477,7 @@ RGBA_QUAD  *pquad;
     fwrite(&bfSize, 1, 2, fp);
     fwrite(&bfFill1, 1, 2, fp);
     fwrite(&bfReserved1, 1, 2, fp);
-    fwrite(&bfReserved1, 1, 2, fp);
+    fwrite(&bfReserved2, 1, 2, fp);
     fwrite(&bfOffBits, 1, 2, fp);
     fwrite(&bfFill2, 1, 2, fp);
 
@@ -477,11 +509,11 @@ RGBA_QUAD  *pquad;
     if (ncolors > 0) {
         if (fwrite(cta, 1, cmaplen, fp) != cmaplen) {
             if (heapcm)
-                FREE(cta);
+                LEPT_FREE(cta);
             return ERROR_INT("colormap write fail", procName, 1);
         }
         if (heapcm)
-            FREE(cta);
+            LEPT_FREE(cta);
     }
 
         /* When you write a binary image with a colormap
@@ -543,27 +575,21 @@ RGBA_QUAD  *pquad;
 /*---------------------------------------------------------------------*
  *                         Read/write to memory                        *
  *---------------------------------------------------------------------*/
-#ifdef HAVE_CONFIG_H
-#include "config_auto.h"
-#endif  /* HAVE_CONFIG_H */
-
-#if HAVE_FMEMOPEN
-extern FILE *open_memstream(char **data, size_t *size);
-extern FILE *fmemopen(void *data, size_t size, const char *mode);
-#endif  /* HAVE_FMEMOPEN */
 
 /*!
- *  pixReadMemBmp()
+ * \brief   pixReadMemBmp()
  *
- *      Input:  cdata (const; bmp-encoded)
- *              size (of data)
- *      Return: pix, or null on error
+ * \param[in]    data const; bmp-encoded
+ * \param[in]    size of data
+ * \return  pix, or NULL on error
  *
- *  Notes:
- *      (1) The @size byte of @data must be a null character.
+ * <pre>
+ * Notes:
+ *      (1) The %size byte of %data must be a null character.
+ * </pre>
  */
 PIX *
-pixReadMemBmp(const l_uint8  *cdata,
+pixReadMemBmp(const l_uint8  *data,
               size_t          size)
 {
 FILE  *fp;
@@ -571,18 +597,11 @@ PIX   *pix;
 
     PROCNAME("pixReadMemBmp");
 
-    if (!cdata)
-        return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
+    if (!data)
+        return (PIX *)ERROR_PTR("data not defined", procName, NULL);
 
-#if HAVE_FMEMOPEN
-    if ((fp = fmemopen((l_uint8 *)cdata, size, "r")) == NULL)
+    if ((fp = fopenReadFromMemory(data, size)) == NULL)
         return (PIX *)ERROR_PTR("stream not opened", procName, NULL);
-#else
-    L_WARNING("work-around: writing to a temp file\n", procName);
-    fp = tmpfile();
-    fwrite(cdata, 1, size, fp);
-    rewind(fp);
-#endif  /* HAVE_FMEMOPEN */
     pix = pixReadStreamBmp(fp);
     fclose(fp);
     if (!pix) L_ERROR("pix not read\n", procName);
@@ -591,16 +610,18 @@ PIX   *pix;
 
 
 /*!
- *  pixWriteMemBmp()
+ * \brief   pixWriteMemBmp()
  *
- *      Input:  &data (<return> data of tiff compressed image)
- *              &size (<return> size of returned data)
- *              pix
- *      Return: 0 if OK, 1 on error
+ * \param[out]   pdata data of tiff compressed image
+ * \param[out]   psize size of returned data
+ * \param[in]    pix
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixWriteStreamBmp() for usage.  This version writes to
  *          memory instead of to a file stream.
+ * </pre>
  */
 l_int32
 pixWriteMemBmp(l_uint8  **pdata,
@@ -624,8 +645,14 @@ FILE    *fp;
         return ERROR_INT("stream not opened", procName, 1);
     ret = pixWriteStreamBmp(fp, pix);
 #else
-    L_WARNING("work-around: writing to a temp file\n", procName);
-    fp = tmpfile();
+    L_INFO("work-around: writing to a temp file\n", procName);
+  #ifdef _WIN32
+    if ((fp = fopenWriteWinTempfile()) == NULL)
+        return ERROR_INT("tmpfile stream not opened", procName, 1);
+  #else
+    if ((fp = tmpfile()) == NULL)
+        return ERROR_INT("tmpfile stream not opened", procName, 1);
+  #endif  /* _WIN32 */
     ret = pixWriteStreamBmp(fp, pix);
     rewind(fp);
     *pdata = l_binaryReadStream(fp, psize);
@@ -634,6 +661,6 @@ FILE    *fp;
     return ret;
 }
 
+
 /* --------------------------------------------*/
 #endif  /* USE_BMPIO */
-/* --------------------------------------------*/
