@@ -24,8 +24,9 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *   boxfunc2.c
+/*!
+ * \file  boxfunc2.c
+ * <pre>
  *
  *      Boxa/Box transform (shift, scale) and orthogonal rotation
  *           BOXA            *boxaTransform()
@@ -43,8 +44,8 @@
  *           BOXAA           *boxaSort2dByIndex()
  *
  *      Boxa statistics
- *           BOX             *boxaGetRankSize()
- *           BOX             *boxaGetMedian()
+ *           l_int32          boxaGetRankVals()
+ *           l_int32          boxaGetMedianVals()
  *           l_int32          boxaGetAverageSize()
  *
  *      Boxa array extraction
@@ -56,7 +57,9 @@
  *           BOXA            *boxaaFlattenToBoxa()
  *           BOXA            *boxaaFlattenAligned()
  *           BOXAA           *boxaEncapsulateAligned()
+ *           BOXAA           *boxaaTranspose()
  *           l_int32          boxaaAlignBox()
+ * </pre>
  */
 
 #include <math.h>
@@ -72,15 +75,17 @@ static const l_int32   MIN_COMPS_FOR_BIN_SORT = 200;
  *      Boxa/Box transform (shift, scale) and orthogonal rotation      *
  *---------------------------------------------------------------------*/
 /*!
- *  boxaTransform()
+ * \brief   boxaTransform()
  *
- *      Input:  boxa
- *              shiftx, shifty
- *              scalex, scaley
- *      Return: boxad, or null on error
+ * \param[in]    boxas
+ * \param[in]    shiftx, shifty
+ * \param[in]    scalex, scaley
+ * \return  boxad, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is a very simple function that first shifts, then scales.
+ * </pre>
  */
 BOXA *
 boxaTransform(BOXA      *boxas,
@@ -113,16 +118,18 @@ BOXA    *boxad;
 
 
 /*!
- *  boxTransform()
+ * \brief   boxTransform()
  *
- *      Input:  box
- *              shiftx, shifty
- *              scalex, scaley
- *      Return: boxd, or null on error
+ * \param[in]    box
+ * \param[in]    shiftx, shifty
+ * \param[in]    scalex, scaley
+ * \return  boxd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is a very simple function that first shifts, then scales.
  *      (2) If the box is invalid, a new invalid box is returned.
+ * </pre>
  */
 BOX *
 boxTransform(BOX       *box,
@@ -146,17 +153,18 @@ boxTransform(BOX       *box,
 
 
 /*!
- *  boxaTransformOrdered()
+ * \brief   boxaTransformOrdered()
  *
- *      Input:  boxa
- *              shiftx, shifty
- *              scalex, scaley
- *              xcen, ycen (center of rotation)
- *              angle (in radians; clockwise is positive)
- *              order (one of 6 combinations: L_TR_SC_RO, ...)
- *      Return: boxd, or null on error
+ * \param[in]    boxas
+ * \param[in]    shiftx, shifty
+ * \param[in]    scalex, scaley
+ * \param[in]    xcen, ycen center of rotation
+ * \param[in]    angle in radians; clockwise is positive
+ * \param[in]    order one of 6 combinations: L_TR_SC_RO, ...
+ * \return  boxd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This allows a sequence of linear transforms on each box.
  *          the transforms are from the affine set, composed of
  *          shift, scaling and rotation, and the order of the
@@ -176,6 +184,7 @@ boxTransform(BOX       *box,
  *          center can be computed from the actual clipping due to
  *          translation of the image origin.
  *      (3) See boxTransformOrdered() for usage and implementation details.
+ * </pre>
  */
 BOXA *
 boxaTransformOrdered(BOXA      *boxas,
@@ -213,17 +222,18 @@ BOXA    *boxad;
 
 
 /*!
- *  boxTransformOrdered()
+ * \brief   boxTransformOrdered()
  *
- *      Input:  boxs
- *              shiftx, shifty
- *              scalex, scaley
- *              xcen, ycen (center of rotation)
- *              angle (in radians; clockwise is positive)
- *              order (one of 6 combinations: L_TR_SC_RO, ...)
- *      Return: boxd, or null on error
+ * \param[in]    boxs
+ * \param[in]    shiftx, shifty
+ * \param[in]    scalex, scaley
+ * \param[in]    xcen, ycen center of rotation
+ * \param[in]    angle in radians; clockwise is positive
+ * \param[in]    order one of 6 combinations: L_TR_SC_RO, ...
+ * \return  boxd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This allows a sequence of linear transforms, composed of
  *          shift, scaling and rotation, where the order of the
  *          transforms is specified.
@@ -243,7 +253,7 @@ BOXA    *boxad;
  *          Then the box UL corner (rx, ry) is
  *            rx = bxcen - 0.5 * rw
  *            ry = bycen - 0.5 * rh
- *      (3) The center of rotation specified by args @xcen and @ycen
+ *      (3) The center of rotation specified by args %xcen and %ycen
  *          is the point BEFORE any translation or scaling.  If the
  *          rotation is not the first operation, this function finds
  *          the actual center at the time of rotation.  It does this
@@ -260,6 +270,7 @@ BOXA    *boxad;
  *          in affinecompose.c, where the rotation center can be
  *          computed from the actual clipping due to translation
  *          of the image origin.
+ * </pre>
  */
 BOX *
 boxTransformOrdered(BOX       *boxs,
@@ -422,16 +433,18 @@ BOX       *boxd;
 
 
 /*!
- *  boxaRotateOrth()
+ * \brief   boxaRotateOrth()
  *
- *      Input:  boxa
- *              w, h (of image in which the boxa is embedded)
- *              rotation (0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
- *                        all rotations are clockwise)
- *      Return: boxad, or null on error
+ * \param[in]    boxas
+ * \param[in]    w, h of image in which the boxa is embedded
+ * \param[in]    rotation 0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
+ *                        all rotations are clockwise
+ * \return  boxad, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See boxRotateOrth() for details.
+ * </pre>
  */
 BOXA *
 boxaRotateOrth(BOXA    *boxas,
@@ -468,18 +481,20 @@ BOXA    *boxad;
 
 
 /*!
- *  boxRotateOrth()
+ * \brief   boxRotateOrth()
  *
- *      Input:  box
- *              w, h (of image in which the box is embedded)
- *              rotation (0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
- *                        all rotations are clockwise)
- *      Return: boxd, or null on error
+ * \param[in]    box
+ * \param[in]    w, h of image in which the box is embedded
+ * \param[in]    rotation 0 = noop, 1 = 90 deg, 2 = 180 deg, 3 = 270 deg;
+ *                        all rotations are clockwise
+ * \return  boxd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Rotate the image with the embedded box by the specified amount.
  *      (2) After rotation, the rotated box is always measured with
  *          respect to the UL corner of the image.
+ * </pre>
  */
 BOX *
 boxRotateOrth(BOX     *box,
@@ -516,19 +531,24 @@ l_int32  bx, by, bw, bh, xdist, ydist;
  *                              Boxa sort                              *
  *---------------------------------------------------------------------*/
 /*!
- *  boxaSort()
+ * \brief   boxaSort()
  *
- *      Input:  boxa
- *              sorttype (L_SORT_BY_X, L_SORT_BY_Y,
+ * \param[in]    boxas
+ * \param[in]    sorttype L_SORT_BY_X, L_SORT_BY_Y,
  *                        L_SORT_BY_RIGHT, L_SORT_BY_BOT,
  *                        L_SORT_BY_WIDTH, L_SORT_BY_HEIGHT,
  *                        L_SORT_BY_MIN_DIMENSION, L_SORT_BY_MAX_DIMENSION,
  *                        L_SORT_BY_PERIMETER, L_SORT_BY_AREA,
- *                        L_SORT_BY_ASPECT_RATIO)
- *              sortorder  (L_SORT_INCREASING, L_SORT_DECREASING)
- *              &naindex (<optional return> index of sorted order into
- *                        original array)
- *      Return: boxad (sorted version of boxas), or null on error
+ *                        L_SORT_BY_ASPECT_RATIO
+ * \param[in]    sortorder  L_SORT_INCREASING, L_SORT_DECREASING
+ * \param[out]   pnaindex [optional] index of sorted order into
+ *                        original array
+ * \return  boxad sorted version of boxas, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) An empty boxa returns a copy, with a warning.
+ * </pre>
  */
 BOXA *
 boxaSort(BOXA    *boxas,
@@ -545,6 +565,10 @@ NUMA      *na, *naindex;
     if (pnaindex) *pnaindex = NULL;
     if (!boxas)
         return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
+    if ((n = boxaGetCount(boxas)) == 0) {
+        L_WARNING("boxas is empty\n", procName);
+        return boxaCopy(boxas, L_COPY);
+    }
     if (sorttype != L_SORT_BY_X && sorttype != L_SORT_BY_Y &&
         sorttype != L_SORT_BY_RIGHT && sorttype != L_SORT_BY_BOT &&
         sorttype != L_SORT_BY_WIDTH && sorttype != L_SORT_BY_HEIGHT &&
@@ -558,7 +582,6 @@ NUMA      *na, *naindex;
         return (BOXA *)ERROR_PTR("invalid sort order", procName, NULL);
 
         /* Use O(n) binsort if possible */
-    n = boxaGetCount(boxas);
     if (n > MIN_COMPS_FOR_BIN_SORT &&
         ((sorttype == L_SORT_BY_X) || (sorttype == L_SORT_BY_Y) ||
          (sorttype == L_SORT_BY_WIDTH) || (sorttype == L_SORT_BY_HEIGHT) ||
@@ -631,22 +654,24 @@ NUMA      *na, *naindex;
 
 
 /*!
- *  boxaBinSort()
+ * \brief   boxaBinSort()
  *
- *      Input:  boxa
- *              sorttype (L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
- *                        L_SORT_BY_HEIGHT, L_SORT_BY_PERIMETER)
- *              sortorder  (L_SORT_INCREASING, L_SORT_DECREASING)
- *              &naindex (<optional return> index of sorted order into
- *                        original array)
- *      Return: boxad (sorted version of boxas), or null on error
+ * \param[in]    boxas
+ * \param[in]    sorttype L_SORT_BY_X, L_SORT_BY_Y, L_SORT_BY_WIDTH,
+ *                        L_SORT_BY_HEIGHT, L_SORT_BY_PERIMETER
+ * \param[in]    sortorder  L_SORT_INCREASING, L_SORT_DECREASING
+ * \param[out]   pnaindex [optional] index of sorted order into
+ *                        original array
+ * \return  boxad sorted version of boxas, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) For a large number of boxes (say, greater than 1000), this
  *          O(n) binsort is much faster than the O(nlogn) shellsort.
  *          For 5000 components, this is over 20x faster than boxaSort().
  *      (2) Consequently, boxaSort() calls this function if it will
  *          likely go much faster.
+ * </pre>
  */
 BOXA *
 boxaBinSort(BOXA    *boxas,
@@ -663,6 +688,10 @@ NUMA    *na, *naindex;
     if (pnaindex) *pnaindex = NULL;
     if (!boxas)
         return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
+    if ((n = boxaGetCount(boxas)) == 0) {
+        L_WARNING("boxas is empty\n", procName);
+        return boxaCopy(boxas, L_COPY);
+    }
     if (sorttype != L_SORT_BY_X && sorttype != L_SORT_BY_Y &&
         sorttype != L_SORT_BY_WIDTH && sorttype != L_SORT_BY_HEIGHT &&
         sorttype != L_SORT_BY_PERIMETER)
@@ -671,7 +700,6 @@ NUMA    *na, *naindex;
         return (BOXA *)ERROR_PTR("invalid sort order", procName, NULL);
 
         /* Generate Numa of appropriate box dimensions */
-    n = boxaGetCount(boxas);
     if ((na = numaCreate(n)) == NULL)
         return (BOXA *)ERROR_PTR("na not made", procName, NULL);
     for (i = 0; i < n; i++) {
@@ -715,11 +743,11 @@ NUMA    *na, *naindex;
 
 
 /*!
- *  boxaSortByIndex()
+ * \brief   boxaSortByIndex()
  *
- *      Input:  boxas
- *              naindex (na that maps from the new boxa to the input boxa)
- *      Return: boxad (sorted), or null on error
+ * \param[in]    boxas
+ * \param[in]    naindex na that maps from the new boxa to the input boxa
+ * \return  boxad sorted, or NULL on error
  */
 BOXA *
 boxaSortByIndex(BOXA  *boxas,
@@ -733,10 +761,13 @@ BOXA    *boxad;
 
     if (!boxas)
         return (BOXA *)ERROR_PTR("boxas not defined", procName, NULL);
+    if ((n = boxaGetCount(boxas)) == 0) {
+        L_WARNING("boxas is empty\n", procName);
+        return boxaCopy(boxas, L_COPY);
+    }
     if (!naindex)
         return (BOXA *)ERROR_PTR("naindex not defined", procName, NULL);
 
-    n = boxaGetCount(boxas);
     boxad = boxaCreate(n);
     for (i = 0; i < n; i++) {
         numaGetIValue(naindex, i, &index);
@@ -749,20 +780,21 @@ BOXA    *boxad;
 
 
 /*!
- *  boxaSort2d()
+ * \brief   boxaSort2d()
  *
- *      Input:  boxas
- *              &naa (<optional return> numaa with sorted indices
- *                    whose values are the indices of the input array)
- *              delta1 (min overlap that permits aggregation of a box
- *                      onto a boxa of horizontally-aligned boxes; pass 1)
- *              delta2 (min overlap that permits aggregation of a box
- *                      onto a boxa of horizontally-aligned boxes; pass 2)
- *              minh1 (components less than this height either join an
- *                     existing boxa or are set aside for pass 2)
- *      Return: baa (2d sorted version of boxa), or null on error
+ * \param[in]    boxas
+ * \param[out]   pnaad [optional] numaa with sorted indices
+ *                    whose values are the indices of the input array
+ * \param[in]    delta1 min overlap that permits aggregation of a box
+ *                      onto a boxa of horizontally-aligned boxes; pass 1
+ * \param[in]    delta2 min overlap that permits aggregation of a box
+ *                      onto a boxa of horizontally-aligned boxes; pass 2
+ * \param[in]    minh1 components less than this height either join an
+ *                     existing boxa or are set aside for pass 2
+ * \return  baa 2d sorted version of boxa, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The final result is a sort where the 'fast scan' direction is
  *          left to right, and the 'slow scan' direction is from top
  *          to bottom.  Each boxa in the baa represents a sorted set
@@ -776,7 +808,7 @@ BOXA    *boxad;
  *          is overlapping are joined.  After that, the boxes in each
  *          boxa are sorted horizontally, and finally the boxa are
  *          sorted vertically.
- *      (3) If delta1 < 0, the first pass allows aggregation when
+ *      (3) If delta1 \< 0, the first pass allows aggregation when
  *          boxes in the same boxa do not overlap vertically.
  *          The distance by which they can miss and still be aggregated
  *          is the absolute value |delta1|.   Similar for delta2 on
@@ -787,6 +819,7 @@ BOXA    *boxad;
  *          with an existing boxa can start a new one.
  *      (6) This can be used to identify lines of text from
  *          character or word bounding boxes.
+ * </pre>
  */
 BOXAA *
 boxaSort2d(BOXA    *boxas,
@@ -807,6 +840,8 @@ NUMAA   *naa, *naa1, *naad;
     if (pnaad) *pnaad = NULL;
     if (!boxas)
         return (BOXAA *)ERROR_PTR("boxas not defined", procName, NULL);
+    if (boxaGetCount(boxas) == 0)
+        return (BOXAA *)ERROR_PTR("boxas is empty", procName, NULL);
 
         /* Sort from left to right */
     if ((boxa = boxaSort(boxas, L_SORT_BY_X, L_SORT_INCREASING, &naindex))
@@ -967,11 +1002,11 @@ NUMAA   *naa, *naa1, *naad;
 
 
 /*!
- *  boxaSort2dByIndex()
+ * \brief   boxaSort2dByIndex()
  *
- *      Input:  boxas
- *              naa (numaa that maps from the new baa to the input boxa)
- *      Return: baa (sorted boxaa), or null on error
+ * \param[in]    boxas
+ * \param[in]    naa numaa that maps from the new baa to the input boxa
+ * \return  baa sorted boxaa, or NULL on error
  */
 BOXAA *
 boxaSort2dByIndex(BOXA   *boxas,
@@ -987,12 +1022,13 @@ NUMA    *na;
 
     if (!boxas)
         return (BOXAA *)ERROR_PTR("boxas not defined", procName, NULL);
+    if ((boxtot = boxaGetCount(boxas)) == 0)
+        return (BOXAA *)ERROR_PTR("boxas is empty", procName, NULL);
     if (!naa)
         return (BOXAA *)ERROR_PTR("naindex not defined", procName, NULL);
 
         /* Check counts */
     ntot = numaaGetNumberCount(naa);
-    boxtot = boxaGetCount(boxas);
     if (ntot != boxtot)
         return (BOXAA *)ERROR_PTR("element count mismatch", procName, NULL);
 
@@ -1019,25 +1055,27 @@ NUMA    *na;
  *                        Boxa array extraction                        *
  *---------------------------------------------------------------------*/
 /*!
- *  boxaExtractAsNuma()
+ * \brief   boxaExtractAsNuma()
  *
- *      Input:  boxa
- *              &nal (<optional return> array of left locations)
- *              &nat (<optional return> array of top locations)
- *              &nar (<optional return> array of right locations)
- *              &nab (<optional return> array of bottom locations)
- *              &naw (<optional return> array of widths)
- *              &nah (<optional return> array of heights)
- *              keepinvalid (1 to keep invalid boxes; 0 to remove them)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    boxa
+ * \param[out]   pnal [optional] array of left locations
+ * \param[out]   pnat [optional] array of top locations
+ * \param[out]   pnar [optional] array of right locations
+ * \param[out]   pnab [optional] array of bottom locations
+ * \param[out]   pnaw [optional] array of widths
+ * \param[out]   pnah [optional] array of heights
+ * \param[in]    keepinvalid 1 to keep invalid boxes; 0 to remove them
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) If you are counting or sorting values, such as determining
  *          rank order, you must remove invalid boxes.
  *      (2) If you are parametrizing the values, or doing an evaluation
  *          where the position in the boxa sequence is important, you
  *          must replace the invalid boxes with valid ones before
  *          doing the extraction. This is easily done with boxaFillSequence().
+ * </pre>
  */
 l_int32
 boxaExtractAsNuma(BOXA    *boxa,
@@ -1092,19 +1130,20 @@ l_int32  i, n, left, top, right, bot, w, h;
 
 
 /*!
- *  boxaExtractAsPta()
+ * \brief   boxaExtractAsPta()
  *
- *      Input:  boxa
- *              &ptal (<optional return> array of left locations vs. index)
- *              &ptat (<optional return> array of top locations vs. index)
- *              &ptar (<optional return> array of right locations vs. index)
- *              &ptab (<optional return> array of bottom locations vs. index)
- *              &ptaw (<optional return> array of widths vs. index)
- *              &ptah (<optional return> array of heights vs. index)
- *              keepinvalid (1 to keep invalid boxes; 0 to remove them)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    boxa
+ * \param[out]   pptal [optional] array of left locations vs. index
+ * \param[out]   pptat [optional] array of top locations vs. index
+ * \param[out]   pptar [optional] array of right locations vs. index
+ * \param[out]   pptab [optional] array of bottom locations vs. index
+ * \param[out]   pptaw [optional] array of widths vs. index
+ * \param[out]   pptah [optional] array of heights vs. index
+ * \param[in]    keepinvalid 1 to keep invalid boxes; 0 to remove them
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) For most applications, such as counting, sorting, fitting
  *          to some parametrized form, plotting or filtering in general,
  *          you should remove the invalid boxes.  Each pta saves the
@@ -1113,6 +1152,7 @@ l_int32  i, n, left, top, right, bot, w, h;
  *          boxaExtractAsNuma(), is not necessary.
  *      (2) If invalid boxes are retained, each one will result in
  *          entries (typically 0) in all selected output pta.
+ * </pre>
  */
 l_int32
 boxaExtractAsPta(BOXA    *boxa,
@@ -1170,14 +1210,18 @@ l_int32  i, n, left, top, right, bot, w, h;
  *                            Boxa statistics                          *
  *---------------------------------------------------------------------*/
 /*!
- *  boxaGetRankSize()
+ * \brief   boxaGetRankVals()
  *
- *      Input:  boxa
- *              fract (use 0.0 for smallest, 1.0 for largest)
- *      Return: box (with rank values for x, y, w, h), or null on error
- *              or if the boxa is empty (has no valid boxes)
+ * \param[in]    boxa
+ * \param[in]    fract use 0.0 for smallest, 1.0 for largest width and height
+ * \param[out]   px  [optional] rank value of x
+ * \param[out]   py  [optional] rank value of y
+ * \param[out]   pw  [optional] rank value of width
+ * \param[out]   ph  [optional] rank value of height
+ * \return  0 if OK, 1 on error or if the boxa is empty or has no valid boxes
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function does not assume that all boxes in the boxa are valid
  *      (2) The four box parameters are sorted independently.
  *          For rank order, the width and height are sorted in increasing
@@ -1187,74 +1231,101 @@ l_int32  i, n, left, top, right, bot, w, h;
  *          to give a "rank order" for x and y by sorting them in
  *          decreasing order.  But in general, the interpretation of a rank
  *          order on x and y is highly application dependent.  In summary:
- *             - x and y are sorted in decreasing order
- *             - w and h are sorted in increasing order
+ *             ~ x and y are sorted in decreasing order
+ *             ~ w and h are sorted in increasing order
+ * </pre>
  */
-BOX *
-boxaGetRankSize(BOXA      *boxa,
-                l_float32  fract)
+l_int32
+boxaGetRankVals(BOXA      *boxa,
+                l_float32  fract,
+                l_int32   *px,
+                l_int32   *py,
+                l_int32   *pw,
+                l_int32   *ph)
 {
 l_float32  xval, yval, wval, hval;
 NUMA      *nax, *nay, *naw, *nah;
-BOX       *box;
 
-    PROCNAME("boxaGetRankSize");
+    PROCNAME("boxaGetRankVals");
 
+    if (px) *px = 0;
+    if (py) *py = 0;
+    if (pw) *pw = 0;
+    if (ph) *ph = 0;
     if (!boxa)
-        return (BOX *)ERROR_PTR("boxa not defined", procName, NULL);
+        return ERROR_INT("boxa not defined", procName, 1);
     if (fract < 0.0 || fract > 1.0)
-        return (BOX *)ERROR_PTR("fract not in [0.0 ... 1.0]", procName, NULL);
+        return ERROR_INT("fract not in [0.0 ... 1.0]", procName, 1);
     if (boxaGetValidCount(boxa) == 0)
-        return (BOX *)ERROR_PTR("no valid boxes in boxa", procName, NULL);
+        return ERROR_INT("no valid boxes in boxa", procName, 1);
 
         /* Use only the valid boxes */
     boxaExtractAsNuma(boxa, &nax, &nay, NULL, NULL, &naw, &nah, 0);
 
-    numaGetRankValue(nax, 1.0 - fract, NULL, 1, &xval);
-    numaGetRankValue(nay, 1.0 - fract, NULL, 1, &yval);
-    numaGetRankValue(naw, fract, NULL, 1, &wval);
-    numaGetRankValue(nah, fract, NULL, 1, &hval);
-    box = boxCreate((l_int32)xval, (l_int32)yval, (l_int32)wval, (l_int32)hval);
-
+    if (px) {
+        numaGetRankValue(nax, 1.0 - fract, NULL, 1, &xval);
+        *px = (l_int32)xval;
+    }
+    if (py) {
+        numaGetRankValue(nay, 1.0 - fract, NULL, 1, &yval);
+        *py = (l_int32)yval;
+    }
+    if (pw) {
+        numaGetRankValue(naw, fract, NULL, 1, &wval);
+        *pw = (l_int32)wval;
+    }
+    if (ph) {
+        numaGetRankValue(nah, fract, NULL, 1, &hval);
+        *ph = (l_int32)hval;
+    }
     numaDestroy(&nax);
     numaDestroy(&nay);
     numaDestroy(&naw);
     numaDestroy(&nah);
-    return box;
+    return 0;
 }
 
 
 /*!
- *  boxaGetMedian()
+ * \brief   boxaGetMedianVals()
  *
- *      Input:  boxa
- *      Return: box (with median values for x, y, w, h), or null on error
- *              or if the boxa is empty.
+ * \param[in]    boxa
+ * \param[out]   px  [optional] median value of x
+ * \param[out]   py  [optional] median value of y
+ * \param[out]   pw  [optional] median value of width
+ * \param[out]   ph  [optional] median value of height
+ * \return  0 if OK, 1 on error or if the boxa is empty or has no valid boxes
  *
- *  Notes:
- *      (1) See boxaGetRankSize()
+ * <pre>
+ * Notes:
+ *      (1) See boxaGetRankVals()
+ * </pre>
  */
-BOX *
-boxaGetMedian(BOXA  *boxa)
+l_int32
+boxaGetMedianVals(BOXA     *boxa,
+                  l_int32  *px,
+                  l_int32  *py,
+                  l_int32  *pw,
+                  l_int32  *ph)
 {
-    PROCNAME("boxaGetMedian");
+    PROCNAME("boxaGetMedianVals");
 
     if (!boxa)
-        return (BOX *)ERROR_PTR("boxa not defined", procName, NULL);
-    if (boxaGetCount(boxa) == 0)
-        return (BOX *)ERROR_PTR("boxa is empty", procName, NULL);
+        return ERROR_INT("boxa not defined", procName, 1);
+    if (boxaGetValidCount(boxa) == 0)
+        return ERROR_INT("no valid boxes in boxa", procName, 1);
 
-    return boxaGetRankSize(boxa, 0.5);
+    return boxaGetRankVals(boxa, 0.5, px, py, pw, ph);
 }
 
 
 /*!
- *  boxaGetAverageSize()
+ * \brief   boxaGetAverageSize()
  *
- *      Input:  boxa
- *              &w  (<optional return> average width)
- *              &h  (<optional return> average height)
- *      Return: 0 if OK, 1 on error or if the boxa is empty
+ * \param[in]    boxa
+ * \param[out]   pw  [optional] average width
+ * \param[out]   ph  [optional] average height
+ * \return  0 if OK, 1 on error or if the boxa is empty
  */
 l_int32
 boxaGetAverageSize(BOXA       *boxa,
@@ -1290,24 +1361,26 @@ l_float32  sumw, sumh;
  *                        Other Boxaa functions                        *
  *---------------------------------------------------------------------*/
 /*!
- *  boxaaGetExtent()
+ * \brief   boxaaGetExtent()
  *
- *      Input:  baa
- *              &w  (<optional return> width)
- *              &h  (<optional return> height)
- *              &box (<optional return>, minimum box containing all boxa
- *                    in boxaa)
- *              &boxa (<optional return>, boxa containing all boxes in each
- *                     boxa in the boxaa)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    baa
+ * \param[out]   pw  [optional] width
+ * \param[out]   ph  [optional] height
+ * \param[out]   pbox [optional]  minimum box containing all boxa
+ *                    in boxaa
+ * \param[out]   pboxa [optional]  boxa containing all boxes in each
+ *                     boxa in the boxaa
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The returned w and h are the minimum size image
  *          that would contain all boxes untranslated.
  *      (2) Each box in the returned boxa is the minimum box required to
  *          hold all the boxes in the respective boxa of baa.
  *      (3) If there are no valid boxes in a boxa, the box corresponding
  *          to its extent has all fields set to 0 (an invalid box).
+ * </pre>
  */
 l_int32
 boxaaGetExtent(BOXAA    *baa,
@@ -1369,14 +1442,15 @@ BOXA    *boxa, *boxa1;
 
 
 /*!
- *  boxaaFlattenToBoxa()
+ * \brief   boxaaFlattenToBoxa()
  *
- *      Input:  baa
- *              &naindex  (<optional return> the boxa index in the baa)
- *              copyflag  (L_COPY or L_CLONE)
- *      Return: boxa, or null on error
+ * \param[in]    baa
+ * \param[out]   pnaindex  [optional] the boxa index in the baa
+ * \param[in]    copyflag  L_COPY or L_CLONE
+ * \return  boxa, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This 'flattens' the baa to a boxa, taking the boxes in
  *          order in the first boxa, then the second, etc.
  *      (2) If a boxa is empty, we generate an invalid, placeholder box
@@ -1384,8 +1458,9 @@ BOXA    *boxa, *boxa1;
  *          where each boxa has either 0 or 1 boxes, and it is necessary
  *          to maintain a 1:1 correspondence between the initial
  *          boxa array and the resulting box array.
- *      (3) If &naindex is defined, we generate a Numa that gives, for
+ *      (3) If \&naindex is defined, we generate a Numa that gives, for
  *          each box in the baa, the index of the boxa to which it belongs.
+ * </pre>
  */
 BOXA *
 boxaaFlattenToBoxa(BOXAA   *baa,
@@ -1435,21 +1510,23 @@ NUMA    *naindex;
 
 
 /*!
- *  boxaaFlattenAligned()
+ * \brief   boxaaFlattenAligned()
  *
- *      Input:  baa
- *              num (number extracted from each)
- *              fillerbox (<optional> that fills if necessary)
- *              copyflag  (L_COPY or L_CLONE)
- *      Return: boxa, or null on error
+ * \param[in]    baa
+ * \param[in]    num number extracted from each
+ * \param[in]    fillerbox [optional] that fills if necessary
+ * \param[in]    copyflag  L_COPY or L_CLONE
+ * \return  boxa, or NULL on error
  *
- *  Notes:
- *      (1) This 'flattens' the baa to a boxa, taking the first @num
+ * <pre>
+ * Notes:
+ *      (1) This 'flattens' the baa to a boxa, taking the first %num
  *          boxes from each boxa.
- *      (2) In each boxa, if there are less than @num boxes, we preserve
+ *      (2) In each boxa, if there are less than %num boxes, we preserve
  *          the alignment between the input baa and the output boxa
- *          by inserting one or more fillerbox(es) or, if @fillerbox == NULL,
+ *          by inserting one or more fillerbox(es) or, if %fillerbox == NULL,
  *          one or more invalid placeholder boxes.
+ * </pre>
  */
 BOXA *
 boxaaFlattenAligned(BOXAA   *baa,
@@ -1475,7 +1552,7 @@ BOX     *box;
         m = boxaGetCount(boxat);
         mval = L_MIN(m, num);
         nshort = num - mval;
-        for (j = 0; j < mval; j++) {  /* take the first @num if possible */
+        for (j = 0; j < mval; j++) {  /* take the first %num if possible */
             box = boxaGetBox(boxat, j, copyflag);
             boxaAddBox(boxad, box, L_INSERT);
         }
@@ -1495,17 +1572,19 @@ BOX     *box;
 
 
 /*!
- *  boxaEncapsulateAligned()
+ * \brief   boxaEncapsulateAligned()
  *
- *      Input:  boxa
- *              num (number put into each boxa in the baa)
- *              copyflag  (L_COPY or L_CLONE)
- *      Return: baa, or null on error
+ * \param[in]    boxa
+ * \param[in]    num number put into each boxa in the baa
+ * \param[in]    copyflag  L_COPY or L_CLONE
+ * \return  baa, or NULL on error
  *
- *  Notes:
- *      (1) This puts @num boxes from the input @boxa into each of a
+ * <pre>
+ * Notes:
+ *      (1) This puts %num boxes from the input %boxa into each of a
  *          set of boxa within an output baa.
- *      (2) This assumes that the boxes in @boxa are in sets of @num each.
+ *      (2) This assumes that the boxes in %boxa are in sets of %num each.
+ * </pre>
  */
 BOXAA *
 boxaEncapsulateAligned(BOXA    *boxa,
@@ -1543,19 +1622,82 @@ BOXAA   *baa;
 
 
 /*!
- *  boxaaAlignBox()
+ * \brief   boxaaTranspose()
  *
- *      Input:  baa
- *              box (to be aligned with the bext boxa in the baa, if possible)
- *              delta (amount by which consecutive components can miss
- *                     in overlap and still be included in the array)
- *              &index (of boxa with best overlap, or if none match,
- *                      this is the index of the next boxa to be generated)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    baas
+ * \return  baad, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
+ *      (1) If you think of a boxaa as a 2D array of boxes that is accessed
+ *          row major, then each row is represented by one of the boxa.
+ *          This function creates a new boxaa related to the input boxaa
+ *          as a column major traversal of the input boxaa.
+ *      (2) For example, if %baas has 2 boxa, each with 10 boxes, then
+ *          %baad will have 10 boxa, each with 2 boxes.
+ *      (3) Require for this transpose operation that each boxa in
+ *          %baas has the same number of boxes.  This operation is useful
+ *          when the i-th boxes in each boxa are meaningfully related.
+ * </pre>
+ */
+BOXAA *
+boxaaTranspose(BOXAA  *baas)
+{
+l_int32   i, j, ny, nb, nbox;
+BOX      *box;
+BOXA     *boxa;
+BOXAA    *baad;
+
+    PROCNAME("boxaaTranspose");
+
+    if (!baas)
+        return (BOXAA *)ERROR_PTR("baas not defined", procName, NULL);
+    if ((ny = boxaaGetCount(baas)) == 0)
+        return (BOXAA *)ERROR_PTR("baas empty", procName, NULL);
+
+        /* Make sure that each boxa in baas has the same number of boxes */
+    for (i = 0; i < ny; i++) {
+        if ((boxa = boxaaGetBoxa(baas, i, L_CLONE)) == NULL)
+            return (BOXAA *)ERROR_PTR("baas is missing a boxa", procName, NULL);
+        nb = boxaGetCount(boxa);
+        boxaDestroy(&boxa);
+        if (i == 0)
+            nbox = nb;
+        else if (nb != nbox)
+            return (BOXAA *)ERROR_PTR("boxa are not all the same size",
+                                      procName, NULL);
+    }
+
+        /* baad[i][j] = baas[j][i] */
+    baad = boxaaCreate(nbox);
+    for (i = 0; i < nbox; i++) {
+        boxa = boxaCreate(ny);
+        for (j = 0; j < ny; j++) {
+            box = boxaaGetBox(baas, j, i, L_COPY);
+            boxaAddBox(boxa, box, L_INSERT);
+        }
+        boxaaAddBoxa(baad, boxa, L_INSERT);
+    }
+    return baad;
+}
+
+
+/*!
+ * \brief   boxaaAlignBox()
+ *
+ * \param[in]    baa
+ * \param[in]    box to be aligned with the bext boxa in the baa, if possible
+ * \param[in]    delta amount by which consecutive components can miss
+ *                     in overlap and still be included in the array
+ * \param[out]   pindex index of boxa with best overlap, or if none match,
+ *                      this is the index of the next boxa to be generated
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
  *      (1) This is not greedy.  It finds the boxa whose vertical
  *          extent has the closest overlap with the input box.
+ * </pre>
  */
 l_int32
 boxaaAlignBox(BOXAA    *baa,
@@ -1569,6 +1711,7 @@ BOXA    *boxa;
 
     PROCNAME("boxaaAlignBox");
 
+    if (pindex) *pindex = 0;
     if (!baa)
         return ERROR_INT("baa not defined", procName, 1);
     if (!box)
