@@ -1,21 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-
-/*
- *  classapp.c
+/*!
+ * \file classapp.c
+ * <pre>
  *
  *      Top-level jb2 correlation and rank-hausdorff
  *
@@ -35,16 +46,15 @@
  *         static l_int32  testLineAlignmentX()
  *         static l_int32  countAlignedMatches()
  *         static void     printRowIndices()
+ * </pre>
  */
 
 #include <string.h>
 #include "allheaders.h"
 
-static const l_int32  JB_WORDS_MIN_WIDTH = 5;  /* pixels */
-static const l_int32  JB_WORDS_MIN_HEIGHT = 3;  /* pixels */
-
-    /* MSVC can't handle arrays dimensioned by static const integers */
-#define  L_BUF_SIZE  512
+static const l_int32 L_BUF_SIZE = 512;        /*!< size of filename buffer    */
+static const l_int32 JB_WORDS_MIN_WIDTH = 5;  /*!< min. word width in pixels  */
+static const l_int32 JB_WORDS_MIN_HEIGHT = 3; /*!< min. word height in pixels */
 
     /* Static comparison functions */
 static l_int32 testLineAlignmentX(NUMA *na1, NUMA *na2, l_int32 shiftx,
@@ -62,23 +72,25 @@ static void printRowIndices(l_int32 *index1, l_int32 n1,
  *          Top-level jb2 correlation and rank-hausdorff            *
  *------------------------------------------------------------------*/
 /*!
- *  jbCorrelation()
+ * \brief   jbCorrelation()
  *
- *       Input:  dirin (directory of input images)
- *               thresh (typically ~0.8)
- *               weight (typically ~0.6)
- *               components (JB_CONN_COMPS, JB_CHARACTERS, JB_WORDS)
- *               rootname (for output files)
- *               firstpage (0-based)
- *               npages (use 0 for all pages in dirin)
- *               renderflag (1 to render from templates; 0 to skip)
- *       Return: 0 if OK, 1 on error
+ * \param[in]    dirin directory of input images
+ * \param[in]    thresh typically ~0.8
+ * \param[in]    weight typically ~0.6
+ * \param[in]    components JB_CONN_COMPS, JB_CHARACTERS, JB_WORDS
+ * \param[in]    rootname for output files
+ * \param[in]    firstpage 0-based
+ * \param[in]    npages use 0 for all pages in dirin
+ * \param[in]    renderflag 1 to render from templates; 0 to skip
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The images must be 1 bpp.  If they are not, you can convert
  *          them using convertFilesTo1bpp().
  *      (2) See prog/jbcorrelation for generating more output (e.g.,
  *          for debugging)
+ * </pre>
  */
 l_int32
 jbCorrelation(const char  *dirin,
@@ -128,7 +140,7 @@ SARRAY     *safiles;
                     numpages, nfiles);
         for (i = 0; i < numpages; i++) {
             pix = pixaGetPix(pixa, i, L_CLONE);
-            snprintf(filename, L_BUF_SIZE, "%s.%05d", rootname, i);
+            snprintf(filename, L_BUF_SIZE, "%s.%04d", rootname, i);
             fprintf(stderr, "filename: %s\n", filename);
             pixWrite(filename, pix, IFF_PNG);
             pixDestroy(&pix);
@@ -144,21 +156,23 @@ SARRAY     *safiles;
 
 
 /*!
- *  jbRankHaus()
+ * \brief   jbRankHaus()
  *
- *       Input:  dirin (directory of input images)
- *               size (of Sel used for dilation; typ. 2)
- *               rank (rank value of match; typ. 0.97)
- *               components (JB_CONN_COMPS, JB_CHARACTERS, JB_WORDS)
- *               rootname (for output files)
- *               firstpage (0-based)
- *               npages (use 0 for all pages in dirin)
- *               renderflag (1 to render from templates; 0 to skip)
- *       Return: 0 if OK, 1 on error
+ * \param[in]    dirin directory of input images
+ * \param[in]    size of Sel used for dilation; typ. 2
+ * \param[in]    rank rank value of match; typ. 0.97
+ * \param[in]    components JB_CONN_COMPS, JB_CHARACTERS, JB_WORDS
+ * \param[in]    rootname for output files
+ * \param[in]    firstpage 0-based
+ * \param[in]    npages use 0 for all pages in dirin
+ * \param[in]    renderflag 1 to render from templates; 0 to skip
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See prog/jbrankhaus for generating more output (e.g.,
  *          for debugging)
+ * </pre>
  */
 l_int32
 jbRankHaus(const char  *dirin,
@@ -208,7 +222,7 @@ SARRAY     *safiles;
                     numpages, nfiles);
         for (i = 0; i < numpages; i++) {
             pix = pixaGetPix(pixa, i, L_CLONE);
-            snprintf(filename, L_BUF_SIZE, "%s.%05d", rootname, i);
+            snprintf(filename, L_BUF_SIZE, "%s.%04d", rootname, i);
             fprintf(stderr, "filename: %s\n", filename);
             pixWrite(filename, pix, IFF_PNG);
             pixDestroy(&pix);
@@ -228,20 +242,21 @@ SARRAY     *safiles;
  *           Extract and classify words in textline order           *
  *------------------------------------------------------------------*/
 /*!
- *  jbWordsInTextlines()
+ * \brief   jbWordsInTextlines()
  *
- *      Input:  dirin (directory of input pages)
- *              reduction (1 for full res; 2 for half-res)
- *              maxwidth (of word mask components, to be kept)
- *              maxheight (of word mask components, to be kept)
- *              thresh (on correlation; 0.80 is reasonable)
- *              weight (for handling thick text; 0.6 is reasonable)
- *              natl (<return> numa with textline index for each component)
- *              firstpage (0-based)
- *              npages (use 0 for all pages in dirin)
- *      Return: classer (for the set of pages)
+ * \param[in]    dirin directory of input pages
+ * \param[in]    reduction 1 for full res; 2 for half-res
+ * \param[in]    maxwidth of word mask components, to be kept
+ * \param[in]    maxheight of word mask components, to be kept
+ * \param[in]    thresh on correlation; 0.80 is reasonable
+ * \param[in]    weight for handling thick text; 0.6 is reasonable
+ * \param[out]   pnatl numa with textline index for each component
+ * \param[in]    firstpage 0-based
+ * \param[in]    npages use 0 for all pages in dirin
+ * \return  classer for the set of pages
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is a high-level function.  See prog/jbwords for example
  *          of usage.
  *      (2) Typically, words can be found reasonably well at a resolution
@@ -249,6 +264,7 @@ SARRAY     *safiles;
  *          Assuming that the input images are 300 ppi, use reduction = 1
  *          for finding words at full res, and reduction = 2 for finding
  *          them at 150 ppi.
+ * </pre>
  */
 JBCLASSER *
 jbWordsInTextlines(const char  *dirin,
@@ -289,17 +305,16 @@ SARRAY     *safiles;
     natl = numaCreate(0);
     *pnatl = natl;
     for (i = 0; i < nfiles; i++) {
-        fname = sarrayGetString(safiles, i, 0);
+        fname = sarrayGetString(safiles, i, L_NOCOPY);
         if ((pix = pixRead(fname)) == NULL) {
-            L_WARNING_INT("image file %d not read", procName, i);
+            L_WARNING("image file %d not read\n", procName, i);
             continue;
         }
         pixGetDimensions(pix, &w, &h, NULL);
         if (reduction == 1) {
             classer->w = w;
             classer->h = h;
-        }
-        else {  /* reduction == 2 */
+        } else {  /* reduction == 2 */
             classer->w = w / 2;
             classer->h = h / 2;
         }
@@ -307,7 +322,7 @@ SARRAY     *safiles;
                                JB_WORDS_MIN_HEIGHT, maxwidth, maxheight,
                                &boxa, &pixa, &nai);
         jbAddPageComponents(classer, pix, boxa, pixa);
-        numaJoin(natl, nai, 0, 0);
+        numaJoin(natl, nai, 0, -1);
         pixDestroy(&pix);
         numaDestroy(&nai);
         boxaDestroy(&boxa);
@@ -320,34 +335,35 @@ SARRAY     *safiles;
 
 
 /*!
- *  pixGetWordsInTextlines()
+ * \brief   pixGetWordsInTextlines()
  *
- *      Input:  pixs (1 bpp, 300 ppi)
- *              reduction (1 for full res; 2 for half-res)
- *              minwidth, minheight (of saved components; smaller are discarded)
- *              maxwidth, maxheight (of saved components; larger are discarded)
- *              &boxad (<return> word boxes sorted in textline line order)
- *              &pixad (<return> word images sorted in textline line order)
- *              &naindex (<return> index of textline for each word)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs 1 bpp, typ. 300 ppi
+ * \param[in]    reduction 1 for input res; 2 for 2x reduction of input res
+ * \param[in]    minwidth, minheight of saved components; smaller are discarded
+ * \param[in]    maxwidth, maxheight of saved components; larger are discarded
+ * \param[out]   pboxad word boxes sorted in textline line order
+ * \param[out]   ppixad word images sorted in textline line order
+ * \param[out]   pnai index of textline for each word
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The input should be at a resolution of about 300 ppi.
- *          The word masks can be computed at either 150 ppi or 300 ppi.
- *          For the former, set reduction = 2.
+ *          The word masks and word images can be computed at either
+ *          150 ppi or 300 ppi.  For the former, set reduction = 2.
  *      (2) The four size constraints on saved components are all
- *          used at 2x reduction.
+ *          scaled by %reduction.
  *      (3) The result are word images (and their b.b.), extracted in
- *          textline order, all at 2x reduction, and with a numa giving
- *          the textline index for each word.
+ *          textline order, at either full res or 2x reduction,
+ *          and with a numa giving the textline index for each word.
  *      (4) The pixa and boxa interfaces should make this type of
  *          application simple to put together.  The steps are:
- *           - generate first estimate of word masks
- *           - get b.b. of these, and remove the small and big ones
- *           - extract pixa of the word mask from these boxes
- *           - extract pixa of the actual word images, using word masks
- *           - sort actual word images in textline order (2d)
- *           - flatten them to a pixa (1d), saving the textline index
+ *           ~ optionally reduce by 2x
+ *           ~ generate first estimate of word masks
+ *           ~ get b.b. of these, and remove the small and big ones
+ *           ~ extract pixa of the word images, using the b.b.
+ *           ~ sort actual word images in textline order (2d)
+ *           ~ flatten them to a pixa (1d), saving the textline index
  *             for each pix
  *      (5) In an actual application, it may be desirable to pre-filter
  *          the input image to remove large components, to extract
@@ -364,6 +380,7 @@ SARRAY     *safiles;
  *          significantly increase their height.  But if there are many
  *          small connected components in a dense texture, this is likely
  *          to generate tall components that will be eliminated in pixf.
+ * </pre>
  */
 l_int32
 pixGetWordsInTextlines(PIX     *pixs,
@@ -376,13 +393,13 @@ pixGetWordsInTextlines(PIX     *pixs,
                        PIXA   **ppixad,
                        NUMA   **pnai)
 {
-l_int32  maxsize;
-BOXA    *boxa1, *boxa2, *boxa3, *boxad;
+l_int32  maxdil;
+BOXA    *boxa1, *boxad;
 BOXAA   *baa;
 NUMA    *nai;
 NUMAA   *naa;
-PIXA    *pixa1, *pixa2, *pixad;
-PIX     *pixt1, *pixt2;
+PIXA    *pixa1, *pixad;
+PIX     *pix1;
 PIXAA   *paa;
 
     PROCNAME("pixGetWordsInTextlines");
@@ -398,38 +415,26 @@ PIXAA   *paa;
         return ERROR_INT("reduction not in {1,2}", procName, 1);
 
     if (reduction == 1) {
-        pixt1 = pixClone(pixs);
-        maxsize = 14;
-    }
-    else {  /* reduction == 2 */
-        pixt1 = pixReduceRankBinaryCascade(pixs, 1, 0, 0, 0);
-        maxsize = 7;
+        pix1 = pixClone(pixs);
+        maxdil = 18;
+    } else {  /* reduction == 2 */
+        pix1 = pixReduceRankBinaryCascade(pixs, 1, 0, 0, 0);
+        maxdil = 9;
     }
 
-        /* First estimate of the word masks */
-    pixt2 = pixWordMaskByDilation(pixt1, maxsize, NULL);
+        /* Get the bounding boxes of the words from the word mask. */
+    pixWordBoxesByDilation(pix1, maxdil, minwidth, minheight,
+                           maxwidth, maxheight, &boxa1, NULL);
 
-        /* Get the bounding boxes of the words. First remove the
-         * small ones, which can be due to punctuation that was
-         * not joined to a word.  Then remove the large ones, which are
-         * also not likely to be words.  Here, pixa1 contains
-         * the masks over each word.  */
-    boxa1 = pixConnComp(pixt2, NULL, 8);
-    boxa2 = boxaSelectBySize(boxa1, minwidth, minheight, L_SELECT_IF_BOTH,
-                             L_SELECT_IF_GTE, NULL);
-    boxa3 = boxaSelectBySize(boxa2, maxwidth, maxheight, L_SELECT_IF_BOTH,
-                             L_SELECT_IF_LTE, NULL);
-    pixa1 = pixaCreateFromBoxa(pixt2, boxa3, NULL);
+        /* Generate a pixa of the word images */
+    pixa1 = pixaCreateFromBoxa(pix1, boxa1, NULL);  /* mask over each word */
 
-        /* Generate a pixa of the actual word images, not the mask images. */
-    pixa2 = pixaClipToPix(pixa1, pixt1);
+        /* Sort the bounding boxes of these words by line.  We use the
+         * index mapping to allow identical sorting of the pixa. */
+    baa = boxaSort2d(boxa1, &naa, -1, -1, 4);
+    paa = pixaSort2dByIndex(pixa1, naa, L_CLONE);
 
-        /* Sort the bounding boxes of these words, saving the
-         * index mapping that will allow us to sort the pixa identically. */
-    baa = boxaSort2d(boxa3, &naa, -1, -1, 4);
-    paa = pixaSort2dByIndex(pixa2, naa, L_CLONE);
-
-        /* Flatten the word pixa */
+        /* Flatten the word paa */
     pixad = pixaaFlattenToPixa(paa, &nai, L_CLONE);
     boxad = pixaGetBoxa(pixad, L_COPY);
 
@@ -437,13 +442,9 @@ PIXAA   *paa;
     *pboxad = boxad;
     *ppixad = pixad;
 
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
+    pixDestroy(&pix1);
     pixaDestroy(&pixa1);
-    pixaDestroy(&pixa2);
     boxaDestroy(&boxa1);
-    boxaDestroy(&boxa2);
-    boxaDestroy(&boxa3);
     boxaaDestroy(&baa);
     pixaaDestroy(&paa);
     numaaDestroy(&naa);
@@ -452,26 +453,26 @@ PIXAA   *paa;
 
 
 /*!
- *  pixGetWordBoxesInTextlines()
+ * \brief   pixGetWordBoxesInTextlines()
  *
- *      Input:  pixs (1 bpp, 300 ppi)
- *              reduction (1 for full res; 2 for half-res)
- *              minwidth, minheight (of saved components; smaller are discarded)
- *              maxwidth, maxheight (of saved components; larger are discarded)
- *              &boxad (<return> word boxes sorted in textline line order)
- *              &naindex (<return> index of textline for each word)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs 1 bpp, typ. 300 ppi
+ * \param[in]    reduction 1 for input res; 2 for 2x reduction of input res
+ * \param[in]    minwidth, minheight of saved components; smaller are discarded
+ * \param[in]    maxwidth, maxheight of saved components; larger are discarded
+ * \param[out]   pboxad word boxes sorted in textline line order
+ * \param[out]   pnai [optional] index of textline for each word
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The input should be at a resolution of about 300 ppi.
  *          The word masks can be computed at either 150 ppi or 300 ppi.
  *          For the former, set reduction = 2.
- *      (2) In an actual application, it may be desirable to pre-filter
- *          the input image to remove large components, to extract
- *          single columns of text, and to deskew them.
- *      (3) This is a special version that just finds the word boxes
- *          in line order, with a numa giving the textline index for
- *          each word.  See pixGetWordsInTextlines() for more details.
+ *      (2) This is a special version of pixGetWordsInTextlines(), that
+ *          just finds the word boxes in line order, with a numa
+ *          giving the textline index for each word.
+ *          See pixGetWordsInTextlines() for more details.
+ * </pre>
  */
 l_int32
 pixGetWordBoxesInTextlines(PIX     *pixs,
@@ -483,59 +484,47 @@ pixGetWordBoxesInTextlines(PIX     *pixs,
                            BOXA   **pboxad,
                            NUMA   **pnai)
 {
-l_int32  maxsize;
-BOXA    *boxa1, *boxa2, *boxa3, *boxad;
+l_int32  maxdil;
+BOXA    *boxa1;
 BOXAA   *baa;
 NUMA    *nai;
-PIX     *pixt1, *pixt2;
+PIX     *pix1;
 
     PROCNAME("pixGetWordBoxesInTextlines");
 
-    if (!pboxad || !pnai)
+    if (pnai) *pnai = NULL;
+    if (!pboxad)
         return ERROR_INT("&boxad and &nai not both defined", procName, 1);
     *pboxad = NULL;
-    *pnai = NULL;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     if (reduction != 1 && reduction != 2)
         return ERROR_INT("reduction not in {1,2}", procName, 1);
 
     if (reduction == 1) {
-        pixt1 = pixClone(pixs);
-        maxsize = 14;
-    }
-    else {  /* reduction == 2 */
-        pixt1 = pixReduceRankBinaryCascade(pixs, 1, 0, 0, 0);
-        maxsize = 7;
+        pix1 = pixClone(pixs);
+        maxdil = 18;
+    } else {  /* reduction == 2 */
+        pix1 = pixReduceRankBinaryCascade(pixs, 1, 0, 0, 0);
+        maxdil = 9;
     }
 
-        /* First estimate of the word masks */
-    pixt2 = pixWordMaskByDilation(pixt1, maxsize, NULL);
-
-        /* Get the bounding boxes of the words, and remove the
-         * small ones, which can be due to punctuation that was
-         * not joined to a word, and the large ones, which are
-         * also not likely to be words. */
-    boxa1 = pixConnComp(pixt2, NULL, 8);
-    boxa2 = boxaSelectBySize(boxa1, minwidth, minheight,
-                             L_SELECT_IF_BOTH, L_SELECT_IF_GTE, NULL);
-    boxa3 = boxaSelectBySize(boxa2, maxwidth, maxheight,
-                             L_SELECT_IF_BOTH, L_SELECT_IF_LTE, NULL);
+        /* Get the bounding boxes of the words from the word mask. */
+    pixWordBoxesByDilation(pix1, maxdil, minwidth, minheight,
+                           maxwidth, maxheight, &boxa1, NULL);
 
         /* 2D sort the bounding boxes of these words. */
-    baa = boxaSort2d(boxa3, NULL, 3, -5, 5);
+    baa = boxaSort2d(boxa1, NULL, 3, -5, 5);
 
         /* Flatten the boxaa, saving the boxa index for each box */
-    boxad = boxaaFlattenToBoxa(baa, &nai, L_CLONE);
+    *pboxad = boxaaFlattenToBoxa(baa, &nai, L_CLONE);
 
-    *pnai = nai;
-    *pboxad = boxad;
-
-    pixDestroy(&pixt1);
-    pixDestroy(&pixt2);
+    if (pnai)
+        *pnai = nai;
+    else
+        numaDestroy(&nai);
+    pixDestroy(&pix1);
     boxaDestroy(&boxa1);
-    boxaDestroy(&boxa2);
-    boxaDestroy(&boxa3);
     boxaaDestroy(&baa);
     return 0;
 }
@@ -545,19 +534,21 @@ PIX     *pixt1, *pixt2;
  *           Use word bounding boxes to compare page images         *
  *------------------------------------------------------------------*/
 /*!
- *  boxaExtractSortedPattern()
+ * \brief   boxaExtractSortedPattern()
  *
- *      Input:  boxa (typ. of word bounding boxes, in textline order)
- *              numa (index of textline for each box in boxa)
- *      Return: naa (numaa, where each numa represents one textline),
- *                   or null on error
+ * \param[in]    boxa typ. of word bounding boxes, in textline order
+ * \param[in]    na   index of textline for each box in boxa
+ * \return  naa NUMAA, where each numa represents one textline,
+ *                   or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The input is expected to come from pixGetWordBoxesInTextlines().
  *      (2) Each numa in the output consists of an average y coordinate
- *          of the first box in the textline, followed by pairs of 
+ *          of the first box in the textline, followed by pairs of
  *          x coordinates representing the left and right edges of each
  *          of the boxes in the textline.
+ * </pre>
  */
 NUMAA *
 boxaExtractSortedPattern(BOXA  *boxa,
@@ -604,21 +595,22 @@ NUMAA   *naa;
 
 
 /*!
- *  numaaCompareImagesByBoxes()
+ * \brief   numaaCompareImagesByBoxes()
  *
- *      Input:  naa1 (for image 1, formatted by boxaExtractSortedPattern())
- *              naa2 (ditto; for image 2)
- *              nperline (number of box regions to be used in each textline)
- *              nreq (number of complete row matches required)
- *              maxshiftx (max allowed x shift between two patterns, in pixels)
- *              maxshifty (max allowed y shift between two patterns, in pixels)
- *              delx (max allowed difference in x data, after alignment)
- *              dely (max allowed difference in y data, after alignment)
- *              &same (<return> 1 if @nreq row matches are found; 0 otherwise)
- *              debugflag (1 for debug output)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    naa1 for image 1, formatted by boxaExtractSortedPattern()
+ * \param[in]    naa2 ditto; for image 2
+ * \param[in]    nperline number of box regions to be used in each textline
+ * \param[in]    nreq number of complete row matches required
+ * \param[in]    maxshiftx max allowed x shift between two patterns, in pixels
+ * \param[in]    maxshifty max allowed y shift between two patterns, in pixels
+ * \param[in]    delx max allowed difference in x data, after alignment
+ * \param[in]    dely max allowed difference in y data, after alignment
+ * \param[out]   psame 1 if %nreq row matches are found; 0 otherwise
+ * \param[in]    debugflag 1 for debug output
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Each input numaa describes a set of sorted bounding boxes
  *          (sorted by textline and, within each textline, from
  *          left to right) in the images from which they are derived.
@@ -626,9 +618,9 @@ NUMAA   *naa;
  *          format in each of the input numaa.
  *      (2) This function does an alignment between the input
  *          descriptions of bounding boxes for two images. The
- *          input parameter @nperline specifies the number of boxes
+ *          input parameter %nperline specifies the number of boxes
  *          to consider in each line when testing for a match, and
- *          @nreq is the required number of lines that must be well-aligned
+ *          %nreq is the required number of lines that must be well-aligned
  *          to get a match.
  *      (3) Testing by alignment has 3 steps:
  *          (a) Generating the location of word bounding boxes from the
@@ -636,14 +628,15 @@ NUMAA   *naa;
  *          (b) Listing all possible pairs of aligned rows, based on
  *              tolerances in horizontal and vertical positions of
  *              the boxes.  Specifically, all pairs of rows are enumerated
- *              whose first @nperline boxes can be brought into close
+ *              whose first %nperline boxes can be brought into close
  *              alignment, based on the delx parameter for boxes in the
- *              line and within the overall the @maxshiftx and @maxshifty
+ *              line and within the overall the %maxshiftx and %maxshifty
  *              constraints.
  *          (c) Each pair, starting with the first, is used to search
- *              for a set of @nreq - 1 other pairs that can all be aligned
+ *              for a set of %nreq - 1 other pairs that can all be aligned
  *              with a difference in global translation of not more
- *              than (@delx, @dely).
+ *              than (%delx, %dely).
+ * </pre>
  */
 l_int32
 numaaCompareImagesByBoxes(NUMAA    *naa1,
@@ -686,12 +679,12 @@ NUMA     *na1, *na2, *nai1, *nai2, *nasx, *nasy;
         /* Find the lines in naa1 and naa2 with sufficient boxes.
          * Also, find the y-values for each of the lines, and the
          * LH x-values of the first box in each line. */
-    line1 = (l_int32 *)CALLOC(n1, sizeof(l_int32));
-    line2 = (l_int32 *)CALLOC(n2, sizeof(l_int32));
-    yloc1 = (l_int32 *)CALLOC(n1, sizeof(l_int32));
-    yloc2 = (l_int32 *)CALLOC(n2, sizeof(l_int32));
-    xleft1 = (l_int32 *)CALLOC(n1, sizeof(l_int32));
-    xleft2 = (l_int32 *)CALLOC(n2, sizeof(l_int32));
+    line1 = (l_int32 *)LEPT_CALLOC(n1, sizeof(l_int32));
+    line2 = (l_int32 *)LEPT_CALLOC(n2, sizeof(l_int32));
+    yloc1 = (l_int32 *)LEPT_CALLOC(n1, sizeof(l_int32));
+    yloc2 = (l_int32 *)LEPT_CALLOC(n2, sizeof(l_int32));
+    xleft1 = (l_int32 *)LEPT_CALLOC(n1, sizeof(l_int32));
+    xleft2 = (l_int32 *)LEPT_CALLOC(n2, sizeof(l_int32));
     for (i = 0; i < n1; i++) {
         na1 = numaaGetNuma(naa1, i, L_CLONE);
         numaGetIValue(na1, 0, yloc1 + i);
@@ -757,12 +750,12 @@ NUMA     *na1, *na2, *nai1, *nai2, *nasx, *nasy;
     countAlignedMatches(nai1, nai2, nasx, nasy, n1, n2, delx, dely,
                         nreq, psame, debugflag);
 
-    FREE(line1);
-    FREE(line2);
-    FREE(yloc1);
-    FREE(yloc2);
-    FREE(xleft1);
-    FREE(xleft2);
+    LEPT_FREE(line1);
+    LEPT_FREE(line2);
+    LEPT_FREE(yloc1);
+    LEPT_FREE(yloc2);
+    LEPT_FREE(xleft1);
+    LEPT_FREE(xleft2);
     numaDestroy(&nai1);
     numaDestroy(&nai2);
     numaDestroy(&nasx);
@@ -810,7 +803,7 @@ l_int32  i, xl1, xr1, xl2, xr2, diffl, diffr;
  *              delx, dely (allowed difference in shifts of the match,
  *                          compared to the reference match)
  *              nreq (number of required aligned matches)
- *              &same (<return> 1 if @nreq row matches are found; 0 otherwise)
+ *              &same (<return> 1 if %nreq row matches are found; 0 otherwise)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -857,8 +850,8 @@ l_int32  *ia1, *ia2, *iasx, *iasy, *index1, *index2;
     ia2 = numaGetIArray(nai2);
     iasx = numaGetIArray(nasx);
     iasy = numaGetIArray(nasy);
-    index1 = (l_int32 *)CALLOC(n1, sizeof(l_int32));  /* keep track of rows */
-    index2 = (l_int32 *)CALLOC(n2, sizeof(l_int32));
+    index1 = (l_int32 *)LEPT_CALLOC(n1, sizeof(l_int32));  /* keep track of rows */
+    index2 = (l_int32 *)LEPT_CALLOC(n2, sizeof(l_int32));
     for (i = 0; i < nm; i++) {
         if (*psame == 1)
             break;
@@ -883,7 +876,7 @@ l_int32  *ia1, *ia2, *iasx, *iasy, *index1, *index2;
             diffx = L_ABS(shiftx - iasx[j]);
             diffy = L_ABS(shifty - iasy[j]);
             if (diffx > delx || diffy > dely) continue;
-                /* We have a match */   
+                /* We have a match */
             nmatch++;
             index1[ia1[j]] = nmatch;  /* mark the rows */
             index2[ia2[j]] = nmatch;
@@ -895,13 +888,13 @@ l_int32  *ia1, *ia2, *iasx, *iasy, *index1, *index2;
             }
         }
     }
-        
-    FREE(ia1);
-    FREE(ia2);
-    FREE(iasx);
-    FREE(iasy);
-    FREE(index1);
-    FREE(index2);
+
+    LEPT_FREE(ia1);
+    LEPT_FREE(ia2);
+    LEPT_FREE(iasx);
+    LEPT_FREE(iasy);
+    LEPT_FREE(index1);
+    LEPT_FREE(index2);
     return 0;
 }
 
@@ -931,5 +924,3 @@ l_int32  i;
     fprintf(stderr, "\n");
     return;
 }
-
-
