@@ -24,8 +24,9 @@
  -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  colorcontent.c
+/*!
+ * \file colorcontent.c
+ * <pre>
  *
  *      Builds an image of the color content, on a per-pixel basis,
  *      as a measure of the amount of divergence of each color
@@ -128,6 +129,7 @@
  *        to selecting the intermediate value of the three differences
  *        between the three components.  For (a) and (b) above,
  *        this value is in /../.
+ * </pre>
  */
 
 #include "allheaders.h"
@@ -138,17 +140,18 @@
  *      component (R,G,B) from gray.                                       *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixColorContent()
+ * \brief   pixColorContent()
  *
- *      Input:  pixs  (32 bpp rgb or 8 bpp colormapped)
- *              rwhite, gwhite, bwhite (color value associated with white point)
- *              mingray (min gray value for which color is measured)
- *              &pixr (<optional return> 8 bpp red 'content')
- *              &pixg (<optional return> 8 bpp green 'content')
- *              &pixb (<optional return> 8 bpp blue 'content')
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs  32 bpp rgb or 8 bpp colormapped
+ * \param[in]    rwhite, gwhite, bwhite color value associated with white point
+ * \param[in]    mingray min gray value for which color is measured
+ * \param[out]   ppixr [optional] 8 bpp red 'content'
+ * \param[out]   ppixg [optional] 8 bpp green 'content'
+ * \param[out]   ppixb [optional] 8 bpp blue 'content'
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This returns the color content in each component, which is
  *          a measure of the deviation from gray, and is defined
  *          as the difference between the component and the average of
@@ -166,6 +169,7 @@
  *      (4) Therefore, use 0 for all four input parameters if the color
  *          magnitude is to be calculated without either white balance
  *          correction or dark filtering.
+ * </pre>
  */
 l_int32
 pixColorContent(PIX     *pixs,
@@ -189,13 +193,13 @@ PIXCMAP   *cmap;
 
     PROCNAME("pixColorContent");
 
+    if (!ppixr && !ppixg && !ppixb)
+        return ERROR_INT("no return val requested", procName, 1);
     if (ppixr) *ppixr = NULL;
     if (ppixg) *ppixg = NULL;
     if (ppixb) *ppixb = NULL;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
-    if (!ppixr && !ppixg && !ppixb)
-        return ERROR_INT("nothing to compute", procName, 1);
     if (mingray < 0) mingray = 0;
     pixGetDimensions(pixs, &w, &h, &d);
     if (mingray > 255)
@@ -288,9 +292,9 @@ PIXCMAP   *cmap;
         numaDestroy(&nar);
         numaDestroy(&nag);
         numaDestroy(&nab);
-        FREE(rtab);
-        FREE(gtab);
-        FREE(btab);
+        LEPT_FREE(rtab);
+        LEPT_FREE(gtab);
+        LEPT_FREE(btab);
     }
     pixDestroy(&pixc);
     return 0;
@@ -302,17 +306,18 @@ PIXCMAP   *cmap;
  *      as a measure of the difference of the pixel color from gray.       *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixColorMagnitude()
+ * \brief   pixColorMagnitude()
  *
- *      Input:  pixs  (32 bpp rgb or 8 bpp colormapped)
- *              rwhite, gwhite, bwhite (color value associated with white point)
- *              type (chooses the method for calculating the color magnitude:
+ * \param[in]    pixs  32 bpp rgb or 8 bpp colormapped
+ * \param[in]    rwhite, gwhite, bwhite color value associated with white point
+ * \param[in]    type chooses the method for calculating the color magnitude:
  *                    L_MAX_DIFF_FROM_AVERAGE_2, L_MAX_MIN_DIFF_FROM_2,
- *                    L_MAX_DIFF)
- *      Return: pixd (8 bpp, amount of color in each source pixel),
+ *                    L_MAX_DIFF
+ * \return  pixd 8 bpp, amount of color in each source pixel,
  *                    or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) For an RGB image, a gray pixel is one where all three components
  *          are equal.  We define the amount of color in an RGB pixel as
  *          a function depending on the absolute value of the differences
@@ -348,6 +353,7 @@ PIXCMAP   *cmap;
  *          They are used to compensate for an unbalanced color white point.
  *          They must either be all 0 or all non-zero.  To turn this
  *          off, set them all to 0.
+ * </pre>
  */
 PIX *
 pixColorMagnitude(PIX     *pixs,
@@ -447,9 +453,9 @@ PIXCMAP   *cmap;
         numaDestroy(&nar);
         numaDestroy(&nag);
         numaDestroy(&nab);
-        FREE(rtab);
-        FREE(gtab);
-        FREE(btab);
+        LEPT_FREE(rtab);
+        LEPT_FREE(gtab);
+        LEPT_FREE(btab);
     }
     pixDestroy(&pixc);
     return pixd;
@@ -461,30 +467,32 @@ PIXCMAP   *cmap;
  *      are not too close to gray pixels.                                  *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixMaskOverColorPixels()
+ * \brief   pixMaskOverColorPixels()
  *
- *      Input:  pixs  (32 bpp rgb or 8 bpp colormapped)
- *              threshdiff (threshold for minimum of the max difference
- *                          between components)
- *              mindist (minimum allowed distance from nearest non-color pixel)
- *      Return: pixd (1 bpp, mask over color pixels), or null on error
+ * \param[in]    pixs  32 bpp rgb or 8 bpp colormapped
+ * \param[in]    threshdiff threshold for minimum of the max difference
+ *                          between components
+ * \param[in]    mindist minimum allowed distance from nearest non-color pixel
+ * \return  pixd 1 bpp, mask over color pixels, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The generated mask identifies each pixel as either color or
  *          non-color.  For a pixel to be color, it must satisfy two
  *          constraints:
  *            (a) The max difference between the r,g and b components must
- *                equal or exceed a threshold @threshdiff.
- *            (b) It must be at least @mindist (in an 8-connected way)
+ *                equal or exceed a threshold %threshdiff.
+ *            (b) It must be at least %mindist (in an 8-connected way)
  *                from the nearest non-color pixel.
- *      (2) The distance constraint (b) is only applied if @mindist > 1.
- *          For example, if @mindist == 2, the color pixels identified
+ *      (2) The distance constraint (b) is only applied if %mindist \> 1.
+ *          For example, if %mindist == 2, the color pixels identified
  *          by (a) are eroded by a 3x3 Sel.  In general, the Sel size
- *          for erosion is 2 * (@mindist - 1) + 1.
+ *          for erosion is 2 * (%mindist - 1) + 1.
  *          Why have this constraint?  In scanned images that are
  *          essentially gray, color artifacts are typically introduced
  *          in transition regions near sharp edges that go from dark
  *          to light, so this allows these transition regions to be removed.
+ * </pre>
  */
 PIX *
 pixMaskOverColorPixels(PIX     *pixs,
@@ -545,13 +553,13 @@ PIXCMAP   *cmap;
  *      within the prescribed range (a cube in RGB color space)            *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixMaskOverColorRange()
+ * \brief   pixMaskOverColorRange()
  *
- *      Input:  pixs  (32 bpp rgb or 8 bpp colormapped)
- *              rmin, rmax (min and max allowed values for red component)
- *              gmin, gmax
- *              bmin, bmax
- *      Return: pixd (1 bpp, mask over color pixels), or null on error
+ * \param[in]    pixs  32 bpp rgb or 8 bpp colormapped
+ * \param[in]    rmin, rmax min and max allowed values for red component
+ * \param[in]    gmin, gmax
+ * \param[in]    bmin, bmax
+ * \return  pixd 1 bpp, mask over color pixels, or NULL on error
  */
 PIX *
 pixMaskOverColorRange(PIX     *pixs,
@@ -608,27 +616,28 @@ PIXCMAP   *cmap;
  *   Finds the fraction of pixels with "color" that are not close to black *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixColorFraction()
+ * \brief   pixColorFraction()
  *
- *      Input:  pixs  (32 bpp rgb)
- *              darkthresh (threshold near black; if the lightest component
+ * \param[in]    pixs  32 bpp rgb
+ * \param[in]    darkthresh threshold near black; if the lightest component
  *                          is below this, the pixel is not considered in
- *                          the statistics; typ. 20)
- *              lightthresh (threshold near white; if the darkest component
+ *                          the statistics; typ. 20
+ * \param[in]    lightthresh threshold near white; if the darkest component
  *                           is above this, the pixel is not considered in
- *                           the statistics; typ. 244)
- *              diffthresh (thresh for the maximum difference between
+ *                           the statistics; typ. 244
+ * \param[in]    diffthresh thresh for the maximum difference between
  *                          component value; below this the pixel is not
- *                          considered to have sufficient color)
- *              factor (subsampling factor)
- *              &pixfract (<return> fraction of pixels in intermediate
+ *                          considered to have sufficient color
+ * \param[in]    factor subsampling factor
+ * \param[out]   ppixfract fraction of pixels in intermediate
  *                         brightness range that were considered
- *                         for color content)
- *              &colorfract (<return> fraction of pixels that meet the
- *                           criterion for sufficient color; 0.0 on error)
- *      Return: 0 if OK, 1 on error
+ *                         for color content
+ * \param[out]   pcolorfract fraction of pixels that meet the
+ *                           criterion for sufficient color; 0.0 on error
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function is asking the question: to what extent does the
  *          image appear to have color?   The amount of color a pixel
  *          appears to have depends on both the deviation of the
@@ -638,10 +647,10 @@ PIXCMAP   *cmap;
  *          deviation from black.
  *      (2) Any pixel that meets these three tests is considered a
  *          colorful pixel:
- *            (a) the lightest component must equal or exceed @darkthresh
- *            (b) the darkest component must not exceed @lightthresh
+ *            (a) the lightest component must equal or exceed %darkthresh
+ *            (b) the darkest component must not exceed %lightthresh
  *            (c) the max difference between components must equal or
- *                exceed @diffthresh.
+ *                exceed %diffthresh.
  *      (3) The dark pixels are removed from consideration because
  *          they don't appear to have color.
  *      (4) The very lightest pixels are removed because if an image
@@ -659,6 +668,7 @@ PIXCMAP   *cmap;
  *          octree quantization of an image with only gray values, the
  *          2^(level) octcubes on the diagonal are the only ones
  *          that can be occupied.
+ * </pre>
  */
 l_int32
 pixColorFraction(PIX        *pixs,
@@ -724,26 +734,27 @@ l_uint32  *data, *line;
  *      in a grayscale image.                                              *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixNumSignificantGrayColors()
+ * \brief   pixNumSignificantGrayColors()
  *
- *      Input:  pixs  (8 bpp gray)
- *              darkthresh (dark threshold for minimum intensity to be
- *                          considered; typ. 20)
- *              lightthresh (threshold near white, for maximum intensity
- *                           to be considered; typ. 236)
- *              minfract (minimum fraction of all pixels to include a level
- *                        as significant; typ. 0.0001; should be < 0.001)
- *              factor (subsample factor; integer >= 1)
- *              &ncolors (<return> number of significant colors; 0 on error)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs  8 bpp gray
+ * \param[in]    darkthresh dark threshold for minimum intensity to be
+ *                          considered; typ. 20
+ * \param[in]    lightthresh threshold near white, for maximum intensity
+ *                           to be considered; typ. 236
+ * \param[in]    minfract minimum fraction of all pixels to include a level
+ *                        as significant; typ. 0.0001; should be < 0.001
+ * \param[in]    factor subsample factor; integer >= 1
+ * \param[out]   pncolors number of significant colors; 0 on error
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function is asking the question: how many perceptually
  *          significant gray color levels is in this pix?
  *          A color level must meet 3 criteria to be significant:
- *            - it can't be too close to black
- *            - it can't be too close to white
- *            - it must have at least some minimum fractional population
+ *            ~ it can't be too close to black
+ *            ~ it can't be too close to white
+ *            ~ it must have at least some minimum fractional population
  *      (2) Use -1 for default values for darkthresh, lightthresh and minfract.
  *      (3) Choose default of darkthresh = 20, because variations in very
  *          dark pixels are not visually significant.
@@ -751,6 +762,7 @@ l_uint32  *data, *line;
  *          that have been jpeg'd typically have near-white pixels in the
  *          8x8 jpeg blocks, and these should not be counted.  It is desirable
  *          to obtain a clean image by quantizing this noise away.
+ * </pre>
  */
 l_int32
 pixNumSignificantGrayColors(PIX       *pixs,
@@ -804,18 +816,19 @@ NUMA    *na;
  *   due to the existence of many colors in low-gradient regions.          *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixColorsForQuantization()
- *      Input:  pixs (8 bpp gray or 32 bpp rgb; with or without colormap)
- *              thresh (binary threshold on edge gradient; 0 for default)
- *              &ncolors (<return> the number of colors found)
- *              &iscolor (<optional return> 1 if significant color is found;
+ * \brief   pixColorsForQuantization()
+ * \param[in]    pixs 8 bpp gray or 32 bpp rgb; with or without colormap
+ * \param[in]    thresh binary threshold on edge gradient; 0 for default
+ * \param[out]   pncolors the number of colors found
+ * \param[out]   piscolor [optional] 1 if significant color is found;
  *                        0 otherwise.  If pixs is 8 bpp, and does not have
- *                        a colormap with color entries, this is 0)
- *              debug (1 to output masked image that is tested for colors;
- *                     0 otherwise)
- *      Return: 0 if OK, 1 on error.
+ *                        a colormap with color entries, this is 0
+ * \param[in]    debug 1 to output masked image that is tested for colors;
+ *                     0 otherwise
+ * \return  0 if OK, 1 on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function finds a measure of the number of colors that are
  *          found in low-gradient regions of an image.  By its
  *          magnitude relative to some threshold (not specified in
@@ -855,7 +868,7 @@ NUMA    *na;
  *          the low gradient regions is found.  If the image has color,
  *          the number of occupied level 4 octcubes is found.
  *      (8) The number of colors in the low-gradient regions increases
- *          monotonically with the threshold @thresh on the edge gradient.
+ *          monotonically with the threshold %thresh on the edge gradient.
  *      (9) Background: grayscale and color quantization is often useful
  *          to achieve highly compressed images with little visible
  *          distortion.  However, gray or color washes (regions of
@@ -872,6 +885,7 @@ NUMA    *na;
  *          compression artifacts.  This function reduces but does not
  *          completely eliminate measurement of jpeg quantization noise
  *          in the white background of grayscale or color images.
+ * </pre>
  */
 l_int32
 pixColorsForQuantization(PIX      *pixs,
@@ -941,13 +955,13 @@ PIXCMAP   *cmap;
         pixsc = pixScaleAreaMap(pixt, 0.25, 0.25);
 
         /* Basic edge mask generation procedure:
-         *   - work on a grayscale image
-         *   - get a 1 bpp edge mask by using an edge filter and
+         *   ~ work on a grayscale image
+         *   ~ get a 1 bpp edge mask by using an edge filter and
          *     thresholding to get fg pixels at the edges
-         *   - for gray, dilate with a 3x3 brick Sel to get mask over
+         *   ~ for gray, dilate with a 3x3 brick Sel to get mask over
          *     all pixels within a distance of 1 pixel from the nearest
          *     edge pixel
-         *   - for color, dilate with a 7x7 brick Sel to get mask over
+         *   ~ for color, dilate with a 7x7 brick Sel to get mask over
          *     all pixels within a distance of 3 pixels from the nearest
          *     edge pixel  */
     if (d == 8)
@@ -994,24 +1008,26 @@ PIXCMAP   *cmap;
  *               Finds the number of unique colors in an image             *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixNumColors()
- *      Input:  pixs (2, 4, 8, 32 bpp)
- *              factor (subsampling factor; integer)
- *              &ncolors (<return> the number of colors found, or 0 if
- *                        there are more than 256)
- *      Return: 0 if OK, 1 on error.
+ * \brief   pixNumColors()
+ * \param[in]    pixs 2, 4, 8, 32 bpp
+ * \param[in]    factor subsampling factor; integer
+ * \param[out]   pncolors the number of colors found, or 0 if
+ *                        there are more than 256
+ * \return  0 if OK, 1 on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This returns the actual number of colors found in the image,
- *          even if there is a colormap.  If @factor == 1 and the
+ *          even if there is a colormap.  If %factor == 1 and the
  *          number of colors differs from the number of entries
  *          in the colormap, a warning is issued.
- *      (2) Use @factor == 1 to find the actual number of colors.
- *          Use @factor > 1 to quickly find the approximate number of colors.
+ *      (2) Use %factor == 1 to find the actual number of colors.
+ *          Use %factor \> 1 to quickly find the approximate number of colors.
  *      (3) For d = 2, 4 or 8 bpp grayscale, this returns the number
  *          of colors found in the image in 'ncolors'.
  *      (4) For d = 32 bpp (rgb), if the number of colors is
  *          greater than 256, this returns 0 in 'ncolors'.
+ * </pre>
  */
 l_int32
 pixNumColors(PIX      *pixs,
@@ -1041,7 +1057,7 @@ PIXCMAP   *cmap;
     wpl = pixGetWpl(pixs);
     sum = 0;
     if (d != 32) {  /* grayscale */
-        inta = (l_int32 *)CALLOC(256, sizeof(l_int32));
+        inta = (l_int32 *)LEPT_CALLOC(256, sizeof(l_int32));
         for (i = 0; i < h; i += factor) {
             line = data + i * wpl;
             for (j = 0; j < w; j += factor) {
@@ -1057,7 +1073,7 @@ PIXCMAP   *cmap;
         for (i = 0; i < 256; i++)
             if (inta[i]) sum++;
         *pncolors = sum;
-        FREE(inta);
+        LEPT_FREE(inta);
 
         cmap = pixGetColormap(pixs);
         if (cmap && factor == 1) {
@@ -1071,7 +1087,7 @@ PIXCMAP   *cmap;
 
         /* 32 bpp rgb; quit if we get above 256 colors */
     hashsize = 5507;  /* big and prime; collisions are not likely */
-    inta = (l_int32 *)CALLOC(hashsize, sizeof(l_int32));
+    inta = (l_int32 *)LEPT_CALLOC(hashsize, sizeof(l_int32));
     for (i = 0; i < h; i += factor) {
         line = data + i * wpl;
         for (j = 0; j < w; j += factor) {
@@ -1082,7 +1098,7 @@ PIXCMAP   *cmap;
                 inta[val] = 1;
                 sum++;
                 if (sum > 256) {
-                    FREE(inta);
+                    LEPT_FREE(inta);
                     return 0;
                 }
             }
@@ -1090,7 +1106,7 @@ PIXCMAP   *cmap;
     }
 
     *pncolors = sum;
-    FREE(inta);
+    LEPT_FREE(inta);
     return 0;
 }
 
@@ -1099,23 +1115,25 @@ PIXCMAP   *cmap;
  *       Find the most "populated" colors in the image (and quantize)      *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixGetMostPopulatedColors()
- *      Input:  pixs (32 bpp rgb)
- *              sigbits (2-6, significant bits retained in the quantizer
- *                       for each component of the input image)
- *              factor (subsampling factor; use 1 for no subsampling)
- *              ncolors (the number of most populated colors to select)
- *              &array (<optional return> array of colors, each as 0xrrggbb00)
- *              &cmap (<optional return> colormap of the colors)
- *      Return: 0 if OK, 1 on error
+ * \brief   pixGetMostPopulatedColors()
+ * \param[in]    pixs 32 bpp rgb
+ * \param[in]    sigbits 2-6, significant bits retained in the quantizer
+ *                       for each component of the input image
+ * \param[in]    factor subsampling factor; use 1 for no subsampling
+ * \param[in]    ncolors the number of most populated colors to select
+ * \param[out]   parray [optional] array of colors, each as 0xrrggbb00
+ * \param[out]   pcmap [optional] colormap of the colors
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
- *      (1) This finds the @ncolors most populated cubes in rgb colorspace,
- *          where the cube size depends on @sigbits as
- *               cube side = (256 >> sigbits)
+ * <pre>
+ * Notes:
+ *      (1) This finds the %ncolors most populated cubes in rgb colorspace,
+ *          where the cube size depends on %sigbits as
+ *               cube side = (256 \>\> sigbits)
  *      (2) The rgb color components are found at the center of the cube.
  *      (3) The output array of colors can be displayed using
  *               pixDisplayColorArray(array, ncolors, ...);
+ * </pre>
  */
 l_int32
 pixGetMostPopulatedColors(PIX        *pixs,
@@ -1130,10 +1148,10 @@ NUMA    *nahisto, *naindex;
 
     PROCNAME("pixGetMostPopulatedColors");
 
+    if (!parray && !pcmap)
+        return ERROR_INT("no return val requested", procName, 1);
     if (parray) *parray = NULL;
     if (pcmap) *pcmap = NULL;
-    if (!parray && !pcmap)
-        return ERROR_INT("no output requested", procName, 1);
     if (!pixs || pixGetDepth(pixs) != 32)
         return ERROR_INT("pixs not defined", procName, 1);
     if (sigbits < 2 || sigbits > 6)
@@ -1152,7 +1170,7 @@ NUMA    *nahisto, *naindex;
 
     n = numaGetCount(naindex);
     ncolors = L_MIN(n, ncolors);
-    if (parray) *parray = (l_uint32 *)CALLOC(ncolors, sizeof(l_uint32));
+    if (parray) *parray = (l_uint32 *)LEPT_CALLOC(ncolors, sizeof(l_uint32));
     if (pcmap) *pcmap = pixcmapCreate(8);
     for (i = 0; i < ncolors; i++) {
         numaGetIValue(naindex, i, &rgbindex);  /* rgb index */
@@ -1167,28 +1185,30 @@ NUMA    *nahisto, *naindex;
 
 
 /*!
- *  pixSimpleColorQuantize()
- *      Input:  pixs (32 bpp rgb)
- *              sigbits (2-4, significant bits retained in the quantizer
- *                       for each component of the input image)
- *              factor (subsampling factor; use 1 for no subsampling)
- *              ncolors (the number of most populated colors to select)
- *      Return: pixd (8 bpp cmapped) or NULL on error
+ * \brief   pixSimpleColorQuantize()
+ * \param[in]    pixs 32 bpp rgb
+ * \param[in]    sigbits 2-4, significant bits retained in the quantizer
+ *                       for each component of the input image
+ * \param[in]    factor subsampling factor; use 1 for no subsampling
+ * \param[in]    ncolors the number of most populated colors to select
+ * \return  pixd 8 bpp cmapped or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) If you want to do color quantization for real, use octcube
  *          or modified median cut.  This function shows that it is
  *          easy to make a simple quantizer based solely on the population
  *          in cells of a given size in rgb color space.
- *      (2) The @ncolors most populated cells at the @sigbits level form
+ *      (2) The %ncolors most populated cells at the %sigbits level form
  *          the colormap for quantizing, and this uses octcube indexing
  *          under the covers to assign each pixel to the nearest color.
- *      (3) @sigbits is restricted to 2, 3 and 4.  At the low end, the
+ *      (3) %sigbits is restricted to 2, 3 and 4.  At the low end, the
  *          color discrimination is very crude; at the upper end, a set of
  *          similar colors can dominate the result.  Interesting results
- *          are generally found for @sigbits = 3 and ncolors ~ 20.
+ *          are generally found for %sigbits = 3 and ncolors ~ 20.
  *      (4) See also pixColorSegment() for a method of quantizing the
  *          colors to generate regions of similar color.
+ * </pre>
  */
 PIX *
 pixSimpleColorQuantize(PIX        *pixs,
@@ -1220,19 +1240,21 @@ PIXCMAP  *cmap;
  *            Constructs a color histogram based on rgb indices            *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixGetRGBHistogram()
- *      Input:  pixs (32 bpp rgb)
- *              sigbits (2-6, significant bits retained in the quantizer
- *                       for each component of the input image)
- *              factor (subsampling factor; use 1 for no subsampling)
- *      Return: numa (histogram of colors, indexed by RGB
- *                    components), or null on error
+ * \brief   pixGetRGBHistogram()
+ * \param[in]    pixs 32 bpp rgb
+ * \param[in]    sigbits 2-6, significant bits retained in the quantizer
+ *                       for each component of the input image
+ * \param[in]    factor subsampling factor; use 1 for no subsampling
+ * \return  numa histogram of colors, indexed by RGB
+ *                    components, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This uses a simple, fast method of indexing into an rgb image.
  *      (2) The output is a 1D histogram of count vs. rgb-index, which
  *          uses red sigbits as the most significant and blue as the least.
  *      (3) This function produces the same result as pixMedianCutHisto().
+ * </pre>
  */
 NUMA *
 pixGetRGBHistogram(PIX     *pixs,
@@ -1278,27 +1300,29 @@ NUMA       *na;
         }
     }
 
-    FREE(rtab);
-    FREE(gtab);
-    FREE(btab);
+    LEPT_FREE(rtab);
+    LEPT_FREE(gtab);
+    LEPT_FREE(btab);
     return na;
 }
 
 
 /*!
- *  makeRGBIndexTables()
+ * \brief   makeRGBIndexTables()
  *
- *      Input:  &rtab, &gtab, &btab (<return> 256-entry index tables)
- *              sigbits (2-6, significant bits retained in the quantizer
- *                       for each component of the input image)
- *      Return: 0 if OK, 1 on error
+ * \param[out]   prtab, pgtab, pbtab 256-entry index tables
+ * \param[in]    sigbits 2-6, significant bits retained in the quantizer
+ *                       for each component of the input image
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) These tables are used to map from rgb sample values to
  *          an rgb index, using
  *             rgbindex = rtab[rval] | gtab[gval] | btab[bval]
  *          where, e.g., if sigbits = 3, the index is a 9 bit integer:
  *             r7 r6 r5 g7 g6 g5 b7 b6 b5
+ * </pre>
  */
 l_int32
 makeRGBIndexTables(l_uint32  **prtab,
@@ -1319,9 +1343,9 @@ l_uint32  *rtab, *gtab, *btab;
     if (sigbits < 2 || sigbits > 6)
         return ERROR_INT("sigbits not in [2 ... 6]", procName, 1);
 
-    rtab = (l_uint32 *)CALLOC(256, sizeof(l_uint32));
-    gtab = (l_uint32 *)CALLOC(256, sizeof(l_uint32));
-    btab = (l_uint32 *)CALLOC(256, sizeof(l_uint32));
+    rtab = (l_uint32 *)LEPT_CALLOC(256, sizeof(l_uint32));
+    gtab = (l_uint32 *)LEPT_CALLOC(256, sizeof(l_uint32));
+    btab = (l_uint32 *)LEPT_CALLOC(256, sizeof(l_uint32));
     *prtab = rtab;
     *pgtab = gtab;
     *pbtab = btab;
@@ -1371,20 +1395,22 @@ l_uint32  *rtab, *gtab, *btab;
 
 
 /*!
- *  getRGBFromIndex()
+ * \brief   getRGBFromIndex()
  *
- *      Input:  index (rgbindex)
- *              sigbits (2-6, significant bits retained in the quantizer
- *                       for each component of the input image)
- *              &rval, &gval, &bval (<return> rgb values)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    index rgbindex
+ * \param[in]    sigbits 2-6, significant bits retained in the quantizer
+ *                       for each component of the input image
+ * \param[out]   prval, pgval, pbval rgb values
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
- *      (1) The @index is expressed in bits, based on the the
- *          @sigbits of the r, g and b components, as
+ * <pre>
+ * Notes:
+ *      (1) The %index is expressed in bits, based on the the
+ *          %sigbits of the r, g and b components, as
  *             r7 r6 ... g7 g6 ... b7 b6 ...
  *      (2) The computed rgb values are in the center of the quantized cube.
  *          The extra bit that is OR'd accomplishes this.
+ * </pre>
  */
 l_int32
 getRGBFromIndex(l_uint32  index,
@@ -1442,22 +1468,23 @@ getRGBFromIndex(l_uint32  index,
  *             Identify images that have highlight (red) color             *
  * ----------------------------------------------------------------------- */
 /*!
- *  pixHasHighlightRed()
+ * \brief   pixHasHighlightRed()
  *
- *      Input:  pixs  (32 bpp rgb)
- *              factor (subsampling; an integer >= 1; use 1 for all pixels)
- *              fract (threshold fraction of all image pixels)
- *              fthresh (threshold on a function of the components; typ. ~2.5)
- *              &hasred (<return> 1 if red pixels are above threshold)
- *              &ratio (<optional return> normalized fraction of threshold
- *                      red pixels that is actually observed)
- *              &pixdb (<optional return> seed pixel mask)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs  32 bpp rgb
+ * \param[in]    factor subsampling; an integer >= 1; use 1 for all pixels
+ * \param[in]    fract threshold fraction of all image pixels
+ * \param[in]    fthresh threshold on a function of the components; typ. ~2.5
+ * \param[out]   phasred 1 if red pixels are above threshold
+ * \param[out]   pratio [optional] normalized fraction of threshold
+ *                      red pixels that is actually observed
+ * \param[out]   ppixdb [optional] seed pixel mask
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Pixels are identified as red if they satisfy two conditions:
- *          (a) The components satisfy (R-B)/B > @fthresh   (red or dark fg)
- *          (b) The red component satisfied R > 128  (red or light bg)
+ *          (a) The components satisfy (R-B)/B \> %fthresh   (red or dark fg)
+ *          (b) The red component satisfied R \> 128  (red or light bg)
  *          Masks are generated for (a) and (b), and the intersection
  *          gives the pixels that are red but not either light bg or
  *          dark fg.
@@ -1466,6 +1493,7 @@ getRGBFromIndex(l_uint32  index,
  *          in red.
  *      (3) A typical value for fthresh = 2.5.  Higher values give less
  *          sensitivity to red, and fewer false positives.
+ * </pre>
  */
 l_int32
 pixHasHighlightRed(PIX        *pixs,
@@ -1485,9 +1513,11 @@ FPIX      *fpix;
 
     if (pratio) *pratio = 0.0;
     if (ppixdb) *ppixdb = NULL;
+    if (phasred) *phasred = 0;
+    if (!pratio && !ppixdb)
+        return ERROR_INT("no return val requested", procName, 1);
     if (!phasred)
         return ERROR_INT("&hasred not defined", procName, 1);
-    *phasred = 0;
     if (!pixs || pixGetDepth(pixs) != 32)
         return ERROR_INT("pixs not defined or not 32 bpp", procName, 1);
     if (fthresh < 1.5 || fthresh > 3.5)
@@ -1527,5 +1557,3 @@ FPIX      *fpix;
     fpixDestroy(&fpix);
     return 0;
 }
-
-
