@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *   dnabasic.c
+/*!
+ * \file  dnabasic.c
+ * <pre>
  *
  *      Dna creation, destruction, copy, clone, etc.
  *          L_DNA       *l_dnaCreate()
@@ -28,7 +40,7 @@
  *
  *      Dna: add/remove number and extend array
  *          l_int32      l_dnaAddNumber()
- *          l_int32      l_dnaExtendArray()
+ *          static l_int32  l_dnaExtendArray()
  *          l_int32      l_dnaInsertNumber()
  *          l_int32      l_dnaRemoveNumber()
  *          l_int32      l_dnaReplaceNumber()
@@ -44,9 +56,9 @@
  *          l_float64   *l_dnaGetDArray()
  *          l_int32      l_dnaGetRefcount()
  *          l_int32      l_dnaChangeRefcount()
- *          l_int32      l_dnaGetXParameters()
- *          l_int32      l_dnaSetXParameters()
- *          l_int32      l_dnaCopyXParameters()
+ *          l_int32      l_dnaGetParameters()
+ *          l_int32      l_dnaSetParameters()
+ *          l_int32      l_dnaCopyParameters()
  *
  *      Serialize Dna for I/O
  *          L_DNA       *l_dnaRead()
@@ -56,11 +68,13 @@
  *
  *      Dnaa creation, destruction
  *          L_DNAA      *l_dnaaCreate()
+ *          L_DNAA      *l_dnaaCreateFull()
+ *          l_int32      l_dnaaTruncate()
  *          void        *l_dnaaDestroy()
  *
  *      Add Dna to Dnaa
  *          l_int32      l_dnaaAddDna()
- *          l_int32      l_dnaaExtendArray()
+ *          static l_int32  l_dnaaExtendArray()
  *
  *      Dnaa accessors
  *          l_int32      l_dnaaGetCount()
@@ -77,15 +91,10 @@
  *          l_int32      l_dnaaWrite()
  *          l_int32      l_dnaaWriteStream()
  *
- *      Other Dna functions
- *          L_DNA       *l_dnaMakeDelta()
- *          NUMA        *l_dnaConvertToNuma()
- *          L_DNA       *numaConvertToDna()
- *
  *    (1) The Dna is a struct holding an array of doubles.  It can also
  *        be used to store l_int32 values, up to the full precision
- *        of int32.  Use it whenever integers larger than a few million
- *        need to be stored.
+ *        of int32.  Always use it whenever integers larger than a
+ *        few million need to be stored.
  *
  *    (2) Always use the accessors in this file, never the fields directly.
  *
@@ -102,18 +111,18 @@
  *
  *       * to obtain a value from the array, use either l_dnaGetIValue()
  *         or l_dnaGetDValue(), depending on whether you are retrieving
- *         an integer or a float.  This avoids doing an explicit cast,
+ *         an integer or a float64.  This avoids doing an explicit cast,
  *         such as
  *           (a) return a l_float64 and cast it to an l_int32
  *           (b) cast the return directly to (l_float64 *) to
  *               satisfy the function prototype, as in
- *                 l_dnaGetDValue(da, index, (l_float64 *)&ival);   [ugly!]
+ *                 l_dnaGetDValue(da, index, (l_float64 *)\&ival);   [ugly!]
  *
- *    (4) int <--> double conversions:
+ *    (4) int \<--\> double conversions:
  *
- *        Conversions go automatically from l_int32 --> l_float64,
+ *        Conversions go automatically from l_int32 --\> l_float64,
  *        without loss of precision.  You must cast (l_int32)
- *        to go from l_float64 --> l_int32 because you're truncating
+ *        to go from l_float64 --\> l_int32 because you're truncating
  *        to the integer value.
  *
  *    (5) As with other arrays in leptonica, the l_dna has both an allocated
@@ -133,23 +142,28 @@
  *        startx and delx are initialized to 0.0 and 1.0, rsp.
  *        For arbitrary spacings, we use a second l_dna, and the two
  *        l_dnas are typically denoted dnay and dnax.
+ * </pre>
  */
 
 #include <string.h>
 #include <math.h>
 #include "allheaders.h"
 
-static const l_int32 INITIAL_PTR_ARRAYSIZE = 50;      /* n'importe quoi */
+static const l_int32 INITIAL_PTR_ARRAYSIZE = 50;      /*!< n'importe quoi */
+
+    /* Static functions */
+static l_int32 l_dnaExtendArray(L_DNA *da);
+static l_int32 l_dnaaExtendArray(L_DNAA *daa);
 
 
 /*--------------------------------------------------------------------------*
  *                 Dna creation, destruction, copy, clone, etc.             *
  *--------------------------------------------------------------------------*/
 /*!
- *  l_dnaCreate()
+ * \brief   l_dnaCreate()
  *
- *      Input:  size of number array to be alloc'd (0 for default)
- *      Return: da, or null on error
+ * \param[in]    n size of number array to be alloc'd; 0 for default
+ * \return  da, or NULL on error
  */
 L_DNA *
 l_dnaCreate(l_int32  n)
@@ -161,9 +175,9 @@ L_DNA  *da;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((da = (L_DNA *)CALLOC(1, sizeof(L_DNA))) == NULL)
+    if ((da = (L_DNA *)LEPT_CALLOC(1, sizeof(L_DNA))) == NULL)
         return (L_DNA *)ERROR_PTR("da not made", procName, NULL);
-    if ((da->array = (l_float64 *)CALLOC(n, sizeof(l_float64))) == NULL)
+    if ((da->array = (l_float64 *)LEPT_CALLOC(n, sizeof(l_float64))) == NULL)
         return (L_DNA *)ERROR_PTR("double array not made", procName, NULL);
 
     da->nalloc = n;
@@ -177,17 +191,19 @@ L_DNA  *da;
 
 
 /*!
- *  l_dnaCreateFromIArray()
+ * \brief   l_dnaCreateFromIArray()
  *
- *      Input:  iarray (integer)
- *              size (of the array)
- *      Return: da, or null on error
+ * \param[in]    iarray integer
+ * \param[in]    size of the array
+ * \return  da, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) We can't insert this int array into the l_dna, because a l_dna
  *          takes a double array.  So this just copies the data from the
  *          input array into the l_dna.  The input array continues to be
  *          owned by the caller.
+ * </pre>
  */
 L_DNA *
 l_dnaCreateFromIArray(l_int32  *iarray,
@@ -212,17 +228,19 @@ L_DNA   *da;
 
 
 /*!
- *  l_dnaCreateFromDArray()
+ * \brief   l_dnaCreateFromDArray()
  *
- *      Input:  da (float)
- *              size (of the array)
- *              copyflag (L_INSERT or L_COPY)
- *      Return: da, or null on error
+ * \param[in]    darray float
+ * \param[in]    size of the array
+ * \param[in]    copyflag L_INSERT or L_COPY
+ * \return  da, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) With L_INSERT, ownership of the input array is transferred
- *          to the returned l_dna, and all @size elements are considered
+ *          to the returned l_dna, and all %size elements are considered
  *          to be valid.
+ * </pre>
  */
 L_DNA *
 l_dnaCreateFromDArray(l_float64  *darray,
@@ -243,11 +261,10 @@ L_DNA   *da;
 
     da = l_dnaCreate(size);
     if (copyflag == L_INSERT) {
-        if (da->array) FREE(da->array);
+        if (da->array) LEPT_FREE(da->array);
         da->array = darray;
         da->n = size;
-    }
-    else {  /* just copy the contents */
+    } else {  /* just copy the contents */
         for (i = 0; i < size; i++)
             l_dnaAddNumber(da, darray[i]);
     }
@@ -257,12 +274,12 @@ L_DNA   *da;
 
 
 /*!
- *  l_dnaMakeSequence()
+ * \brief   l_dnaMakeSequence()
  *
- *      Input:  startval
- *              increment
- *              size (of sequence)
- *      Return: l_dna of sequence of evenly spaced values, or null on error
+ * \param[in]    startval
+ * \param[in]    increment
+ * \param[in]    size of sequence
+ * \return  l_dna of sequence of evenly spaced values, or NULL on error
  */
 L_DNA *
 l_dnaMakeSequence(l_float64  startval,
@@ -288,14 +305,16 @@ L_DNA     *da;
 
 
 /*!
- *  l_dnaDestroy()
+ * \brief   l_dnaDestroy()
  *
- *      Input:  &da (<to be nulled if it exists>)
- *      Return: void
+ * \param[in,out]   pda to be nulled if it exists
+ * \return  void
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Decrements the ref count and, if 0, destroys the l_dna.
  *      (2) Always nulls the input ptr.
+ * </pre>
  */
 void
 l_dnaDestroy(L_DNA  **pda)
@@ -305,7 +324,7 @@ L_DNA  *da;
     PROCNAME("l_dnaDestroy");
 
     if (pda == NULL) {
-        L_WARNING("ptr address is NULL", procName);
+        L_WARNING("ptr address is NULL\n", procName);
         return;
     }
 
@@ -316,8 +335,8 @@ L_DNA  *da;
     l_dnaChangeRefcount(da, -1);
     if (l_dnaGetRefcount(da) <= 0) {
         if (da->array)
-            FREE(da->array);
-        FREE(da);
+            LEPT_FREE(da->array);
+        LEPT_FREE(da);
     }
 
     *pda = NULL;
@@ -326,10 +345,15 @@ L_DNA  *da;
 
 
 /*!
- *  l_dnaCopy()
+ * \brief   l_dnaCopy()
  *
- *      Input:  da
- *      Return: copy of l_dna, or null on error
+ * \param[in]    da
+ * \return  copy of da, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This removes unused ptrs above da-\>n.
+ * </pre>
  */
 L_DNA *
 l_dnaCopy(L_DNA  *da)
@@ -342,7 +366,7 @@ L_DNA   *dac;
     if (!da)
         return (L_DNA *)ERROR_PTR("da not defined", procName, NULL);
 
-    if ((dac = l_dnaCreate(da->nalloc)) == NULL)
+    if ((dac = l_dnaCreate(da->n)) == NULL)
         return (L_DNA *)ERROR_PTR("dac not made", procName, NULL);
     dac->startx = da->startx;
     dac->delx = da->delx;
@@ -355,10 +379,10 @@ L_DNA   *dac;
 
 
 /*!
- *  l_dnaClone()
+ * \brief   l_dnaClone()
  *
- *      Input:  da
- *      Return: ptr to same l_dna, or null on error
+ * \param[in]    da
+ * \return  ptr to same da, or NULL on error
  */
 L_DNA *
 l_dnaClone(L_DNA  *da)
@@ -374,15 +398,17 @@ l_dnaClone(L_DNA  *da)
 
 
 /*!
- *  l_dnaEmpty()
+ * \brief   l_dnaEmpty()
  *
- *      Input:  da
- *      Return: 0 if OK; 1 on error
+ * \param[in]    da
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This does not change the allocation of the array.
  *          It just clears the number of stored numbers, so that
  *          the array appears to be empty.
+ * </pre>
  */
 l_int32
 l_dnaEmpty(L_DNA  *da)
@@ -402,11 +428,11 @@ l_dnaEmpty(L_DNA  *da)
  *                  Dna: add/remove number and extend array                 *
  *--------------------------------------------------------------------------*/
 /*!
- *  l_dnaAddNumber()
+ * \brief   l_dnaAddNumber()
  *
- *      Input:  da
- *              val  (float or int to be added; stored as a float)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \param[in]    val  float or int to be added; stored as a float
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaAddNumber(L_DNA     *da,
@@ -429,12 +455,12 @@ l_int32  n;
 
 
 /*!
- *  l_dnaExtendArray()
+ * \brief   l_dnaExtendArray()
  *
- *      Input:  da
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \return  0 if OK, 1 on error
  */
-l_int32
+static l_int32
 l_dnaExtendArray(L_DNA  *da)
 {
     PROCNAME("l_dnaExtendArray");
@@ -453,19 +479,21 @@ l_dnaExtendArray(L_DNA  *da)
 
 
 /*!
- *  l_dnaInsertNumber()
+ * \brief   l_dnaInsertNumber()
  *
- *      Input:  da
- *              index (location in da to insert new value)
- *              val  (float64 or integer to be added)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \param[in]    index location in da to insert new value
+ * \param[in]    val  float64 or integer to be added
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
- *      (1) This shifts da[i] --> da[i + 1] for all i >= index,
+ * <pre>
+ * Notes:
+ *      (1) This shifts da[i] --\> da[i + 1] for all i \>= index,
  *          and then inserts val as da[index].
  *      (2) It should not be used repeatedly on large arrays,
  *          because the function is O(n).
  *
+ * </pre>
  */
 l_int32
 l_dnaInsertNumber(L_DNA      *da,
@@ -493,16 +521,18 @@ l_int32  i, n;
 
 
 /*!
- *  l_dnaRemoveNumber()
+ * \brief   l_dnaRemoveNumber()
  *
- *      Input:  da
- *              index (element to be removed)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \param[in]    index element to be removed
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
- *      (1) This shifts da[i] --> da[i - 1] for all i > index.
+ * <pre>
+ * Notes:
+ *      (1) This shifts da[i] --\> da[i - 1] for all i \> index.
  *      (2) It should not be used repeatedly on large arrays,
  *          because the function is O(n).
+ * </pre>
  */
 l_int32
 l_dnaRemoveNumber(L_DNA   *da,
@@ -526,12 +556,12 @@ l_int32  i, n;
 
 
 /*!
- *  l_dnaReplaceNumber()
+ * \brief   l_dnaReplaceNumber()
  *
- *      Input:  da
- *              index (element to be replaced)
- *              val (new value to replace old one)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \param[in]    index element to be replaced
+ * \param[in]    val new value to replace old one
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaReplaceNumber(L_DNA     *da,
@@ -557,10 +587,10 @@ l_int32  n;
  *                             Dna accessors                            *
  *----------------------------------------------------------------------*/
 /*!
- *  l_dnaGetCount()
+ * \brief   l_dnaGetCount()
  *
- *      Input:  da
- *      Return: count, or 0 if no numbers or on error
+ * \param[in]    da
+ * \return  count, or 0 if no numbers or on error
  */
 l_int32
 l_dnaGetCount(L_DNA  *da)
@@ -574,18 +604,20 @@ l_dnaGetCount(L_DNA  *da)
 
 
 /*!
- *  l_dnaSetCount()
+ * \brief   l_dnaSetCount()
  *
- *      Input:  da
- *              newcount
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \param[in]    newcount
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
- *      (1) If newcount <= da->nalloc, this resets da->n.
+ * <pre>
+ * Notes:
+ *      (1) If newcount \<= da-\>nalloc, this resets da-\>n.
  *          Using newcount = 0 is equivalent to l_dnaEmpty().
- *      (2) If newcount > da->nalloc, this causes a realloc
- *          to a size da->nalloc = newcount.
+ *      (2) If newcount \> da-\>nalloc, this causes a realloc
+ *          to a size da-\>nalloc = newcount.
  *      (3) All the previously unused values in da are set to 0.0.
+ * </pre>
  */
 l_int32
 l_dnaSetCount(L_DNA   *da,
@@ -608,16 +640,18 @@ l_dnaSetCount(L_DNA   *da,
 
 
 /*!
- *  l_dnaGetDValue()
+ * \brief   l_dnaGetDValue()
  *
- *      Input:  da
- *              index (into l_dna)
- *              &val  (<return> double value; 0.0 on error)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    da
+ * \param[in]    index into l_dna
+ * \param[out]   pval  double value; 0.0 on error
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Caller may need to check the function return value to
  *          decide if a 0.0 in the returned ival is valid.
+ * </pre>
  */
 l_int32
 l_dnaGetDValue(L_DNA      *da,
@@ -641,16 +675,18 @@ l_dnaGetDValue(L_DNA      *da,
 
 
 /*!
- *  l_dnaGetIValue()
+ * \brief   l_dnaGetIValue()
  *
- *      Input:  da
- *              index (into l_dna)
- *              &ival  (<return> integer value; 0 on error)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    da
+ * \param[in]    index into l_dna
+ * \param[out]   pival  integer value; 0 on error
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Caller may need to check the function return value to
  *          decide if a 0 in the returned ival is valid.
+ * </pre>
  */
 l_int32
 l_dnaGetIValue(L_DNA    *da,
@@ -677,12 +713,12 @@ l_float64  val;
 
 
 /*!
- *  l_dnaSetValue()
+ * \brief   l_dnaSetValue()
  *
- *      Input:  da
- *              index  (to element to be set)
- *              val  (to set element)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    da
+ * \param[in]    index  to element to be set
+ * \param[in]    val  to set element
+ * \return  0 if OK; 1 on error
  */
 l_int32
 l_dnaSetValue(L_DNA     *da,
@@ -702,12 +738,12 @@ l_dnaSetValue(L_DNA     *da,
 
 
 /*!
- *  l_dnaShiftValue()
+ * \brief   l_dnaShiftValue()
  *
- *      Input:  da
- *              index (to element to change relative to the current value)
- *              diff  (increment if diff > 0 or decrement if diff < 0)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    da
+ * \param[in]    index to element to change relative to the current value
+ * \param[in]    diff  increment if diff > 0 or decrement if diff < 0
+ * \return  0 if OK; 1 on error
  */
 l_int32
 l_dnaShiftValue(L_DNA     *da,
@@ -727,12 +763,13 @@ l_dnaShiftValue(L_DNA     *da,
 
 
 /*!
- *  l_dnaGetIArray()
+ * \brief   l_dnaGetIArray()
  *
- *      Input:  da
- *      Return: a copy of the bare internal array, integerized
- *              by rounding, or null on error
- *  Notes:
+ * \param[in]    da
+ * \return  a copy of the bare internal array, integerized
+ *              by rounding, or NULL on error
+ * <pre>
+ * Notes:
  *      (1) A copy of the array is made, because we need to
  *          generate an integer array from the bare double array.
  *          The caller is responsible for freeing the array.
@@ -742,6 +779,7 @@ l_dnaShiftValue(L_DNA     *da,
  *          using the bare internal array, rather than continually
  *          calling accessors on the l_dna.  It is typically used
  *          on an array of size 256.
+ * </pre>
  */
 l_int32 *
 l_dnaGetIArray(L_DNA  *da)
@@ -755,7 +793,7 @@ l_int32  *array;
         return (l_int32 *)ERROR_PTR("da not defined", procName, NULL);
 
     n = l_dnaGetCount(da);
-    if ((array = (l_int32 *)CALLOC(n, sizeof(l_int32))) == NULL)
+    if ((array = (l_int32 *)LEPT_CALLOC(n, sizeof(l_int32))) == NULL)
         return (l_int32 *)ERROR_PTR("array not made", procName, NULL);
     for (i = 0; i < n; i++) {
         l_dnaGetIValue(da, i, &ival);
@@ -767,24 +805,26 @@ l_int32  *array;
 
 
 /*!
- *  l_dnaGetDArray()
+ * \brief   l_dnaGetDArray()
  *
- *      Input:  da
- *              copyflag (L_NOCOPY or L_COPY)
- *      Return: either the bare internal array or a copy of it,
- *              or null on error
+ * \param[in]    da
+ * \param[in]    copyflag L_NOCOPY or L_COPY
+ * \return  either the bare internal array or a copy of it,
+ *              or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) If copyflag == L_COPY, it makes a copy which the caller
  *          is responsible for freeing.  Otherwise, it operates
  *          directly on the bare array of the l_dna.
  *      (2) Very important: for L_NOCOPY, any writes to the array
  *          will be in the l_dna.  Do not write beyond the size of
- *          the count field, because it will not be accessable
+ *          the count field, because it will not be accessible
  *          from the l_dna!  If necessary, be sure to set the count
  *          field to a larger number (such as the alloc size)
  *          BEFORE calling this function.  Creating with l_dnaMakeConstant()
  *          is another way to insure full initialization.
+ * </pre>
  */
 l_float64 *
 l_dnaGetDArray(L_DNA   *da,
@@ -798,11 +838,11 @@ l_float64  *array;
     if (!da)
         return (l_float64 *)ERROR_PTR("da not defined", procName, NULL);
 
-    if (copyflag == L_NOCOPY)
+    if (copyflag == L_NOCOPY) {
         array = da->array;
-    else {  /* copyflag == L_COPY */
+    } else {  /* copyflag == L_COPY */
         n = l_dnaGetCount(da);
-        if ((array = (l_float64 *)CALLOC(n, sizeof(l_float64))) == NULL)
+        if ((array = (l_float64 *)LEPT_CALLOC(n, sizeof(l_float64))) == NULL)
             return (l_float64 *)ERROR_PTR("array not made", procName, NULL);
         for (i = 0; i < n; i++)
             array[i] = da->array[i];
@@ -813,10 +853,10 @@ l_float64  *array;
 
 
 /*!
- *  l_dnaGetRefCount()
+ * \brief   l_dnaGetRefCount()
  *
- *      Input:  da
- *      Return: refcount, or UNDEF on error
+ * \param[in]    da
+ * \return  refcount, or UNDEF on error
  */
 l_int32
 l_dnaGetRefcount(L_DNA  *da)
@@ -830,11 +870,11 @@ l_dnaGetRefcount(L_DNA  *da)
 
 
 /*!
- *  l_dnaChangeRefCount()
+ * \brief   l_dnaChangeRefCount()
  *
- *      Input:  da
- *              delta (change to be applied)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \param[in]    delta change to be applied
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaChangeRefcount(L_DNA   *da,
@@ -850,20 +890,24 @@ l_dnaChangeRefcount(L_DNA   *da,
 
 
 /*!
- *  l_dnaGetXParameters()
+ * \brief   l_dnaGetParameters()
  *
- *      Input:  da
- *              &startx (<optional return> startx)
- *              &delx (<optional return> delx)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    da
+ * \param[out]   pstartx [optional] startx
+ * \param[out]   pdelx [optional] delx
+ * \return  0 if OK, 1 on error
  */
 l_int32
-l_dnaGetXParameters(L_DNA     *da,
+l_dnaGetParameters(L_DNA     *da,
                    l_float64  *pstartx,
                    l_float64  *pdelx)
 {
-    PROCNAME("l_dnaGetXParameters");
+    PROCNAME("l_dnaGetParameters");
 
+    if (pstartx) *pstartx = 0.0;
+    if (pdelx) *pdelx = 1.0;
+    if (!pstartx && !pdelx)
+        return ERROR_INT("neither &startx nor &delx are defined", procName, 1);
     if (!da)
         return ERROR_INT("da not defined", procName, 1);
 
@@ -874,21 +918,21 @@ l_dnaGetXParameters(L_DNA     *da,
 
 
 /*!
- *  l_dnaSetXParameters()
+ * \brief   l_dnaSetParameters()
  *
- *      Input:  da
- *              startx (x value corresponding to da[0])
- *              delx (difference in x values for the situation where the
+ * \param[in]    da
+ * \param[in]    startx x value corresponding to da[0]
+ * \param[in]    delx difference in x values for the situation where the
  *                    elements of da correspond to the evaulation of a
- *                    function at equal intervals of size @delx)
- *      Return: 0 if OK, 1 on error
+ *                    function at equal intervals of size %delx
+ * \return  0 if OK, 1 on error
  */
 l_int32
-l_dnaSetXParameters(L_DNA     *da,
-                    l_float64  startx,
-                    l_float64  delx)
+l_dnaSetParameters(L_DNA     *da,
+                   l_float64  startx,
+                   l_float64  delx)
 {
-    PROCNAME("l_dnaSetXParameters");
+    PROCNAME("l_dnaSetParameters");
 
     if (!da)
         return ERROR_INT("da not defined", procName, 1);
@@ -900,25 +944,25 @@ l_dnaSetXParameters(L_DNA     *da,
 
 
 /*!
- *  l_dnaCopyXParameters()
+ * \brief   l_dnaCopyParameters()
  *
- *      Input:  dad (destination DNuma)
- *              das (source DNuma)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    dad destination DNuma
+ * \param[in]    das source DNuma
+ * \return  0 if OK, 1 on error
  */
 l_int32
-l_dnaCopyXParameters(L_DNA  *dad,
-                     L_DNA  *das)
+l_dnaCopyParameters(L_DNA  *dad,
+                    L_DNA  *das)
 {
 l_float64  start, binsize;
 
-    PROCNAME("l_dnaCopyXParameters");
+    PROCNAME("l_dnaCopyParameters");
 
     if (!das || !dad)
         return ERROR_INT("das and dad not both defined", procName, 1);
 
-    l_dnaGetXParameters(das, &start, &binsize);
-    l_dnaSetXParameters(dad, start, binsize);
+    l_dnaGetParameters(das, &start, &binsize);
+    l_dnaSetParameters(dad, start, binsize);
     return 0;
 }
 
@@ -927,10 +971,10 @@ l_float64  start, binsize;
  *                        Serialize Dna for I/O                         *
  *----------------------------------------------------------------------*/
 /*!
- *  l_dnaRead()
+ * \brief   l_dnaRead()
  *
- *      Input:  filename
- *      Return: da, or null on error
+ * \param[in]    filename
+ * \return  da, or NULL on error
  */
 L_DNA *
 l_dnaRead(const char  *filename)
@@ -957,10 +1001,15 @@ L_DNA  *da;
 
 
 /*!
- *  l_dnaReadStream()
+ * \brief   l_dnaReadStream()
  *
- *      Input:  stream
- *      Return: da, or null on error
+ * \param[in]    fp file stream
+ * \return  da, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) fscanf takes %lf to read a double; fprintf takes %f to write it.
+ * </pre>
  */
 L_DNA *
 l_dnaReadStream(FILE  *fp)
@@ -993,17 +1042,17 @@ L_DNA     *da;
 
         /* Optional data */
     if (fscanf(fp, "startx = %lf, delx = %lf\n", &startx, &delx) == 2)
-        l_dnaSetXParameters(da, startx, delx);
+        l_dnaSetParameters(da, startx, delx);
 
     return da;
 }
 
 
 /*!
- *  l_dnaWrite()
+ * \brief   l_dnaWrite()
  *
- *      Input:  filename, da
- *      Return: 0 if OK, 1 on error
+ * \param[in]    filename, da
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaWrite(const char  *filename,
@@ -1029,10 +1078,11 @@ FILE  *fp;
 
 
 /*!
- *  l_dnaWriteStream()
+ * \brief   l_dnaWriteStream()
  *
- *      Input:  stream, da
- *      Return: 0 if OK, 1 on error
+ * \param[in]    fp file stream
+ * \param[in]    da
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaWriteStream(FILE   *fp,
@@ -1052,13 +1102,13 @@ l_float64  startx, delx;
     fprintf(fp, "\nL_Dna Version %d\n", DNA_VERSION_NUMBER);
     fprintf(fp, "Number of numbers = %d\n", n);
     for (i = 0; i < n; i++)
-        fprintf(fp, "  [%d] = %lf\n", i, da->array[i]);
+        fprintf(fp, "  [%d] = %f\n", i, da->array[i]);
     fprintf(fp, "\n");
 
         /* Optional data */
-    l_dnaGetXParameters(da, &startx, &delx);
+    l_dnaGetParameters(da, &startx, &delx);
     if (startx != 0.0 || delx != 1.0)
-        fprintf(fp, "startx = %lf, delx = %lf\n", startx, delx);
+        fprintf(fp, "startx = %f, delx = %f\n", startx, delx);
 
     return 0;
 }
@@ -1068,10 +1118,10 @@ l_float64  startx, delx;
  *                       Dnaa creation, destruction                         *
  *--------------------------------------------------------------------------*/
 /*!
- *  l_dnaaCreate()
+ * \brief   l_dnaaCreate()
  *
- *      Input:  size of l_dna ptr array to be alloc'd (0 for default)
- *      Return: daa, or null on error
+ * \param[in]    n size of l_dna ptr array to be alloc'd 0 for default
+ * \return  daa, or NULL on error
  *
  */
 L_DNAA *
@@ -1084,9 +1134,9 @@ L_DNAA  *daa;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((daa = (L_DNAA *)CALLOC(1, sizeof(L_DNAA))) == NULL)
+    if ((daa = (L_DNAA *)LEPT_CALLOC(1, sizeof(L_DNAA))) == NULL)
         return (L_DNAA *)ERROR_PTR("daa not made", procName, NULL);
-    if ((daa->dna = (L_DNA **)CALLOC(n, sizeof(L_DNA *))) == NULL)
+    if ((daa->dna = (L_DNA **)LEPT_CALLOC(n, sizeof(L_DNA *))) == NULL)
         return (L_DNAA *)ERROR_PTR("l_dna ptr array not made", procName, NULL);
 
     daa->nalloc = n;
@@ -1097,10 +1147,84 @@ L_DNAA  *daa;
 
 
 /*!
- *  l_dnaaDestroy()
+ * \brief   l_dnaaCreateFull()
  *
- *      Input: &dnaa <to be nulled if it exists>
- *      Return: void
+ * \param[in]    nptr: size of dna ptr array to be alloc'd
+ * \param[in]    n: size of individual dna arrays to be alloc'd 0 for default
+ * \return  daa, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This allocates a dnaa and fills the array with allocated dnas.
+ *          In use, after calling this function, use
+ *              l_dnaaAddNumber(dnaa, index, val);
+ *          to add val to the index-th dna in dnaa.
+ * </pre>
+ */
+L_DNAA *
+l_dnaaCreateFull(l_int32  nptr,
+                 l_int32  n)
+{
+l_int32  i;
+L_DNAA  *daa;
+L_DNA   *da;
+
+    daa = l_dnaaCreate(nptr);
+    for (i = 0; i < nptr; i++) {
+        da = l_dnaCreate(n);
+        l_dnaaAddDna(daa, da, L_INSERT);
+    }
+
+    return daa;
+}
+
+
+/*!
+ * \brief   l_dnaaTruncate()
+ *
+ * \param[in]    daa
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This identifies the largest index containing a dna that
+ *          has any numbers within it, destroys all dna beyond that
+ *          index, and resets the count.
+ * </pre>
+ */
+l_int32
+l_dnaaTruncate(L_DNAA  *daa)
+{
+l_int32  i, n, nn;
+L_DNA   *da;
+
+    PROCNAME("l_dnaaTruncate");
+
+    if (!daa)
+        return ERROR_INT("daa not defined", procName, 1);
+
+    n = l_dnaaGetCount(daa);
+    for (i = n - 1; i >= 0; i--) {
+        da = l_dnaaGetDna(daa, i, L_CLONE);
+        if (!da)
+            continue;
+        nn = l_dnaGetCount(da);
+        l_dnaDestroy(&da);  /* the clone */
+        if (nn == 0)
+            l_dnaDestroy(&daa->dna[i]);
+        else
+            break;
+    }
+    daa->n = i + 1;
+    return 0;
+}
+
+
+/*!
+ * \brief   l_dnaaDestroy()
+ *
+ * \param[in,out] pdaa to be nulled if it exists
+ * \return  void
  */
 void
 l_dnaaDestroy(L_DNAA  **pdaa)
@@ -1111,7 +1235,7 @@ L_DNAA  *daa;
     PROCNAME("l_dnaaDestroy");
 
     if (pdaa == NULL) {
-        L_WARNING("ptr address is NULL!", procName);
+        L_WARNING("ptr address is NULL!\n", procName);
         return;
     }
 
@@ -1120,8 +1244,8 @@ L_DNAA  *daa;
 
     for (i = 0; i < daa->n; i++)
         l_dnaDestroy(&daa->dna[i]);
-    FREE(daa->dna);
-    FREE(daa);
+    LEPT_FREE(daa->dna);
+    LEPT_FREE(daa);
     *pdaa = NULL;
 
     return;
@@ -1132,17 +1256,17 @@ L_DNAA  *daa;
  *                             Add Dna to Dnaa                              *
  *--------------------------------------------------------------------------*/
 /*!
- *  l_dnaaAddDna()
+ * \brief   l_dnaaAddDna()
  *
- *      Input:  daa
- *              da   (to be added)
- *              copyflag  (L_INSERT, L_COPY, L_CLONE)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    daa
+ * \param[in]    da   to be added
+ * \param[in]    copyflag  L_INSERT, L_COPY, L_CLONE
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaaAddDna(L_DNAA  *daa,
-               L_DNA   *da,
-               l_int32  copyflag)
+             L_DNA   *da,
+             l_int32  copyflag)
 {
 l_int32  n;
 L_DNA   *dac;
@@ -1154,16 +1278,16 @@ L_DNA   *dac;
     if (!da)
         return ERROR_INT("da not defined", procName, 1);
 
-    if (copyflag == L_INSERT)
+    if (copyflag == L_INSERT) {
         dac = da;
-    else if (copyflag == L_COPY) {
+    } else if (copyflag == L_COPY) {
         if ((dac = l_dnaCopy(da)) == NULL)
             return ERROR_INT("dac not made", procName, 1);
-    }
-    else if (copyflag == L_CLONE)
+    } else if (copyflag == L_CLONE) {
         dac = l_dnaClone(da);
-    else
+    } else {
         return ERROR_INT("invalid copyflag", procName, 1);
+    }
 
     n = l_dnaaGetCount(daa);
     if (n >= daa->nalloc)
@@ -1175,12 +1299,12 @@ L_DNA   *dac;
 
 
 /*!
- *  l_dnaaExtendArray()
+ * \brief   l_dnaaExtendArray()
  *
- *      Input:  daa
- *      Return: 0 if OK, 1 on error
+ * \param[in]    daa
+ * \return  0 if OK, 1 on error
  */
-l_int32
+static l_int32
 l_dnaaExtendArray(L_DNAA  *daa)
 {
     PROCNAME("l_dnaaExtendArray");
@@ -1202,10 +1326,10 @@ l_dnaaExtendArray(L_DNAA  *daa)
  *                           DNumaa accessors                           *
  *----------------------------------------------------------------------*/
 /*!
- *  l_dnaaGetCount()
+ * \brief   l_dnaaGetCount()
  *
- *      Input:  daa
- *      Return: count (number of l_dna), or 0 if no l_dna or on error
+ * \param[in]    daa
+ * \return  count number of l_dna, or 0 if no l_dna or on error
  */
 l_int32
 l_dnaaGetCount(L_DNAA  *daa)
@@ -1219,11 +1343,11 @@ l_dnaaGetCount(L_DNAA  *daa)
 
 
 /*!
- *  l_dnaaGetDnaCount()
+ * \brief   l_dnaaGetDnaCount()
  *
- *      Input:  daa
- *              index (of l_dna in daa)
- *      Return: count of numbers in the referenced l_dna, or 0 on error.
+ * \param[in]    daa
+ * \param[in]    index of l_dna in daa
+ * \return  count of numbers in the referenced l_dna, or 0 on error.
  */
 l_int32
 l_dnaaGetDnaCount(L_DNAA   *daa,
@@ -1240,10 +1364,10 @@ l_dnaaGetDnaCount(L_DNAA   *daa,
 
 
 /*!
- *  l_dnaaGetNumberCount()
+ * \brief   l_dnaaGetNumberCount()
  *
- *      Input:  daa
- *      Return: count (total number of numbers in the l_dnaa),
+ * \param[in]    daa
+ * \return  count total number of numbers in the l_dnaa,
  *                     or 0 if no numbers or on error
  */
 l_int32
@@ -1269,12 +1393,12 @@ l_int32  n, sum, i;
 
 
 /*!
- *  l_dnaaGetDna()
+ * \brief   l_dnaaGetDna()
  *
- *      Input:  daa
- *              index  (to the index-th l_dna)
- *              accessflag   (L_COPY or L_CLONE)
- *      Return: l_dna, or null on error
+ * \param[in]    daa
+ * \param[in]    index  to the index-th l_dna
+ * \param[in]    accessflag   L_COPY or L_CLONE
+ * \return  l_dna, or NULL on error
  */
 L_DNA *
 l_dnaaGetDna(L_DNAA  *daa,
@@ -1298,17 +1422,19 @@ l_dnaaGetDna(L_DNAA  *daa,
 
 
 /*!
- *  l_dnaaReplaceDna()
+ * \brief   l_dnaaReplaceDna()
  *
- *      Input:  daa
- *              index  (to the index-th l_dna)
- *              l_dna (insert and replace any existing one)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    daa
+ * \param[in]    index  to the index-th l_dna
+ * \param[in]    da insert and replace any existing one
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Any existing l_dna is destroyed, and the input one
  *          is inserted in its place.
  *      (2) If the index is invalid, return 1 (error)
+ * </pre>
  */
 l_int32
 l_dnaaReplaceDna(L_DNAA  *daa,
@@ -1334,13 +1460,13 @@ l_int32  n;
 
 
 /*!
- *  l_dnaaGetValue()
+ * \brief   l_dnaaGetValue()
  *
- *      Input:  daa
- *              i (index of l_dna within l_dnaa)
- *              j (index into l_dna)
- *              val (<return> double value)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    daa
+ * \param[in]    i index of l_dna within l_dnaa
+ * \param[in]    j index into l_dna
+ * \param[out]   pval double value
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaaGetValue(L_DNAA     *daa,
@@ -1370,15 +1496,17 @@ L_DNA   *da;
 
 
 /*!
- *  l_dnaaAddNumber()
+ * \brief   l_dnaaAddNumber()
  *
- *      Input:  daa
- *              index (of l_dna within l_dnaa)
- *              val  (number to be added; stored as a double)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    daa
+ * \param[in]    index of l_dna within l_dnaa
+ * \param[in]    val  number to be added; stored as a double
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Adds to an existing l_dna only.
+ * </pre>
  */
 l_int32
 l_dnaaAddNumber(L_DNAA    *daa,
@@ -1407,10 +1535,10 @@ L_DNA   *da;
  *                        Serialize Dna for I/O                         *
  *----------------------------------------------------------------------*/
 /*!
- *  l_dnaaRead()
+ * \brief   l_dnaaRead()
  *
- *      Input:  filename
- *      Return: daa, or null on error
+ * \param[in]    filename
+ * \return  daa, or NULL on error
  */
 L_DNAA *
 l_dnaaRead(const char  *filename)
@@ -1437,10 +1565,10 @@ L_DNAA  *daa;
 
 
 /*!
- *  l_dnaaReadStream()
+ * \brief   l_dnaaReadStream()
  *
- *      Input:  stream
- *      Return: daa, or null on error
+ * \param[in]    fp file stream
+ * \return  daa, or NULL on error
  */
 L_DNAA *
 l_dnaaReadStream(FILE  *fp)
@@ -1477,10 +1605,10 @@ L_DNAA    *daa;
 
 
 /*!
- *  l_dnaaWrite()
+ * \brief   l_dnaaWrite()
  *
- *      Input:  filename, daa
- *      Return: 0 if OK, 1 on error
+ * \param[in]    filename, daa
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaaWrite(const char  *filename,
@@ -1506,10 +1634,11 @@ FILE  *fp;
 
 
 /*!
- *  l_dnaaWriteStream()
+ * \brief   l_dnaaWriteStream()
  *
- *      Input:  stream, daa
- *      Return: 0 if OK, 1 on error
+ * \param[in]    fp file stream
+ * \param[in]    daa
+ * \return  0 if OK, 1 on error
  */
 l_int32
 l_dnaaWriteStream(FILE    *fp,
@@ -1539,90 +1668,3 @@ L_DNA   *da;
     return 0;
 }
 
-
-/*----------------------------------------------------------------------*
- *                         Other Dna functions                          *
- *----------------------------------------------------------------------*/
-/*!
- *  l_dnaMakeDelta()
- *
- *      Input:  das (input l_dna)
- *      Return: dad (of difference values val[i+1] - val[i]),
- *                   or null on error
- */
-L_DNA *
-l_dnaMakeDelta(L_DNA  *das)
-{
-l_int32  i, n, prev, cur;
-L_DNA   *dad;
-
-    PROCNAME("l_dnaMakeDelta");
-
-    if (!das)
-        return (L_DNA *)ERROR_PTR("das not defined", procName, NULL);
-    n = l_dnaGetCount(das);
-    dad = l_dnaCreate(n - 1);
-    prev = 0;
-    for (i = 1; i < n; i++) {
-        l_dnaGetIValue(das, i, &cur);
-        l_dnaAddNumber(dad, cur - prev);
-        prev = cur;
-    }
-    return dad;
-}
-
-
-/*!
- *  l_dnaConvertToNuma()
- *
- *      Input:  da
- *      Return: na, or null on error
- */
-NUMA *
-l_dnaConvertToNuma(L_DNA  *da)
-{
-l_int32    i, n;
-l_float64  val;
-NUMA      *na;
-
-    PROCNAME("l_dnaConvertToNuma");
-
-    if (!da)
-        return (NUMA *)ERROR_PTR("da not defined", procName, NULL);
-
-    n = l_dnaGetCount(da);
-    na = numaCreate(n);
-    for (i = 0; i < n; i++) {
-        l_dnaGetDValue(da, i, &val);
-        numaAddNumber(na, val);
-    }
-    return na;
-}
-
-
-/*!
- *  numaConvertToDna
- *
- *      Input:  na
- *      Return: da, or null on error
- */
-L_DNA *
-numaConvertToDna(NUMA  *na)
-{
-l_int32    i, n;
-l_float32  val;
-L_DNA     *da;
-
-    PROCNAME("numaConvertToDna");
-
-    if (!na)
-        return (L_DNA *)ERROR_PTR("na not defined", procName, NULL);
-
-    n = numaGetCount(na);
-    da = l_dnaCreate(n);
-    for (i = 0; i < n; i++) {
-        numaGetFValue(na, i, &val);
-        l_dnaAddNumber(da, val);
-    }
-    return da;
-}
