@@ -25,8 +25,9 @@
  *====================================================================*/
 
 
-/*
- *  fhmtauto.c
+/*!
+ * \file fhmtauto.c
+ * <pre>
  *
  *    Main function calls:
  *       l_int32             fhmtautogen()
@@ -73,11 +74,11 @@
  *    (4) In an application, you now use this interface.  Again
  *        for the example files generated, using integer "1":
  *
- *           PIX   *pixHMTDwa_1(PIX *pixd, PIX *pixs, char *selname);
+ *           PIX   *pixHMTDwa_1(PIX *pixd, PIX *pixs, const char *selname);
  *
  *              or
  *
- *           PIX   *pixFHMTGen_1(PIX *pixd, PIX *pixs, char *selname);
+ *           PIX   *pixFHMTGen_1(PIX *pixd, PIX *pixs, const char *selname);
  *
  *        where the selname is one of the set that were defined
  *        as the name field of sels.  This set is listed at the
@@ -86,6 +87,7 @@
  *        verifies the correctness of the implementation by
  *        comparing the dwa result with that of full-image
  *        rasterops.
+ * </pre>
  */
 
 #include <string.h>
@@ -187,17 +189,19 @@ static char wplstrm[][10] = {"- wpls", "- wpls2", "- wpls3", "- wpls4",
 
 
 /*!
- *  fhmtautogen()
+ * \brief   fhmtautogen()
  *
- *      Input:  sela
- *              fileindex
- *              filename (<optional>; can be null)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    sela
+ * \param[in]    fileindex
+ * \param[in]    filename [optional]; can be null
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function generates all the code for implementing
  *          dwa morphological operations using all the sels in the sela.
  *      (2) See fhmtautogen1() and fhmtautogen2() for details.
+ * </pre>
  */
 l_int32
 fhmtautogen(SELA        *sela,
@@ -219,25 +223,27 @@ l_int32  ret1, ret2;
 
 
 /*!
- *  fhmtautogen1()
+ * \brief   fhmtautogen1()
  *
- *      Input:  sel array
- *              fileindex
- *              filename (<optional>; can be null)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    sela array
+ * \param[in]    fileindex
+ * \param[in]    filename [optional]; can be null
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function uses hmttemplate1.txt to create a
  *          top-level file that contains two functions that carry
  *          out the hit-miss transform for any of the sels in
  *          the input sela.
  *      (2) The fileindex parameter is inserted into the output
  *          filename, as described below.
- *      (3) If filename == NULL, the output file is fhmtgen.<n>.c,
- *          where <n> is equal to the 'fileindex' parameter.
- *      (4) If filename != NULL, the output file is <filename>.<n>.c.
+ *      (3) If filename == NULL, the output file is fhmtgen.\<n\>.c,
+ *          where \<n\> is equal to the 'fileindex' parameter.
+ *      (4) If filename != NULL, the output file is \<filename\>.\<n\>.c.
  *      (5) Each sel must have at least one hit.  A sel with only misses
  *          generates code that will abort the operation if it is called.
+ * </pre>
  */
 l_int32
 fhmtautogen1(SELA        *sela,
@@ -271,14 +277,14 @@ SARRAY  *sa1, *sa2, *sa3;
         return ERROR_INT("filestr not made", procName, 1);
     if ((sa2 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
         return ERROR_INT("sa2 not made", procName, 1);
-    FREE(filestr);
+    LEPT_FREE(filestr);
 
         /* Make strings containing function call names */
     sprintf(bigbuf, "PIX *pixHMTDwa_%d(PIX *pixd, PIX *pixs, "
-                    "char *selname);", fileindex);
+                    "const char *selname);", fileindex);
     str_proto1 = stringNew(bigbuf);
     sprintf(bigbuf, "PIX *pixFHMTGen_%d(PIX *pixd, PIX *pixs, "
-                    "char *selname);", fileindex);
+                    "const char *selname);", fileindex);
     str_proto2 = stringNew(bigbuf);
     sprintf(bigbuf, "l_int32 fhmtgen_low_%d(l_uint32 *datad, l_int32 w,\n"
             "                      l_int32 h, l_int32 wpld,\n"
@@ -293,9 +299,9 @@ SARRAY  *sa1, *sa2, *sa3;
     str_doc3 = stringNew(bigbuf);
     sprintf(bigbuf, " *  pixFHMTGen_%d()", fileindex);
     str_doc4 = stringNew(bigbuf);
-    sprintf(bigbuf, "pixHMTDwa_%d(PIX   *pixd,", fileindex);
+    sprintf(bigbuf, "pixHMTDwa_%d(PIX         *pixd,", fileindex);
     str_def1 = stringNew(bigbuf);
-    sprintf(bigbuf, "pixFHMTGen_%d(PIX   *pixd,", fileindex);
+    sprintf(bigbuf, "pixFHMTGen_%d(PIX         *pixd,", fileindex);
     str_def2 = stringNew(bigbuf);
     sprintf(bigbuf, "    PROCNAME(\"pixHMTDwa_%d\");", fileindex);
     str_proc1 = stringNew(bigbuf);
@@ -340,12 +346,12 @@ SARRAY  *sa1, *sa2, *sa3;
     sprintf(bigbuf, "static char  SEL_NAMES[][80] = {");
     sarrayAddString(sa3, bigbuf, L_COPY);
     for (i = 0; i < nsels - 1; i++) {
-        sprintf(bigbuf,
-        "                             \"%s\",", sarrayGetString(sa1, i, 0));
+        sprintf(bigbuf, "                             \"%s\",",
+                sarrayGetString(sa1, i, L_NOCOPY));
         sarrayAddString(sa3, bigbuf, L_COPY);
     }
-    sprintf(bigbuf,
-        "                             \"%s\"};", sarrayGetString(sa1, i, 0));
+    sprintf(bigbuf, "                             \"%s\"};",
+            sarrayGetString(sa1, i, L_NOCOPY));
     sarrayAddString(sa3, bigbuf, L_COPY);
 
         /* Start pixHMTDwa_*() function description */
@@ -389,36 +395,38 @@ SARRAY  *sa1, *sa2, *sa3;
         return ERROR_INT("filestr from sa3 not made", procName, 1);
     nbytes = strlen(filestr);
     if (filename)
-        sprintf(bigbuf, "%s.%d.c", filename, fileindex);
+        snprintf(bigbuf, L_BUF_SIZE, "%s.%d.c", filename, fileindex);
     else
         sprintf(bigbuf, "%s.%d.c", OUTROOT, fileindex);
     l_binaryWrite(bigbuf, "w", filestr, nbytes);
     sarrayDestroy(&sa1);
     sarrayDestroy(&sa2);
     sarrayDestroy(&sa3);
-    FREE(filestr);
+    LEPT_FREE(filestr);
     return 0;
 }
 
 
 /*!
- *  fhmtautogen2()
+ * \brief   fhmtautogen2()
  *
- *      Input:  sel array
- *              fileindex
- *              filename (<optional>; can be null)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    sela array
+ * \param[in]    fileindex
+ * \param[in]    filename [optional]; can be null
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function uses hmttemplate2.txt to create a
  *          low-level file that contains the low-level functions for
  *          implementing the hit-miss transform for every sel
  *          in the input sela.
  *      (2) The fileindex parameter is inserted into the output
  *          filename, as described below.
- *      (3) If filename == NULL, the output file is fhmtgenlow.<n>.c,
- *          where <n> is equal to the 'fileindex' parameter.
- *      (4) If filename != NULL, the output file is <filename>low.<n>.c.
+ *      (3) If filename == NULL, the output file is fhmtgenlow.\<n\>.c,
+ *          where \<n\> is equal to the 'fileindex' parameter.
+ *      (4) If filename != NULL, the output file is \<filename\>low.\<n\>.c.
+ * </pre>
  */
 l_int32
 fhmtautogen2(SELA        *sela,
@@ -451,23 +459,23 @@ SEL     *sel;
         return ERROR_INT("filestr not made", procName, 1);
     if ((sa1 = sarrayCreateLinesFromString(filestr, 1)) == NULL)
         return ERROR_INT("sa1 not made", procName, 1);
-    FREE(filestr);
+    LEPT_FREE(filestr);
 
         /* Make the array of static function names */
     if ((sa2 = sarrayCreate(nsels)) == NULL)
         return ERROR_INT("sa2 not made", procName, 1);
     for (i = 0; i < nsels; i++) {
         sprintf(bigbuf, "fhmt_%d_%d", fileindex, i);
-        sarrayAddString(sa2, bigbuf, 1);
+        sarrayAddString(sa2, bigbuf, L_COPY);
     }
 
         /* Make the static prototype strings */
     if ((sa3 = sarrayCreate(2 * nsels)) == NULL)
         return ERROR_INT("sa3 not made", procName, 1);
     for (i = 0; i < nsels; i++) {
-        fname = sarrayGetString(sa2, i, 0);
+        fname = sarrayGetString(sa2, i, L_NOCOPY);
         sprintf(bigbuf, "static void  %s%s", fname, PROTOARGS);
-        sarrayAddString(sa3, bigbuf, 1);
+        sarrayAddString(sa3, bigbuf, L_COPY);
     }
 
         /* Make strings containing function names */
@@ -551,7 +559,7 @@ SEL     *sel;
             return ERROR_INT("sel not returned", procName, 1);
         if ((sa5 = sarrayMakeWplsCode(sel)) == NULL)
             return ERROR_INT("sa5 not made", procName, 1);
-        sarrayConcatenate(sa4, sa5);
+        sarrayJoin(sa4, sa5);
         sarrayDestroy(&sa5);
 
             /* Make sure sel has at least one hit */
@@ -577,7 +585,7 @@ SEL     *sel;
             /* Insert barrel-op code for *dptr */
         if ((sa6 = sarrayMakeInnerLoopDWACode(sel, nhits, nmisses)) == NULL)
             return ERROR_INT("sa6 not made", procName, 1);
-        sarrayConcatenate(sa4, sa6);
+        sarrayJoin(sa4, sa6);
         sarrayDestroy(&sa6);
 
             /* Finish the function code */
@@ -589,7 +597,7 @@ SEL     *sel;
         return ERROR_INT("filestr from sa4 not made", procName, 1);
     nbytes = strlen(filestr);
     if (filename)
-        sprintf(bigbuf, "%slow.%d.c", filename, fileindex);
+        snprintf(bigbuf, L_BUF_SIZE, "%slow.%d.c", filename, fileindex);
     else
         sprintf(bigbuf, "%slow.%d.c", OUTROOT, fileindex);
     l_binaryWrite(bigbuf, "w", filestr, nbytes);
@@ -597,7 +605,7 @@ SEL     *sel;
     sarrayDestroy(&sa2);
     sarrayDestroy(&sa3);
     sarrayDestroy(&sa4);
-    FREE(filestr);
+    LEPT_FREE(filestr);
 
     return 0;
 }
@@ -608,7 +616,7 @@ SEL     *sel;
  *                            Helper code for sel                           *
  *--------------------------------------------------------------------------*/
 /*!
- *  sarrayMakeWplsCode()
+ * \brief   sarrayMakeWplsCode()
  */
 static SARRAY *
 sarrayMakeWplsCode(SEL  *sel)
@@ -625,7 +633,7 @@ SARRAY  *sa;
     ymax = 0;
     for (i = 0; i < sel->sy; i++) {
         for (j = 0; j < sel->sx; j++) {
-            if (sel->data[i][j] == 1) {
+            if (sel->data[i][j] == 1 || sel->data[i][j] == 2) {
                 dely = L_ABS(i - sel->cy);
                 ymax = L_MAX(ymax, dely);
             }
@@ -641,34 +649,34 @@ SARRAY  *sa;
 
         /* Declarations */
     if (ymax > 4)
-        sarrayAddString(sa, wpldecls[2], 1);
+        sarrayAddString(sa, wpldecls[2], L_COPY);
     if (ymax > 8)
-        sarrayAddString(sa, wpldecls[6], 1);
+        sarrayAddString(sa, wpldecls[6], L_COPY);
     if (ymax > 12)
-        sarrayAddString(sa, wpldecls[10], 1);
+        sarrayAddString(sa, wpldecls[10], L_COPY);
     if (ymax > 16)
-        sarrayAddString(sa, wpldecls[14], 1);
+        sarrayAddString(sa, wpldecls[14], L_COPY);
     if (ymax > 20)
-        sarrayAddString(sa, wpldecls[18], 1);
+        sarrayAddString(sa, wpldecls[18], L_COPY);
     if (ymax > 24)
-        sarrayAddString(sa, wpldecls[22], 1);
+        sarrayAddString(sa, wpldecls[22], L_COPY);
     if (ymax > 28)
-        sarrayAddString(sa, wpldecls[26], 1);
+        sarrayAddString(sa, wpldecls[26], L_COPY);
     if (ymax > 1)
-        sarrayAddString(sa, wpldecls[ymax - 2], 1);
+        sarrayAddString(sa, wpldecls[ymax - 2], L_COPY);
 
-    sarrayAddString(sa, emptystring, 1);
+    sarrayAddString(sa, emptystring, L_COPY);
 
         /* Definitions */
     for (i = 2; i <= ymax; i++)
-        sarrayAddString(sa, wpldefs[i - 2], 1);
+        sarrayAddString(sa, wpldefs[i - 2], L_COPY);
 
     return sa;
 }
 
 
 /*!
- *  sarrayMakeInnerLoopDWACode()
+ * \brief   sarrayMakeInnerLoopDWACode()
  */
 static SARRAY *
 sarrayMakeInnerLoopDWACode(SEL     *sel,
@@ -711,8 +719,8 @@ SARRAY  *sa;
                     sprintf(bigbuf, "                    %s %s", string, land);
                 else  /* nfound == ntot */
                     sprintf(bigbuf, "                    %s;", string);
-                sarrayAddString(sa, bigbuf, 1);
-                FREE(string);
+                sarrayAddString(sa, bigbuf, L_COPY);
+                LEPT_FREE(string);
             }
         }
     }
@@ -722,7 +730,7 @@ SARRAY  *sa;
 
 
 /*!
- *  makeBarrelshiftString()
+ * \brief   makeBarrelshiftString()
  */
 static char *
 makeBarrelshiftString(l_int32  delx,    /* j - cx */
