@@ -25,8 +25,9 @@
  *====================================================================*/
 
 
-/*
- *  morphapp.c
+/*!
+ * \file morphapp.c
+ * <pre>
  *
  *      These are some useful and/or interesting composite
  *      image processing operations, of the type that are often
@@ -60,6 +61,9 @@
  *      Display of matched patterns
  *            PIX       *pixDisplayMatchedPattern()
  *
+ *      Extension of pixa by iterative erosion or dilation
+ *            PIXA      *pixaExtendIterative()
+ *
  *      Iterative morphological seed filling (don't use for real work)
  *            PIX       *pixSeedfillMorph()
  *
@@ -75,6 +79,7 @@
  *      Centroid of component
  *            PTA       *pixaCentroids()
  *            l_int32    pixCentroid()
+ * </pre>
  */
 
 #include "allheaders.h"
@@ -86,15 +91,17 @@
  *                   Extraction of boundary pixels                 *
  *-----------------------------------------------------------------*/
 /*!
- *  pixExtractBoundary()
+ * \brief   pixExtractBoundary()
  *
- *      Input:  pixs (1 bpp)
- *              type (0 for background pixels; 1 for foreground pixels)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 1 bpp
+ * \param[in]    type 0 for background pixels; 1 for foreground pixels
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Extracts the fg or bg boundary pixels for each component.
  *          Components are assumed to end at the boundary of pixs.
+ * </pre>
  */
 PIX *
 pixExtractBoundary(PIX     *pixs,
@@ -120,19 +127,21 @@ PIX  *pixd;
  *           Selective morph sequence operation under mask         *
  *-----------------------------------------------------------------*/
 /*!
- *  pixMorphSequenceMasked()
+ * \brief   pixMorphSequenceMasked()
  *
- *      Input:  pixs (1 bpp)
- *              pixm (<optional> 1 bpp mask)
- *              sequence (string specifying sequence of operations)
- *              dispsep (horizontal separation in pixels between
- *                       successive displays; use zero to suppress display)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 1 bpp
+ * \param[in]    pixm [optional] 1 bpp mask
+ * \param[in]    sequence string specifying sequence of operations
+ * \param[in]    dispsep horizontal separation in pixels between
+ *                       successive displays; use zero to suppress display
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This applies the morph sequence to the image, but only allows
  *          changes in pixs for pixels under the background of pixm.
  *      (5) If pixm is NULL, this is just pixMorphSequence().
+ * </pre>
  */
 PIX *
 pixMorphSequenceMasked(PIX         *pixs,
@@ -159,17 +168,18 @@ PIX  *pixd;
  *             Morph sequence operation on each component          *
  *-----------------------------------------------------------------*/
 /*!
- *  pixMorphSequenceByComponent()
+ * \brief   pixMorphSequenceByComponent()
  *
- *      Input:  pixs (1 bpp)
- *              sequence (string specifying sequence)
- *              connectivity (4 or 8)
- *              minw  (minimum width to consider; use 0 or 1 for any width)
- *              minh  (minimum height to consider; use 0 or 1 for any height)
- *              &boxa (<optional> return boxa of c.c. in pixs)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 1 bpp
+ * \param[in]    sequence string specifying sequence
+ * \param[in]    connectivity 4 or 8
+ * \param[in]    minw  minimum width to consider; use 0 or 1 for any width
+ * \param[in]    minh  minimum height to consider; use 0 or 1 for any height
+ * \param[out]   pboxa [optional] return boxa of c.c. in pixs
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixMorphSequence() for composing operation sequences.
  *      (2) This operates separately on each c.c. in the input pix.
  *      (3) The dilation does NOT increase the c.c. size; it is clipped
@@ -178,6 +188,7 @@ PIX  *pixd;
  *      (4) You can specify that the width and/or height must equal
  *          or exceed a minimum size for the operation to take place.
  *      (5) Use NULL for boxa to avoid returning the boxa.
+ * </pre>
  */
 PIX *
 pixMorphSequenceByComponent(PIX         *pixs,
@@ -231,21 +242,23 @@ PIXA    *pixas, *pixad;
 
 
 /*!
- *  pixaMorphSequenceByComponent()
+ * \brief   pixaMorphSequenceByComponent()
  *
- *      Input:  pixas (of 1 bpp pix)
- *              sequence (string specifying sequence)
- *              minw  (minimum width to consider; use 0 or 1 for any width)
- *              minh  (minimum height to consider; use 0 or 1 for any height)
- *      Return: pixad, or null on error
+ * \param[in]    pixas of 1 bpp pix
+ * \param[in]    sequence string specifying sequence
+ * \param[in]    minw  minimum width to consider; use 0 or 1 for any width
+ * \param[in]    minh  minimum height to consider; use 0 or 1 for any height
+ * \return  pixad, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixMorphSequence() for composing operation sequences.
  *      (2) This operates separately on each c.c. in the input pixa.
  *      (3) You can specify that the width and/or height must equal
  *          or exceed a minimum size for the operation to take place.
  *      (4) The input pixa should have a boxa giving the locations
  *          of the pix components.
+ * </pre>
  */
 PIXA *
 pixaMorphSequenceByComponent(PIXA        *pixas,
@@ -280,10 +293,14 @@ PIXA    *pixad;
     for (i = 0; i < n; i++) {
         pixaGetPixDimensions(pixas, i, &w, &h, NULL);
         if (w >= minw && h >= minh) {
-            if ((pix1 = pixaGetPix(pixas, i, L_CLONE)) == NULL)
+            if ((pix1 = pixaGetPix(pixas, i, L_CLONE)) == NULL) {
+                pixaDestroy(&pixad);
                 return (PIXA *)ERROR_PTR("pix1 not found", procName, NULL);
-            if ((pix2 = pixMorphCompSequence(pix1, sequence, 0)) == NULL)
+            }
+            if ((pix2 = pixMorphCompSequence(pix1, sequence, 0)) == NULL) {
+                pixaDestroy(&pixad);
                 return (PIXA *)ERROR_PTR("pix2 not made", procName, NULL);
+            }
             pixaAddPix(pixad, pix2, L_INSERT);
             box = pixaGetBox(pixas, i, L_COPY);
             pixaAddBox(pixad, box, L_INSERT);
@@ -299,18 +316,19 @@ PIXA    *pixad;
  *              Morph sequence operation on each region            *
  *-----------------------------------------------------------------*/
 /*!
- *  pixMorphSequenceByRegion()
+ * \brief   pixMorphSequenceByRegion()
  *
- *      Input:  pixs (1 bpp)
- *              pixm (mask specifying regions)
- *              sequence (string specifying sequence)
- *              connectivity (4 or 8, used on mask)
- *              minw  (minimum width to consider; use 0 or 1 for any width)
- *              minh  (minimum height to consider; use 0 or 1 for any height)
- *              &boxa (<optional> return boxa of c.c. in pixm)
- *      Return: pixd, or null on error
+ * \param[in]    pixs 1 bpp
+ * \param[in]    pixm mask specifying regions
+ * \param[in]    sequence string specifying sequence
+ * \param[in]    connectivity 4 or 8, used on mask
+ * \param[in]    minw  minimum width to consider; use 0 or 1 for any width
+ * \param[in]    minh  minimum height to consider; use 0 or 1 for any height
+ * \param[out]   pboxa [optional] return boxa of c.c. in pixm
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixMorphCompSequence() for composing operation sequences.
  *      (2) This operates separately on the region in pixs corresponding
  *          to each c.c. in the mask pixm.  It differs from
@@ -322,7 +340,8 @@ PIXA    *pixad;
  *          to make regions independent after the operation.
  *      (4) You can specify that the width and/or height of a region must
  *          equal or exceed a minimum size for the operation to take place.
- *      (5) Use NULL for @pboxa to avoid returning the boxa.
+ *      (5) Use NULL for %pboxa to avoid returning the boxa.
+ * </pre>
  */
 PIX *
 pixMorphSequenceByRegion(PIX         *pixs,
@@ -340,6 +359,7 @@ PIXA    *pixam, *pixad;
 
     PROCNAME("pixMorphSequenceByRegion");
 
+    if (pboxa) *pboxa = NULL;
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (!pixm)
@@ -381,16 +401,17 @@ PIXA    *pixam, *pixad;
 
 
 /*!
- *  pixaMorphSequenceByRegion()
+ * \brief   pixaMorphSequenceByRegion()
  *
- *      Input:  pixs (1 bpp)
- *              pixam (of 1 bpp mask elements)
- *              sequence (string specifying sequence)
- *              minw  (minimum width to consider; use 0 or 1 for any width)
- *              minh  (minimum height to consider; use 0 or 1 for any height)
- *      Return: pixad, or null on error
+ * \param[in]    pixs 1 bpp
+ * \param[in]    pixam of 1 bpp mask elements
+ * \param[in]    sequence string specifying sequence
+ * \param[in]    minw  minimum width to consider; use 0 or 1 for any width
+ * \param[in]    minh  minimum height to consider; use 0 or 1 for any height
+ * \return  pixad, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixMorphSequence() for composing operation sequences.
  *      (2) This operates separately on each region in the input pixs
  *          defined by the components in pixam.
@@ -399,6 +420,7 @@ PIXA    *pixam, *pixad;
  *          operation to take place.
  *      (4) The input pixam should have a boxa giving the locations
  *          of the regions in pixs.
+ * </pre>
  */
 PIXA *
 pixaMorphSequenceByRegion(PIX         *pixs,
@@ -467,13 +489,13 @@ PIXA    *pixad;
  *      Union and intersection of parallel composite operations    *
  *-----------------------------------------------------------------*/
 /*!
- *  pixUnionOfMorphOps()
+ * \brief   pixUnionOfMorphOps()
  *
- *      Input:  pixs (binary)
- *              sela
- *              type (L_MORPH_DILATE, etc.)
- *      Return: pixd (union of the specified morphological operation
- *                    on pixs for each Sel in the Sela), or null on error
+ * \param[in]    pixs binary
+ * \param[in]    sela
+ * \param[in]    type L_MORPH_DILATE, etc.
+ * \return  pixd union of the specified morphological operation
+ *                    on pixs for each Sel in the Sela, or NULL on error
  */
 PIX *
 pixUnionOfMorphOps(PIX     *pixs,
@@ -520,13 +542,13 @@ SEL     *sel;
 
 
 /*!
- *  pixIntersectionOfMorphOps()
+ * \brief   pixIntersectionOfMorphOps()
  *
- *      Input:  pixs (binary)
- *              sela
- *              type (L_MORPH_DILATE, etc.)
- *      Return: pixd (intersection of the specified morphological operation
- *                    on pixs for each Sel in the Sela), or null on error
+ * \param[in]    pixs binary
+ * \param[in]    sela
+ * \param[in]    type L_MORPH_DILATE, etc.
+ * \return  pixd intersection of the specified morphological operation
+ *                    on pixs for each Sel in the Sela, or NULL on error
  */
 PIX *
 pixIntersectionOfMorphOps(PIX     *pixs,
@@ -578,13 +600,13 @@ SEL     *sel;
  *             Selective connected component filling               *
  *-----------------------------------------------------------------*/
 /*!
- *  pixSelectiveConnCompFill()
+ * \brief   pixSelectiveConnCompFill()
  *
- *      Input:  pixs (binary)
- *              connectivity (4 or 8)
- *              minw  (minimum width to consider; use 0 or 1 for any width)
- *              minh  (minimum height to consider; use 0 or 1 for any height)
- *      Return: pix (with holes filled in selected c.c.), or null on error
+ * \param[in]    pixs binary
+ * \param[in]    connectivity 4 or 8
+ * \param[in]    minw  minimum width to consider; use 0 or 1 for any width
+ * \param[in]    minh  minimum height to consider; use 0 or 1 for any height
+ * \return  pix with holes filled in selected c.c., or NULL on error
  */
 PIX *
 pixSelectiveConnCompFill(PIX     *pixs,
@@ -635,18 +657,19 @@ PIXA    *pixa;
  *                    Removal of matched patterns                  *
  *-----------------------------------------------------------------*/
 /*!
- *  pixRemoveMatchedPattern()
+ * \brief   pixRemoveMatchedPattern()
  *
- *      Input:  pixs (input image, 1 bpp)
- *              pixp (pattern to be removed from image, 1 bpp)
- *              pixe (image after erosion by Sel that approximates pixp, 1 bpp)
- *              x0, y0 (center of Sel)
- *              dsize (number of pixels on each side by which pixp is
+ * \param[in]    pixs input image, 1 bpp
+ * \param[in]    pixp pattern to be removed from image, 1 bpp
+ * \param[in]    pixe image after erosion by Sel that approximates pixp, 1 bpp
+ * \param[in]    x0, y0 center of Sel
+ * \param[in]    dsize number of pixels on each side by which pixp is
  *                     dilated before being subtracted from pixs;
- *                     valid values are {0, 1, 2, 3, 4})
- *      Return: 0 if OK, 1 on error
+ *                     valid values are {0, 1, 2, 3, 4}
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *    (1) This is in-place.
  *    (2) You can use various functions in selgen to create a Sel
  *        that is used to generate pixe from pixs.
@@ -654,6 +677,7 @@ PIXA    *pixa;
  *        It finds the centroid of each c.c., and subtracts
  *        (the appropriately dilated version of) pixp, with the center
  *        of the Sel used to align pixp with pixs.
+ * </pre>
  */
 l_int32
 pixRemoveMatchedPattern(PIX     *pixs,
@@ -735,20 +759,21 @@ SEL     *sel;
  *                    Display of matched patterns                  *
  *-----------------------------------------------------------------*/
 /*!
- *  pixDisplayMatchedPattern()
+ * \brief   pixDisplayMatchedPattern()
  *
- *      Input:  pixs (input image, 1 bpp)
- *              pixp (pattern to be removed from image, 1 bpp)
- *              pixe (image after erosion by Sel that approximates pixp, 1 bpp)
- *              x0, y0 (center of Sel)
- *              color (to paint the matched patterns; 0xrrggbb00)
- *              scale (reduction factor for output pixd)
- *              nlevels (if scale < 1.0, threshold to this number of levels)
- *      Return: pixd (8 bpp, colormapped), or null on error
+ * \param[in]    pixs input image, 1 bpp
+ * \param[in]    pixp pattern to be removed from image, 1 bpp
+ * \param[in]    pixe image after erosion by Sel that approximates pixp, 1 bpp
+ * \param[in]    x0, y0 center of Sel
+ * \param[in]    color to paint the matched patterns; 0xrrggbb00
+ * \param[in]    scale reduction factor for output pixd
+ * \param[in]    nlevels if scale < 1.0, threshold to this number of levels
+ * \return  pixd 8 bpp, colormapped, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *    (1) A 4 bpp colormapped image is generated.
- *    (2) If scale <= 1.0, do scale to gray for the output, and threshold
+ *    (2) If scale \<= 1.0, do scale to gray for the output, and threshold
  *        to nlevels of gray.
  *    (3) You can use various functions in selgen to create a Sel
  *        that will generate pixe from pixs.
@@ -757,6 +782,7 @@ SEL     *sel;
  *        pixels using pixp (appropriately aligned) as a stencil.
  *        Alignment is done using the origin of the Sel and the
  *        centroid of the eroded image to place the stencil pixp.
+ * </pre>
  */
 PIX *
 pixDisplayMatchedPattern(PIX       *pixs,
@@ -845,24 +871,104 @@ PIXCMAP  *cmap;
 }
 
 
+/*-----------------------------------------------------------------*
+ *       Extension of pixa by iterative erosion or dilation        *
+ *-----------------------------------------------------------------*/
+/*!
+ * \brief   pixaExtendIterative()
+ *
+ * \param[in]    pixas
+ * \param[in]    type L_MORPH_DILATE, L_MORPH_ERODE
+ * \param[in]    niters
+ * \param[in]    sel used for dilation, erosion; uses 2x2 if null
+ * \param[in]    include 1 to include a copy of the input pixas in pixad;
+ *                       0 to omit
+ * \return  pixad of derived pix, using all iterations, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *    (1) This dilates or erodes every pix in %pixas, iteratively,
+ *        using the input Sel (or, if null, a 2x2 Sel by default),
+ *        and puts the results in %pixad.
+ *    (2) If %niters \<= 0, this is a no-op; it returns a clone of pixas.
+ *    (3) If %include == 1, the output %pixad contains all the pix
+ *        in %pixas.  Otherwise, it doesn't, but pixaJoin() can be
+ *        used later to join pixas with pixad.
+ * </pre>
+ */
+PIXA *
+pixaExtendIterative(PIXA    *pixas,
+                    l_int32  type,
+                    l_int32  niters,
+                    SEL     *sel,
+                    l_int32  include)
+{
+l_int32  maxdepth, i, j, n;
+PIX     *pix0, *pix1, *pix2;
+SEL     *selt;
+PIXA    *pixad;
+
+    PROCNAME("pixaExtendIterative");
+
+    if (!pixas)
+        return (PIXA *)ERROR_PTR("pixas undefined", procName, NULL);
+    if (niters <= 0) {
+        L_INFO("niters = %d; nothing to do\n", procName, niters);
+        return pixaCopy(pixas, L_CLONE);
+    }
+    if (type != L_MORPH_DILATE && type != L_MORPH_ERODE)
+        return (PIXA *)ERROR_PTR("invalid type", procName, NULL);
+    pixaGetDepthInfo(pixas, &maxdepth, NULL);
+    if (maxdepth > 1)
+        return (PIXA *)ERROR_PTR("some pix have bpp > 1", procName, NULL);
+
+    if (!sel)
+        selt = selCreateBrick(2, 2, 0, 0, SEL_HIT);  /* default */
+    else
+        selt = selCopy(sel);
+    n = pixaGetCount(pixas);
+    pixad = pixaCreate(n * niters);
+    for (i = 0; i < n; i++) {
+        pix1 = pixaGetPix(pixas, i, L_CLONE);
+        if (include) pixaAddPix(pixad, pix1, L_COPY);
+        pix0 = pix1;  /* need to keep the handle to destroy the clone */
+        for (j = 0; j < niters; j++) {
+            if (type == L_MORPH_DILATE) {
+                pix2 = pixDilate(NULL, pix1, selt);
+            } else {  /* L_MORPH_ERODE */
+                pix2 = pixErode(NULL, pix1, selt);
+            }
+            pixaAddPix(pixad, pix2, L_INSERT);
+            pix1 = pix2;  /* owned by pixad; do not destroy */
+        }
+        pixDestroy(&pix0);
+    }
+
+    selDestroy(&selt);
+    return pixad;
+}
+
+
 /*-----------------------------------------------------------------*
  *             Iterative morphological seed filling                *
  *-----------------------------------------------------------------*/
 /*!
- *  pixSeedfillMorph()
+ * \brief   pixSeedfillMorph()
  *
- *      Input:  pixs (seed)
- *              pixm (mask)
- *              maxiters (use 0 to go to completion)
- *              connectivity (4 or 8)
- *      Return: pixd (after filling into the mask) or null on error
+ * \param[in]    pixs seed
+ * \param[in]    pixm mask
+ * \param[in]    maxiters use 0 to go to completion
+ * \param[in]    connectivity 4 or 8
+ * \return  pixd after filling into the mask or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *    (1) This is in general a very inefficient method for filling
  *        from a seed into a mask.  Use it for a small number of iterations,
  *        but if you expect more than a few iterations, use
  *        pixSeedfillBinary().
  *    (2) We use a 3x3 brick SEL for 8-cc filling and a 3x3 plus SEL for 4-cc.
+ * </pre>
  */
 PIX *
 pixSeedfillMorph(PIX     *pixs,
@@ -886,7 +992,7 @@ SEL     *sel_3;
     if (pixSizesEqual(pixs, pixm) == 0)
         return (PIX *)ERROR_PTR("pix sizes unequal", procName, NULL);
 
-    if ((sel_3 = selCreateBrick(3, 3, 1, 1, 1)) == NULL)
+    if ((sel_3 = selCreateBrick(3, 3, 1, 1, SEL_HIT)) == NULL)
         return (PIX *)ERROR_PTR("sel_3 not made", procName, NULL);
     if (connectivity == 4) {  /* remove corner hits to make a '+' */
         selSetElement(sel_3, 0, 0, SEL_DONT_CARE);
@@ -918,13 +1024,13 @@ SEL     *sel_3;
  *                   Granulometry on binary images                 *
  *-----------------------------------------------------------------*/
 /*!
- *  pixRunHistogramMorph()
+ * \brief   pixRunHistogramMorph()
  *
- *      Input:  pixs
- *              runtype (L_RUN_OFF, L_RUN_ON)
- *              direction (L_HORIZ, L_VERT)
- *              maxsize  (size of largest runlength counted)
- *      Return: numa of run-lengths
+ * \param[in]    pixs
+ * \param[in]    runtype L_RUN_OFF, L_RUN_ON
+ * \param[in]    direction L_HORIZ, L_VERT
+ * \param[in]    maxsize  size of largest runlength counted
+ * \return  numa of run-lengths
  */
 NUMA *
 pixRunHistogramMorph(PIX     *pixs,
@@ -951,15 +1057,17 @@ SEL       *sel_2a;
         return (NUMA *)ERROR_PTR("pixs must be binary", procName, NULL);
 
     if (direction == L_HORIZ)
-        sel_2a = selCreateBrick(1, 2, 0, 0, 1);
+        sel_2a = selCreateBrick(1, 2, 0, 0, SEL_HIT);
     else   /* direction == L_VERT */
-        sel_2a = selCreateBrick(2, 1, 0, 0, 1);
+        sel_2a = selCreateBrick(2, 1, 0, 0, SEL_HIT);
     if (!sel_2a)
         return (NUMA *)ERROR_PTR("sel_2a not made", procName, NULL);
 
     if (runtype == L_RUN_OFF) {
-        if ((pix1 = pixCopy(NULL, pixs)) == NULL)
+        if ((pix1 = pixCopy(NULL, pixs)) == NULL) {
+            selDestroy(&sel_2a);
             return (NUMA *)ERROR_PTR("pix1 not made", procName, NULL);
+        }
         pixInvert(pix1, pix1);
     } else {  /* runtype == L_RUN_ON */
         pix1 = pixClone(pixs);
@@ -997,7 +1105,6 @@ SEL       *sel_2a;
     pixDestroy(&pix3);
     selDestroy(&sel_2a);
     numaDestroy(&na);
-
     return nah;
 }
 
@@ -1006,22 +1113,24 @@ SEL       *sel_2a;
  *            Composite operations on grayscale images             *
  *-----------------------------------------------------------------*/
 /*!
- *  pixTophat()
+ * \brief   pixTophat()
  *
- *      Input:  pixs
- *              hsize (of Sel; must be odd; origin implicitly in center)
- *              vsize (ditto)
- *              type   (L_TOPHAT_WHITE: image - opening
- *                      L_TOPHAT_BLACK: closing - image)
- *      Return: pixd, or null on error
+ * \param[in]    pixs
+ * \param[in]    hsize of Sel; must be odd; origin implicitly in center
+ * \param[in]    vsize ditto
+ * \param[in]    type   L_TOPHAT_WHITE: image - opening
+ *                      L_TOPHAT_BLACK: closing - image
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Sel is a brick with all elements being hits
  *      (2) If hsize = vsize = 1, returns an image with all 0 data.
  *      (3) The L_TOPHAT_WHITE flag emphasizes small bright regions,
  *          whereas the L_TOPHAT_BLACK flag emphasizes small dark regions.
  *          The L_TOPHAT_WHITE tophat can be accomplished by doing a
  *          L_TOPHAT_BLACK tophat on the inverse, or v.v.
+ * </pre>
  */
 PIX *
 pixTophat(PIX     *pixs,
@@ -1076,14 +1185,15 @@ PIX  *pixt, *pixd;
 
 
 /*!
- *  pixHDome()
+ * \brief   pixHDome()
  *
- *      Input:  pixs (8 bpp, filling mask)
- *              height (of seed below the filling maskhdome; must be >= 0)
- *              connectivity (4 or 8)
- *      Return: pixd (8 bpp), or null on error
+ * \param[in]    pixs 8 bpp, filling mask
+ * \param[in]    height of seed below the filling maskhdome; must be >= 0
+ * \param[in]    connectivity 4 or 8
+ * \return  pixd 8 bpp, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) It is more efficient to use a connectivity of 4 for the fill.
  *      (2) This fills bumps to some level, and extracts the unfilled
  *          part of the bump.  To extract the troughs of basins, first
@@ -1111,15 +1221,16 @@ PIX  *pixt, *pixd;
  *            (b) Fill in pixd starting with this seed, clipping by pixs,
  *                in the way described in seedfillGrayLow().  The filling
  *                stops before the peaks in pixs are filled.
- *                For peaks that have a height > p, pixd is filled to
+ *                For peaks that have a height \> p, pixd is filled to
  *                the level equal to the (top-of-the-peak - p).
- *                For peaks of height < p, the peak is left unfilled
+ *                For peaks of height \< p, the peak is left unfilled
  *                from its highest saddle point (the leak to the outside).
  *            (c) Subtract the filled seed (pixd) from the filling mask (pixs).
  *          Note that in this procedure, everything is done starting
  *          with the filling mask, pixs.
  *      (6) For segmentation, the resulting image, pixd, can be thresholded
  *          and used as a seed for another filling operation.
+ * </pre>
  */
 PIX *
 pixHDome(PIX     *pixs,
@@ -1150,16 +1261,17 @@ PIX  *pixsd, *pixd;
 
 
 /*!
- *  pixFastTophat()
+ * \brief   pixFastTophat()
  *
- *      Input:  pixs
- *              xsize (width of max/min op, smoothing; any integer >= 1)
- *              ysize (height of max/min op, smoothing; any integer >= 1)
- *              type   (L_TOPHAT_WHITE: image - min
- *                      L_TOPHAT_BLACK: max - image)
- *      Return: pixd, or null on error
+ * \param[in]    pixs
+ * \param[in]    xsize width of max/min op, smoothing; any integer >= 1
+ * \param[in]    ysize height of max/min op, smoothing; any integer >= 1
+ * \param[in]    type   L_TOPHAT_WHITE: image - min
+ *                      L_TOPHAT_BLACK: max - image
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Don't be fooled. This is NOT a tophat.  It is a tophat-like
  *          operation, where the result is similar to what you'd get
  *          if you used an erosion instead of an opening, or a dilation
@@ -1174,6 +1286,7 @@ PIX  *pixsd, *pixd;
  *          If you need the preciseness of the tophat, don't use this.
  *      (4) The L_TOPHAT_WHITE flag emphasizes small bright regions,
  *          whereas the L_TOPHAT_BLACK flag emphasizes small dark regions.
+ * </pre>
  */
 PIX *
 pixFastTophat(PIX     *pixs,
@@ -1228,14 +1341,14 @@ PIX  *pix1, *pix2, *pix3, *pixd;
 
 
 /*!
- *  pixMorphGradient()
+ * \brief   pixMorphGradient()
  *
- *      Input:  pixs
- *              hsize (of Sel; must be odd; origin implicitly in center)
- *              vsize (ditto)
- *              smoothing  (half-width of convolution smoothing filter.
- *                          The width is (2 * smoothing + 1), so 0 is no-op.
- *      Return: pixd, or null on error
+ * \param[in]    pixs
+ * \param[in]    hsize of Sel; must be odd; origin implicitly in center
+ * \param[in]    vsize ditto
+ * \param[in]    smoothing  half-width of convolution smoothing filter.
+ *                          The width is (2 * smoothing + 1, so 0 is no-op.
+ * \return  pixd, or NULL on error
  */
 PIX *
 pixMorphGradient(PIX     *pixs,
@@ -1278,16 +1391,18 @@ PIX  *pixg, *pixd;
  *                       Centroid of component                     *
  *-----------------------------------------------------------------*/
 /*!
- *  pixaCentroids()
+ * \brief   pixaCentroids()
  *
- *      Input:  pixa of components (1 or 8 bpp)
- *      Return: pta of centroids relative to the UL corner of
- *              each pix, or null on error
+ * \param[in]    pixa of components 1 or 8 bpp
+ * \return  pta of centroids relative to the UL corner of
+ *              each pix, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) An error message is returned if any pix has something other
  *          than 1 bpp or 8 bpp depth, and the centroid from that pix
  *          is saved as (0, 0).
+ * </pre>
  */
 PTA *
 pixaCentroids(PIXA  *pixa)
@@ -1319,25 +1434,27 @@ PTA       *pta;
         ptaAddPt(pta, x, y);
     }
 
-    FREE(centtab);
-    FREE(sumtab);
+    LEPT_FREE(centtab);
+    LEPT_FREE(sumtab);
     return pta;
 }
 
 
 /*!
- *  pixCentroid()
+ * \brief   pixCentroid()
  *
- *      Input:  pix (1 or 8 bpp)
- *              centtab (<optional> table for finding centroids; can be null)
- *              sumtab (<optional> table for finding pixel sums; can be null)
- *              &xave, &yave (<return> coordinates of centroid, relative to
- *                            the UL corner of the pix)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix 1 or 8 bpp
+ * \param[in]    centtab [optional] table for finding centroids; can be null
+ * \param[in]    sumtab [optional] table for finding pixel sums; can be null
+ * \param[out]   pxave, pyave coordinates of centroid, relative to
+ *                            the UL corner of the pix
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Any table not passed in will be made internally and destroyed
  *          after use.
+ * </pre>
  */
 l_int32
 pixCentroid(PIX        *pix,
@@ -1439,7 +1556,7 @@ l_int32   *ctab, *stab;
         }
     }
 
-    if (!centtab) FREE(ctab);
-    if (!sumtab) FREE(stab);
+    if (!centtab) LEPT_FREE(ctab);
+    if (!sumtab) LEPT_FREE(stab);
     return 0;
 }
