@@ -1,16 +1,27 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 /*
@@ -35,9 +46,7 @@
 #include <string.h>
 #include "allheaders.h"
 
-    /* MS VC++ can't handle array initialization with static consts ! */
-#define L_BUF_SIZE      512    /* max token size */
-
+static const l_int32  L_BUF_SIZE = 512;    /* max token size */
 
 static l_int32 getNextNonCommentLine(SARRAY *sa, l_int32 start, l_int32 *pnext);
 static l_int32 getNextNonBlankLine(SARRAY *sa, l_int32 start, l_int32 *pnext);
@@ -72,10 +81,10 @@ static l_int32 getOffsetForMatchingRP(SARRAY *sa, l_int32 start,
  *
  *  Notes:
  *      (1) We parse the output of cpp:
- *              cpp -ansi <filein> 
- *          Three plans were attempted, with success on the third. 
+ *              cpp -ansi <filein>
+ *          Three plans were attempted, with success on the third.
  *      (2) Plan 1.  A cursory examination of the cpp output indicated that
- *          every function was preceeded by a cpp comment statement.
+ *          every function was preceded by a cpp comment statement.
  *          So we just need to look at statements beginning after comments.
  *          Unfortunately, this is NOT the case.  Some functions start
  *          without cpp comment lines, typically when there are no
@@ -121,10 +130,10 @@ static l_int32 getOffsetForMatchingRP(SARRAY *sa, l_int32 start,
  *      (6) When a prototype is extracted, it is put in a canonical
  *          form (i.e., cleaned up).  Finally, we check that it is
  *          not static and save it.  (If static, it is ignored).
- *      (7) The @prestring for unix is NULL; it is included here so that
+ *      (7) The %prestring for unix is NULL; it is included here so that
  *          you can use Microsoft's declaration for importing or
  *          exporting to a dll.  See environ.h for examples of use.
- *          Here, we set: @prestring = "LEPT_DLL ".  Note in particular
+ *          Here, we set: %prestring = "LEPT_DLL ".  Note in particular
  *          the space character that will separate 'LEPT_DLL' from
  *          the standard unix prototype that follows.
  */
@@ -170,19 +179,19 @@ SARRAY  *sa, *saout, *satest;
              * 'extern' keywords we are looking for, if they exist,
              * would be the second word. */
         satest = sarrayCreateWordsFromString(str);
-        secondword = sarrayGetString(satest, 1, 0);
+        secondword = sarrayGetString(satest, 1, L_NOCOPY);
         if (strcmp(secondword, "static") &&  /* not static */
             strcmp(secondword, "extern")) {  /* not extern */
             if (prestring) {  /* prepend it to the prototype */
                 newstr = stringJoin(prestring, str);
                 sarrayAddString(saout, newstr, L_INSERT);
-                FREE(str);
-            }
-            else
+                LEPT_FREE(str);
+            } else {
                 sarrayAddString(saout, str, L_INSERT);
+            }
+        } else {
+            LEPT_FREE(str);
         }
-        else
-            FREE(str);
         sarrayDestroy(&satest);
 
         skipToEndOfFunction(sa, stop, charindex, &next);
@@ -191,7 +200,7 @@ SARRAY  *sa, *saout, *satest;
 
         /* Flatten into a string with newlines between prototypes */
     parsestr = sarrayToString(saout, 1);
-    FREE(strdata);
+    LEPT_FREE(strdata);
     sarrayDestroy(&sa);
     sarrayDestroy(&saout);
 
@@ -199,7 +208,7 @@ SARRAY  *sa, *saout, *satest;
 }
 
 
-/* 
+/*
  *  getNextNonCommentLine()
  *
  *      Input:  sa (output from cpp, by line)
@@ -232,7 +241,7 @@ l_int32  i, n;
 
     n = sarrayGetCount(sa);
     for (i = start; i < n; i++) {
-        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+        if ((str = sarrayGetString(sa, i, L_NOCOPY)) == NULL)
             return ERROR_INT("str not returned; shouldn't happen", procName, 1);
         if (str[0] != '#') {
             *pnext = i;
@@ -244,7 +253,7 @@ l_int32  i, n;
 }
 
 
-/* 
+/*
  *  getNextNonBlankLine()
  *
  *      Input:  sa (output from cpp, by line)
@@ -278,7 +287,7 @@ l_int32  i, j, n, len;
 
     n = sarrayGetCount(sa);
     for (i = start; i < n; i++) {
-        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+        if ((str = sarrayGetString(sa, i, L_NOCOPY)) == NULL)
             return ERROR_INT("str not returned; shouldn't happen", procName, 1);
         len = strlen(str);
         for (j = 0; j < len; j++) {
@@ -294,7 +303,7 @@ l_int32  i, j, n, len;
 }
 
 
-/* 
+/*
  *  getNextNonDoubleSlashLine()
  *
  *      Input:  sa (output from cpp, by line)
@@ -328,7 +337,7 @@ l_int32  i, n, len;
 
     n = sarrayGetCount(sa);
     for (i = start; i < n; i++) {
-        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+        if ((str = sarrayGetString(sa, i, L_NOCOPY)) == NULL)
             return ERROR_INT("str not returned; shouldn't happen", procName, 1);
         len = strlen(str);
         if (len < 2 || str[0] != '/' || str[1] != '/') {
@@ -446,7 +455,7 @@ l_int32  toffsetlp, toffsetrp, toffsetlb, toffsetsc;
 
             /* Check if a left brace occurs before a left parenthesis;
              * if so, skip it */
-        if (toffsetlb < toffsetlp) {  
+        if (toffsetlb < toffsetlp) {
             skipToMatchingBrace(sa, next + soffsetlb, boffsetlb,
                 &rbline, &rbindex);
             skipToSemicolon(sa, rbline, rbindex, &scline);
@@ -457,7 +466,7 @@ l_int32  toffsetlp, toffsetrp, toffsetlb, toffsetsc;
             /* Check if a semicolon occurs before a left brace or
              * a left parenthesis; if so, skip it */
         if ((soffsetsc != -1) &&
-            (toffsetsc < toffsetlb || toffsetsc < toffsetlp)) {  
+            (toffsetsc < toffsetlb || toffsetsc < toffsetlp)) {
             skipToSemicolon(sa, next, 0, &scline);
             begin = scline + 1;
             continue;
@@ -508,18 +517,18 @@ l_int32  i;
 
     sap = sarrayCreate(0);
     for (i = start; i < stop; i++) {
-        str = sarrayGetString(sa, i, 1);
-        sarrayAddString(sap, str, 0);
+        str = sarrayGetString(sa, i, L_COPY);
+        sarrayAddString(sap, str, L_INSERT);
     }
-    str = sarrayGetString(sa, stop, 1);
+    str = sarrayGetString(sa, stop, L_COPY);
     str[charindex + 1] = '\0';
     newstr = stringJoin(str, ";");
-    sarrayAddString(sap, newstr, 0);
-    FREE(str);
+    sarrayAddString(sap, newstr, L_INSERT);
+    LEPT_FREE(str);
     protostr = sarrayToString(sap, 2);
     sarrayDestroy(&sap);
     cleanstr = cleanProtoSignature(protostr);
-    FREE(protostr);
+    LEPT_FREE(protostr);
 
     return cleanstr;
 }
@@ -552,9 +561,9 @@ SARRAY  *sa, *saout;
     sa = sarrayCreateWordsFromString(instr);
     nwords = sarrayGetCount(sa);
     saout = sarrayCreate(0);
-    sarrayAddString(saout, externstring, 1);
+    sarrayAddString(saout, externstring, L_COPY);
     for (i = 0; i < nwords; i++) {
-        str = sarrayGetString(sa, i, 0);
+        str = sarrayGetString(sa, i, L_NOCOPY);
         nchars = strlen(str);
         index = 0;
         for (j = 0; j < nchars; j++) {
@@ -564,16 +573,15 @@ SARRAY  *sa, *saout;
                 buf[index++] = ' ';
                 buf[index++] = '(';
                 buf[index++] = ' ';
-            }
-            else if (str[j] == ')') {
+            } else if (str[j] == ')') {
                 buf[index++] = ' ';
                 buf[index++] = ')';
-            }
-            else 
+            } else {
                 buf[index++] = str[j];
+            }
         }
         buf[index] = '\0';
-        sarrayAddString(saout, buf, 1);
+        sarrayAddString(saout, buf, L_COPY);
     }
 
         /* Flatten to a prototype string with spaces added after
@@ -665,7 +673,7 @@ l_int32  i, j, jstart, n, sumbrace, found, instring, nchars;
     sumbrace = 1;
     found = FALSE;
     for (i = start; i < n; i++) {
-        str = sarrayGetString(sa, i, 0);
+        str = sarrayGetString(sa, i, L_NOCOPY);
         jstart = 0;
         if (i == start)
             jstart = lbindex + 1;
@@ -679,9 +687,9 @@ l_int32  i, j, jstart, n, sumbrace, found, instring, nchars;
                 instring = 1 - instring;
                 /* Record the braces if they are neither a literal character
                  * nor within a string. */
-            if (str[j] == '{' && str[j+1] != '\'' && !instring)
+            if (str[j] == '{' && str[j+1] != '\'' && !instring) {
                 sumbrace++;
-            else if (str[j] == '}' && str[j+1] != '\'' && !instring) {
+            } else if (str[j] == '}' && str[j+1] != '\'' && !instring) {
                 sumbrace--;
                 if (sumbrace == 0) {
                     found = TRUE;
@@ -735,7 +743,7 @@ l_int32  i, j, n, jstart, nchars, found;
     n = sarrayGetCount(sa);
     found = FALSE;
     for (i = start; i < n; i++) {
-        str = sarrayGetString(sa, i, 0);
+        str = sarrayGetString(sa, i, L_NOCOPY);
         jstart = 0;
         if (i == start)
             jstart = charindex + 1;
@@ -806,10 +814,10 @@ l_int32  i, j, n, nchars, totchars, found;
     *ptoffset = 100000000;
 
     n = sarrayGetCount(sa);
-    found = FALSE; 
+    found = FALSE;
     totchars = 0;
     for (i = start; i < n; i++) {
-        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+        if ((str = sarrayGetString(sa, i, L_NOCOPY)) == NULL)
             return ERROR_INT("str not returned; shouldn't happen", procName, 1);
         nchars = strlen(str);
         for (j = 0; j < nchars; j++) {
@@ -895,12 +903,12 @@ l_int32  i, j, n, nchars, totchars, leftmatch, firstline, jstart, found;
     *ptoffset = 100000000;
 
     n = sarrayGetCount(sa);
-    found = FALSE; 
+    found = FALSE;
     totchars = toffsetlp;
     leftmatch = 1;  /* count of (LP - RP); we're finished when it goes to 0. */
     firstline = start + soffsetlp;
     for (i = firstline; i < n; i++) {
-        if ((str = sarrayGetString(sa, i, 0)) == NULL)
+        if ((str = sarrayGetString(sa, i, L_NOCOPY)) == NULL)
             return ERROR_INT("str not returned; shouldn't happen", procName, 1);
         nchars = strlen(str);
         jstart = 0;
@@ -932,4 +940,3 @@ l_int32  i, j, n, nchars, totchars, leftmatch, firstline, jstart, found;
 
     return 0;
 }
-
