@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  pix1.c
+/*!
+ * \file pix1.c
+ * <pre>
  *
  *    The pixN.c {N = 1,2,3,4,5} files are sorted by the type of operation.
  *    The primary functions in these files are:
@@ -63,6 +75,9 @@
  *          l_int32       pixGetDimensions()
  *          l_int32       pixSetDimensions()
  *          l_int32       pixCopyDimensions()
+ *          l_int32       pixGetSpp()
+ *          l_int32       pixSetSpp()
+ *          l_int32       pixCopySpp()
  *          l_int32       pixGetWpl()
  *          l_int32       pixSetWpl()
  *          l_int32       pixGetRefcount()
@@ -78,6 +93,7 @@
  *          l_int32       pixGetInputFormat()
  *          l_int32       pixSetInputFormat()
  *          l_int32       pixCopyInputFormat()
+ *          l_int32       pixSetSpecial()
  *          char         *pixGetText()
  *          l_int32       pixSetText()
  *          l_int32       pixAddText()
@@ -146,7 +162,7 @@
  *      But if the refcount of pixs is greater than 1, it just copies
  *      the data and decrements the ref count.
  *
- *  (3) Use pixSwapAndDestroy(pixd, &pixs) to replace pixs by an
+ *  (3) Use pixSwapAndDestroy(pixd, \&pixs) to replace pixs by an
  *      existing pixd.  This is similar to pixTransferAllData(), but
  *      simpler, in that it never makes any copies and if pixs is
  *      cloned, the other references are not changed by this operation.
@@ -156,7 +172,7 @@
  *      to convert from a pix to some other data structure with minimal
  *      heap allocation.  After the data is extracated, the pixels can
  *      be munged and used in another context.  However, the danger
- *      here is that the pix might have a refcount > 1, in which case
+ *      here is that the pix might have a refcount \> 1, in which case
  *      a copy of the data must be made and the input pix left unchanged.
  *      If there are no clones, the image data can be extracted without
  *      a copy, and the data ptr in the pix must be nulled before
@@ -170,10 +186,9 @@
  *  on the pix data field, look carefully at the behavior of the image
  *  data accessors and keep in mind that when you invoke pixDestroy(),
  *  the pix considers itself the owner of all its heap data.
+ * </pre>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include "allheaders.h"
 
@@ -191,12 +206,22 @@ static void pixFree(PIX *pix);
  *  The default functions are malloc and free.  Use setPixMemoryManager()  *
  *  to specify other functions to use.                                     *
  *-------------------------------------------------------------------------*/
+
+/*! Pix memory manager */
+    /*
+     * <pre>
+     * Notes:
+     *      (1) The allocator and deallocator function types,
+     *          alloc_fn and dealloc_fn, are defined in pix.h.
+     * </pre>
+     */
 struct PixMemoryManager
 {
-    void     *(*allocator)(size_t);
-    void      (*deallocator)(void *);
+    alloc_fn    allocator;
+    dealloc_fn  deallocator;
 };
 
+/*! Default Pix memory manager */
 static struct PixMemoryManager  pix_mem_manager = {
     &malloc,
     &free
@@ -229,47 +254,49 @@ pix_free(void  *ptr)
 }
 
 /*!
- *  setPixMemoryManager()
+ * \brief   setPixMemoryManager()
  *
- *      Input: allocator (<optional>; use null to skip)
- *             deallocator (<optional>; use null to skip)
- *      Return: void
+ * \param[in]   allocator [optional]; use NULL to skip
+ * \param[in]   deallocator [optional]; use NULL to skip
+ * \return  void
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Use this to change the alloc and/or dealloc functions;
  *          e.g., setPixMemoryManager(my_malloc, my_free).
+ *      (2) The C99 standard (section 6.7.5.3, par. 8) says:
+ *            A declaration of a parameter as "function returning type"
+ *            shall be adjusted to "pointer to function returning type"
+ *          so that it can be in either of these two forms:
+ *            (a) type (function-ptr(type, ...))
+ *            (b) type ((*function-ptr)(type, ...))
+ *          because form (a) is implictly converted to form (b), as in the
+ *          definition of struct PixMemoryManager above.  So, for example,
+ *          we should be able to declare either of these:
+ *            (a) void *(allocator(size_t))
+ *            (b) void *((*allocator)(size_t))
+ *          However, MSVC++ only accepts the second version.
+ * </pre>
  */
-#ifndef _MSC_VER
 void
-setPixMemoryManager(void  *(allocator(size_t)),
-                    void  (deallocator(void *)))
+setPixMemoryManager(alloc_fn   allocator,
+                    dealloc_fn deallocator)
 {
     if (allocator) pix_mem_manager.allocator = allocator;
     if (deallocator) pix_mem_manager.deallocator = deallocator;
     return;
 }
-#else  /* _MSC_VER */
-    /* MSVC++ wants type (*fun)(types...) syntax */
-void
-setPixMemoryManager(void  *((*allocator)(size_t)),
-                    void  ((*deallocator)(void *)))
-{
-    if (allocator) pix_mem_manager.allocator = allocator;
-    if (deallocator) pix_mem_manager.deallocator = deallocator;
-    return;
-}
-#endif /* _MSC_VER */
 
 
 /*--------------------------------------------------------------------*
  *                              Pix Creation                          *
  *--------------------------------------------------------------------*/
 /*!
- *  pixCreate()
+ * \brief   pixCreate()
  *
- *      Input:  width, height, depth
- *      Return: pixd (with data allocated and initialized to 0),
- *                    or null on error
+ * \param[in]    width, height, depth
+ * \return  pixd with data allocated and initialized to 0,
+ *                    or NULL on error
  */
 PIX *
 pixCreate(l_int32  width,
@@ -288,15 +315,17 @@ PIX  *pixd;
 
 
 /*!
- *  pixCreateNoInit()
+ * \brief   pixCreateNoInit()
  *
- *      Input:  width, height, depth
- *      Return: pixd (with data allocated but not initialized),
- *                    or null on error
+ * \param[in]    width, height, depth
+ * \return  pixd with data allocated but not initialized,
+ *                    or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Must set pad bits to avoid reading unitialized data, because
  *          some optimized routines (e.g., pixConnComp()) read from pad bits.
+ * </pre>
  */
 PIX *
 pixCreateNoInit(l_int32  width,
@@ -311,7 +340,7 @@ l_uint32  *data;
     if ((pixd = pixCreateHeader(width, height, depth)) == NULL)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
     wpl = pixGetWpl(pixd);
-    if ((data = (l_uint32 *)pix_malloc(4 * wpl * height)) == NULL) {
+    if ((data = (l_uint32 *)pix_malloc(4LL * wpl * height)) == NULL) {
         pixDestroy(&pixd);
         return (PIX *)ERROR_PTR("pix_malloc fail for data", procName, NULL);
     }
@@ -322,15 +351,17 @@ l_uint32  *data;
 
 
 /*!
- *  pixCreateTemplate()
+ * \brief   pixCreateTemplate()
  *
- *      Input:  pixs
- *      Return: pixd, or null on error
+ * \param[in]    pixs
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Makes a Pix of the same size as the input Pix, with the
  *          data array allocated and initialized to 0.
  *      (2) Copies the other fields, including colormap if it exists.
+ * </pre>
  */
 PIX *
 pixCreateTemplate(PIX  *pixs)
@@ -350,15 +381,17 @@ PIX  *pixd;
 
 
 /*!
- *  pixCreateTemplateNoInit()
+ * \brief   pixCreateTemplateNoInit()
  *
- *      Input:  pixs
- *      Return: pixd, or null on error
+ * \param[in]    pixs
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Makes a Pix of the same size as the input Pix, with
  *          the data array allocated but not initialized to 0.
  *      (2) Copies the other fields, including colormap if it exists.
+ * </pre>
  */
 PIX *
 pixCreateTemplateNoInit(PIX  *pixs)
@@ -374,6 +407,7 @@ PIX     *pixd;
     pixGetDimensions(pixs, &w, &h, &d);
     if ((pixd = pixCreateNoInit(w, h, d)) == NULL)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+    pixCopySpp(pixd, pixs);
     pixCopyResolution(pixd, pixs);
     pixCopyColormap(pixd, pixs);
     pixCopyText(pixd, pixs);
@@ -384,18 +418,32 @@ PIX     *pixd;
 
 
 /*!
- *  pixCreateHeader()
+ * \brief   pixCreateHeader()
  *
- *      Input:  width, height, depth
- *      Return: pixd (with no data allocated), or null on error
+ * \param[in]    width, height, depth
+ * \return  pixd with no data allocated, or NULL on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) It is assumed that all 32 bit pix have 3 spp.  If there is
+ *          a valid alpha channel, this will be set to 4 spp later.
+ *      (2) If the number of bytes to be allocated is larger than the
+ *          maximum value in an int32, we can get overflow, resulting
+ *          in a smaller amount of memory actually being allocated.
+ *          Later, an attempt to access memory that wasn't allocated will
+ *          cause a crash.  So to avoid crashing a program (or worse)
+ *          with bad (or malicious) input, this is where we limit the
+ *          requested allocation of image data in a typesafe way.
+ * </pre>
  */
 PIX *
 pixCreateHeader(l_int32  width,
                 l_int32  height,
                 l_int32  depth)
 {
-l_int32  wpl;
-PIX     *pixd;
+l_int32   wpl;
+l_uint64  wpl64, bignum;
+PIX      *pixd;
 
     PROCNAME("pixCreateHeader");
 
@@ -408,13 +456,33 @@ PIX     *pixd;
     if (height <= 0)
         return (PIX *)ERROR_PTR("height must be > 0", procName, NULL);
 
-    if ((pixd = (PIX *)CALLOC(1, sizeof(PIX))) == NULL)
-        return (PIX *)ERROR_PTR("CALLOC fail for pixd", procName, NULL);
+        /* Avoid overflow in malloc arg, malicious or otherwise */
+    wpl = 0;
+    wpl64 = ((l_uint64)width * (l_uint64)depth + 31) / 32;
+    if (wpl64 > ((1LL << 29) - 1)) {
+        L_ERROR("requested w = %d, h = %d, d = %d\n",
+                procName, width, height, depth);
+        return (PIX *)ERROR_PTR("wpl >= 2^29", procName, NULL);
+    } else {
+      wpl = (l_int32)wpl64;
+    }
+    bignum = 4L * wpl * height;   /* number of bytes to be requested */
+    if (bignum > ((1LL << 31) - 1)) {
+        L_ERROR("requested w = %d, h = %d, d = %d\n",
+                procName, width, height, depth);
+        return (PIX *)ERROR_PTR("requested bytes >= 2^31", procName, NULL);
+    }
+
+    if ((pixd = (PIX *)LEPT_CALLOC(1, sizeof(PIX))) == NULL)
+        return (PIX *)ERROR_PTR("LEPT_CALLOC fail for pixd", procName, NULL);
     pixSetWidth(pixd, width);
     pixSetHeight(pixd, height);
     pixSetDepth(pixd, depth);
-    wpl = (width * depth + 31) / 32;
     pixSetWpl(pixd, wpl);
+    if (depth == 24 || depth == 32)
+        pixSetSpp(pixd, 3);
+    else
+        pixSetSpp(pixd, 1);
 
     pixd->refcount = 1;
     pixd->informat = IFF_UNKNOWN;
@@ -423,12 +491,13 @@ PIX     *pixd;
 
 
 /*!
- *  pixClone()
+ * \brief   pixClone()
  *
- *      Input:  pix
- *      Return: same pix (ptr), or null on error
+ * \param[in]    pixs
+ * \return  same pix ptr, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) A "clone" is simply a handle (ptr) to an existing pix.
  *          It is implemented because (a) images can be large and
  *          hence expensive to copy, and (b) extra handles to a data
@@ -443,6 +512,7 @@ PIX     *pixd;
  *              decrements the ref count, nulls the handle, and
  *              only destroys the pix when pixDestroy() has been
  *              called on all handles.
+ * </pre>
  */
 PIX *
 pixClone(PIX  *pixs)
@@ -461,14 +531,16 @@ pixClone(PIX  *pixs)
  *                           Pix Destruction                          *
  *--------------------------------------------------------------------*/
 /*!
- *  pixDestroy()
+ * \brief   pixDestroy()
  *
- *      Input:  &pix <will be nulled>
- *      Return: void
+ * \param[in,out]   ppix will be nulled
+ * \return  void
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Decrements the ref count and, if 0, destroys the pix.
  *      (2) Always nulls the input ptr.
+ * </pre>
  */
 void
 pixDestroy(PIX  **ppix)
@@ -478,13 +550,12 @@ PIX  *pix;
     PROCNAME("pixDestroy");
 
     if (!ppix) {
-        L_WARNING("ptr address is null!", procName);
+        L_WARNING("ptr address is null!\n", procName);
         return;
     }
 
     if ((pix = *ppix) == NULL)
         return;
-
     pixFree(pix);
     *ppix = NULL;
     return;
@@ -492,13 +563,15 @@ PIX  *pix;
 
 
 /*!
- *  pixFree()
+ * \brief   pixFree()
  *
- *      Input:  pix
- *      Return: void
+ * \param[in]    pix
+ * \return  void
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Decrements the ref count and, if 0, destroys the pix.
+ * </pre>
  */
 static void
 pixFree(PIX  *pix)
@@ -513,9 +586,9 @@ char      *text;
         if ((data = pixGetData(pix)) != NULL)
             pix_free(data);
         if ((text = pixGetText(pix)) != NULL)
-            FREE(text);
+            LEPT_FREE(text);
         pixDestroyColormap(pix);
-        FREE(pix);
+        LEPT_FREE(pix);
     }
     return;
 }
@@ -525,14 +598,15 @@ char      *text;
  *                                 Pix Copy                                *
  *-------------------------------------------------------------------------*/
 /*!
- *  pixCopy()
+ * \brief   pixCopy()
  *
- *      Input:  pixd (<optional>; can be null, or equal to pixs,
- *                    or different from pixs)
- *              pixs
- *      Return: pixd, or null on error
+ * \param[in]    pixd [optional]; can be null, or equal to pixs,
+ *                    or different from pixs
+ * \param[in]    pixs
+ * \return  pixd, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) There are three cases:
  *            (a) pixd == null  (makes a new pix; refcount = 1)
  *            (b) pixd == pixs  (no-op)
@@ -551,6 +625,7 @@ char      *text;
  *          and the copy proceeds.  The refcount of pixd is unchanged.
  *      (4) This operation, like all others that may involve a pre-existing
  *          pixd, will side-effect any existing clones of pixd.
+ * </pre>
  */
 PIX *
 pixCopy(PIX  *pixd,   /* can be null */
@@ -585,6 +660,7 @@ l_uint32  *datas, *datad;
 
         /* Copy non-image data fields */
     pixCopyColormap(pixd, pixs);
+    pixCopySpp(pixd, pixs);
     pixCopyResolution(pixd, pixs);
     pixCopyInputFormat(pixd, pixs);
     pixCopyText(pixd, pixs);
@@ -598,16 +674,18 @@ l_uint32  *datas, *datad;
 
 
 /*!
- *  pixResizeImageData()
+ * \brief   pixResizeImageData()
  *
- *      Input:  pixd (gets new uninitialized buffer for image data)
- *              pixs (determines the size of the buffer; not changed)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixd gets new uninitialized buffer for image data
+ * \param[in]    pixs determines the size of the buffer; not changed
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This removes any existing image data from pixd and
  *          allocates an uninitialized buffer that will hold the
  *          amount of image data that is in pixs.
+ * </pre>
  */
 l_int32
 pixResizeImageData(PIX  *pixd,
@@ -642,14 +720,16 @@ l_uint32  *data;
 
 
 /*!
- *  pixCopyColormap()
+ * \brief   pixCopyColormap()
  *
- *      Input:  src and dest Pix
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixd, pixs dest and src Pix
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This always destroys any colormap in pixd (except if
  *          the operation is a no-op.
+ * </pre>
  */
 l_int32
 pixCopyColormap(PIX  *pixd,
@@ -679,10 +759,10 @@ PIXCMAP  *cmaps, *cmapd;
 
 
 /*!
- *  pixSizesEqual()
+ * \brief   pixSizesEqual()
  *
- *      Input:  two pix
- *      Return: 1 if the two pix have same {h, w, d}; 0 otherwise.
+ * \param[in]    pix1, pix2  two pix
+ * \return  1 if the two pix have same {h, w, d}; 0 otherwise.
  */
 l_int32
 pixSizesEqual(PIX  *pix1,
@@ -706,15 +786,16 @@ pixSizesEqual(PIX  *pix1,
 
 
 /*!
- *  pixTransferAllData()
+ * \brief   pixTransferAllData()
  *
- *      Input:  pixd (must be different from pixs)
- *              &pixs (will be nulled if refcount goes to 0)
- *              copytext (1 to copy the text field; 0 to skip)
- *              copyformat (1 to copy the informat field; 0 to skip)
- *      Return: 0 if OK, 1 on error
+ * \param[in]      pixd must be different from pixs
+ * \param[in,out]  ppixs will be nulled if refcount goes to 0
+ * \param[in]      copytext 1 to copy the text field; 0 to skip
+ * \param[in]      copyformat 1 to copy the informat field; 0 to skip
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This does a complete data transfer from pixs to pixd,
  *          followed by the destruction of pixs (refcount permitting).
  *      (2) If the refcount of pixs is 1, pixs is destroyed.  Otherwise,
@@ -730,27 +811,32 @@ pixSizesEqual(PIX  *pix1,
  *          in place of the input Pix.  There are two ways you can do this:
  *          (a) The straightforward way is to change the function
  *              signature to take the address of the Pix ptr:
+ * \code
  *                  void function-inplace(PIX **ppix, ...) {
  *                      PIX *pixt = function-makenew(*ppix);
  *                      pixDestroy(ppix);
  *                      *ppix = pixt;
  *                      return;
  *                  }
+ * \endcode
  *              Here, the input and returned pix are different, as viewed
  *              by the calling function, and the inplace function is
  *              expected to destroy the input pix to avoid a memory leak.
  *          (b) Keep the signature the same and use pixTransferAllData()
  *              to return the new Pix in the input Pix struct:
+ * \code
  *                  void function-inplace(PIX *pix, ...) {
  *                      PIX *pixt = function-makenew(pix);
  *                      pixTransferAllData(pix, &pixt, 0, 0);
  *                               // pixDestroy() is called on pixt
  *                      return;
  *                  }
+ * \endcode
  *              Here, the input and returned pix are the same, as viewed
  *              by the calling function, and the inplace function must
  *              never destroy the input pix, because the calling function
  *              maintains an unchanged handle to it.
+ * </pre>
  */
 l_int32
 pixTransferAllData(PIX     *pixd,
@@ -791,6 +877,7 @@ PIX     *pixs;
             pixCopyText(pixd, pixs);
     }
 
+    pixCopySpp(pixd, pixs);
     pixCopyResolution(pixd, pixs);
     pixCopyDimensions(pixd, pixs);
     if (copyformat)
@@ -804,14 +891,15 @@ PIX     *pixs;
 
 
 /*!
- *  pixSwapAndDestroy()
+ * \brief   pixSwapAndDestroy()
  *
- *      Input:  &pixd (<optional, return> input pixd can be null,
- *                     and it must be different from pixs)
- *              &pixs (will be nulled after the swap)
- *      Return: 0 if OK, 1 on error
+ * \param[out]     ppixd [optional] input pixd can be null,
+ *                       and it must be different from pixs
+ * \param[in,out]  ppixs will be nulled after the swap
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Simple operation to change the handle name safely.
  *          After this operation, the original image in pixd has
  *          been destroyed, pixd points to what was pixs, and
@@ -820,12 +908,15 @@ PIX     *pixs;
  *          If pixs is cloned, the other handles still point to
  *          the original image, with the ref count reduced by 1.
  *      (3) Usage example:
+ * \code
  *            Pix *pix1 = pixRead("...");
  *            Pix *pix2 = function(pix1, ...);
  *            pixSwapAndDestroy(&pix1, &pix2);
  *            pixDestroy(&pix1);  // holds what was in pix2
+ * \endcode
  *          Example with clones ([] shows ref count of image generated
  *                               by the function):
+ * \code
  *            Pix *pixs = pixRead("...");
  *            Pix *pix1 = pixClone(pixs);
  *            Pix *pix2 = function(pix1, ...);   [1]
@@ -834,6 +925,8 @@ PIX     *pixs;
  *            pixDestroy(&pixs);  // still holds read image
  *            pixDestroy(&pix1);  // holds what was in pix2  [2] --> [1]
  *            pixDestroy(&pix3);  // holds what was in pix2  [1] --> [0]
+ * \endcode
+ * </pre>
  */
 l_int32
 pixSwapAndDestroy(PIX  **ppixd,
@@ -949,11 +1042,11 @@ pixSetDepth(PIX     *pix,
 
 
 /*!
- *  pixGetDimensions()
+ * \brief   pixGetDimensions()
  *
- *      Input:  pix
- *              &w, &h, &d (<optional return>; each can be null)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix
+ * \param[out]   pw, ph, pd [optional]  each can be null
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixGetDimensions(PIX      *pix,
@@ -976,11 +1069,11 @@ pixGetDimensions(PIX      *pix,
 
 
 /*!
- *  pixSetDimensions()
+ * \brief   pixSetDimensions()
  *
- *      Input:  pix
- *              w, h, d (use 0 to skip the setting for any of these)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix
+ * \param[in]    w, h, d use 0 to skip the setting for any of these
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixSetDimensions(PIX     *pix,
@@ -1000,11 +1093,11 @@ pixSetDimensions(PIX     *pix,
 
 
 /*!
- *  pixCopyDimensions()
+ * \brief   pixCopyDimensions()
  *
- *      Input:  pixd
- *              pixd
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixd
+ * \param[in]    pixd
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixCopyDimensions(PIX  *pixd,
@@ -1027,6 +1120,72 @@ pixCopyDimensions(PIX  *pixd,
 }
 
 
+l_int32
+pixGetSpp(PIX  *pix)
+{
+    PROCNAME("pixGetSpp");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, UNDEF);
+
+    return pix->spp;
+}
+
+
+/*
+ *  pixSetSpp()
+ *      Input:  pix
+ *              spp (1, 3 or 4)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) For a 32 bpp pix, this can be used to ignore the
+ *          alpha sample (spp == 3) or to use it (spp == 4).
+ *          For example, to write a spp == 4 image without the alpha
+ *          sample (as an rgb pix), call pixSetSpp(pix, 3) and
+ *          then write it out as a png.
+ */
+l_int32
+pixSetSpp(PIX     *pix,
+          l_int32  spp)
+{
+    PROCNAME("pixSetSpp");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (spp < 1)
+        return ERROR_INT("spp must be >= 1", procName, 1);
+
+    pix->spp = spp;
+    return 0;
+}
+
+
+/*!
+ * \brief   pixCopySpp()
+ *
+ * \param[in]    pixd
+ * \param[in]    pixs
+ * \return  0 if OK, 1 on error
+ */
+l_int32
+pixCopySpp(PIX  *pixd,
+           PIX  *pixs)
+{
+    PROCNAME("pixCopySpp");
+
+    if (!pixd)
+        return ERROR_INT("pixd not defined", procName, 1);
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+    if (pixs == pixd)
+        return 0;   /* no-op */
+
+    pixSetSpp(pixd, pixGetSpp(pixs));
+    return 0;
+}
+
+
 l_int32
 pixGetWpl(PIX  *pix)
 {
@@ -1128,11 +1287,11 @@ pixSetYRes(PIX     *pix,
 
 
 /*!
- *  pixGetResolution()
+ * \brief   pixGetResolution()
  *
- *      Input:  pix
- *              &xres, &yres (<optional return>; each can be null)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix
+ * \param[out]   pxres, pyres [optional]  each can be null
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixGetResolution(PIX      *pix,
@@ -1141,6 +1300,10 @@ pixGetResolution(PIX      *pix,
 {
     PROCNAME("pixGetResolution");
 
+    if (pxres) *pxres = 0;
+    if (pyres) *pyres = 0;
+    if (!pxres && !pyres)
+        return ERROR_INT("no output requested", procName, 1);
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
     if (pxres) *pxres = pix->xres;
@@ -1150,11 +1313,11 @@ pixGetResolution(PIX      *pix,
 
 
 /*!
- *  pixSetResolution()
+ * \brief   pixSetResolution()
  *
- *      Input:  pix
- *              xres, yres (use 0 to skip the setting for either of these)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix
+ * \param[in]    xres, yres use 0 to skip the setting for either of these
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixSetResolution(PIX     *pix,
@@ -1227,7 +1390,6 @@ pixSetInputFormat(PIX     *pix,
 
     if (!pix)
         return ERROR_INT("pix not defined", procName, 1);
-
     pix->informat = informat;
     return 0;
 }
@@ -1251,15 +1413,30 @@ pixCopyInputFormat(PIX  *pixd,
 }
 
 
+l_int32
+pixSetSpecial(PIX     *pix,
+              l_int32  special)
+{
+    PROCNAME("pixSetSpecial");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    pix->special = special;
+    return 0;
+}
+
+
 /*!
- *  pixGetText()
+ * \brief   pixGetText()
  *
- *      Input:  pix
- *      Return: ptr to existing text string
+ * \param[in]    pix
+ * \return  ptr to existing text string
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The text string belongs to the pix.  The caller must
  *          NOT free it!
+ * </pre>
  */
 char *
 pixGetText(PIX  *pix)
@@ -1273,15 +1450,17 @@ pixGetText(PIX  *pix)
 
 
 /*!
- *  pixSetText()
+ * \brief   pixSetText()
  *
- *      Input:  pix
- *              textstring (can be null)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix
+ * \param[in]    textstring can be null
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This removes any existing textstring and puts a copy of
  *          the input textstring there.
+ * </pre>
  */
 l_int32
 pixSetText(PIX         *pix,
@@ -1298,16 +1477,18 @@ pixSetText(PIX         *pix,
 
 
 /*!
- *  pixAddText()
+ * \brief   pixAddText()
  *
- *      Input:  pix
- *              textstring
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix
+ * \param[in]    textstring
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This adds the new textstring to any existing text.
  *      (2) Either or both the existing text and the new text
  *          string can be null.
+ * </pre>
  */
 l_int32
 pixAddText(PIX         *pix,
@@ -1322,7 +1503,7 @@ char  *newstring;
 
     newstring = stringJoin(pixGetText(pix), textstring);
     stringReplace(&pix->text, newstring);
-    FREE(newstring);
+    LEPT_FREE(newstring);
     return 0;
 }
 
@@ -1357,17 +1538,19 @@ pixGetColormap(PIX  *pix)
 
 
 /*!
- *  pixSetColormap()
+ * \brief   pixSetColormap()
  *
- *      Input:  pix
- *              colormap (to be assigned)
- *      Return: 0 if OK, 1 on error.
+ * \param[in]    pix
+ * \param[in]    colormap to be assigned
+ * \return  0 if OK, 1 on error.
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Unlike with the pix data field, pixSetColormap() destroys
  *          any existing colormap before assigning the new one.
  *          Because colormaps are not ref counted, it is important that
  *          the new colormap does not belong to any other pix.
+ * </pre>
  */
 l_int32
 pixSetColormap(PIX      *pix,
@@ -1385,10 +1568,10 @@ pixSetColormap(PIX      *pix,
 
 
 /*!
- *  pixDestroyColormap()
+ * \brief   pixDestroyColormap()
  *
- *      Input:  pix
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pix
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixDestroyColormap(PIX  *pix)
@@ -1409,11 +1592,11 @@ PIXCMAP  *cmap;
 
 
 /*!
- *  pixGetData()
+ * \brief   pixGetData()
  *
  *  Notes:
  *      (1) This gives a new handle for the data.  The data is still
- *          owned by the pix, so do not call FREE() on it.
+ *          owned by the pix, so do not call LEPT_FREE() on it.
  */
 l_uint32 *
 pixGetData(PIX  *pix)
@@ -1427,7 +1610,7 @@ pixGetData(PIX  *pix)
 
 
 /*!
- *  pixSetData()
+ * \brief   pixSetData()
  *
  *  Notes:
  *      (1) This does not free any existing data.  To free existing
@@ -1448,7 +1631,7 @@ pixSetData(PIX       *pix,
 
 
 /*!
- *  pixExtractData()
+ * \brief   pixExtractData()
  *
  *  Notes:
  *      (1) This extracts the pix image data for use in another context.
@@ -1473,8 +1656,7 @@ l_uint32  *data, *datas;
     if (count == 1) {  /* extract */
         data = pixGetData(pixs);
         pixSetData(pixs, NULL);
-    }
-    else {  /* refcount > 1; copy */
+    } else {  /* refcount > 1; copy */
         bytes = 4 * pixGetWpl(pixs) * pixGetHeight(pixs);
         datas = pixGetData(pixs);
         if ((data = (l_uint32 *)pix_malloc(bytes)) == NULL)
@@ -1487,7 +1669,7 @@ l_uint32  *data, *datas;
 
 
 /*!
- *  pixFreeData()
+ * \brief   pixFreeData()
  *
  *  Notes:
  *      (1) This frees the data and sets the pix data ptr to null.
@@ -1517,13 +1699,14 @@ l_uint32  *data;
  *                          Pix line ptrs                             *
  *--------------------------------------------------------------------*/
 /*!
- *  pixGetLinePtrs()
+ * \brief   pixGetLinePtrs()
  *
- *      Input:  pix
- *              &size (<optional return> array size, which is the pix height)
- *      Return: array of line ptrs, or null on error
+ * \param[in]    pix
+ * \param[out]   psize [optional] array size, which is the pix height
+ * \return  array of line ptrs, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is intended to be used for fast random pixel access.
  *          For example, for an 8 bpp image,
  *              val = GET_DATA_BYTE(lines8[i], j);
@@ -1554,7 +1737,7 @@ l_uint32  *data;
  *              void **lineg8 = pixGetLinePtrs(pixg, NULL);
  *              val = GET_DATA_BYTE(lineg8[i], j);  // fast access; BYTE, 8
  *              ...
- *              FREE(lineg8);  // don't forget this
+ *              LEPT_FREE(lineg8);  // don't forget this
  *      (5) These are convenient for accessing bytes sequentially in an
  *          8 bpp grayscale image.  People who write image processing code
  *          on 8 bpp images are accustomed to grabbing pixels directly out
@@ -1572,7 +1755,7 @@ l_uint32  *data;
  *                  }
  *              }
  *              pixEndianByteSwap(pix);  // restore big-endian order
- *              FREE(lineptrs);
+ *              LEPT_FREE(lineptrs);
  *          This can be done even more simply as follows:
  *              l_uint8 **lineptrs = pixSetupByteProcessing(pix, &w, &h);
  *              for (i = 0; i < h; i++) {
@@ -1583,6 +1766,7 @@ l_uint32  *data;
  *                  }
  *              }
  *              pixCleanupByteProcessing(pix, lineptrs);
+ * </pre>
  */
 void **
 pixGetLinePtrs(PIX      *pix,
@@ -1594,12 +1778,13 @@ void     **lines;
 
     PROCNAME("pixGetLinePtrs");
 
+    if (psize) *psize = 0;
     if (!pix)
         return (void **)ERROR_PTR("pix not defined", procName, NULL);
 
     h = pixGetHeight(pix);
     if (psize) *psize = h;
-    if ((lines = (void **)CALLOC(h, sizeof(void *))) == NULL)
+    if ((lines = (void **)LEPT_CALLOC(h, sizeof(void *))) == NULL)
         return (void **)ERROR_PTR("lines not made", procName, NULL);
     wpl = pixGetWpl(pix);
     data = pixGetData(pix);
@@ -1616,18 +1801,19 @@ void     **lines;
 extern const char *ImageFileFormatExtensions[];
 
 /*!
- *  pixPrintStreamInfo()
+ * \brief   pixPrintStreamInfo()
  *
- *      Input:  fp (file stream)
- *              pix
- *              text (<optional> identifying string; can be null)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    fp file stream
+ * \param[in]    pix
+ * \param[in]    text [optional] identifying string; can be null
+ * \return  0 if OK, 1 on error
  */
 l_int32
 pixPrintStreamInfo(FILE        *fp,
                    PIX         *pix,
                    const char  *text)
 {
+char     *textdata;
 l_int32   informat;
 PIXCMAP  *cmap;
 
@@ -1640,10 +1826,12 @@ PIXCMAP  *cmap;
 
     if (text)
         fprintf(fp, "  Pix Info for %s:\n", text);
-    fprintf(fp, "    width = %d, height = %d, depth = %d\n",
-               pixGetWidth(pix), pixGetHeight(pix), pixGetDepth(pix));
+    fprintf(fp, "    width = %d, height = %d, depth = %d, spp = %d\n",
+               pixGetWidth(pix), pixGetHeight(pix), pixGetDepth(pix),
+               pixGetSpp(pix));
     fprintf(fp, "    wpl = %d, data = %p, refcount = %d\n",
                pixGetWpl(pix), pixGetData(pix), pixGetRefcount(pix));
+    fprintf(fp, "    xres = %d, yres = %d\n", pixGetXRes(pix), pixGetYRes(pix));
     if ((cmap = pixGetColormap(pix)) != NULL)
         pixcmapWriteStream(fp, cmap);
     else
@@ -1651,6 +1839,8 @@ PIXCMAP  *cmap;
     informat = pixGetInputFormat(pix);
     fprintf(fp, "    input format: %d (%s)\n", informat,
             ImageFileFormatExtensions[informat]);
+    if ((textdata = pixGetText(pix)) != NULL)
+        fprintf(fp, "    text: %s\n", textdata);
 
     return 0;
 }
