@@ -1,68 +1,95 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
 
-/*
- *   ptabasic.c
+/*!
+ * \file  ptabasic.c
+ * <pre>
  *
  *      Pta creation, destruction, copy, clone, empty
- *           PTA      *ptaCreate()
- *           PTA      *ptaCreateFromNuma()
- *           void      ptaDestroy()
- *           PTA      *ptaCopy()
- *           PTA      *ptaCopyRange()
- *           PTA      *ptaClone()
- *           l_int32   ptaEmpty()
+ *           PTA            *ptaCreate()
+ *           PTA            *ptaCreateFromNuma()
+ *           void            ptaDestroy()
+ *           PTA            *ptaCopy()
+ *           PTA            *ptaCopyRange()
+ *           PTA            *ptaClone()
+ *           l_int32         ptaEmpty()
  *
  *      Pta array extension
- *           l_int32   ptaAddPt()
- *           l_int32   ptaExtendArrays()
+ *           l_int32         ptaAddPt()
+ *           static l_int32  ptaExtendArrays()
  *
- *      Pta Accessors
- *           l_int32   ptaGetRefcount()
- *           l_int32   ptaChangeRefcount()
- *           l_int32   ptaGetCount()
- *           l_int32   ptaGetPt()
- *           l_int32   ptaGetIPt()
- *           l_int32   ptaSetPt()
- *           l_int32   ptaGetArrays()
+ *      Pta insertion and removal
+ *           l_int32         ptaInsertPt()
+ *           l_int32         ptaRemovePt()
+ *
+ *      Pta accessors
+ *           l_int32         ptaGetRefcount()
+ *           l_int32         ptaChangeRefcount()
+ *           l_int32         ptaGetCount()
+ *           l_int32         ptaGetPt()
+ *           l_int32         ptaGetIPt()
+ *           l_int32         ptaSetPt()
+ *           l_int32         ptaGetArrays()
  *
  *      Pta serialized for I/O
- *           PTA      *ptaRead()
- *           PTA      *ptaReadStream()
- *           l_int32   ptaWrite()
- *           l_int32   ptaWriteStream()
+ *           PTA            *ptaRead()
+ *           PTA            *ptaReadStream()
+ *           PTA            *ptaReadMem()
+ *           l_int32         ptaWrite()
+ *           l_int32         ptaWriteStream()
+ *           l_int32         ptaWriteMem()
  *
  *      Ptaa creation, destruction
- *           PTAA     *ptaaCreate()
- *           void      ptaaDestroy()
+ *           PTAA           *ptaaCreate()
+ *           void            ptaaDestroy()
  *
  *      Ptaa array extension
- *           l_int32   ptaaAddPta()
- *           l_int32   ptaaExtendArray()
+ *           l_int32         ptaaAddPta()
+ *           static l_int32  ptaaExtendArray()
  *
- *      Ptaa Accessors
- *           l_int32   ptaaGetCount()
- *           l_int32   ptaaGetPta()
- *           l_int32   ptaaGetPt()
+ *      Ptaa accessors
+ *           l_int32         ptaaGetCount()
+ *           l_int32         ptaaGetPta()
+ *           l_int32         ptaaGetPt()
+ *
+ *      Ptaa array modifiers
+ *           l_int32         ptaaInitFull()
+ *           l_int32         ptaaReplacePta()
+ *           l_int32         ptaaAddPt()
+ *           l_int32         ptaaTruncate()
  *
  *      Ptaa serialized for I/O
- *           PTAA     *ptaaRead()
- *           PTAA     *ptaaReadStream()
- *           l_int32   ptaaWrite()
- *           l_int32   ptaaWriteStream()
+ *           PTAA           *ptaaRead()
+ *           PTAA           *ptaaReadStream()
+ *           PTAA           *ptaaReadMem()
+ *           l_int32         ptaaWrite()
+ *           l_int32         ptaaWriteStream()
+ *           l_int32         ptaaWriteMem()
+ * </pre>
  */
 
 #include <string.h>
@@ -70,15 +97,19 @@
 
 static const l_int32  INITIAL_PTR_ARRAYSIZE = 20;   /* n'import quoi */
 
+    /* Static functions */
+static l_int32 ptaExtendArrays(PTA *pta);
+static l_int32 ptaaExtendArray(PTAA *ptaa);
+
 
 /*---------------------------------------------------------------------*
  *                Pta creation, destruction, copy, clone               *
  *---------------------------------------------------------------------*/
 /*!
- *  ptaCreate()
+ * \brief   ptaCreate()
  *
- *      Input:  n  (initial array sizes)
- *      Return: pta, or null on error.
+ * \param[in]    n  initial array sizes
+ * \return  pta, or NULL on error.
  */
 PTA *
 ptaCreate(l_int32  n)
@@ -90,15 +121,15 @@ PTA  *pta;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((pta = (PTA *)CALLOC(1, sizeof(PTA))) == NULL)
+    if ((pta = (PTA *)LEPT_CALLOC(1, sizeof(PTA))) == NULL)
         return (PTA *)ERROR_PTR("pta not made", procName, NULL);
     pta->n = 0;
     pta->nalloc = n;
     ptaChangeRefcount(pta, 1);  /* sets to 1 */
 
-    if ((pta->x = (l_float32 *)CALLOC(n, sizeof(l_float32))) == NULL)
+    if ((pta->x = (l_float32 *)LEPT_CALLOC(n, sizeof(l_float32))) == NULL)
         return (PTA *)ERROR_PTR("x array not made", procName, NULL);
-    if ((pta->y = (l_float32 *)CALLOC(n, sizeof(l_float32))) == NULL)
+    if ((pta->y = (l_float32 *)LEPT_CALLOC(n, sizeof(l_float32))) == NULL)
         return (PTA *)ERROR_PTR("y array not made", procName, NULL);
 
     return pta;
@@ -106,11 +137,11 @@ PTA  *pta;
 
 
 /*!
- *  ptaCreateFromNuma()
+ * \brief   ptaCreateFromNuma()
  *
- *      Input:  nax (<optional> can be null)
- *              nay
- *      Return: pta, or null on error.
+ * \param[in]    nax [optional] can be null
+ * \param[in]    nay
+ * \return  pta, or NULL on error.
  */
 PTA *
 ptaCreateFromNuma(NUMA  *nax,
@@ -129,7 +160,7 @@ PTA       *pta;
         return (PTA *)ERROR_PTR("nax and nay sizes differ", procName, NULL);
 
     pta = ptaCreate(n);
-    numaGetXParameters(nay, &startx, &delx);
+    numaGetParameters(nay, &startx, &delx);
     for (i = 0; i < n; i++) {
         if (nax)
             numaGetFValue(nax, i, &xval);
@@ -144,14 +175,16 @@ PTA       *pta;
 
 
 /*!
- *  ptaDestroy()
+ * \brief   ptaDestroy()
  *
- *      Input:  &pta (<to be nulled>)
- *      Return: void
+ * \param[in,out]   ppta to be nulled
+ * \return  void
  *
- *  Note:
- *      - Decrements the ref count and, if 0, destroys the pta.
- *      - Always nulls the input ptr.
+ * <pre>
+ * Notes:
+ *      (1) Decrements the ref count and, if 0, destroys the pta.
+ *      (2) Always nulls the input ptr.
+ * </pre>
  */
 void
 ptaDestroy(PTA  **ppta)
@@ -161,7 +194,7 @@ PTA  *pta;
     PROCNAME("ptaDestroy");
 
     if (ppta == NULL) {
-        L_WARNING("ptr address is NULL!", procName);
+        L_WARNING("ptr address is NULL!\n", procName);
         return;
     }
 
@@ -170,9 +203,9 @@ PTA  *pta;
 
     ptaChangeRefcount(pta, -1);
     if (ptaGetRefcount(pta) <= 0) {
-        FREE(pta->x);
-        FREE(pta->y);
-        FREE(pta);
+        LEPT_FREE(pta->x);
+        LEPT_FREE(pta->y);
+        LEPT_FREE(pta);
     }
 
     *ppta = NULL;
@@ -181,10 +214,10 @@ PTA  *pta;
 
 
 /*!
- *  ptaCopy()
+ * \brief   ptaCopy()
  *
- *      Input:  pta
- *      Return: copy of pta, or null on error
+ * \param[in]    pta
+ * \return  copy of pta, or NULL on error
  */
 PTA *
 ptaCopy(PTA  *pta)
@@ -211,12 +244,12 @@ PTA       *npta;
 
 
 /*!
- *  ptaCopyRange()
+ * \brief   ptaCopyRange()
  *
- *      Input:  ptas
- *              istart  (starting index in ptas)
- *              iend  (ending index in ptas; use 0 to copy to end)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    ptas
+ * \param[in]    istart  starting index in ptas
+ * \param[in]    iend  ending index in ptas; use 0 to copy to end
+ * \return  0 if OK, 1 on error
  */
 PTA *
 ptaCopyRange(PTA     *ptas,
@@ -252,10 +285,10 @@ PTA     *ptad;
 
 
 /*!
- *  ptaClone()
+ * \brief   ptaClone()
  *
- *      Input:  pta
- *      Return: ptr to same pta, or null on error
+ * \param[in]    pta
+ * \return  ptr to same pta, or NULL on error
  */
 PTA *
 ptaClone(PTA  *pta)
@@ -271,12 +304,15 @@ ptaClone(PTA  *pta)
 
 
 /*!
- *  ptaEmpty()
+ * \brief   ptaEmpty()
  *
- *      Input:  pta
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pta
+ * \return  0 if OK, 1 on error
  *
- *  Note: this only resets the "n" field, for reuse
+ * <pre>
+ * Notes:
+ *      This only resets the Pta::n field, for reuse
+ * </pre>
  */
 l_int32
 ptaEmpty(PTA  *pta)
@@ -294,11 +330,11 @@ ptaEmpty(PTA  *pta)
  *                         Pta array extension                         *
  *---------------------------------------------------------------------*/
 /*!
- *  ptaAddPt()
+ * \brief   ptaAddPt()
  *
- *      Input:  pta
- *              x, y
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pta
+ * \param[in]    x, y
+ * \return  0 if OK, 1 on error
  */
 l_int32
 ptaAddPt(PTA       *pta,
@@ -324,12 +360,12 @@ l_int32  n;
 
 
 /*!
- *  ptaExtendArrays()
+ * \brief   ptaExtendArrays()
  *
- *      Input:  pta
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pta
+ * \return  0 if OK; 1 on error
  */
-l_int32
+static l_int32
 ptaExtendArrays(PTA  *pta)
 {
     PROCNAME("ptaExtendArrays");
@@ -351,6 +387,84 @@ ptaExtendArrays(PTA  *pta)
 }
 
 
+/*---------------------------------------------------------------------*
+ *                     Pta insertion and removal                       *
+ *---------------------------------------------------------------------*/
+/*!
+ * \brief   ptaInsertPt()
+ *
+ * \param[in]    pta
+ * \param[in]    index at which pt is to be inserted
+ * \param[in]    x, y point values
+ * \return  0 if OK; 1 on error
+ */
+l_int32
+ptaInsertPt(PTA     *pta,
+            l_int32  index,
+            l_int32  x,
+            l_int32  y)
+{
+l_int32  i, n;
+
+    PROCNAME("ptaInsertPt");
+
+    if (!pta)
+        return ERROR_INT("pta not defined", procName, 1);
+    n = ptaGetCount(pta);
+    if (index < 0 || index > n)
+        return ERROR_INT("index not in {0...n}", procName, 1);
+
+    if (n > pta->nalloc)
+        ptaExtendArrays(pta);
+    pta->n++;
+    for (i = n; i > index; i--) {
+        pta->x[i] = pta->x[i - 1];
+        pta->y[i] = pta->y[i - 1];
+    }
+    pta->x[index] = x;
+    pta->y[index] = y;
+    return 0;
+}
+
+
+/*!
+ * \brief   ptaRemovePt()
+ *
+ * \param[in]    pta
+ * \param[in]    index of point to be removed
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This shifts pta[i] --> pta[i - 1] for all i > index.
+ *      (2) It should not be used repeatedly on large arrays,
+ *          because the function is O(n).
+ * </pre>
+ */
+l_int32
+ptaRemovePt(PTA     *pta,
+            l_int32  index)
+{
+l_int32  i, n;
+
+    PROCNAME("ptaRemovePt");
+
+    if (!pta)
+        return ERROR_INT("pta not defined", procName, 1);
+    n = ptaGetCount(pta);
+    if (index < 0 || index >= n)
+        return ERROR_INT("index not in {0...n - 1}", procName, 1);
+
+        /* Remove the point */
+    for (i = index + 1; i < n; i++) {
+        pta->x[i - 1] = pta->x[i];
+        pta->y[i - 1] = pta->y[i];
+    }
+    pta->n--;
+    return 0;
+}
+
+
 /*---------------------------------------------------------------------*
  *                           Pta accessors                             *
  *---------------------------------------------------------------------*/
@@ -379,10 +493,10 @@ ptaChangeRefcount(PTA     *pta,
 
 
 /*!
- *  ptaGetCount()
+ * \brief   ptaGetCount()
  *
- *      Input:  pta
- *      Return: count, or 0 if no pta
+ * \param[in]    pta
+ * \return  count, or 0 if no pta
  */
 l_int32
 ptaGetCount(PTA  *pta)
@@ -397,13 +511,13 @@ ptaGetCount(PTA  *pta)
 
 
 /*!
- *  ptaGetPt()
+ * \brief   ptaGetPt()
  *
- *      Input:  pta
- *              index  (into arrays)
- *              &x (<optional return> float x value)
- *              &y (<optional return> float y value)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pta
+ * \param[in]    index  into arrays
+ * \param[out]   px [optional] float x value
+ * \param[out]   py [optional] float y value
+ * \return  0 if OK; 1 on error
  */
 l_int32
 ptaGetPt(PTA        *pta,
@@ -427,13 +541,13 @@ ptaGetPt(PTA        *pta,
 
 
 /*!
- *  ptaGetIPt()
+ * \brief   ptaGetIPt()
  *
- *      Input:  pta
- *              index  (into arrays)
- *              &x (<optional return> integer x value)
- *              &y (<optional return> integer y value)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pta
+ * \param[in]    index  into arrays
+ * \param[out]   px [optional] integer x value
+ * \param[out]   py [optional] integer y value
+ * \return  0 if OK; 1 on error
  */
 l_int32
 ptaGetIPt(PTA      *pta,
@@ -457,12 +571,12 @@ ptaGetIPt(PTA      *pta,
 
 
 /*!
- *  ptaSetPt()
+ * \brief   ptaSetPt()
  *
- *      Input:  pta
- *              index  (into arrays)
- *              x, y
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pta
+ * \param[in]    index  into arrays
+ * \param[in]    x, y
+ * \return  0 if OK; 1 on error
  */
 l_int32
 ptaSetPt(PTA       *pta,
@@ -484,15 +598,17 @@ ptaSetPt(PTA       *pta,
 
 
 /*!
- *  ptaGetArrays()
+ * \brief   ptaGetArrays()
  *
- *      Input:  pta
- *              &nax (<optional return> numa of x array)
- *              &nay (<optional return> numa of y array)
- *      Return: 0 if OK; 1 on error or if pta is empty
+ * \param[in]    pta
+ * \param[out]   pnax [optional] numa of x array
+ * \param[out]   pnay [optional] numa of y array
+ * \return  0 if OK; 1 on error or if pta is empty
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This copies the internal arrays into new Numas.
+ * </pre>
  */
 l_int32
 ptaGetArrays(PTA    *pta,
@@ -537,10 +653,10 @@ NUMA    *nax, *nay;
  *                       Pta serialized for I/O                        *
  *---------------------------------------------------------------------*/
 /*!
- *  ptaRead()
+ * \brief   ptaRead()
  *
- *      Input:  filename
- *      Return: pta, or null on error
+ * \param[in]    filename
+ * \return  pta, or NULL on error
  */
 PTA *
 ptaRead(const char  *filename)
@@ -566,10 +682,10 @@ PTA   *pta;
 
 
 /*!
- *  ptaReadStream()
+ * \brief   ptaReadStream()
  *
- *      Input:  stream
- *      Return: pta, or null on error
+ * \param[in]    fp file stream
+ * \return  pta, or NULL on error
  */
 PTA *
 ptaReadStream(FILE  *fp)
@@ -602,8 +718,7 @@ PTA       *pta;
             if (fscanf(fp, "   (%f, %f)\n", &x, &y) != 2)
                 return (PTA *)ERROR_PTR("error reading floats", procName, NULL);
             ptaAddPt(pta, x, y);
-        }
-        else {   /* data is integer */
+        } else {   /* data is integer */
             if (fscanf(fp, "   (%d, %d)\n", &ix, &iy) != 2)
                 return (PTA *)ERROR_PTR("error reading ints", procName, NULL);
             ptaAddPt(pta, ix, iy);
@@ -615,12 +730,40 @@ PTA       *pta;
 
 
 /*!
- *  ptaWrite()
+ * \brief   ptaReadMem()
  *
- *      Input:  filename
- *              pta
- *              type  (0 for float values; 1 for integer values)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    data  serialization in ascii
+ * \param[in]    size  of data in bytes; can use strlen to get it
+ * \return  pta, or NULL on error
+ */
+PTA *
+ptaReadMem(const l_uint8  *data,
+           size_t          size)
+{
+FILE  *fp;
+PTA   *pta;
+
+    PROCNAME("ptaReadMem");
+
+    if (!data)
+        return (PTA *)ERROR_PTR("data not defined", procName, NULL);
+    if ((fp = fopenReadFromMemory(data, size)) == NULL)
+        return (PTA *)ERROR_PTR("stream not opened", procName, NULL);
+
+    pta = ptaReadStream(fp);
+    fclose(fp);
+    if (!pta) L_ERROR("pta not read\n", procName);
+    return pta;
+}
+
+
+/*!
+ * \brief   ptaWrite()
+ *
+ * \param[in]    filename
+ * \param[in]    pta
+ * \param[in]    type  0 for float values; 1 for integer values
+ * \return  0 if OK, 1 on error
  */
 l_int32
 ptaWrite(const char  *filename,
@@ -647,12 +790,12 @@ FILE  *fp;
 
 
 /*!
- *  ptaWriteStream()
+ * \brief   ptaWriteStream()
  *
- *      Input:  stream
- *              pta
- *              type  (0 for float values; 1 for integer values)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    fp file stream
+ * \param[in]    pta
+ * \param[in]    type  0 for float values; 1 for integer values
+ * \return  0 if OK; 1 on error
  */
 l_int32
 ptaWriteStream(FILE    *fp,
@@ -679,8 +822,7 @@ l_float32  x, y;
         if (type == 0) {  /* data is float */
             ptaGetPt(pta, i, &x, &y);
             fprintf(fp, "   (%f, %f)\n", x, y);
-        }
-        else {   /* data is integer */
+        } else {   /* data is integer */
             ptaGetIPt(pta, i, &ix, &iy);
             fprintf(fp, "   (%d, %d)\n", ix, iy);
         }
@@ -690,14 +832,70 @@ l_float32  x, y;
 }
 
 
+/*!
+ * \brief   ptaWriteMem()
+ *
+ * \param[out]   pdata data of serialized pta; ascii
+ * \param[out]   psize size of returned data
+ * \param[in]    pta
+ * \param[in]    type  0 for float values; 1 for integer values
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Serializes a pta in memory and puts the result in a buffer.
+ * </pre>
+ */
+l_int32
+ptaWriteMem(l_uint8  **pdata,
+            size_t    *psize,
+            PTA       *pta,
+            l_int32    type)
+{
+l_int32  ret;
+FILE    *fp;
+
+    PROCNAME("ptaWriteMem");
+
+    if (pdata) *pdata = NULL;
+    if (psize) *psize = 0;
+    if (!pdata)
+        return ERROR_INT("&data not defined", procName, 1);
+    if (!psize)
+        return ERROR_INT("&size not defined", procName, 1);
+    if (!pta)
+        return ERROR_INT("pta not defined", procName, 1);
+
+#if HAVE_FMEMOPEN
+    if ((fp = open_memstream((char **)pdata, psize)) == NULL)
+        return ERROR_INT("stream not opened", procName, 1);
+    ret = ptaWriteStream(fp, pta, type);
+#else
+    L_INFO("work-around: writing to a temp file\n", procName);
+  #ifdef _WIN32
+    if ((fp = fopenWriteWinTempfile()) == NULL)
+        return ERROR_INT("tmpfile stream not opened", procName, 1);
+  #else
+    if ((fp = tmpfile()) == NULL)
+        return ERROR_INT("tmpfile stream not opened", procName, 1);
+  #endif  /* _WIN32 */
+    ret = ptaWriteStream(fp, pta, type);
+    rewind(fp);
+    *pdata = l_binaryReadStream(fp, psize);
+#endif  /* HAVE_FMEMOPEN */
+    fclose(fp);
+    return ret;
+}
+
+
 /*---------------------------------------------------------------------*
  *                     PTAA creation, destruction                      *
  *---------------------------------------------------------------------*/
 /*!
- *  ptaaCreate()
+ * \brief   ptaaCreate()
  *
- *      Input:  n  (initial number of ptrs)
- *      Return: ptaa, or null on error
+ * \param[in]    n  initial number of ptrs
+ * \return  ptaa, or NULL on error
  */
 PTAA *
 ptaaCreate(l_int32  n)
@@ -709,12 +907,12 @@ PTAA  *ptaa;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((ptaa = (PTAA *)CALLOC(1, sizeof(PTAA))) == NULL)
+    if ((ptaa = (PTAA *)LEPT_CALLOC(1, sizeof(PTAA))) == NULL)
         return (PTAA *)ERROR_PTR("ptaa not made", procName, NULL);
     ptaa->n = 0;
     ptaa->nalloc = n;
 
-    if ((ptaa->pta = (PTA **)CALLOC(n, sizeof(PTA *))) == NULL)
+    if ((ptaa->pta = (PTA **)LEPT_CALLOC(n, sizeof(PTA *))) == NULL)
         return (PTAA *)ERROR_PTR("pta ptrs not made", procName, NULL);
 
     return ptaa;
@@ -722,10 +920,10 @@ PTAA  *ptaa;
 
 
 /*!
- *  ptaaDestroy()
+ * \brief   ptaaDestroy()
  *
- *      Input:  &ptaa <to be nulled>
- *      Return: void
+ * \param[in,out]   pptaa to be nulled
+ * \return  void
  */
 void
 ptaaDestroy(PTAA  **pptaa)
@@ -736,7 +934,7 @@ PTAA    *ptaa;
     PROCNAME("ptaaDestroy");
 
     if (pptaa == NULL) {
-        L_WARNING("ptr address is NULL!", procName);
+        L_WARNING("ptr address is NULL!\n", procName);
         return;
     }
 
@@ -745,9 +943,9 @@ PTAA    *ptaa;
 
     for (i = 0; i < ptaa->n; i++)
         ptaDestroy(&ptaa->pta[i]);
-    FREE(ptaa->pta);
+    LEPT_FREE(ptaa->pta);
 
-    FREE(ptaa);
+    LEPT_FREE(ptaa);
     *pptaa = NULL;
     return;
 }
@@ -757,12 +955,12 @@ PTAA    *ptaa;
  *                          PTAA array extension                       *
  *---------------------------------------------------------------------*/
 /*!
- *  ptaaAddPta()
+ * \brief   ptaaAddPta()
  *
- *      Input:  ptaa
- *              pta  (to be added)
- *              copyflag  (L_INSERT, L_COPY, L_CLONE)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    ptaa
+ * \param[in]    pta  to be added
+ * \param[in]    copyflag  L_INSERT, L_COPY, L_CLONE
+ * \return  0 if OK, 1 on error
  */
 l_int32
 ptaaAddPta(PTAA    *ptaa,
@@ -779,18 +977,17 @@ PTA     *ptac;
     if (!pta)
         return ERROR_INT("pta not defined", procName, 1);
 
-    if (copyflag == L_INSERT)
+    if (copyflag == L_INSERT) {
         ptac = pta;
-    else if (copyflag == L_COPY) {
+    } else if (copyflag == L_COPY) {
         if ((ptac = ptaCopy(pta)) == NULL)
             return ERROR_INT("ptac not made", procName, 1);
-    }
-    else if (copyflag == L_CLONE) {
+    } else if (copyflag == L_CLONE) {
         if ((ptac = ptaClone(pta)) == NULL)
             return ERROR_INT("pta clone not made", procName, 1);
-    }
-    else
+    } else {
         return ERROR_INT("invalid copyflag", procName, 1);
+    }
 
     n = ptaaGetCount(ptaa);
     if (n >= ptaa->nalloc)
@@ -803,12 +1000,12 @@ PTA     *ptac;
 
 
 /*!
- *  ptaaExtendArray()
+ * \brief   ptaaExtendArray()
  *
- *      Input:  ptaa
- *      Return: 0 if OK, 1 on error
+ * \param[in]    ptaa
+ * \return  0 if OK, 1 on error
  */
-l_int32
+static l_int32
 ptaaExtendArray(PTAA  *ptaa)
 {
     PROCNAME("ptaaExtendArray");
@@ -830,10 +1027,10 @@ ptaaExtendArray(PTAA  *ptaa)
  *                          Ptaa accessors                             *
  *---------------------------------------------------------------------*/
 /*!
- *  ptaaGetCount()
+ * \brief   ptaaGetCount()
  *
- *      Input:  ptaa
- *      Return: count, or 0 if no ptaa
+ * \param[in]    ptaa
+ * \return  count, or 0 if no ptaa
  */
 l_int32
 ptaaGetCount(PTAA  *ptaa)
@@ -848,12 +1045,12 @@ ptaaGetCount(PTAA  *ptaa)
 
 
 /*!
- *  ptaaGetPta()
+ * \brief   ptaaGetPta()
  *
- *      Input:  ptaa
- *              index  (to the i-th pta)
- *              accessflag  (L_COPY or L_CLONE)
- *      Return: pta, or null on error
+ * \param[in]    ptaa
+ * \param[in]    index  to the i-th pta
+ * \param[in]    accessflag  L_COPY or L_CLONE
+ * \return  pta, or NULL on error
  */
 PTA *
 ptaaGetPta(PTAA    *ptaa,
@@ -877,14 +1074,14 @@ ptaaGetPta(PTAA    *ptaa,
 
 
 /*!
- *  ptaaGetPt()
+ * \brief   ptaaGetPt()
  *
- *      Input:  ptaa
- *              ipta  (to the i-th pta)
- *              jpt (index to the j-th pt in the pta)
- *              &x (<optional return> float x value)
- *              &y (<optional return> float y value)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    ptaa
+ * \param[in]    ipta  to the i-th pta
+ * \param[in]    jpt index to the j-th pt in the pta
+ * \param[out]   px [optional] float x value
+ * \param[out]   py [optional] float y value
+ * \return  0 if OK; 1 on error
  */
 l_int32
 ptaaGetPt(PTAA       *ptaa,
@@ -916,14 +1113,160 @@ PTA  *pta;
 }
 
 
+/*---------------------------------------------------------------------*
+ *                        Ptaa array modifiers                         *
+ *---------------------------------------------------------------------*/
+/*!
+ * \brief   ptaaInitFull()
+ *
+ * \param[in]    ptaa can have non-null ptrs in the ptr array
+ * \param[in]    pta to be replicated into the entire ptr array
+ * \return  0 if OK; 1 on error
+ */
+l_int32
+ptaaInitFull(PTAA  *ptaa,
+             PTA   *pta)
+{
+l_int32  n, i;
+PTA     *ptat;
+
+    PROCNAME("ptaaInitFull");
+
+    if (!ptaa)
+        return ERROR_INT("ptaa not defined", procName, 1);
+    if (!pta)
+        return ERROR_INT("pta not defined", procName, 1);
+
+    n = ptaa->nalloc;
+    ptaa->n = n;
+    for (i = 0; i < n; i++) {
+        ptat = ptaCopy(pta);
+        ptaaReplacePta(ptaa, i, ptat);
+    }
+    return 0;
+}
+
+
+/*!
+ * \brief   ptaaReplacePta()
+ *
+ * \param[in]    ptaa
+ * \param[in]    index  to the index-th pta
+ * \param[in]    pta insert and replace any existing one
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Any existing pta is destroyed, and the input one
+ *          is inserted in its place.
+ *      (2) If the index is invalid, return 1 (error)
+ * </pre>
+ */
+l_int32
+ptaaReplacePta(PTAA    *ptaa,
+               l_int32  index,
+               PTA     *pta)
+{
+l_int32  n;
+
+    PROCNAME("ptaaReplacePta");
+
+    if (!ptaa)
+        return ERROR_INT("ptaa not defined", procName, 1);
+    if (!pta)
+        return ERROR_INT("pta not defined", procName, 1);
+    n = ptaaGetCount(ptaa);
+    if (index < 0 || index >= n)
+        return ERROR_INT("index not valid", procName, 1);
+
+    ptaDestroy(&ptaa->pta[index]);
+    ptaa->pta[index] = pta;
+    return 0;
+}
+
+
+/*!
+ * \brief   ptaaAddPt()
+ *
+ * \param[in]    ptaa
+ * \param[in]    ipta  to the i-th pta
+ * \param[in]    x,y point coordinates
+ * \return  0 if OK; 1 on error
+ */
+l_int32
+ptaaAddPt(PTAA      *ptaa,
+          l_int32    ipta,
+          l_float32  x,
+          l_float32  y)
+{
+PTA  *pta;
+
+    PROCNAME("ptaaAddPt");
+
+    if (!ptaa)
+        return ERROR_INT("ptaa not defined", procName, 1);
+    if (ipta < 0 || ipta >= ptaa->n)
+        return ERROR_INT("index ipta not valid", procName, 1);
+
+    pta = ptaaGetPta(ptaa, ipta, L_CLONE);
+    ptaAddPt(pta, x, y);
+    ptaDestroy(&pta);
+    return 0;
+}
+
+
+/*!
+ * \brief   ptaaTruncate()
+ *
+ * \param[in]    ptaa
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) This identifies the largest index containing a pta that
+ *          has any points within it, destroys all pta above that index,
+ *          and resets the count.
+ * </pre>
+ */
+l_int32
+ptaaTruncate(PTAA  *ptaa)
+{
+l_int32  i, n, np;
+PTA     *pta;
+
+    PROCNAME("ptaaTruncate");
+
+    if (!ptaa)
+        return ERROR_INT("ptaa not defined", procName, 1);
+
+    n = ptaaGetCount(ptaa);
+    for (i = n - 1; i >= 0; i--) {
+        pta = ptaaGetPta(ptaa, i, L_CLONE);
+        if (!pta) {
+            ptaa->n--;
+            continue;
+        }
+        np = ptaGetCount(pta);
+        ptaDestroy(&pta);
+        if (np == 0) {
+            ptaDestroy(&ptaa->pta[i]);
+            ptaa->n--;
+        } else {
+            break;
+        }
+    }
+    return 0;
+}
+
+
 /*---------------------------------------------------------------------*
  *                       Ptaa serialized for I/O                       *
  *---------------------------------------------------------------------*/
 /*!
- *  ptaaRead()
+ * \brief   ptaaRead()
  *
- *      Input:  filename
- *      Return: ptaa, or null on error
+ * \param[in]    filename
+ * \return  ptaa, or NULL on error
  */
 PTAA *
 ptaaRead(const char  *filename)
@@ -949,10 +1292,10 @@ PTAA  *ptaa;
 
 
 /*!
- *  ptaaReadStream()
+ * \brief   ptaaReadStream()
  *
- *      Input:  stream
- *      Return: ptaa, or null on error
+ * \param[in]    fp file stream
+ * \return  ptaa, or NULL on error
  */
 PTAA *
 ptaaReadStream(FILE  *fp)
@@ -986,12 +1329,40 @@ PTAA    *ptaa;
 
 
 /*!
- *  ptaaWrite()
+ * \brief   ptaaReadMem()
  *
- *      Input:  filename
- *              ptaa
- *              type  (0 for float values; 1 for integer values)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    data  serialization in ascii
+ * \param[in]    size  of data in bytes; can use strlen to get it
+ * \return  ptaa, or NULL on error
+ */
+PTAA *
+ptaaReadMem(const l_uint8  *data,
+            size_t          size)
+{
+FILE  *fp;
+PTAA  *ptaa;
+
+    PROCNAME("ptaaReadMem");
+
+    if (!data)
+        return (PTAA *)ERROR_PTR("data not defined", procName, NULL);
+    if ((fp = fopenReadFromMemory(data, size)) == NULL)
+        return (PTAA *)ERROR_PTR("stream not opened", procName, NULL);
+
+    ptaa = ptaaReadStream(fp);
+    fclose(fp);
+    if (!ptaa) L_ERROR("ptaa not read\n", procName);
+    return ptaa;
+}
+
+
+/*!
+ * \brief   ptaaWrite()
+ *
+ * \param[in]    filename
+ * \param[in]    ptaa
+ * \param[in]    type  0 for float values; 1 for integer values
+ * \return  0 if OK, 1 on error
  */
 l_int32
 ptaaWrite(const char  *filename,
@@ -1018,12 +1389,12 @@ FILE  *fp;
 
 
 /*!
- *  ptaaWriteStream()
+ * \brief   ptaaWriteStream()
  *
- *      Input:  stream
- *              ptaa
- *              type  (0 for float values; 1 for integer values)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    fp file stream
+ * \param[in]    ptaa
+ * \param[in]    type  0 for float values; 1 for integer values
+ * \return  0 if OK; 1 on error
  */
 l_int32
 ptaaWriteStream(FILE    *fp,
@@ -1051,3 +1422,60 @@ PTA     *pta;
 
     return 0;
 }
+
+
+/*!
+ * \brief   ptaaWriteMem()
+ *
+ * \param[out]   pdata data of serialized ptaa; ascii
+ * \param[out]   psize size of returned data
+ * \param[in]    ptaa
+ * \param[in]    type  0 for float values; 1 for integer values
+ * \return  0 if OK, 1 on error
+ *
+ * <pre>
+ * Notes:
+ *      (1) Serializes a ptaa in memory and puts the result in a buffer.
+ * </pre>
+ */
+l_int32
+ptaaWriteMem(l_uint8  **pdata,
+             size_t    *psize,
+             PTAA      *ptaa,
+             l_int32    type)
+{
+l_int32  ret;
+FILE    *fp;
+
+    PROCNAME("ptaaWriteMem");
+
+    if (pdata) *pdata = NULL;
+    if (psize) *psize = 0;
+    if (!pdata)
+        return ERROR_INT("&data not defined", procName, 1);
+    if (!psize)
+        return ERROR_INT("&size not defined", procName, 1);
+    if (!ptaa)
+        return ERROR_INT("ptaa not defined", procName, 1);
+
+#if HAVE_FMEMOPEN
+    if ((fp = open_memstream((char **)pdata, psize)) == NULL)
+        return ERROR_INT("stream not opened", procName, 1);
+    ret = ptaaWriteStream(fp, ptaa, type);
+#else
+    L_INFO("work-around: writing to a temp file\n", procName);
+  #ifdef _WIN32
+    if ((fp = fopenWriteWinTempfile()) == NULL)
+        return ERROR_INT("tmpfile stream not opened", procName, 1);
+  #else
+    if ((fp = tmpfile()) == NULL)
+        return ERROR_INT("tmpfile stream not opened", procName, 1);
+  #endif  /* _WIN32 */
+    ret = ptaaWriteStream(fp, ptaa, type);
+    rewind(fp);
+    *pdata = l_binaryReadStream(fp, psize);
+#endif  /* HAVE_FMEMOPEN */
+    fclose(fp);
+    return ret;
+}
+
