@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *   ptra.c
+/*!
+ * \file  ptra.c
+ * <pre>
  *
  *      Ptra creation and destruction
  *          L_PTRA      *ptraCreate()
@@ -22,9 +34,8 @@
  *
  *      Add/insert/remove/replace generic ptr object
  *          l_int32      ptraAdd()
- *          l_int32      ptraExtendArray()
+ *          static l_int32  ptraExtendArray()
  *          l_int32      ptraInsert()
- *          void        *ptraGetHandle()
  *          void        *ptraRemove()
  *          void        *ptraRemoveLast()
  *          void        *ptraReplace()
@@ -75,9 +86,9 @@
  *        distance to the next null ptr in the array.
  *    (7) Null ptrs are valid input args for both insertion and
  *        replacement; this allows arbitrary swapping.
- *    (8) The item in the array with the largest index is at pa->imax.
+ *    (8) The item in the array with the largest index is at pa-\>imax.
  *        This can be any value from -1 (initialized; all array ptrs
- *        are null) up to pa->nalloc - 1 (the last ptr in the array).
+ *        are null) up to pa-\>nalloc - 1 (the last ptr in the array).
  *    (9) In referring to the array: the first ptr is the "top" or
  *        "beginning"; the last pointer is the "bottom" or "end";
  *        items are shifted "up" towards the top when compaction occurs;
@@ -87,7 +98,7 @@
  *        are general:
  *         * You can insert an item into any ptr location in the
  *           allocated ptr array, as well as into the next ptr address
- *           beyond the allocated array (in which case a realloc will occur).   
+ *           beyond the allocated array (in which case a realloc will occur).
  *         * You can remove or replace an item from any ptr location
  *           in the allocated ptr array.
  *         * When inserting into an occupied location, you have
@@ -104,21 +115,25 @@
  *        A typical usage is to allow an O(n) horizontal sort of Pix,
  *        where the size of the Ptra array is the width of the image,
  *        and each Ptra is an array of all the Pix at a specific x location.
+ * </pre>
  */
 
 #include "allheaders.h"
 
 static const l_int32 INITIAL_PTR_ARRAYSIZE = 20;      /* n'importe quoi */
 
+    /* Static function */
+static l_int32 ptraExtendArray(L_PTRA *pa);
+
 
 /*--------------------------------------------------------------------------*
  *                       Ptra creation and destruction                      *
  *--------------------------------------------------------------------------*/
 /*!
- *  ptraCreate()
+ * \brief   ptraCreate()
  *
- *      Input:  size of ptr array to be alloc'd (0 for default)
- *      Return: pa, or null on error
+ * \param[in]    n size of ptr array to be alloc'd 0 for default
+ * \return  pa, or NULL on error
  */
 L_PTRA *
 ptraCreate(l_int32  n)
@@ -130,9 +145,9 @@ L_PTRA  *pa;
     if (n <= 0)
         n = INITIAL_PTR_ARRAYSIZE;
 
-    if ((pa = (L_PTRA *)CALLOC(1, sizeof(L_PTRA))) == NULL)
+    if ((pa = (L_PTRA *)LEPT_CALLOC(1, sizeof(L_PTRA))) == NULL)
         return (L_PTRA *)ERROR_PTR("pa not made", procName, NULL);
-    if ((pa->array = (void **)CALLOC(n, sizeof(void *))) == NULL)
+    if ((pa->array = (void **)LEPT_CALLOC(n, sizeof(void *))) == NULL)
         return (L_PTRA *)ERROR_PTR("ptr array not made", procName, NULL);
 
     pa->nalloc = n;
@@ -144,26 +159,28 @@ L_PTRA  *pa;
 
 
 /*!
- *  ptraDestroy()
+ * \brief   ptraDestroy()
  *
- *      Input:  &ptra (<to be nulled>)
- *              freeflag (TRUE to free each remaining item in the array)
- *              warnflag (TRUE to warn if any remaining items are not destroyed)
- *      Return: void
+ * \param[in,out]   ppa ptra to be nulled
+ * \param[in]       freeflag TRUE to free each remaining item in the array
+ * \param[in]       warnflag TRUE to warn if any remaining items are not destroyed
+ * \return  void
  *
- *  Notes:
- *      (1) If @freeflag == TRUE, frees each item in the array.
- *      (2) If @freeflag == FALSE and warnflag == TRUE, and there are
+ * <pre>
+ * Notes:
+ *      (1) If %freeflag == TRUE, frees each item in the array.
+ *      (2) If %freeflag == FALSE and %warnflag == TRUE, and there are
  *          items on the array, this gives a warning and destroys the array.
  *          If these items are not owned elsewhere, this will cause
  *          a memory leak of all the items that were on the array.
  *          So if the items are not owned elsewhere and require their
  *          own destroy function, they must be destroyed before the ptra.
- *      (3) If warnflag == FALSE, no warnings will be issued.  This is
+ *      (3) If %warnflag == FALSE, no warnings will be issued.  This is
  *          useful if the items are owned elsewhere, such as a
  *          PixMemoryStore().
  *      (4) To destroy the ptra, we destroy the ptr array, then
  *          the ptra, and then null the contents of the input ptr.
+ * </pre>
  */
 void
 ptraDestroy(L_PTRA  **ppa,
@@ -177,7 +194,7 @@ L_PTRA  *pa;
     PROCNAME("ptraDestroy");
 
     if (ppa == NULL) {
-        L_WARNING("ptr address is NULL", procName);
+        L_WARNING("ptr address is NULL\n", procName);
         return;
     }
     if ((pa = *ppa) == NULL)
@@ -188,16 +205,16 @@ L_PTRA  *pa;
         if (freeflag) {
             for (i = 0; i <= pa->imax; i++) {
                 if ((item = ptraRemove(pa, i, L_NO_COMPACTION)) != NULL)
-                    FREE(item);
+                    LEPT_FREE(item);
             }
+        } else if (warnflag) {
+            L_WARNING("potential memory leak of %d items in ptra\n",
+                      procName, nactual);
         }
-        else if (warnflag)
-            L_WARNING_INT("potential memory leak of %d items in ptra",
-                          procName, nactual);
     }
 
-    FREE(pa->array);
-    FREE(pa);
+    LEPT_FREE(pa->array);
+    LEPT_FREE(pa);
     *ppa = NULL;
     return;
 }
@@ -207,18 +224,20 @@ L_PTRA  *pa;
  *               Add/insert/remove/replace generic ptr object               *
  *--------------------------------------------------------------------------*/
 /*!
- *  ptraAdd()
+ * \brief   ptraAdd()
  *
- *      Input:  ptra
- *              item  (generic ptr to a struct)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pa ptra
+ * \param[in]    item  generic ptr to a struct
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This adds the element to the next location beyond imax,
  *          which is the largest occupied ptr in the array.  This is
  *          what you expect from a stack, where all ptrs up to and
  *          including imax are occupied, but here the occuption of
  *          items in the array is entirely arbitrary.
+ * </pre>
  */
 l_int32
 ptraAdd(L_PTRA  *pa,
@@ -232,7 +251,7 @@ l_int32  imax;
         return ERROR_INT("pa not defined", procName, 1);
     if (!item)
         return ERROR_INT("item not defined", procName, 1);
-    
+
     ptraGetMaxIndex(pa, &imax);
     if (imax >= pa->nalloc - 1 && ptraExtendArray(pa))
         return ERROR_INT("extension failure", procName, 1);
@@ -244,12 +263,12 @@ l_int32  imax;
 
 
 /*!
- *  ptraExtendArray()
+ * \brief   ptraExtendArray()
  *
- *      Input:  ptra 
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pa
+ * \return  0 if OK, 1 on error
  */
-l_int32
+static l_int32
 ptraExtendArray(L_PTRA  *pa)
 {
     PROCNAME("ptraExtendArray");
@@ -268,34 +287,35 @@ ptraExtendArray(L_PTRA  *pa)
 
 
 /*!
- *  ptraInsert()
+ * \brief   ptraInsert()
  *
- *      Input:  ptra 
- *              index (location in ptra to insert new value)
- *              item  (generic ptr to a struct; can be null)
- *              shiftflag (L_AUTO_DOWNSHIFT, L_MIN_DOWNSHIFT, L_FULL_DOWNSHIFT)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pa ptra
+ * \param[in]    index location in ptra to insert new value
+ * \param[in]    item  generic ptr to a struct; can be null
+ * \param[in]    shiftflag L_AUTO_DOWNSHIFT, L_MIN_DOWNSHIFT, L_FULL_DOWNSHIFT
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This checks first to see if the location is valid, and
  *          then if there is presently an item there.  If there is not,
  *          it is simply inserted into that location.
  *      (2) If there is an item at the insert location, items must be
  *          moved down to make room for the insert.  In the downward
- *          shift there are three options, given by @shiftflag.
- *            - If @shiftflag == L_AUTO_DOWNSHIFT, a decision is made
+ *          shift there are three options, given by %shiftflag.
+ *            ~ If %shiftflag == L_AUTO_DOWNSHIFT, a decision is made
  *              whether, in a cascade of items, to downshift a minimum
- *              amount or for all items above @index.  The decision is
+ *              amount or for all items above %index.  The decision is
  *              based on the expectation of finding holes (null ptrs)
- *              between @index and the bottom of the array.
+ *              between %index and the bottom of the array.
  *              Assuming the holes are distributed uniformly, if 2 or more
  *              holes are expected, we do a minimum shift.
- *            - If @shiftflag == L_MIN_DOWNSHIFT, the downward shifting
+ *            ~ If %shiftflag == L_MIN_DOWNSHIFT, the downward shifting
  *              cascade of items progresses a minimum amount, until
  *              the first empty slot is reached.  This mode requires
  *              some computation before the actual shifting is done.
- *            - If @shiftflag == L_FULL_DOWNSHIFT, a shifting cascade is
- *              performed where pa[i] --> pa[i + 1] for all i >= index.
+ *            ~ If %shiftflag == L_FULL_DOWNSHIFT, a shifting cascade is
+ *              performed where pa[i] --\> pa[i + 1] for all i \>= index.
  *              Then, the item is inserted at pa[index].
  *      (3) If you are not using L_AUTO_DOWNSHIFT, the rule of thumb is
  *          to use L_FULL_DOWNSHIFT if the array is compacted (each
@@ -311,6 +331,7 @@ ptraExtendArray(L_PTRA  *pa)
  *          you can support an array of Numa, where at each ptr location
  *          you store either 0 or 1 Numa, and the Numa can be added
  *          randomly to the ptr array.
+ * </pre>
  */
 l_int32
 ptraInsert(L_PTRA  *pa,
@@ -359,12 +380,12 @@ l_float32  nexpected;
         /* If there are no holes, do a full downshift.
          * Otherwise, if L_AUTO_DOWNSHIFT, use the expected number
          * of holes between index and n to determine the shift mode */
-    if (imax + 1 == pa->nactual)
+    if (imax + 1 == pa->nactual) {
         shiftflag = L_FULL_DOWNSHIFT;
-    else if (shiftflag == L_AUTO_DOWNSHIFT) {
-        if (imax < 10)
+    } else if (shiftflag == L_AUTO_DOWNSHIFT) {
+        if (imax < 10) {
             shiftflag = L_FULL_DOWNSHIFT;  /* no big deal */
-        else {
+        } else {
             nexpected = (l_float32)(imax - pa->nactual) *
                          (l_float32)((imax - index) / imax);
             shiftflag = (nexpected > 2.0) ? L_MIN_DOWNSHIFT : L_FULL_DOWNSHIFT;
@@ -376,9 +397,9 @@ l_float32  nexpected;
              if (pa->array[ihole] == NULL)
                  break;
         }
-    }
-    else   /* L_FULL_DOWNSHIFT */
+    } else {  /* L_FULL_DOWNSHIFT */
         ihole = imax + 1;
+    }
 
     for (i = ihole; i > index; i--)
         pa->array[i] = pa->array[i - 1];
@@ -391,52 +412,24 @@ l_float32  nexpected;
 
 
 /*!
- *  ptraGetHandle()
+ * \brief   ptraRemove()
  *
- *      Input:  ptra
- *              index (element to be retrieved)
- *      Return: item, or null on error
+ * \param[in]    pa ptra
+ * \param[in]    index element to be removed
+ * \param[in]    flag L_NO_COMPACTION, L_COMPACTION
+ * \return  item, or NULL on error
  *
- *  Notes:
- *      (1) This returns a ptr to the item.  You must cast it to
- *          the type of item.  Do not destroy it; the item belongs
- *          to the Ptra.
- *      (2) This can access all possible items on the ptr array.
- *          If an item doesn't exist, it returns null.
- */
-void *
-ptraGetHandle(L_PTRA  *pa,
-              l_int32  index)
-{
-    PROCNAME("ptraGetHandle");
-
-    if (!pa)
-        return (void *)ERROR_PTR("pa not defined", procName, NULL);
-    if (index < 0 || index >= pa->nalloc)
-        return (void *)ERROR_PTR("index not in [0 ... nalloc-1]",
-                                 procName, NULL);
-
-    return pa->array[index];
-}
-
-
-/*!
- *  ptraRemove()
- *
- *      Input:  ptra
- *              index (element to be removed)
- *              flag (L_NO_COMPACTION, L_COMPACTION)
- *      Return: item, or null on error
- *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) If flag == L_NO_COMPACTION, this removes the item and
  *          nulls the ptr on the array.  If it takes the last item
- *          in the array, pa->n is reduced to the next item.
+ *          in the array, pa-\>n is reduced to the next item.
  *      (2) If flag == L_COMPACTION, this compacts the array for
- *          for all i >= index.  It should not be used repeatedly on
+ *          for all i \>= index.  It should not be used repeatedly on
  *          large arrays, because compaction is O(n).
  *      (3) The ability to remove without automatic compaction allows
  *          removal with cost O(1).
+ * </pre>
  */
 void *
 ptraRemove(L_PTRA  *pa,
@@ -458,7 +451,7 @@ void    *item;
     if (item)
         pa->nactual--;
     pa->array[index] = NULL;
-  
+
         /* If we took the last item, need to reduce pa->n */
     fromend = (index == imax);
     if (fromend) {
@@ -467,7 +460,6 @@ void    *item;
                 break;
         }
         pa->imax = i;
-        imax = i + 1;
     }
 
         /* Compact from index to the end of the array */
@@ -483,10 +475,10 @@ void    *item;
 
 
 /*!
- *  ptraRemoveLast()
+ * \brief   ptraRemoveLast()
  *
- *      Input:  ptra
- *      Return: item, or null on error or if the array is empty
+ * \param[in]    pa ptra
+ * \return  item, or NULL on error or if the array is empty
  */
 void *
 ptraRemoveLast(L_PTRA  *pa)
@@ -508,14 +500,14 @@ l_int32  imax;
 
 
 /*!
- *  ptraReplace()
+ * \brief   ptraReplace()
  *
- *      Input:  ptra
- *              index (element to be replaced)
- *              item  (new generic ptr to a struct; can be null)
- *              freeflag (TRUE to free old item; FALSE to return it)
- *      Return: item  (old item, if it exists and is not freed),
- *                     or null on error
+ * \param[in]    pa ptra
+ * \param[in]    index element to be replaced
+ * \param[in]    item  new generic ptr to a struct; can be null
+ * \param[in]    freeflag TRUE to free old item; FALSE to return it
+ * \return  item  old item, if it exists and is not freed,
+ *                     or NULL on error
  */
 void *
 ptraReplace(L_PTRA  *pa,
@@ -545,18 +537,18 @@ void    *olditem;
         return olditem;
 
     if (olditem)
-        FREE(olditem);
+        LEPT_FREE(olditem);
     return NULL;
 }
 
 
 /*!
- *  ptraSwap()
+ * \brief   ptraSwap()
  *
- *      Input:  ptra
- *              index1
- *              index2
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pa ptra
+ * \param[in]    index1
+ * \param[in]    index2
+ * \return  0 if OK, 1 on error
  */
 l_int32
 ptraSwap(L_PTRA  *pa,
@@ -584,14 +576,16 @@ void    *item;
 
 
 /*!
- *  ptraCompactArray()
+ * \brief   ptraCompactArray()
  *
- *      Input:  ptra 
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pa
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This compacts the items on the array, filling any empty ptrs.
  *      (2) This does not change the size of the array of ptrs.
+ * </pre>
  */
 l_int32
 ptraCompactArray(L_PTRA  *pa)
@@ -613,7 +607,7 @@ l_int32  i, imax, nactual, index;
     }
     pa->imax = index - 1;
     if (nactual != index)
-        L_ERROR_INT("index = %d; != nactual", procName, index);
+        L_ERROR("index = %d; != nactual\n", procName, index);
 
     return 0;
 }
@@ -623,10 +617,10 @@ l_int32  i, imax, nactual, index;
  *                        Other array operations                        *
  *----------------------------------------------------------------------*/
 /*!
- *  ptraReverse()
+ * \brief   ptraReverse()
  *
- *      Input:  ptra
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pa ptra
+ * \return  0 if OK, 1 on error
  */
 l_int32
 ptraReverse(L_PTRA  *pa)
@@ -646,11 +640,11 @@ l_int32  i, imax;
 
 
 /*!
- *  ptraJoin()
+ * \brief   ptraJoin()
  *
- *      Input:  ptra1 (add to this one)
- *              ptra2 (appended to ptra1, and emptied of items; can be null)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pa1 add to this one
+ * \param[in]    pa2 appended to pa1, and emptied of items; can be null
+ * \return  0 if OK, 1 on error
  */
 l_int32
 ptraJoin(L_PTRA  *pa1,
@@ -671,7 +665,7 @@ void    *item;
         item = ptraRemove(pa2, i, L_NO_COMPACTION);
         ptraAdd(pa1, item);
     }
-    
+
     return 0;
 }
 
@@ -681,24 +675,26 @@ void    *item;
  *                        Simple ptra accessors                         *
  *----------------------------------------------------------------------*/
 /*!
- *  ptraGetMaxIndex()
+ * \brief   ptraGetMaxIndex()
  *
- *      Input:  ptra
- *              &maxindex (<return> index of last item in the array);
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pa ptra
+ * \param[out]   pmaxindex index of last item in the array;
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
- *      (1) The largest index to an item in the array is @maxindex.
- *          @maxindex is one less than the number of items that would be
+ * <pre>
+ * Notes:
+ *      (1) The largest index to an item in the array is %maxindex.
+ *          %maxindex is one less than the number of items that would be
  *          in the array if there were no null pointers between 0
- *          and @maxindex - 1.  However, because the internal ptr array
- *          need not be compacted, there may be null pointers at
- *          indices below @maxindex; for example, if items have
+ *          and %maxindex - 1.  However, because the internal ptr array
+ *          need not be compacted, there may be NULL pointers at
+ *          indices below %maxindex; for example, if items have
  *          been removed.
  *      (2) When an item is added to the end of the array, it goes
- *          into pa->array[maxindex + 1], and maxindex is then
+ *          into pa-\>array[maxindex + 1], and maxindex is then
  *          incremented by 1.
- *      (3) If there are no items in the array, this returns @maxindex = -1.
+ *      (3) If there are no items in the array, this returns %maxindex = -1.
+ * </pre>
  */
 l_int32
 ptraGetMaxIndex(L_PTRA   *pa,
@@ -713,18 +709,20 @@ ptraGetMaxIndex(L_PTRA   *pa,
     *pmaxindex = pa->imax;
     return 0;
 }
-        
+
 
 /*!
- *  ptraGetActualCount()
+ * \brief   ptraGetActualCount()
  *
- *      Input:  ptra
- *              &count (<return> actual number of items on the ptr array)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    pa ptra
+ * \param[out]   pcount actual number of items on the ptr array
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
- *      (1) The actual number of items on the ptr array, pa->nactual,
- *          will be smaller than pa->n if the array is not compacted.
+ * <pre>
+ * Notes:
+ *      (1) The actual number of items on the ptr array, pa-\>nactual,
+ *          will be smaller than pa-\>n if the array is not compacted.
+ * </pre>
  */
 l_int32
 ptraGetActualCount(L_PTRA   *pa,
@@ -743,15 +741,20 @@ ptraGetActualCount(L_PTRA   *pa,
 
 
 /*!
- *  ptraGetPtrToItem()
+ * \brief   ptraGetPtrToItem()
  *
- *      Input:  ptra
- *              index (element to fetch pointer to)
- *      Return: item (just a pointer to it)
+ * \param[in]    pa ptra
+ * \param[in]    index of element to be retrieved
+ * \return  a ptr to the element, or NULL on error
  *
- *  Notes:
- *      (1) The item remains on the Ptra and is 'owned' by it, so
- *          the item must not be destroyed.
+ * <pre>
+ * Notes:
+ *      (1) This returns a ptr to the item.  You must cast it to
+ *          the type of item.  Do not destroy it; the item belongs
+ *          to the Ptra.
+ *      (2) This can access all possible items on the ptr array.
+ *          If an item doesn't exist, it returns null.
+ * </pre>
  */
 void *
 ptraGetPtrToItem(L_PTRA  *pa,
@@ -761,8 +764,9 @@ ptraGetPtrToItem(L_PTRA  *pa,
 
     if (!pa)
         return (void *)ERROR_PTR("pa not defined", procName, NULL);
-    if (index < 0 || index > pa->imax)
-        return (void *)ERROR_PTR("index not in [0 ... imax]", procName, NULL);
+    if (index < 0 || index >= pa->nalloc)
+        return (void *)ERROR_PTR("index not in [0 ... nalloc-1]",
+                                 procName, NULL);
 
     return pa->array[index];
 }
@@ -772,14 +776,16 @@ ptraGetPtrToItem(L_PTRA  *pa,
  *                      Ptraa creation and destruction                      *
  *--------------------------------------------------------------------------*/
 /*!
- *  ptraaCreate()
+ * \brief   ptraaCreate()
  *
- *      Input:  size of ptr array to be alloc'd
- *      Return: paa, or null on error
+ * \param[in]    n size of ptr array to be alloc'd
+ * \return  paa, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) The ptraa is generated with a fixed size, that can not change.
  *          The ptra can be generated and inserted randomly into this array.
+ * </pre>
  */
 L_PTRAA *
 ptraaCreate(l_int32  n)
@@ -791,9 +797,9 @@ L_PTRAA  *paa;
     if (n <= 0)
         return (L_PTRAA *)ERROR_PTR("n must be > 0", procName, NULL);
 
-    if ((paa = (L_PTRAA *)CALLOC(1, sizeof(L_PTRAA))) == NULL)
+    if ((paa = (L_PTRAA *)LEPT_CALLOC(1, sizeof(L_PTRAA))) == NULL)
         return (L_PTRAA *)ERROR_PTR("paa not made", procName, NULL);
-    if ((paa->ptra = (L_PTRA **)CALLOC(n, sizeof(L_PTRA *))) == NULL)
+    if ((paa->ptra = (L_PTRA **)LEPT_CALLOC(n, sizeof(L_PTRA *))) == NULL)
         return (L_PTRAA *)ERROR_PTR("ptr array not made", procName, NULL);
 
     paa->nalloc = n;
@@ -802,17 +808,19 @@ L_PTRAA  *paa;
 
 
 /*!
- *  ptraaDestroy()
+ * \brief   ptraaDestroy()
  *
- *      Input:  &paa (<to be nulled>)
- *              freeflag (TRUE to free each remaining item in each ptra)
- *              warnflag (TRUE to warn if any remaining items are not destroyed)
- *      Return: void
+ * \param[in,out]   ppaa to be nulled
+ * \param[in]    freeflag TRUE to free each remaining item in each ptra
+ * \param[in]    warnflag TRUE to warn if any remaining items are not destroyed
+ * \return  void
  *
- *  Notes:
- *      (1) See ptraDestroy() for use of @freeflag and @warnflag.
+ * <pre>
+ * Notes:
+ *      (1) See ptraDestroy() for use of %freeflag and %warnflag.
  *      (2) To destroy the ptraa, we destroy each ptra, then the ptr array,
  *          then the ptraa, and then null the contents of the input ptr.
+ * </pre>
  */
 void
 ptraaDestroy(L_PTRAA  **ppaa,
@@ -826,7 +834,7 @@ L_PTRAA  *paa;
     PROCNAME("ptraaDestroy");
 
     if (ppaa == NULL) {
-        L_WARNING("ptr address is NULL", procName);
+        L_WARNING("ptr address is NULL\n", procName);
         return;
     }
     if ((paa = *ppaa) == NULL)
@@ -838,8 +846,8 @@ L_PTRAA  *paa;
         ptraDestroy(&pa, freeflag, warnflag);
     }
 
-    FREE(paa->ptra);
-    FREE(paa);
+    LEPT_FREE(paa->ptra);
+    LEPT_FREE(paa);
     *ppaa = NULL;
     return;
 }
@@ -849,11 +857,11 @@ L_PTRAA  *paa;
  *                             Ptraa accessors                              *
  *--------------------------------------------------------------------------*/
 /*!
- *  ptraaGetSize()
+ * \brief   ptraaGetSize()
  *
- *      Input:  ptraa
- *              &size (<return> size of ptr array)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    paa
+ * \param[out]   psize size of ptr array
+ * \return  0 if OK; 1 on error
  */
 l_int32
 ptraaGetSize(L_PTRAA  *paa,
@@ -872,17 +880,19 @@ ptraaGetSize(L_PTRAA  *paa,
 
 
 /*!
- *  ptraaInsertPtra()
+ * \brief   ptraaInsertPtra()
  *
- *      Input:  ptraa
- *              index (location in array for insertion)
- *              ptra (to be inserted)
- *      Return: 0 if OK; 1 on error
+ * \param[in]    paa ptraa
+ * \param[in]    index location in array for insertion
+ * \param[in]    pa to be inserted
+ * \return  0 if OK; 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Caller should check return value.  On success, the Ptra
  *          is inserted in the Ptraa and is owned by it.  However,
  *          on error, the Ptra remains owned by the caller.
+ * </pre>
  */
 l_int32
 ptraaInsertPtra(L_PTRAA  *paa,
@@ -909,21 +919,23 @@ l_int32  n;
 
 
 /*!
- *  ptraaGetPtra()
+ * \brief   ptraaGetPtra()
  *
- *      Input:  ptraa
- *              index (location in array)
- *              accessflag (L_HANDLE_ONLY, L_REMOVE)
- *      Return: ptra (at index location), or NULL on error or if there
+ * \param[in]    paa ptraa
+ * \param[in]    index location in array
+ * \param[in]    accessflag L_HANDLE_ONLY, L_REMOVE
+ * \return  ptra at index location, or NULL on error or if there
  *              is no ptra there.
  *
- *  Notes:
- *      (1) This returns the ptra ptr.  If @accessflag == L_HANDLE_ONLY,
- *          the ptra is left on the ptraa.  If @accessflag == L_REMOVE,
+ * <pre>
+ * Notes:
+ *      (1) This returns the ptra ptr.  If %accessflag == L_HANDLE_ONLY,
+ *          the ptra is left on the ptraa.  If %accessflag == L_REMOVE,
  *          the ptr in the ptraa is set to NULL, and the caller
  *          is responsible for disposing of the ptra (either putting it
  *          back on the ptraa, or destroying it).
  *      (2) This returns NULL if there is no Ptra at the index location.
+ * </pre>
  */
 L_PTRA *
 ptraaGetPtra(L_PTRAA  *paa,
@@ -954,16 +966,18 @@ L_PTRA  *pa;
  *                             Ptraa conversion                             *
  *--------------------------------------------------------------------------*/
 /*!
- *  ptraaFlattenToPtra()
+ * \brief   ptraaFlattenToPtra()
  *
- *      Input:  ptraa
- *      Return: ptra, or null on error
+ * \param[in]    paa ptraa
+ * \return  ptra, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This 'flattens' the ptraa to a ptra, taking the items in
  *          each ptra, in order, starting with the first ptra, etc.
  *      (2) As a side-effect, the ptra are all removed from the ptraa
  *          and destroyed, leaving an empty ptraa.
+ * </pre>
  */
 L_PTRA *
 ptraaFlattenToPtra(L_PTRAA  *paa)
