@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  seedfilllow.c
+/*!
+ * \file seedfilllow.c
+ * <pre>
  *
  *      Seedfill:
  *      Gray seedfill (source: Luc Vincent:fast-hybrid-grayscale-reconstruction)
@@ -30,10 +42,9 @@
  *      Seed spread:
  *               void   seedspreadLow()
  *
+ * </pre>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <math.h>
 #include "allheaders.h"
 
@@ -49,7 +60,7 @@ typedef struct L_Pixel  L_PIXEL;
  *                 Vincent's Iterative Binary Seedfill                   *
  *-----------------------------------------------------------------------*/
 /*!
- *  seedfillBinaryLow()
+ * \brief   seedfillBinaryLow()
  *
  *  Notes:
  *      (1) This is an in-place fill, where the seed image is
@@ -239,7 +250,8 @@ l_uint32  *lines, *linem;
         break;
 
     default:
-        L_ERROR("connectivity must be 4 or 8", procName);
+        L_ERROR("connectivity must be 4 or 8\n", procName);
+        return;
     }
 
     return;
@@ -251,7 +263,7 @@ l_uint32  *lines, *linem;
  *                 Vincent's Hybrid Grayscale Seedfill                *
  *-----------------------------------------------------------------------*/
 /*!
- *  seedfillGrayLow()
+ * \brief   seedfillGrayLow()
  *
  *  Notes:
  *      (1) The pixels are numbered as follows:
@@ -313,6 +325,11 @@ L_QUEUE  *lq_pixel;
 
     PROCNAME("seedfillGrayLow");
 
+    if (connectivity != 4 && connectivity != 8) {
+        L_ERROR("connectivity must be 4 or 8\n", procName);
+        return;
+    }
+
     imax = h - 1;
     jmax = w - 1;
 
@@ -393,7 +410,7 @@ L_QUEUE  *lq_pixel;
                         }
                     }
                     if (boolval) {
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -417,7 +434,7 @@ L_QUEUE  *lq_pixel;
             pixel = (L_PIXEL *)lqueueRemove(lq_pixel);
             i = pixel->x;
             j = pixel->y;
-            FREE(pixel);
+            LEPT_FREE(pixel);
             lines = datas + i * wpls;
             linem = datam + i * wplm;
 
@@ -427,7 +444,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem - wplm, j);
                     if (val > val2 && val2 != maskval) {
                         SET_DATA_BYTE(lines - wpls, j, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i - 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -439,7 +456,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j - 1);
                     if (val > val4 && val4 != maskval) {
                         SET_DATA_BYTE(lines, j - 1, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j - 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -450,7 +467,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem + wplm, j);
                     if (val > val7 && val7 != maskval) {
                         SET_DATA_BYTE(lines + wpls, j, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i + 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -461,7 +478,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j + 1);
                     if (val > val5 && val5 != maskval) {
                         SET_DATA_BYTE(lines, j + 1, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j + 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -573,7 +590,7 @@ L_QUEUE  *lq_pixel;
                         }
                     }
                     if (boolval) {
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -597,7 +614,7 @@ L_QUEUE  *lq_pixel;
             pixel = (L_PIXEL *)lqueueRemove(lq_pixel);
             i = pixel->x;
             j = pixel->y;
-            FREE(pixel);
+            LEPT_FREE(pixel);
             lines = datas + i * wpls;
             linem = datam + i * wplm;
 
@@ -607,8 +624,9 @@ L_QUEUE  *lq_pixel;
                         val1 = GET_DATA_BYTE(lines - wpls, j - 1);
                         maskval = GET_DATA_BYTE(linem - wplm, j - 1);
                         if (val > val1 && val1 != maskval) {
-                            SET_DATA_BYTE(lines - wpls, j - 1, L_MIN(val, maskval));
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            SET_DATA_BYTE(lines - wpls, j - 1,
+                                          L_MIN(val, maskval));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i - 1;
                             pixel->y = j - 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -618,8 +636,9 @@ L_QUEUE  *lq_pixel;
                         val3 = GET_DATA_BYTE(lines - wpls, j + 1);
                         maskval = GET_DATA_BYTE(linem - wplm, j + 1);
                         if (val > val3 && val3 != maskval) {
-                            SET_DATA_BYTE(lines - wpls, j + 1, L_MIN(val, maskval));
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            SET_DATA_BYTE(lines - wpls, j + 1,
+                                          L_MIN(val, maskval));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i - 1;
                             pixel->y = j + 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -629,7 +648,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem - wplm, j);
                     if (val > val2 && val2 != maskval) {
                         SET_DATA_BYTE(lines - wpls, j, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i - 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -641,7 +660,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j - 1);
                     if (val > val4 && val4 != maskval) {
                         SET_DATA_BYTE(lines, j - 1, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j - 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -652,8 +671,9 @@ L_QUEUE  *lq_pixel;
                         val6 = GET_DATA_BYTE(lines + wpls, j - 1);
                         maskval = GET_DATA_BYTE(linem + wplm, j - 1);
                         if (val > val6 && val6 != maskval) {
-                            SET_DATA_BYTE(lines + wpls, j - 1, L_MIN(val, maskval));
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            SET_DATA_BYTE(lines + wpls, j - 1,
+                                          L_MIN(val, maskval));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i + 1;
                             pixel->y = j - 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -663,8 +683,9 @@ L_QUEUE  *lq_pixel;
                         val8 = GET_DATA_BYTE(lines + wpls, j + 1);
                         maskval = GET_DATA_BYTE(linem + wplm, j + 1);
                         if (val > val8 && val8 != maskval) {
-                            SET_DATA_BYTE(lines + wpls, j + 1, L_MIN(val, maskval));
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            SET_DATA_BYTE(lines + wpls, j + 1,
+                                          L_MIN(val, maskval));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i + 1;
                             pixel->y = j + 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -674,7 +695,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem + wplm, j);
                     if (val > val7 && val7 != maskval) {
                         SET_DATA_BYTE(lines + wpls, j, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i + 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -685,7 +706,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j + 1);
                     if (val > val5 && val5 != maskval) {
                         SET_DATA_BYTE(lines, j + 1, L_MIN(val, maskval));
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j + 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -698,8 +719,8 @@ L_QUEUE  *lq_pixel;
         break;
 
     default:
-        L_ERROR("connectivity must be 4 or 8", procName);
-        lqueueDestroy(&lq_pixel, TRUE);
+        L_ERROR("shouldn't get here!\n", procName);
+        break;
     }
 
     lqueueDestroy(&lq_pixel, TRUE);
@@ -708,7 +729,7 @@ L_QUEUE  *lq_pixel;
 
 
 /*!
- *  seedfillGrayInvLow()
+ * \brief   seedfillGrayInvLow()
  *
  *  Notes:
  *      (1) The pixels are numbered as follows:
@@ -718,7 +739,7 @@ L_QUEUE  *lq_pixel;
  *          This low-level filling operation consists of two scans,
  *          raster and anti-raster, covering the entire seed image.
  *          During the anti-raster scan, every pixel p such that its
- *          current value could still be propogated during the next
+ *          current value could still be propagated during the next
  *          raster scanning is put into the FIFO-queue.
  *          Next step is the propagation step where where we update
  *          and propagate the values using FIFO structure created in
@@ -758,6 +779,11 @@ L_QUEUE  *lq_pixel;
 
     PROCNAME("seedfillGrayInvLow");
 
+    if (connectivity != 4 && connectivity != 8) {
+        L_ERROR("connectivity must be 4 or 8\n", procName);
+        return;
+    }
+
     imax = h - 1;
     jmax = w - 1;
 
@@ -845,7 +871,7 @@ L_QUEUE  *lq_pixel;
                         }
                     }
                     if (boolval) {
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -869,7 +895,7 @@ L_QUEUE  *lq_pixel;
             pixel = (L_PIXEL *)lqueueRemove(lq_pixel);
             i = pixel->x;
             j = pixel->y;
-            FREE(pixel);
+            LEPT_FREE(pixel);
             lines = datas + i * wpls;
             linem = datam + i * wplm;
 
@@ -879,7 +905,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem - wplm, j);
                     if (val > val2 && val > maskval) {
                         SET_DATA_BYTE(lines - wpls, j, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i - 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -891,7 +917,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j - 1);
                     if (val > val4 && val > maskval) {
                         SET_DATA_BYTE(lines, j - 1, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j - 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -902,7 +928,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem + wplm, j);
                     if (val > val7 && val > maskval) {
                         SET_DATA_BYTE(lines + wpls, j, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i + 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -913,7 +939,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j + 1);
                     if (val > val5 && val > maskval) {
                         SET_DATA_BYTE(lines, j + 1, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j + 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -1032,7 +1058,7 @@ L_QUEUE  *lq_pixel;
                         }
                     }
                     if (boolval) {
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -1056,7 +1082,7 @@ L_QUEUE  *lq_pixel;
             pixel = (L_PIXEL *)lqueueRemove(lq_pixel);
             i = pixel->x;
             j = pixel->y;
-            FREE(pixel);
+            LEPT_FREE(pixel);
             lines = datas + i * wpls;
             linem = datam + i * wplm;
 
@@ -1067,7 +1093,7 @@ L_QUEUE  *lq_pixel;
                         maskval = GET_DATA_BYTE(linem - wplm, j - 1);
                         if (val > val1 && val > maskval) {
                             SET_DATA_BYTE(lines - wpls, j - 1, val);
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i - 1;
                             pixel->y = j - 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -1078,7 +1104,7 @@ L_QUEUE  *lq_pixel;
                         maskval = GET_DATA_BYTE(linem - wplm, j + 1);
                         if (val > val3 && val > maskval) {
                             SET_DATA_BYTE(lines - wpls, j + 1, val);
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i - 1;
                             pixel->y = j + 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -1088,7 +1114,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem - wplm, j);
                     if (val > val2 && val > maskval) {
                         SET_DATA_BYTE(lines - wpls, j, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i - 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -1100,7 +1126,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j - 1);
                     if (val > val4 && val > maskval) {
                         SET_DATA_BYTE(lines, j - 1, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j - 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -1112,7 +1138,7 @@ L_QUEUE  *lq_pixel;
                         maskval = GET_DATA_BYTE(linem + wplm, j - 1);
                         if (val > val6 && val > maskval) {
                             SET_DATA_BYTE(lines + wpls, j - 1, val);
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i + 1;
                             pixel->y = j - 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -1123,7 +1149,7 @@ L_QUEUE  *lq_pixel;
                         maskval = GET_DATA_BYTE(linem + wplm, j + 1);
                         if (val > val8 && val > maskval) {
                             SET_DATA_BYTE(lines + wpls, j + 1, val);
-                            pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                            pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                             pixel->x = i + 1;
                             pixel->y = j + 1;
                             lqueueAdd(lq_pixel, pixel);
@@ -1133,7 +1159,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem + wplm, j);
                     if (val > val7 && val > maskval) {
                         SET_DATA_BYTE(lines + wpls, j, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i + 1;
                         pixel->y = j;
                         lqueueAdd(lq_pixel, pixel);
@@ -1144,7 +1170,7 @@ L_QUEUE  *lq_pixel;
                     maskval = GET_DATA_BYTE(linem, j + 1);
                     if (val > val5 && val > maskval) {
                         SET_DATA_BYTE(lines, j + 1, val);
-                        pixel = (L_PIXEL *)CALLOC(1, sizeof(L_PIXEL));
+                        pixel = (L_PIXEL *)LEPT_CALLOC(1, sizeof(L_PIXEL));
                         pixel->x = i;
                         pixel->y = j + 1;
                         lqueueAdd(lq_pixel, pixel);
@@ -1157,8 +1183,8 @@ L_QUEUE  *lq_pixel;
         break;
 
     default:
-        lqueueDestroy(&lq_pixel, TRUE);
-        L_ERROR("connectivity must be 4 or 8", procName);
+        L_ERROR("shouldn't get here!\n", procName);
+        break;
     }
 
     lqueueDestroy(&lq_pixel, TRUE);
@@ -1169,7 +1195,7 @@ L_QUEUE  *lq_pixel;
  *                 Vincent's Iterative Grayscale Seedfill                *
  *-----------------------------------------------------------------------*/
 /*!
- *  seedfillGrayLowSimple()
+ * \brief   seedfillGrayLowSimple()
  *
  *  Notes:
  *      (1) The pixels are numbered as follows:
@@ -1327,7 +1353,7 @@ l_uint32  *lines, *linem;
         break;
 
     default:
-        L_ERROR("connectivity must be 4 or 8", procName);
+        L_ERROR("connectivity must be 4 or 8\n", procName);
     }
 
     return;
@@ -1335,7 +1361,7 @@ l_uint32  *lines, *linem;
 
 
 /*!
- *  seedfillGrayInvLowSimple()
+ * \brief   seedfillGrayInvLowSimple()
  *
  *  Notes:
  *      (1) The pixels are numbered as follows:
@@ -1485,7 +1511,7 @@ l_uint32  *lines, *linem;
         break;
 
     default:
-        L_ERROR("connectivity must be 4 or 8", procName);
+        L_ERROR("connectivity must be 4 or 8\n", procName);
     }
 
     return;
@@ -1496,7 +1522,7 @@ l_uint32  *lines, *linem;
  *                   Vincent's Distance Function method                  *
  *-----------------------------------------------------------------------*/
 /*!
- *  distanceFunctionLow()
+ * \brief   distanceFunctionLow()
  */
 void
 distanceFunctionLow(l_uint32  *datad,
@@ -1548,8 +1574,7 @@ l_uint32  *lined;
                     }
                 }
             }
-        }
-        else {  /* d == 16 */
+        } else {  /* d == 16 */
                 /* UL --> LR scan */
             for (i = 1; i < imax; i++) {
                 lined = datad + i * wpld;
@@ -1617,8 +1642,7 @@ l_uint32  *lined;
                     }
                 }
             }
-        }
-        else {  /* d == 16 */
+        } else {  /* d == 16 */
                 /* UL --> LR scan */
             for (i = 1; i < imax; i++) {
                 lined = datad + i * wpld;
@@ -1658,7 +1682,7 @@ l_uint32  *lined;
         break;
 
     default:
-        L_ERROR("connectivity must be 4 or 8", procName);
+        L_ERROR("connectivity must be 4 or 8\n", procName);
         break;
     }
 
@@ -1670,7 +1694,7 @@ l_uint32  *lined;
  *                 Seed spread (based on distance function)              *
  *-----------------------------------------------------------------------*/
 /*!
- *  seedspreadLow()
+ * \brief   seedspreadLow()
  *
  *    See pixSeedspread() for a brief description of the algorithm here.
  */
@@ -1801,7 +1825,7 @@ l_uint32  *linet, *lined;
         }
         break;
     default:
-        L_ERROR("connectivity must be 4 or 8", procName);
+        L_ERROR("connectivity must be 4 or 8\n", procName);
         break;
     }
 
