@@ -1,20 +1,32 @@
-#/*====================================================================*
-# -  Copyright (C) 2001 Leptonica.  All rights reserved.
-# -  This software is distributed in the hope that it will be
-# -  useful, but with NO WARRANTY OF ANY KIND.
-# -  No author or distributor accepts responsibility to anyone for the
-# -  consequences of using this software, or for whether it serves any
-# -  particular purpose or works at all, unless he or she says so in
-# -  writing.  Everyone is granted permission to copy, modify and
-# -  redistribute this source code, for commercial or non-commercial
-# -  purposes, with the following restrictions: (1) the origin of this
-# -  source code must not be misrepresented; (2) modified versions must
-# -  be plainly marked as such; and (3) this notice may not be removed
-# -  or altered from any source or modified source distribution.
-# *====================================================================*/
+/*====================================================================*
+ -  Copyright (C) 2001 Leptonica.  All rights reserved.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *====================================================================*/
 
-/*
- *  skew.c
+/*!
+ * \file skew.c
+ * <pre>
  *
  *      Top-level deskew interfaces
  *          PIX       *pixDeskew()
@@ -40,7 +52,7 @@
  *          l_int32    pixFindNormalizedSquareSum()
  *
  *
- *      ==============================================================    
+ *      ==============================================================
  *      Page skew detection
  *
  *      Skew is determined by pixel profiles, which are computed
@@ -79,6 +91,7 @@
  *      significantly over the page.  Local skew determination
  *      is not very important except for locating lines of
  *      handwritten text that may be mixed with printed text.
+ * </pre>
  */
 
 #include <math.h>
@@ -121,7 +134,7 @@ static const l_int32  DEFAULT_BINARY_THRESHOLD = 130;
 #define  DEBUG_PRINT_BINARY     0
 #define  DEBUG_PRINT_ORTH       0
 #define  DEBUG_THRESHOLD        0
-#define  DEBUG_PLOT_SCORES      0
+#define  DEBUG_PLOT_SCORES      0  /* requires the gnuplot executable */
 #endif  /* ~NO_CONSOLE_IO */
 
 
@@ -130,17 +143,19 @@ static const l_int32  DEFAULT_BINARY_THRESHOLD = 130;
  *                       Top-level deskew interfaces                     *
  *-----------------------------------------------------------------------*/
 /*!
- *  pixDeskew()
+ * \brief   pixDeskew()
  *
- *      Input:  pixs (any depth)
- *              redsearch (for binary search: reduction factor = 1, 2 or 4;
- *                         use 0 for default)
- *      Return: pixd (deskewed pix), or null on error
+ * \param[in]    pixs any depth
+ * \param[in]    redsearch for binary search: reduction factor = 1, 2 or 4;
+ *                         use 0 for default
+ * \return  pixd deskewed pix, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This binarizes if necessary and finds the skew angle.  If the
  *          angle is large enough and there is sufficient confidence,
  *          it returns a deskewed image; otherwise, it returns a clone.
+ * </pre>
  */
 PIX *
 pixDeskew(PIX     *pixs,
@@ -160,21 +175,23 @@ pixDeskew(PIX     *pixs,
 
 
 /*!
- *  pixFindSkewAndDeskew()
+ * \brief   pixFindSkewAndDeskew()
  *
- *      Input:  pixs (any depth)
- *              redsearch (for binary search: reduction factor = 1, 2 or 4;
- *                         use 0 for default)
- *              &angle   (<optional return> angle required to deskew,
- *                        in degrees; use NULL to skip)
- *              &conf    (<optional return> conf value is ratio
- *                        of max/min scores; use NULL to skip)
- *      Return: pixd (deskewed pix), or null on error
+ * \param[in]    pixs any depth
+ * \param[in]    redsearch for binary search: reduction factor = 1, 2 or 4;
+ *                         use 0 for default
+ * \param[out]   pangle   [optional] angle required to deskew,
+ *                        in degrees; use NULL to skip
+ * \param[out]   pconf    [optional] conf value is ratio
+ *                        of max/min scores; use NULL to skip
+ * \return  pixd deskewed pix, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This binarizes if necessary and finds the skew angle.  If the
  *          angle is large enough and there is sufficient confidence,
  *          it returns a deskewed image; otherwise, it returns a clone.
+ * </pre>
  */
 PIX *
 pixFindSkewAndDeskew(PIX        *pixs,
@@ -196,27 +213,29 @@ pixFindSkewAndDeskew(PIX        *pixs,
 
 
 /*!
- *  pixDeskewGeneral()
+ * \brief   pixDeskewGeneral()
  *
- *      Input:  pixs  (any depth)
- *              redsweep  (for linear search: reduction factor = 1, 2 or 4;
- *                         use 0 for default)
- *              sweeprange (in degrees in each direction from 0;
- *                          use 0.0 for default)
- *              sweepdelta (in degrees; use 0.0 for default)
- *              redsearch  (for binary search: reduction factor = 1, 2 or 4;
- *                          use 0 for default;)
- *              thresh (for binarizing the image; use 0 for default)
- *              &angle   (<optional return> angle required to deskew,
- *                        in degrees; use NULL to skip)
- *              &conf    (<optional return> conf value is ratio
- *                        of max/min scores; use NULL to skip)
- *      Return: pixd (deskewed pix), or null on error
+ * \param[in]    pixs  any depth
+ * \param[in]    redsweep  for linear search: reduction factor = 1, 2 or 4;
+ *                         use 0 for default
+ * \param[in]    sweeprange in degrees in each direction from 0;
+ *                          use 0.0 for default
+ * \param[in]    sweepdelta in degrees; use 0.0 for default
+ * \param[in]    redsearch  for binary search: reduction factor = 1, 2 or 4;
+ *                          use 0 for default;
+ * \param[in]    thresh for binarizing the image; use 0 for default
+ * \param[out]   pangle   [optional] angle required to deskew,
+ *                        in degrees; use NULL to skip
+ * \param[out]   pconf    [optional] conf value is ratio
+ *                        of max/min scores; use NULL to skip
+ * \return  pixd deskewed pix, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This binarizes if necessary and finds the skew angle.  If the
  *          angle is large enough and there is sufficient confidence,
  *          it returns a deskewed image; otherwise, it returns a clone.
+ * </pre>
  */
 PIX *
 pixDeskewGeneral(PIX        *pixs,
@@ -234,6 +253,8 @@ PIX       *pixb, *pixd;
 
     PROCNAME("pixDeskewGeneral");
 
+    if (pangle) *pangle = 0.0;
+    if (pconf) *pconf = 0.0;
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
     if (redsweep == 0)
@@ -265,10 +286,8 @@ PIX       *pixb, *pixd;
                                     sweeprange, sweepdelta,
                                     DEFAULT_MINBS_DELTA);
     pixDestroy(&pixb);
-    if (pangle)
-        *pangle = angle;
-    if (pconf)
-        *pconf = conf;
+    if (pangle) *pangle = angle;
+    if (pconf) *pconf = conf;
     if (ret)
         return pixClone(pixs);
 
@@ -287,19 +306,21 @@ PIX       *pixb, *pixd;
  *                  Simple top-level angle-finding interface             *
  *-----------------------------------------------------------------------*/
 /*!
- *  pixFindSkew()
+ * \brief   pixFindSkew()
  *
- *      Input:  pixs  (1 bpp)
- *              &angle   (<return> angle required to deskew, in degrees)
- *              &conf    (<return> confidence value is ratio max/min scores)
- *      Return: 0 if OK, 1 on error or if angle measurment not valid
+ * \param[in]    pixs  1 bpp
+ * \param[out]   pangle   angle required to deskew, in degrees
+ * \param[out]   pconf    confidence value is ratio max/min scores
+ * \return  0 if OK, 1 on error or if angle measurment not valid
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This is a simple high-level interface, that uses default
  *          values of the parameters for reasonable speed and accuracy.
  *      (2) The angle returned is the negative of the skew angle of
  *          the image.  It is the angle required for deskew.
  *          Clockwise rotations are positive angles.
+ * </pre>
  */
 l_int32
 pixFindSkew(PIX        *pixs,
@@ -308,14 +329,14 @@ pixFindSkew(PIX        *pixs,
 {
     PROCNAME("pixFindSkew");
 
+    if (pangle) *pangle = 0.0;
+    if (pconf) *pconf = 0.0;
+    if (!pangle || !pconf)
+        return ERROR_INT("&angle and/or &conf not defined", procName, 1);
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     if (pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not 1 bpp", procName, 1);
-    if (!pangle)
-        return ERROR_INT("&angle not defined", procName, 1);
-    if (!pconf)
-        return ERROR_INT("&conf not defined", procName, 1);
 
     return pixFindSkewSweepAndSearch(pixs, pangle, pconf,
                                      DEFAULT_SWEEP_REDUCTION,
@@ -330,18 +351,20 @@ pixFindSkew(PIX        *pixs,
  *                       Basic angle-finding functions                   *
  *-----------------------------------------------------------------------*/
 /*!
- *  pixFindSkewSweep()
+ * \brief   pixFindSkewSweep()
  *
- *      Input:  pixs  (1 bpp)
- *              &angle   (<return> angle required to deskew, in degrees)
- *              reduction  (factor = 1, 2, 4 or 8)
- *              sweeprange   (half the full range; assumed about 0; in degrees)
- *              sweepdelta   (angle increment of sweep; in degrees)
- *      Return: 0 if OK, 1 on error or if angle measurment not valid
+ * \param[in]    pixs  1 bpp
+ * \param[out]   pangle   angle required to deskew, in degrees
+ * \param[in]    reduction  factor = 1, 2, 4 or 8
+ * \param[in]    sweeprange   half the full range; assumed about 0; in degrees
+ * \param[in]    sweepdelta   angle increment of sweep; in degrees
+ * \return  0 if OK, 1 on error or if angle measurment not valid
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This examines the 'score' for skew angles with equal intervals.
  *      (2) Caller must check the return value for validity of the result.
+ * </pre>
  */
 l_int32
 pixFindSkewSweep(PIX        *pixs,
@@ -358,16 +381,16 @@ PIX       *pix, *pixt;
 
     PROCNAME("pixFindSkewSweep");
 
+    if (!pangle)
+        return ERROR_INT("&angle not defined", procName, 1);
+    *pangle = 0.0;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     if (pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not 1 bpp", procName, 1);
-    if (!pangle)
-        return ERROR_INT("&angle not defined", procName, 1);
     if (reduction != 1 && reduction != 2 && reduction != 4 && reduction != 8)
         return ERROR_INT("reduction must be in {1,2,4,8}", procName, 1);
 
-    *pangle = 0.0;  /* init */
     deg2rad = 3.1415926535 / 180.;
     ret = 0;
 
@@ -411,7 +434,7 @@ PIX       *pix, *pixt;
         pixFindDifferentialSquareSum(pixt, &sum);
 
 #if  DEBUG_PRINT_SCORES
-        L_INFO_FLOAT2("sum(%7.2f) = %7.0f", procName, theta, sum);
+        L_INFO("sum(%7.2f) = %7.0f\n", procName, theta, sum);
 #endif  /* DEBUG_PRINT_SCORES */
 
             /* Save the result in the output arrays */
@@ -426,8 +449,8 @@ PIX       *pix, *pixt;
     *pangle = maxangle;
 
 #if  DEBUG_PRINT_SWEEP
-    L_INFO_FLOAT2(" From sweep: angle = %7.3f, score = %7.3f", procName,
-                  maxangle, maxscore);
+    L_INFO(" From sweep: angle = %7.3f, score = %7.3f\n", procName,
+           maxangle, maxscore);
 #endif  /* DEBUG_PRINT_SWEEP */
 
 #if  DEBUG_PLOT_SCORES
@@ -455,35 +478,37 @@ cleanup:
     pixDestroy(&pixt);
     numaDestroy(&nascore);
     numaDestroy(&natheta);
-    return 0;
+    return ret;
 }
 
 
 /*!
- *  pixFindSkewSweepAndSearch()
+ * \brief   pixFindSkewSweepAndSearch()
  *
- *      Input:  pixs  (1 bpp)
- *              &angle   (<return> angle required to deskew; in degrees)
- *              &conf    (<return> confidence given by ratio of max/min score)
- *              redsweep  (sweep reduction factor = 1, 2, 4 or 8)
- *              redsearch  (binary search reduction factor = 1, 2, 4 or 8;
- *                          and must not exceed redsweep)
- *              sweeprange   (half the full range, assumed about 0; in degrees)
- *              sweepdelta   (angle increment of sweep; in degrees)
- *              minbsdelta   (min binary search increment angle; in degrees)
- *      Return: 0 if OK, 1 on error or if angle measurment not valid
+ * \param[in]    pixs  1 bpp
+ * \param[out]   pangle   angle required to deskew; in degrees
+ * \param[out]   pconf    confidence given by ratio of max/min score
+ * \param[in]    redsweep  sweep reduction factor = 1, 2, 4 or 8
+ * \param[in]    redsearch  binary search reduction factor = 1, 2, 4 or 8;
+ *                          and must not exceed redsweep
+ * \param[in]    sweeprange   half the full range, assumed about 0; in degrees
+ * \param[in]    sweepdelta   angle increment of sweep; in degrees
+ * \param[in]    minbsdelta   min binary search increment angle; in degrees
+ * \return  0 if OK, 1 on error or if angle measurment not valid
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This finds the skew angle, doing first a sweep through a set
  *          of equal angles, and then doing a binary search until
  *          convergence.
  *      (2) Caller must check the return value for validity of the result.
  *      (3) In computing the differential line sum variance score, we sum
  *          the result over scanlines, but we always skip:
- *           - at least one scanline
- *           - not more than 10% of the image height
- *           - not more than 5% of the image width
+ *           ~ at least one scanline
+ *           ~ not more than 10% of the image height
+ *           ~ not more than 5% of the image width
  *      (4) See also notes in pixFindSkewSweepAndSearchScore()
+ * </pre>
  */
 l_int32
 pixFindSkewSweepAndSearch(PIX        *pixs,
@@ -496,35 +521,36 @@ pixFindSkewSweepAndSearch(PIX        *pixs,
                           l_float32   minbsdelta)
 {
     return pixFindSkewSweepAndSearchScore(pixs, pangle, pconf, NULL,
-                                          redsweep, redsearch, 0.0, sweeprange, 
+                                          redsweep, redsearch, 0.0, sweeprange,
                                           sweepdelta, minbsdelta);
 }
 
 
 /*!
- *  pixFindSkewSweepAndSearchScore()
+ * \brief   pixFindSkewSweepAndSearchScore()
  *
- *      Input:  pixs  (1 bpp)
- *              &angle   (<return> angle required to deskew; in degrees)
- *              &conf    (<return> confidence given by ratio of max/min score)
- *              &endscore (<optional return> max score; use NULL to ignore)
- *              redsweep  (sweep reduction factor = 1, 2, 4 or 8)
- *              redsearch  (binary search reduction factor = 1, 2, 4 or 8;
- *                          and must not exceed redsweep)
- *              sweepcenter  (angle about which sweep is performed; in degrees)
- *              sweeprange   (half the full range, taken about sweepcenter;
- *                            in degrees)
- *              sweepdelta   (angle increment of sweep; in degrees)
- *              minbsdelta   (min binary search increment angle; in degrees)
- *      Return: 0 if OK, 1 on error or if angle measurment not valid
+ * \param[in]    pixs  1 bpp
+ * \param[out]   pangle   angle required to deskew; in degrees
+ * \param[out]   pconf    confidence given by ratio of max/min score
+ * \param[out]   pendscore [optional] max score; use NULL to ignore
+ * \param[in]    redsweep  sweep reduction factor = 1, 2, 4 or 8
+ * \param[in]    redsearch  binary search reduction factor = 1, 2, 4 or 8;
+ *                          and must not exceed redsweep
+ * \param[in]    sweepcenter  angle about which sweep is performed; in degrees
+ * \param[in]    sweeprange   half the full range, taken about sweepcenter;
+ *                            in degrees
+ * \param[in]    sweepdelta   angle increment of sweep; in degrees
+ * \param[in]    minbsdelta   min binary search increment angle; in degrees
+ * \return  0 if OK, 1 on error or if angle measurment not valid
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This finds the skew angle, doing first a sweep through a set
  *          of equal angles, and then doing a binary search until convergence.
- *      (2) There are two built-in constants that determine if the 
+ *      (2) There are two built-in constants that determine if the
  *          returned confidence is nonzero:
- *            - MIN_VALID_MAXSCORE (minimum allowed maxscore)
- *            - MINSCORE_THRESHOLD_CONSTANT (determines minimum allowed
+ *            ~ MIN_VALID_MAXSCORE (minimum allowed maxscore)
+ *            ~ MINSCORE_THRESHOLD_CONSTANT (determines minimum allowed
  *                 minscore, by multiplying by (height * width^2)
  *          If either of these conditions is not satisfied, the returned
  *          confidence value will be zero.  The maxscore is optionally
@@ -536,6 +562,7 @@ pixFindSkewSweepAndSearch(PIX        *pixs,
  *          variance.  It should be compared to a threshold, which depends
  *          on the application.  Values between 3.0 and 6.0 are common.
  *      (4) By default, the shear is about the UL corner.
+ * </pre>
  */
 l_int32
 pixFindSkewSweepAndSearchScore(PIX        *pixs,
@@ -558,24 +585,25 @@ pixFindSkewSweepAndSearchScore(PIX        *pixs,
 
 
 /*!
- *  pixFindSkewSweepAndSearchScorePivot()
+ * \brief   pixFindSkewSweepAndSearchScorePivot()
  *
- *      Input:  pixs  (1 bpp)
- *              &angle   (<return> angle required to deskew; in degrees)
- *              &conf    (<return> confidence given by ratio of max/min score)
- *              &endscore (<optional return> max score; use NULL to ignore)
- *              redsweep  (sweep reduction factor = 1, 2, 4 or 8)
- *              redsearch  (binary search reduction factor = 1, 2, 4 or 8;
- *                          and must not exceed redsweep)
- *              sweepcenter  (angle about which sweep is performed; in degrees)
- *              sweeprange   (half the full range, taken about sweepcenter;
- *                            in degrees)
- *              sweepdelta   (angle increment of sweep; in degrees)
- *              minbsdelta   (min binary search increment angle; in degrees)
- *              pivot  (L_SHEAR_ABOUT_CORNER, L_SHEAR_ABOUT_CENTER)
- *      Return: 0 if OK, 1 on error or if angle measurment not valid
+ * \param[in]    pixs  1 bpp
+ * \param[out]   pangle   angle required to deskew; in degrees
+ * \param[out]   pconf    confidence given by ratio of max/min score
+ * \param[out]   pendscore [optional] max score; use NULL to ignore
+ * \param[in]    redsweep  sweep reduction factor = 1, 2, 4 or 8
+ * \param[in]    redsearch  binary search reduction factor = 1, 2, 4 or 8;
+ *                          and must not exceed redsweep
+ * \param[in]    sweepcenter  angle about which sweep is performed; in degrees
+ * \param[in]    sweeprange   half the full range, taken about sweepcenter;
+ *                            in degrees
+ * \param[in]    sweepdelta   angle increment of sweep; in degrees
+ * \param[in]    minbsdelta   min binary search increment angle; in degrees
+ * \param[in]    pivot  L_SHEAR_ABOUT_CORNER, L_SHEAR_ABOUT_CENTER
+ * \return  0 if OK, 1 on error or if angle measurment not valid
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See notes in pixFindSkewSweepAndSearchScore().
  *      (2) This allows choice of shear pivoting from either the UL corner
  *          or the center.  For small angles, the ability to discriminate
@@ -583,6 +611,7 @@ pixFindSkewSweepAndSearchScore(PIX        *pixs,
  *          for large angles (say, greater than 20 degrees), it is better
  *          to shear about the center because a shear from the UL corner
  *          loses too much of the image.
+ * </pre>
  */
 l_int32
 pixFindSkewSweepAndSearchScorePivot(PIX        *pixs,
@@ -611,14 +640,13 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
 
     PROCNAME("pixFindSkewSweepAndSearchScorePivot");
 
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
-    if (pixGetDepth(pixs) != 1)
-        return ERROR_INT("pixs not 1 bpp", procName, 1);
-    if (!pangle)
-        return ERROR_INT("&angle not defined", procName, 1);
-    if (!pconf)
-        return ERROR_INT("&conf not defined", procName, 1);
+    if (pendscore) *pendscore = 0.0;
+    if (pangle) *pangle = 0.0;
+    if (pconf) *pconf = 0.0;
+    if (!pangle || !pconf)
+        return ERROR_INT("&angle and/or &conf not defined", procName, 1);
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
     if (redsweep != 1 && redsweep != 2 && redsweep != 4 && redsweep != 8)
         return ERROR_INT("redsweep must be in {1,2,4,8}", procName, 1);
     if (redsearch != 1 && redsearch != 2 && redsearch != 4 && redsearch != 8)
@@ -628,8 +656,6 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
     if (pivot != L_SHEAR_ABOUT_CORNER && pivot != L_SHEAR_ABOUT_CENTER)
         return ERROR_INT("invalid pivot", procName, 1);
 
-    *pangle = 0.0;
-    *pconf = 0.0;
     deg2rad = 3.1415926535 / 180.;
     ret = 0;
 
@@ -651,9 +677,9 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
 
         /* Generate reduced image for sweep, if requested */
     ratio = redsweep / redsearch;
-    if (ratio == 1)
+    if (ratio == 1) {
         pixsw = pixClone(pixsch);
-    else {  /* ratio > 1 */
+    } else {  /* ratio > 1 */
         if (ratio == 2)
             pixsw = pixReduceRankBinaryCascade(pixsch, 1, 0, 0, 0);
         else if (ratio == 4)
@@ -700,7 +726,7 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
         pixFindDifferentialSquareSum(pixt1, &sum);
 
 #if  DEBUG_PRINT_SCORES
-        L_INFO_FLOAT2("sum(%7.2f) = %7.0f", procName, theta, sum);
+        L_INFO("sum(%7.2f) = %7.0f\n", procName, theta, sum);
 #endif  /* DEBUG_PRINT_SCORES */
 
             /* Save the result in the output arrays */
@@ -713,8 +739,8 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
     numaGetFValue(natheta, maxindex, &maxangle);
 
 #if  DEBUG_PRINT_SWEEP
-    L_INFO_FLOAT2(" From sweep: angle = %7.3f, score = %7.3f", procName,
-                  maxangle, maxscore);
+    L_INFO(" From sweep: angle = %7.3f, score = %7.3f\n", procName,
+           maxangle, maxscore);
 #endif  /* DEBUG_PRINT_SWEEP */
 
 #if  DEBUG_PLOT_SCORES
@@ -734,7 +760,7 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
         /* Check if the max is at the end of the sweep. */
     n = numaGetCount(natheta);
     if (maxindex == 0 || maxindex == n - 1) {
-        L_WARNING("max found at sweep edge", procName);
+        L_WARNING("max found at sweep edge\n", procName);
         goto cleanup;
     }
 
@@ -754,8 +780,7 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
         pixVShearCorner(pixt2, pixsch, deg2rad * (centerangle + sweepdelta),
                         L_BRING_IN_WHITE);
         pixFindDifferentialSquareSum(pixt2, &bsearchscore[4]);
-    }
-    else {
+    } else {
         pixVShearCenter(pixt2, pixsch, deg2rad * centerangle, L_BRING_IN_WHITE);
         pixFindDifferentialSquareSum(pixt2, &bsearchscore[2]);
         pixVShearCenter(pixt2, pixsch, deg2rad * (centerangle - sweepdelta),
@@ -788,7 +813,7 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
         pixFindDifferentialSquareSum(pixt2, &bsearchscore[1]);
         numaAddNumber(nascore, bsearchscore[1]);
         numaAddNumber(natheta, leftcenterangle);
-        
+
             /* Get the right intermediate score */
         rightcenterangle = centerangle + delta;
         if (pivot == L_SHEAR_ABOUT_CORNER)
@@ -800,7 +825,7 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
         pixFindDifferentialSquareSum(pixt2, &bsearchscore[3]);
         numaAddNumber(nascore, bsearchscore[3]);
         numaAddNumber(natheta, rightcenterangle);
-        
+
             /* Find the maximum of the five scores and its location.
              * Note that the maximum must be in the center
              * three values, not in the end two. */
@@ -827,7 +852,7 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
     *pangle = centerangle;
 
 #if  DEBUG_PRINT_SCORES
-    L_INFO_FLOAT(" Binary search score = %7.3f", procName, bsearchscore[2]);
+    L_INFO(" Binary search score = %7.3f\n", procName, bsearchscore[2]);
 #endif  /* DEBUG_PRINT_SCORES */
 
     if (pendscore)  /* save if requested */
@@ -850,9 +875,9 @@ PIX       *pixsw, *pixsch, *pixt1, *pixt2;
     minthresh = MINSCORE_THRESHOLD_CONSTANT * width * width * height;
 
 #if  DEBUG_THRESHOLD
-    L_INFO_FLOAT2(" minthresh = %10.2f, minscore = %10.2f", procName,
-            minthresh, minscore);
-    L_INFO_FLOAT(" maxscore = %10.2f", procName, maxscore);
+    L_INFO(" minthresh = %10.2f, minscore = %10.2f\n", procName,
+           minthresh, minscore);
+    L_INFO(" maxscore = %10.2f\n", procName, maxscore);
 #endif  /* DEBUG_THRESHOLD */
 
     if (minscore > minthresh)
@@ -927,7 +952,7 @@ cleanup:
  *          clockwise.  For exploring the full range of possibilities,
  *          suggest using sweeprange = 47.0 degrees, giving some overlap
  *          at 45 and 135 degrees.  From these results, and discounting
- *          the the second confidence by @confprior, it selects the
+ *          the the second confidence by %confprior, it selects the
  *          angle for maximal differential variance.  If the angle
  *          is larger than pi/4, the angle found after 90 degree rotation
  *          is selected.
@@ -948,16 +973,16 @@ cleanup:
  *            (b) If there are vertical lines in the margins, do not
  *                work below 150 ppi.  The signal from the text lines must
  *                exceed that from the margin lines.
- *      (4) Choosing the @confprior parameter depends on knowing something
+ *      (4) Choosing the %confprior parameter depends on knowing something
  *          about the source of image.  However, we're not using
  *          real probabilities here, so its use is qualitative.
  *          If landscape and portrait are equally likely, use
- *          @confprior = 0.0.  If the likelihood of portrait (non-rotated)
+ *          %confprior = 0.0.  If the likelihood of portrait (non-rotated)
  *          is 100 times higher than that of landscape, we want to reduce
  *          the chance that we rotate to landscape in a situation where
  *          the landscape signal is accidentally larger than the
  *          portrait signal.  To do this use a positive value of
- *          @confprior; say 1.5.
+ *          %confprior; say 1.5.
  */
 l_int32
 pixFindSkewOrthogonalRange(PIX        *pixs,
@@ -975,9 +1000,10 @@ PIX       *pixr;
 
     PROCNAME("pixFindSkewOrthogonalRange");
 
+    if (pangle) *pangle = 0.0;
+    if (pconf) *pconf = 0.0;
     if (!pangle || !pconf)
-        return ERROR_INT("&angle and &conf not both defined", procName, 1);
-    *pangle = *pconf = 0.0;
+        return ERROR_INT("&angle and/or &conf not defined", procName, 1);
     if (!pixs || pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
 
@@ -1017,17 +1043,19 @@ PIX       *pixr;
  *                  Differential square sum function              *
  *----------------------------------------------------------------*/
 /*!
- *  pixFindDifferentialSquareSum()
+ * \brief   pixFindDifferentialSquareSum()
  *
- *      Input:  pixs
- *              &sum  (<return> result)
- *      Return: 0 if OK, 1 on error
+ * \param[in]    pixs
+ * \param[out]   psum  result
+ * \return  0 if OK, 1 on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) At the top and bottom, we skip:
- *           - at least one scanline
- *           - not more than 10% of the image height
- *           - not more than 5% of the image width
+ *           ~ at least one scanline
+ *           ~ not more than 10% of the image height
+ *           ~ not more than 5% of the image width
+ * </pre>
  */
 l_int32
 pixFindDifferentialSquareSum(PIX        *pixs,
@@ -1040,6 +1068,9 @@ NUMA      *na;
 
     PROCNAME("pixFindDifferentialSquareSum");
 
+    if (!psum)
+        return ERROR_INT("&sum not defined", procName, 1);
+    *psum = 0.0;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
 
@@ -1077,17 +1108,18 @@ NUMA      *na;
  *                        Normalized square sum                   *
  *----------------------------------------------------------------*/
 /*!
- *  pixFindNormalizedSquareSum()
+ * \brief   pixFindNormalizedSquareSum()
  *
- *      Input:  pixs
- *              &hratio (<optional return> ratio of normalized horiz square sum
- *                       to result if the pixel distribution were uniform)
- *              &vratio (<optional return> ratio of normalized vert square sum
- *                       to result if the pixel distribution were uniform)
- *              &fract  (<optional return> ratio of fg pixels to total pixels)
- *      Return: 0 if OK, 1 on error or if there are no fg pixels
+ * \param[in]    pixs
+ * \param[out]   phratio [optional] ratio of normalized horiz square sum
+ *                       to result if the pixel distribution were uniform
+ * \param[out]   pvratio [optional] ratio of normalized vert square sum
+ *                       to result if the pixel distribution were uniform
+ * \param[out]   pfract  [optional] ratio of fg pixels to total pixels
+ * \return  0 if OK, 1 on error or if there are no fg pixels
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) Let the image have h scanlines and N fg pixels.
  *          If the pixels were uniformly distributed on scanlines,
  *          the sum of squares of fg pixels on each scanline would be
@@ -1096,6 +1128,7 @@ NUMA      *na;
  *          pixels will be larger.  We return in hratio and vratio the
  *          ratio of these two values.
  *      (2) If there are no fg pixels, hratio and vratio are returned as 0.0.
+ * </pre>
  */
 l_int32
 pixFindNormalizedSquareSum(PIX        *pixs,
@@ -1110,15 +1143,15 @@ PIX       *pixt;
 
     PROCNAME("pixFindNormalizedSquareSum");
 
+    if (phratio) *phratio = 0.0;
+    if (pvratio) *pvratio = 0.0;
+    if (pfract) *pfract = 0.0;
+    if (!phratio && !pvratio)
+        return ERROR_INT("nothing to do", procName, 1);
     if (!pixs || pixGetDepth(pixs) != 1)
         return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
     pixGetDimensions(pixs, &w, &h, NULL);
 
-    if (!phratio && !pvratio)
-        return ERROR_INT("nothing to do", procName, 1);
-    if (phratio) *phratio = 0.0;
-    if (pvratio) *pvratio = 0.0;
-
     empty = 0;
     if (phratio) {
         na = pixCountPixelsByRow(pixs, NULL);
@@ -1132,9 +1165,9 @@ PIX       *pixt;
                 sumsq += val * val;
             }
             *phratio = sumsq / uniform;
-        }
-        else
+        } else {
             empty = 1;
+        }
         numaDestroy(&na);
     }
 
@@ -1152,14 +1185,12 @@ PIX       *pixt;
                 sumsq += val * val;
             }
             *pvratio = sumsq / uniform;
-        }
-        else
+        } else {
             empty = 1;
+        }
         pixDestroy(&pixt);
         numaDestroy(&na);
     }
 
     return empty;
 }
-
-
