@@ -1,44 +1,57 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  tiffiostub.c
+/*!
+ * \file tiffiostub.c
+ * <pre>
  *
  *     Stubs for tiffio.c functions
+ * </pre>
  */
 
-#include "allheaders.h"
-
 #ifdef HAVE_CONFIG_H
 #include "config_auto.h"
 #endif  /* HAVE_CONFIG_H */
 
+#include "allheaders.h"
+
 /* --------------------------------------------*/
 #if  !HAVE_LIBTIFF   /* defined in environ.h */
 /* --------------------------------------------*/
 
 PIX * pixReadTiff(const char *filename, l_int32 n)
 {
-    return (PIX * )ERROR_PTR("function not present", "pixReadTiff", NULL);
+    return (PIX *)ERROR_PTR("function not present", "pixReadTiff", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
 
 PIX * pixReadStreamTiff(FILE *fp, l_int32 n)
 {
-    return (PIX * )ERROR_PTR("function not present", "pixReadStreamTiff", NULL);
+    return (PIX *)ERROR_PTR("function not present", "pixReadStreamTiff", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
@@ -67,10 +80,18 @@ l_int32 pixWriteStreamTiff(FILE *fp, PIX *pix, l_int32 comptype)
 
 /* ----------------------------------------------------------------------*/
 
+PIX * pixReadFromMultipageTiff(const char *filename, size_t *poffset)
+{
+    return (PIX *)ERROR_PTR("function not present",
+                            "pixReadFromMultipageTiff", NULL);
+}
+
+/* ----------------------------------------------------------------------*/
+
 PIXA * pixaReadMultipageTiff(const char *filename)
 {
-    return (PIXA * )ERROR_PTR("function not present",
-                              "pixaReadMultipageTiff", NULL);
+    return (PIXA *)ERROR_PTR("function not present",
+                             "pixaReadMultipageTiff", NULL);
 }
 
 /* ----------------------------------------------------------------------*/
@@ -162,6 +183,15 @@ PIX * pixReadMemTiff(const l_uint8 *cdata, size_t size, l_int32 n)
 
 /* ----------------------------------------------------------------------*/
 
+PIX * pixReadMemFromMultipageTiff(const l_uint8 *cdata, size_t size,
+                                  size_t *poffset)
+{
+    return (PIX *)ERROR_PTR("function not present",
+                            "pixReadMemFromMultipageTiff", NULL);
+}
+
+/* ----------------------------------------------------------------------*/
+
 l_int32 pixWriteMemTiff(l_uint8 **pdata, size_t *psize, PIX *pix,
                         l_int32 comptype)
 {
