@@ -1,20 +1,32 @@
 /*====================================================================*
  -  Copyright (C) 2001 Leptonica.  All rights reserved.
- -  This software is distributed in the hope that it will be
- -  useful, but with NO WARRANTY OF ANY KIND.
- -  No author or distributor accepts responsibility to anyone for the
- -  consequences of using this software, or for whether it serves any
- -  particular purpose or works at all, unless he or she says so in
- -  writing.  Everyone is granted permission to copy, modify and
- -  redistribute this source code, for commercial or non-commercial
- -  purposes, with the following restrictions: (1) the origin of this
- -  source code must not be misrepresented; (2) modified versions must
- -  be plainly marked as such; and (3) this notice may not be removed
- -  or altered from any source or modified source distribution.
+ -
+ -  Redistribution and use in source and binary forms, with or without
+ -  modification, are permitted provided that the following conditions
+ -  are met:
+ -  1. Redistributions of source code must retain the above copyright
+ -     notice, this list of conditions and the following disclaimer.
+ -  2. Redistributions in binary form must reproduce the above
+ -     copyright notice, this list of conditions and the following
+ -     disclaimer in the documentation and/or other materials
+ -     provided with the distribution.
+ -
+ -  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ -  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ -  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ -  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ANY
+ -  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ -  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ -  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ -  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
+ -  OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ -  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ -  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *====================================================================*/
 
-/*
- *  warper.c
+/*!
+ * \file warper.c
+ * <pre>
  *
  *      High-level captcha interface
  *          PIX               *pixSimpleCaptcha()
@@ -47,10 +59,9 @@
  *
  *      Stereo from a pair of images
  *          PIX               *pixStereoFromPair()
+ * </pre>
  */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <math.h>
 #include "allheaders.h"
 
@@ -74,21 +85,23 @@ static const l_float32  L_DEFAULT_BLUE_WEIGHT  = 0.3;
  *                High-level example captcha interface                  *
  *----------------------------------------------------------------------*/
 /*!
- *  pixSimpleCaptcha()
+ * \brief   pixSimpleCaptcha()
  *
- *      Input:  pixs (8 bpp; no colormap)
- *              border (added white pixels on each side)
- *              nterms (number of x and y harmonic terms)
- *              seed (of random number generator)
- *              color (for colorizing; in 0xrrggbb00 format; use 0 for black)
- *              cmapflag (1 for colormap output; 0 for rgb)
- *      Return: pixd (8 bpp cmap or 32 bpp rgb), or null on error
+ * \param[in]    pixs 8 bpp; no colormap
+ * \param[in]    border added white pixels on each side
+ * \param[in]    nterms number of x and y harmonic terms
+ * \param[in]    seed of random number generator
+ * \param[in]    color for colorizing; in 0xrrggbb00 format; use 0 for black
+ * \param[in]    cmapflag 1 for colormap output; 0 for rgb
+ * \return  pixd 8 bpp cmap or 32 bpp rgb, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This uses typical default values for generating captchas.
  *          The magnitudes of the harmonic warp are typically to be
  *          smaller when more terms are used, even though the phases
  *          are random.  See, for example, prog/warptest.c.
+ * </pre>
  */
 PIX *
 pixSimpleCaptcha(PIX      *pixs,
@@ -99,10 +112,10 @@ pixSimpleCaptcha(PIX      *pixs,
                  l_int32   cmapflag)
 {
 l_int32    k;
-l_float32  xmag[] = {7.0, 5.0, 4.0, 3.0};
-l_float32  ymag[] = {10.0, 8.0, 6.0, 5.0};
-l_float32  xfreq[] = {0.12, 0.10, 0.10, 0.11};
-l_float32  yfreq[] = {0.15, 0.13, 0.13, 0.11};
+l_float32  xmag[] = {7.0f, 5.0f, 4.0f, 3.0f};
+l_float32  ymag[] = {10.0f, 8.0f, 6.0f, 5.0f};
+l_float32  xfreq[] = {0.12f, 0.10f, 0.10f, 0.11f};
+l_float32  yfreq[] = {0.15f, 0.13f, 0.13f, 0.11f};
 PIX       *pixg, *pixgb, *pixw, *pixd;
 
     PROCNAME("pixSimpleCaptcha");
@@ -130,18 +143,19 @@ PIX       *pixg, *pixgb, *pixw, *pixd;
  *                     Random sinusoidal warping                        *
  *----------------------------------------------------------------------*/
 /*!
- *  pixRandomHarmonicWarp()
+ * \brief   pixRandomHarmonicWarp()
  *
- *      Input:  pixs (8 bpp; no colormap)
- *              xmag, ymag (maximum magnitude of x and y distortion)
- *              xfreq, yfreq (maximum magnitude of x and y frequency)
- *              nx, ny (number of x and y harmonic terms)
- *              seed (of random number generator)
- *              grayval (color brought in from the outside;
- *                       0 for black, 255 for white)
- *      Return: pixd (8 bpp; no colormap), or null on error
+ * \param[in]    pixs 8 bpp; no colormap
+ * \param[in]    xmag, ymag maximum magnitude of x and y distortion
+ * \param[in]    xfreq, yfreq maximum magnitude of x and y frequency
+ * \param[in]    nx, ny number of x and y harmonic terms
+ * \param[in]    seed of random number generator
+ * \param[in]    grayval color brought in from the outside;
+ *                       0 for black, 255 for white
+ * \return  pixd 8 bpp; no colormap, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) To generate the warped image p(x',y'), set up the transforms
  *          that are in getWarpTransform().  For each (x',y') in the
  *          dest, the warp function computes the originating location
@@ -161,6 +175,7 @@ PIX       *pixg, *pixgb, *pixw, *pixd;
  *                    xfreq = 0.10   yfreq = 0.13
  *                    nx = 3         ny = 3
  *          Other examples can be found in prog/warptest.c.
+ * </pre>
  */
 PIX *
 pixRandomHarmonicWarp(PIX       *pixs,
@@ -210,7 +225,7 @@ PIX        *pixd;
         }
     }
 
-    FREE(randa);
+    LEPT_FREE(randa);
     return pixd;
 }
 
@@ -226,7 +241,7 @@ l_float64  *randa;
 
     PROCNAME("generateRandomNumberArray");
 
-    if ((randa = (l_float64 *)CALLOC(size, sizeof(l_float64))) == NULL)
+    if ((randa = (l_float64 *)LEPT_CALLOC(size, sizeof(l_float64))) == NULL)
         return (l_float64 *)ERROR_PTR("calloc fail for randa", procName, NULL);
 
         /* Return random values between 0.5 and 1.0 */
@@ -237,7 +252,7 @@ l_float64  *randa;
 
 
 /*!
- *  applyWarpTransform()
+ * \brief   applyWarpTransform()
  *
  *  Notes:
  *      (1) Uses the internal sin function.
@@ -290,22 +305,24 @@ static l_float32 getSinFromLUT(l_float32 *tab, l_int32 npts,
                                l_float32 radang);
 
 /*!
- *  pixRandomHarmonicWarpLUT()
+ * \brief   pixRandomHarmonicWarpLUT()
  *
- *      Input:  pixs (8 bpp; no colormap)
- *              xmag, ymag (maximum magnitude of x and y distortion)
- *              xfreq, yfreq (maximum magnitude of x and y frequency)
- *              nx, ny (number of x and y harmonic terms)
- *              seed (of random number generator)
- *              grayval (color brought in from the outside;
- *                       0 for black, 255 for white)
- *      Return: pixd (8 bpp; no colormap), or null on error
+ * \param[in]    pixs 8 bpp; no colormap
+ * \param[in]    xmag, ymag maximum magnitude of x and y distortion
+ * \param[in]    xfreq, yfreq maximum magnitude of x and y frequency
+ * \param[in]    nx, ny number of x and y harmonic terms
+ * \param[in]    seed of random number generator
+ * \param[in]    grayval color brought in from the outside;
+ *                       0 for black, 255 for white
+ * \return  pixd 8 bpp; no colormap, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See notes and inline comments in pixRandomHarmonicWarp().
  *          This version uses a LUT for the sin function.  It is not
  *          appreciably faster than using the built-in sin function,
  *          and is here for comparison only.
+ * </pre>
  */
 PIX *
 pixRandomHarmonicWarpLUT(PIX       *pixs,
@@ -361,13 +378,13 @@ PIX        *pixd;
     }
 
     numaDestroy(&na);
-    FREE(randa);
+    LEPT_FREE(randa);
     return pixd;
 }
 
 
 /*!
- *  applyWarpTransformLUT()
+ * \brief   applyWarpTransformLUT()
  *
  *  Notes:
  *      (1) Uses an LUT for computing sin(theta).  There is little speed
@@ -432,7 +449,7 @@ NUMA      *na;
     na = numaCreate(n);
     *pna = na;
     delx = 3.14159265 / (l_float32)npts;
-    numaSetXParameters(na, 0.0, delx);
+    numaSetParameters(na, 0.0, delx);
     for (i = 0; i < n / 2; i++)
          numaAddNumber(na, (l_float32)sin((l_float64)i * delx));
     for (i = 0; i < n / 2; i++) {
@@ -477,54 +494,55 @@ l_float32  twopi, invtwopi, findex, diff;
  *                          Stereoscopic warping                             *
  *---------------------------------------------------------------------------*/
 /*!
- *  pixWarpStereoscopic()
+ * \brief   pixWarpStereoscopic()
  *
- *      Input:  pixs (any depth, colormap ok)
- *              zbend (horizontal separation in pixels of red and cyan
+ * \param[in]    pixs any depth, colormap ok
+ * \param[in]    zbend horizontal separation in pixels of red and cyan
  *                    at the left and right sides, that gives rise to
- *                    quadratic curvature out of the image plane)
- *              zshiftt (uniform pixel translation difference between
+ *                    quadratic curvature out of the image plane
+ * \param[in]    zshiftt uniform pixel translation difference between
  *                      red and cyan, that pushes the top of the image
- *                      plane away from the viewer (zshiftt > 0) or 
- *                      towards the viewer (zshiftt < 0))
- *              zshiftb (uniform pixel translation difference between
+ *                      plane away from the viewer (zshiftt > 0) or
+ *                      towards the viewer (zshiftt < 0)
+ * \param[in]    zshiftb uniform pixel translation difference between
  *                      red and cyan, that pushes the bottom of the image
- *                      plane away from the viewer (zshiftb > 0) or 
- *                      towards the viewer (zshiftb < 0))
- *              ybendt (multiplicative parameter for in-plane vertical
+ *                      plane away from the viewer (zshiftb > 0) or
+ *                      towards the viewer (zshiftb < 0)
+ * \param[in]    ybendt multiplicative parameter for in-plane vertical
  *                      displacement at the left or right edge at the top:
- *                        y = ybendt * (2x/w - 1)^2 )
- *              ybendb (same as ybendt, except at the left or right edge
- *                      at the bottom)
- *              redleft (1 if the red filter is on the left; 0 otherwise)
- *      Return: pixd (32 bpp), or null on error
+ *                        y = ybendt * (2x/w - 1)^2
+ * \param[in]    ybendb same as ybendt, except at the left or right edge
+ *                      at the bottom
+ * \param[in]    redleft 1 if the red filter is on the left; 0 otherwise
+ * \return  pixd 32 bpp, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This function splits out the red channel, mucks around with
  *          it, then recombines with the unmolested cyan channel.
  *      (2) By using a quadratically increasing shift of the red
  *          pixels horizontally and away from the vertical centerline,
  *          the image appears to bend quadratically out of the image
  *          plane, symmetrically with respect to the vertical center
- *          line.  A positive value of @zbend causes the plane to be
+ *          line.  A positive value of %zbend causes the plane to be
  *          curved away from the viewer.  We use linearly interpolated
  *          stretching to avoid the appearance of kinks in the curve.
- *      (3) The parameters @zshiftt and @zshiftb tilt the image plane
+ *      (3) The parameters %zshiftt and %zshiftb tilt the image plane
  *          about a horizontal line through the center, and at the
  *          same time move that line either in toward the viewer or away.
  *          This is implemented by a combination of horizontal shear
  *          about the center line (for the tilt) and horizontal
  *          translation (to move the entire plane in or out).
- *          A positive value of @zshiftt moves the top of the plane
- *          away from the viewer, and a positive value of @zshiftb
+ *          A positive value of %zshiftt moves the top of the plane
+ *          away from the viewer, and a positive value of %zshiftb
  *          moves the bottom of the plane away.  We use linear interpolated
  *          shear to avoid visible vertical steps in the tilted image.
  *      (4) The image can be bent in the plane and about the vertical
  *          centerline.  The centerline does not shift, and the
- *          parameter @ybend gives the relative shift at left and right
- *          edges, with a downward shift for positive values of @ybend.
+ *          parameter %ybend gives the relative shift at left and right
+ *          edges, with a downward shift for positive values of %ybend.
  *      (6) When writing out a steroscopic (red/cyan) image in jpeg,
- *          first call l_jpegSetNoChromaSampling() to get sufficient
+ *          first call pixSetChromaSampling(pix, 0) to get sufficient
  *          resolution in the red channel.
  *      (7) Typical values are:
  *             zbend = 20
@@ -562,6 +580,7 @@ l_float32  twopi, invtwopi, findex, diff;
  *            Pix *pixg = pixCreate(w, h, 8);
  *            pixSetAllArbitrary(pixg, 128);
  *            pixd = pixBlendWithGrayMask(pixrs, pixc, pixg, 0, 0);
+ * </pre>
  */
 PIX *
 pixWarpStereoscopic(PIX     *pixs,
@@ -610,9 +629,9 @@ PIX       *pixd;
         pixDestroy(&pixv2);
         pixDestroy(&pixv3);
         pixDestroy(&pixv4);
-    }
-    else
+    } else {
         pixt2 = pixClone(pixt);
+    }
 
         /* Split out the 3 components */
     pixr = pixGetRGBComponent(pixt2, COLOR_RED);
@@ -632,9 +651,9 @@ PIX       *pixd;
 
         /* Shift the red pixels horizontally by an amount that
          * increases quadratically from the centerline. */
-    if (zbend == 0)
+    if (zbend == 0) {
         pixrs = pixClone(pixr);
-    else {
+    } else {
         pixr1 = pixClipRectangle(pixr, boxleft, NULL);
         pixr2 = pixClipRectangle(pixr, boxright, NULL);
         pixr3 = pixStretchHorizontal(pixr1, L_WARP_TO_LEFT, L_QUADRATIC_WARP,
@@ -653,11 +672,11 @@ PIX       *pixd;
         /* Perform a combination of horizontal shift and shear of
          * red pixels.  The causes the plane of the image to tilt and
          * also move forward or backward. */
-    if (zshiftt == 0 && zshiftb == 0)
+    if (zshiftt == 0 && zshiftb == 0) {
         pixrss = pixClone(pixrs);
-    else if (zshiftt == zshiftb)
+    } else if (zshiftt == zshiftb) {
         pixrss = pixTranslate(NULL, pixrs, zshiftt, 0, L_BRING_IN_WHITE);
-    else {
+    } else {
         angle = (l_float32)(zshiftb - zshiftt) / (l_float32)pixGetHeight(pixrs);
         zshift = (zshiftt + zshiftb) / 2;
         pixt3 = pixTranslate(NULL, pixrs, zshift, 0, L_BRING_IN_WHITE);
@@ -683,30 +702,32 @@ PIX       *pixd;
  *              Linear and quadratic horizontal stretching              *
  *----------------------------------------------------------------------*/
 /*!
- *  pixStretchHorizontal()
+ * \brief   pixStretchHorizontal()
  *
- *      Input:  pixs (1, 8 or 32 bpp)
- *              dir (L_WARP_TO_LEFT or L_WARP_TO_RIGHT)
- *              type (L_LINEAR_WARP or L_QUADRATIC_WARP)
- *              hmax (horizontal displacement at edge)
- *              operation (L_SAMPLED or L_INTERPOLATED)
- *              incolor (L_BRING_IN_WHITE or L_BRING_IN_BLACK)
- *      Return: pixd (stretched/compressed), or null on error
+ * \param[in]    pixs 1, 8 or 32 bpp
+ * \param[in]    dir L_WARP_TO_LEFT or L_WARP_TO_RIGHT
+ * \param[in]    type L_LINEAR_WARP or L_QUADRATIC_WARP
+ * \param[in]    hmax horizontal displacement at edge
+ * \param[in]    operation L_SAMPLED or L_INTERPOLATED
+ * \param[in]    incolor L_BRING_IN_WHITE or L_BRING_IN_BLACK
+ * \return  pixd stretched/compressed, or NULL on error
  *
- *  Notes:
- *      (1) If @hmax > 0, this is an increase in the coordinate value of
+ * <pre>
+ * Notes:
+ *      (1) If %hmax \> 0, this is an increase in the coordinate value of
  *          pixels in pixd, relative to the same pixel in pixs.
- *      (2) If @dir == L_WARP_TO_LEFT, the pixels on the right edge of
- *          the image are not moved. So, for example, if @hmax > 0
- *          and @dir == L_WARP_TO_LEFT, the pixels in pixd are
+ *      (2) If %dir == L_WARP_TO_LEFT, the pixels on the right edge of
+ *          the image are not moved. So, for example, if %hmax \> 0
+ *          and %dir == L_WARP_TO_LEFT, the pixels in pixd are
  *          contracted toward the right edge of the image, relative
  *          to those in pixs.
- *      (3) If @type == L_LINEAR_WARP, the pixel positions are moved
+ *      (3) If %type == L_LINEAR_WARP, the pixel positions are moved
  *          to the left or right by an amount that varies linearly with
  *          the horizontal location.
- *      (4) If @operation == L_SAMPLED, the dest pixels are taken from
+ *      (4) If %operation == L_SAMPLED, the dest pixels are taken from
  *          the nearest src pixel.  Otherwise, we use linear interpolation
  *          between pairs of sampled pixels.
+ * </pre>
  */
 PIX *
 pixStretchHorizontal(PIX     *pixs,
@@ -734,7 +755,7 @@ l_int32  d;
     if (incolor != L_BRING_IN_WHITE && incolor != L_BRING_IN_BLACK)
         return (PIX *)ERROR_PTR("invalid incolor", procName, NULL);
     if (d == 1 && operation == L_INTERPOLATED) {
-        L_WARNING("Using sampling for 1 bpp", procName);
+        L_WARNING("Using sampling for 1 bpp\n", procName);
         operation = L_INTERPOLATED;
     }
 
@@ -746,17 +767,19 @@ l_int32  d;
 
 
 /*!
- *  pixStretchHorizontalSampled()
+ * \brief   pixStretchHorizontalSampled()
  *
- *      Input:  pixs (1, 8 or 32 bpp)
- *              dir (L_WARP_TO_LEFT or L_WARP_TO_RIGHT)
- *              type (L_LINEAR_WARP or L_QUADRATIC_WARP)
- *              hmax (horizontal displacement at edge)
- *              incolor (L_BRING_IN_WHITE or L_BRING_IN_BLACK)
- *      Return: pixd (stretched/compressed), or null on error
+ * \param[in]    pixs 1, 8 or 32 bpp
+ * \param[in]    dir L_WARP_TO_LEFT or L_WARP_TO_RIGHT
+ * \param[in]    type L_LINEAR_WARP or L_QUADRATIC_WARP
+ * \param[in]    hmax horizontal displacement at edge
+ * \param[in]    incolor L_BRING_IN_WHITE or L_BRING_IN_BLACK
+ * \return  pixd stretched/compressed, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixStretchHorizontal() for details.
+ * </pre>
  */
 PIX *
 pixStretchHorizontalSampled(PIX     *pixs,
@@ -796,8 +819,7 @@ PIX       *pixd;
                 j = jd - (hmax * (wm - jd)) / wm;
             else  /* L_QUADRATIC_WARP */
                 j = jd - (hmax * (wm - jd) * (wm - jd)) / (wm * wm);
-        }
-        else if (dir == L_WARP_TO_RIGHT) {
+        } else if (dir == L_WARP_TO_RIGHT) {
             if (type == L_LINEAR_WARP)
                 j = jd - (hmax * jd) / wm;
             else  /* L_QUADRATIC_WARP */
@@ -832,7 +854,7 @@ PIX       *pixd;
             }
             break;
         default:
-            L_ERROR_INT("invalid depth: %d", procName, d);
+            L_ERROR("invalid depth: %d\n", procName, d);
             pixDestroy(&pixd);
             return NULL;
         }
@@ -843,17 +865,19 @@ PIX       *pixd;
 
 
 /*!
- *  pixStretchHorizontalLI()
+ * \brief   pixStretchHorizontalLI()
  *
- *      Input:  pixs (1, 8 or 32 bpp)
- *              dir (L_WARP_TO_LEFT or L_WARP_TO_RIGHT)
- *              type (L_LINEAR_WARP or L_QUADRATIC_WARP)
- *              hmax (horizontal displacement at edge)
- *              incolor (L_BRING_IN_WHITE or L_BRING_IN_BLACK)
- *      Return: pixd (stretched/compressed), or null on error
+ * \param[in]    pixs 1, 8 or 32 bpp
+ * \param[in]    dir L_WARP_TO_LEFT or L_WARP_TO_RIGHT
+ * \param[in]    type L_LINEAR_WARP or L_QUADRATIC_WARP
+ * \param[in]    hmax horizontal displacement at edge
+ * \param[in]    incolor L_BRING_IN_WHITE or L_BRING_IN_BLACK
+ * \return  pixd stretched/compressed, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixStretchHorizontal() for details.
+ * </pre>
  */
 PIX *
 pixStretchHorizontalLI(PIX     *pixs,
@@ -895,8 +919,7 @@ PIX       *pixd;
                 j = 64 * jd - 64 * (hmax * (wm - jd)) / wm;
             else  /* L_QUADRATIC_WARP */
                 j = 64 * jd - 64 * (hmax * (wm - jd) * (wm - jd)) / (wm * wm);
-        }
-        else if (dir == L_WARP_TO_RIGHT) {
+        } else if (dir == L_WARP_TO_RIGHT) {
             if (type == L_LINEAR_WARP)
                 j = 64 * jd - 64 * (hmax * jd) / wm;
             else  /* L_QUADRATIC_WARP */
@@ -917,8 +940,7 @@ PIX       *pixd;
                            jf * GET_DATA_BYTE(lines, jp + 1) + 31) / 63;
                     SET_DATA_BYTE(lined, jd, val);
                 }
-            }
-            else {  /* jp == wm */
+            } else {  /* jp == wm */
                 for (i = 0; i < h; i++) {
                     lines = datas + i * wpls;
                     lined = datad + i * wpld;
@@ -942,8 +964,7 @@ PIX       *pixd;
                            jf * ((word1 >> L_BLUE_SHIFT) & 0xff) + 31) / 63;
                     composeRGBPixel(rval, gval, bval, lined + jd);
                 }
-            }
-            else {  /* jp == wm */
+            } else {  /* jp == wm */
                 for (i = 0; i < h; i++) {
                     lines = datas + i * wpls;
                     lined = datad + i * wpld;
@@ -952,7 +973,7 @@ PIX       *pixd;
             }
             break;
         default:
-            L_ERROR_INT("invalid depth: %d", procName, d);
+            L_ERROR("invalid depth: %d\n", procName, d);
             pixDestroy(&pixd);
             return NULL;
         }
@@ -966,31 +987,33 @@ PIX       *pixd;
  *                       Quadratic vertical shear                       *
  *----------------------------------------------------------------------*/
 /*!
- *  pixQuadraticVShear()
+ * \brief   pixQuadraticVShear()
  *
- *      Input:  pixs (1, 8 or 32 bpp)
- *              dir (L_WARP_TO_LEFT or L_WARP_TO_RIGHT)
- *              vmaxt (max vertical displacement at edge and at top)
- *              vmaxb (max vertical displacement at edge and at bottom)
- *              operation (L_SAMPLED or L_INTERPOLATED)
- *              incolor (L_BRING_IN_WHITE or L_BRING_IN_BLACK)
- *      Return: pixd (stretched), or null on error
+ * \param[in]    pixs 1, 8 or 32 bpp
+ * \param[in]    dir L_WARP_TO_LEFT or L_WARP_TO_RIGHT
+ * \param[in]    vmaxt max vertical displacement at edge and at top
+ * \param[in]    vmaxb max vertical displacement at edge and at bottom
+ * \param[in]    operation L_SAMPLED or L_INTERPOLATED
+ * \param[in]    incolor L_BRING_IN_WHITE or L_BRING_IN_BLACK
+ * \return  pixd stretched, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) This gives a quadratic bending, upward or downward, as you
  *          move to the left or right.
- *      (2) If @dir == L_WARP_TO_LEFT, the right edge is unchanged, and
+ *      (2) If %dir == L_WARP_TO_LEFT, the right edge is unchanged, and
  *          the left edge pixels are moved maximally up or down.
- *      (3) Parameters @vmaxt and @vmaxb control the maximum amount of
+ *      (3) Parameters %vmaxt and %vmaxb control the maximum amount of
  *          vertical pixel shear at the top and bottom, respectively.
- *          If @vmaxt > 0, the vertical displacement of pixels at the
- *          top is downward.  Likewise, if @vmaxb > 0, the vertical
+ *          If %vmaxt \> 0, the vertical displacement of pixels at the
+ *          top is downward.  Likewise, if %vmaxb \> 0, the vertical
  *          displacement of pixels at the bottom is downward.
- *      (4) If @operation == L_SAMPLED, the dest pixels are taken from
+ *      (4) If %operation == L_SAMPLED, the dest pixels are taken from
  *          the nearest src pixel.  Otherwise, we use linear interpolation
  *          between pairs of sampled pixels.
  *      (5) This is for quadratic shear.  For uniform (linear) shear,
  *          use the standard shear operators.
+ * </pre>
  */
 PIX *
 pixQuadraticVShear(PIX     *pixs,
@@ -1020,7 +1043,7 @@ l_int32    w, h, d;
         return pixCopy(NULL, pixs);
 
     if (operation == L_INTERPOLATED && d == 1) {
-        L_WARNING("no interpolation for 1 bpp; using sampling", procName);
+        L_WARNING("no interpolation for 1 bpp; using sampling\n", procName);
         operation = L_SAMPLED;
     }
 
@@ -1032,17 +1055,19 @@ l_int32    w, h, d;
 
 
 /*!
- *  pixQuadraticVShearSampled()
+ * \brief   pixQuadraticVShearSampled()
  *
- *      Input:  pixs (1, 8 or 32 bpp)
- *              dir (L_WARP_TO_LEFT or L_WARP_TO_RIGHT)
- *              vmaxt (max vertical displacement at edge and at top)
- *              vmaxb (max vertical displacement at edge and at bottom)
- *              incolor (L_BRING_IN_WHITE or L_BRING_IN_BLACK)
- *      Return: pixd (stretched), or null on error
+ * \param[in]    pixs 1, 8 or 32 bpp
+ * \param[in]    dir L_WARP_TO_LEFT or L_WARP_TO_RIGHT
+ * \param[in]    vmaxt max vertical displacement at edge and at top
+ * \param[in]    vmaxb max vertical displacement at edge and at bottom
+ * \param[in]    incolor L_BRING_IN_WHITE or L_BRING_IN_BLACK
+ * \return  pixd stretched, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixQuadraticVShear() for details.
+ * </pre>
  */
 PIX *
 pixQuadraticVShearSampled(PIX     *pixs,
@@ -1085,8 +1110,7 @@ PIX       *pixd;
         if (dir == L_WARP_TO_LEFT) {
             delrowt = (l_float32)(vmaxt * (wm - j) * (wm - j)) * denom2;
             delrowb = (l_float32)(vmaxb * (wm - j) * (wm - j)) * denom2;
-        }
-        else if (dir == L_WARP_TO_RIGHT) {
+        } else if (dir == L_WARP_TO_RIGHT) {
             delrowt = (l_float32)(vmaxt * j * j) * denom2;
             delrowb = (l_float32)(vmaxb * j * j) * denom2;
         }
@@ -1126,7 +1150,7 @@ PIX       *pixd;
             }
             break;
         default:
-            L_ERROR_INT("invalid depth: %d", procName, d);
+            L_ERROR("invalid depth: %d\n", procName, d);
             pixDestroy(&pixd);
             return NULL;
         }
@@ -1137,17 +1161,19 @@ PIX       *pixd;
 
 
 /*!
- *  pixQuadraticVShearLI()
+ * \brief   pixQuadraticVShearLI()
  *
- *      Input:  pixs (8 or 32 bpp, or colormapped)
- *              dir (L_WARP_TO_LEFT or L_WARP_TO_RIGHT)
- *              vmaxt (max vertical displacement at edge and at top)
- *              vmaxb (max vertical displacement at edge and at bottom)
- *              incolor (L_BRING_IN_WHITE or L_BRING_IN_BLACK)
- *      Return: pixd (stretched), or null on error
+ * \param[in]    pixs 8 or 32 bpp, or colormapped
+ * \param[in]    dir L_WARP_TO_LEFT or L_WARP_TO_RIGHT
+ * \param[in]    vmaxt max vertical displacement at edge and at top
+ * \param[in]    vmaxb max vertical displacement at edge and at bottom
+ * \param[in]    incolor L_BRING_IN_WHITE or L_BRING_IN_BLACK
+ * \return  pixd stretched, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) See pixQuadraticVShear() for details.
+ * </pre>
  */
 PIX *
 pixQuadraticVShearLI(PIX     *pixs,
@@ -1208,8 +1234,7 @@ PIXCMAP   *cmap;
         if (dir == L_WARP_TO_LEFT) {
             delrowt = (l_float32)(vmaxt * (wm - j) * (wm - j)) * denom2;
             delrowb = (l_float32)(vmaxb * (wm - j) * (wm - j)) * denom2;
-        }
-        else if (dir == L_WARP_TO_RIGHT) {
+        } else if (dir == L_WARP_TO_RIGHT) {
             delrowt = (l_float32)(vmaxt * j * j) * denom2;
             delrowb = (l_float32)(vmaxb * j * j) * denom2;
         }
@@ -1224,11 +1249,12 @@ PIXCMAP   *cmap;
                 if (yp < 0 || yp > hm) continue;
                 lines = datas + yp * wpls;
                 lined = datad + id * wpld;
-                if (yp < hm)
+                if (yp < hm) {
                     val = ((63 - yf) * GET_DATA_BYTE(lines, j) +
                            yf * GET_DATA_BYTE(lines + wpls, j) + 31) / 63;
-                else  /* yp == hm */
+                } else {  /* yp == hm */
                     val = GET_DATA_BYTE(lines, j);
+                }
                 SET_DATA_BYTE(lined, j, val);
             }
             break;
@@ -1251,14 +1277,13 @@ PIXCMAP   *cmap;
                     bval = ((63 - yf) * ((word0 >> L_BLUE_SHIFT) & 0xff) +
                            yf * ((word1 >> L_BLUE_SHIFT) & 0xff) + 31) / 63;
                     composeRGBPixel(rval, gval, bval, lined + j);
-                }
-                else {  /* yp == hm */
+                } else {  /* yp == hm */
                     lined[j] = lines[j];
                 }
             }
             break;
         default:
-            L_ERROR_INT("invalid depth: %d", procName, d);
+            L_ERROR("invalid depth: %d\n", procName, d);
             pixDestroy(&pix);
             pixDestroy(&pixd);
             return NULL;
@@ -1274,21 +1299,22 @@ PIXCMAP   *cmap;
  *                     Stereo from a pair of images                     *
  *----------------------------------------------------------------------*/
 /*!
- *  pixStereoFromPair()
+ * \brief   pixStereoFromPair()
  *
- *      Input:  pix1 (32 bpp rgb)
- *              pix2 (32 bpp rgb)
- *              rwt, gwt, bwt (weighting factors used for each component in
-                               pix1 to determine the output red channel)
- *      Return: pixd (stereo enhanced), or null on error
+ * \param[in]    pix1 32 bpp rgb
+ * \param[in]    pix2 32 bpp rgb
+ * \param[in]    rwt, gwt, bwt weighting factors used for each component in
+                               pix1 to determine the output red channel
+ * \return  pixd stereo enhanced, or NULL on error
  *
- *  Notes:
+ * <pre>
+ * Notes:
  *      (1) pix1 and pix2 are a pair of stereo images, ideally taken
  *          concurrently in the same plane, with some lateral translation.
- *      (2) The output red channel is determined from @pix1.
+ *      (2) The output red channel is determined from %pix1.
  *          The output green and blue channels are taken from the green
- *          and blue channels, respectively, of @pix2.
- *      (3) The weights determine how much of each component in @pix1
+ *          and blue channels, respectively, of %pix2.
+ *      (3) The weights determine how much of each component in %pix1
  *          goes into the output red channel.  The sum of weights
  *          must be 1.0.  If it's not, we scale the weights to
  *          satisfy this criterion.
@@ -1298,8 +1324,9 @@ PIXCMAP   *cmap;
  *            bval = b2   (from pix2)
  *      (5) The simplest method is to use rwt = 1.0, gwt = 0.0, bwt = 0.0,
  *          but this causes unpleasant visual artifacts with red in the image.
- *          Use of green and blue from @pix1 in the red channel,
+ *          Use of green and blue from %pix1 in the red channel,
  *          instead of red, tends to fix that problem.
+ * </pre>
  */
 PIX *
 pixStereoFromPair(PIX       *pix1,
@@ -1330,7 +1357,7 @@ PIX       *pixd;
     }
     sum = rwt + gwt + bwt;
     if (L_ABS(sum - 1.0) > 0.0001) {  /* maintain ratios with sum == 1.0 */
-        L_WARNING("weights don't sum to 1; maintaining ratios", procName);
+        L_WARNING("weights don't sum to 1; maintaining ratios\n", procName);
         rwt = rwt / sum;
         gwt = gwt / sum;
         bwt = bwt / sum;
@@ -1361,4 +1388,3 @@ PIX       *pixd;
 
     return pixd;
 }
-
